@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -102,6 +102,8 @@ static void account_edit_close		(GtkWidget *widget, gpointer data);
 static gint account_delete_event	(GtkWidget	*widget,
 					 GdkEventAny	*event,
 					 gpointer	 data);
+static void account_size_allocate_cb(GtkWidget *widget,
+					 GtkAllocation *allocation);
 static gboolean account_key_pressed	(GtkWidget	*widget,
 					 GdkEventKey	*event,
 					 gpointer	 data);
@@ -347,6 +349,11 @@ void account_set_menu(void)
 	main_window_set_account_menu(account_list);
 }
 
+void account_set_menu_only_toolbar(void)
+{
+	main_window_set_account_menu_only_toolbar(account_list);
+}
+
 void account_foreach(AccountFunc func, gpointer user_data)
 {
 	GList *cur;
@@ -509,6 +516,37 @@ void account_set_missing_folder(void)
 	}
 }
 
+#define CHECK_CHANGE_FOLDER(folder) {						\
+	if (folder && !strncmp(folder, old_id, strlen(old_id))) {		\
+		if (strlen(folder) == strlen(old_id)) {				\
+			g_free(folder);						\
+			folder = g_strdup(new_id);				\
+		} else if (strlen(folder) > strlen(old_id)			\
+		  && folder[strlen(old_id)] == G_DIR_SEPARATOR) {		\
+			gchar *new_path = g_strdup_printf("%s%s",		\
+					new_id, (folder + strlen(old_id)));	\
+			g_free(folder);						\
+			folder = new_path;					\
+		} 								\
+	}									\
+}
+
+void account_rename_path(const gchar *old_id, const gchar *new_id)
+{
+	GList *cur = account_list;
+	for (; cur != NULL; cur = g_list_next(cur)) {
+		PrefsAccount *ap = (PrefsAccount *)cur->data;
+		CHECK_CHANGE_FOLDER(ap->inbox);
+		CHECK_CHANGE_FOLDER(ap->local_inbox);
+		CHECK_CHANGE_FOLDER(ap->queue_folder);
+		CHECK_CHANGE_FOLDER(ap->sent_folder);
+		CHECK_CHANGE_FOLDER(ap->draft_folder);
+		CHECK_CHANGE_FOLDER(ap->trash_folder);
+	}
+}
+
+#undef CHECK_CHANGE_FOLDER
+
 FolderItem *account_get_special_folder(PrefsAccount *ac_prefs,
 				       SpecialFolderItemType type)
 {
@@ -598,6 +636,17 @@ void account_destroy(PrefsAccount *ac_prefs)
 	}
 }
 
+/*!
+ *\brief	Save Gtk object size to prefs dataset
+ */
+static void account_size_allocate_cb(GtkWidget *widget,
+					 GtkAllocation *allocation)
+{
+	g_return_if_fail(allocation != NULL);
+
+	prefs_common.accountswin_width = allocation->width;
+	prefs_common.accountswin_height = allocation->height;
+}
 
 static void account_edit_create(void)
 {
@@ -621,10 +670,11 @@ static void account_edit_create(void)
 	GtkWidget *hbbox;
 	GtkWidget *close_btn;
 
+	static GdkGeometry geometry;
+
 	debug_print("Creating account edit window...\n");
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_widget_set_size_request (window, 500, 350);
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
 	gtk_window_set_title (GTK_WINDOW (window), _("Edit accounts"));
 	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
@@ -645,7 +695,7 @@ static void account_edit_create(void)
 
 	label = gtk_label_new
 		(_("New messages will be checked in this order. Check the boxes\n"
-		   "on the 'G' column to enable message retrieval by 'Get all'."));
+		   "on the 'G' column to enable message retrieval by 'Get Mail'."));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4);
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
@@ -742,6 +792,19 @@ static void account_edit_create(void)
 
 	account_create_list_view_images(list_view);
 
+	g_signal_connect(G_OBJECT(window), "size_allocate",
+			 G_CALLBACK(account_size_allocate_cb), NULL);
+
+	if (!geometry.min_height) {
+		geometry.min_width = 500;
+		geometry.min_height = 350;
+	}
+
+	gtk_window_set_geometry_hints(GTK_WINDOW(window), NULL, &geometry,
+				      GDK_HINT_MIN_SIZE);
+	gtk_widget_set_size_request(window, prefs_common.accountswin_width,
+				    prefs_common.accountswin_height);
+
 	edit_account.window    = window;
 	edit_account.list_view = list_view;
 	edit_account.close_btn = close_btn;
@@ -945,8 +1008,8 @@ static void account_delete(GtkWidget *widget, gpointer data)
 		   ac_prefs->account_name ? ac_prefs->account_name :
 		   _("(Untitled)"));
 	if (alertpanel_full(_("Delete account"), buf,
-		 	    GTK_STOCK_YES, GTK_STOCK_NO, NULL, FALSE,
-			    NULL, ALERT_WARNING, G_ALERTALTERNATE) != G_ALERTDEFAULT)
+		 	    GTK_STOCK_CANCEL, GTK_STOCK_DELETE, NULL, FALSE,
+			    NULL, ALERT_WARNING, G_ALERTDEFAULT) != G_ALERTALTERNATE)
 		return;
 
 	if (ac_prefs->folder) {
