@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto & The Sylpheed Claws Team
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto & The Sylpheed Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -54,6 +54,7 @@
 #include "textview.h"
 #include "matcher_parser.h" /* CLAWS */
 #include "filtering.h"
+#include "procheader.h"
 
 typedef struct _Children		Children;
 typedef struct _ChildInfo		ChildInfo;
@@ -99,6 +100,10 @@ struct _ChildInfo
 	gint		 new_out;
 
 	GString		*output;
+	void (*callback)(void *data);
+	void *data;
+
+	GSList		*msginfo_list;
 };
 
 static void action_update_menu		(GtkItemFactory	*ifactory,
@@ -125,7 +130,9 @@ static gboolean execute_actions		(gchar		*action,
 					 GSList		*msg_list, 
 					 GtkWidget	*text,
 					 gint		 body_pos,
-					 MimeInfo	*partinfo);
+					 MimeInfo	*partinfo, 
+					 void (*callback)(void *data),
+					 void *data);
 
 static gchar *parse_action_cmd		(gchar		*action,
 					 MsgInfo	*msginfo,
@@ -473,7 +480,8 @@ static void action_update_menu(GtkItemFactory *ifactory,
 		action   = g_strdup((gchar *)cur->data);
 		action_p = strstr(action, ": ");
 		if (action_p && action_p[2] &&
-		    action_get_type(&action_p[2]) != ACTION_ERROR) {
+		    (action_get_type(&action_p[2]) != ACTION_ERROR) &&
+		    (action[0] != '/')) {
 			action_p[0] = '\0';
 			menu_path = g_strdup_printf("%s/%s", branch_path,
 						    action);
@@ -511,7 +519,8 @@ static void compose_actions_execute_cb(Compose *compose, guint action_nb,
 		return;
 	}
 
-	execute_actions(action, NULL, compose->text, 0, NULL);
+	execute_actions(action, NULL, compose->text, 0, NULL, 
+		compose_action_cb, compose);
 }
 
 static void mainwin_actions_execute_cb(MainWindow *mainwin, guint action_nb,
@@ -573,7 +582,8 @@ static void message_actions_execute(MessageView *msgview, guint action_nb,
 		 * filtering */
 		execute_filtering_actions(action, msg_list);
 	else
-		execute_actions(action, msg_list, text, body_pos, partinfo);
+		execute_actions(action, msg_list, text, body_pos, partinfo,
+			NULL, NULL);
 }
 
 static gboolean execute_filtering_actions(gchar *action, GSList *msglist)
@@ -581,7 +591,14 @@ static gboolean execute_filtering_actions(gchar *action, GSList *msglist)
 	GSList *action_list, *p;
 	const gchar *sbegin, *send;
 	gchar *action_string;
-	
+	SummaryView *summaryview = NULL;
+	MainWindow *mainwin = NULL;
+
+	if (mainwindow_get_mainwindow()) {
+		summaryview = mainwindow_get_mainwindow()->summaryview;
+		mainwin = mainwindow_get_mainwindow();
+	}
+
 	if (NULL == (sbegin = strstr2(action, "%as{")))
 		return FALSE;
 	sbegin += sizeof "%as{" - 1;
@@ -594,9 +611,27 @@ static gboolean execute_filtering_actions(gchar *action, GSList *msglist)
 	if (action_list == NULL) return FALSE;
 	
 	/* apply actions on each message info */
-	for (p = msglist; p && p->data; p = g_slist_next(p))
+	for (p = msglist; p && p->data; p = g_slist_next(p)) {
 		filteringaction_apply_action_list(action_list, (MsgInfo *) p->data);
 		
+	}
+
+	if (summaryview) {
+		summary_lock(summaryview);				
+		main_window_cursor_wait(mainwin);		
+		gtk_clist_freeze(GTK_CLIST(summaryview->ctree));	
+		folder_item_update_freeze();				
+	}
+
+	filtering_move_and_copy_msgs(msglist);
+
+	if (summaryview) {
+		folder_item_update_thaw();				
+		gtk_clist_thaw(GTK_CLIST(summaryview->ctree));		
+		main_window_cursor_normal(mainwin);	
+		summary_unlock(summaryview);				
+		summary_show(summaryview, summaryview->folder_item);
+	}
 	for (p = action_list; p; p = g_slist_next(p))
 		if (p->data) filteringaction_free(p->data);	
 	g_slist_free(action_list);		
@@ -605,7 +640,8 @@ static gboolean execute_filtering_actions(gchar *action, GSList *msglist)
 
 static gboolean execute_actions(gchar *action, GSList *msg_list,
 				GtkWidget *text,
-				gint body_pos, MimeInfo *partinfo)
+				gint body_pos, MimeInfo *partinfo,
+				void (*callback)(void *data), void *data)
 {
 	GSList *children_list = NULL;
 	gint is_ok  = TRUE;
@@ -716,6 +752,8 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 				break;
 			}
 			if ((child_info = fork_child(cmd, msg_str, children))) {
+				/* Pass msginfo to catch_status () */
+				child_info->msginfo_list = g_slist_append (NULL, msginfo);
 				children_list = g_slist_append(children_list,
 							       child_info);
 			}
@@ -726,8 +764,9 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 				       user_str, user_hidden_str, sel_str);
 		if (cmd) {
 			if ((child_info = fork_child(cmd, msg_str, children))) {
+				child_info->msginfo_list = g_slist_copy (msg_list);
 				children_list = g_slist_append(children_list,
-							       child_info);
+								child_info);
 			}
 			g_free(cmd);
 		} else
@@ -751,6 +790,8 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 
 		for (cur = children_list; cur; cur = cur->next) {
 			child_info = (ChildInfo *) cur->data;
+			child_info->callback = callback;
+			child_info->data = data;
 			child_info->tag_status = 
 				gdk_input_add(child_info->chld_status,
 					      GDK_INPUT_READ,
@@ -759,7 +800,6 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 
 		create_io_dialog(children);
 	}
-
 	return is_ok;
 }
 
@@ -1061,6 +1101,8 @@ static void childinfo_close_pipes(ChildInfo *child_info)
 static void free_children(Children *children)
 {
 	ChildInfo *child_info;
+	void (*callback)(void *data) = NULL;
+	void *data = NULL;
 
 	debug_print("Freeing children data %p\n", children);
 
@@ -1070,8 +1112,14 @@ static void free_children(Children *children)
 		g_free(child_info->cmd);
 		g_string_free(child_info->output, TRUE);
 		children->list = g_slist_remove(children->list, child_info);
+		callback = child_info->callback;
+		data = child_info->data;
 		g_free(child_info);
 	}
+
+	if (callback)
+		callback(data);
+	
 	g_free(children);
 }
 
@@ -1288,6 +1336,49 @@ static void catch_status(gpointer data, gint source, GdkInputCondition cond)
 	childinfo_close_pipes(child_info);
 	child_info->pid = 0;
 
+	if (child_info->children->action_type & (ACTION_SINGLE | ACTION_MULTIPLE)
+	    && child_info->msginfo_list) {
+	/* Actions on message *files* might change size and
+	* time stamp, and thus invalidate the cache */
+	SummaryView *summaryview  = NULL;
+	GSList      *cur;
+	MsgInfo     *msginfo, *nmi;	/* newmsginfo */
+	char        *file;
+
+	if (mainwindow_get_mainwindow ())
+		summaryview = mainwindow_get_mainwindow ()->summaryview;
+	for (cur = child_info->msginfo_list; cur; cur = cur->next) {
+		msginfo = (MsgInfo *)cur->data;
+		if (!(msginfo && /* Stuff used valid? */
+		    msginfo->folder && msginfo->folder->cache)) 
+			continue;
+		file = procmsg_get_message_file_path (msginfo);
+		if (!file) 
+			continue;
+		nmi = procheader_parse_file (file, msginfo->flags, FALSE, FALSE);
+		if (!nmi) 
+			continue; /* Deleted? */
+		if (msginfo->mtime != nmi->mtime || msginfo->size != nmi->size) {
+			msginfo->mtime  = nmi->mtime;
+			msginfo->size   = nmi->size;
+			msgcache_update_msg (msginfo->folder->cache, msginfo);
+		}
+		procmsg_msginfo_free (nmi);
+
+		/* refresh the message display, but only if this message is still
+		* in focus */
+		if (summaryview
+		    && summaryview->displayed
+		    && summaryview->folder_item == msginfo->folder
+		    && summary_get_msgnum (summaryview, summaryview->displayed) == msginfo->msgnum) {
+			summary_show (summaryview, summaryview->folder_item);
+			summary_select_node (summaryview, summaryview->displayed, TRUE, TRUE);
+		}
+	}
+	g_slist_free (child_info->msginfo_list);
+	child_info->msginfo_list = NULL;
+	}
+
 	wait_for_children(child_info->children);
 }
 	
