@@ -1,7 +1,7 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
  *
- * Copyright (C) 2000-2005 by Alfons Hoogervorst & The Sylpheed Claws Team. 
+ * Copyright (C) 2000-2006 by Alfons Hoogervorst & The Sylpheed Claws Team.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -46,6 +46,7 @@
 #include "addr_compl.h"
 #include "utils.h"
 #include "prefs_common.h"
+#include "sylpheed.h"
 #include <pthread.h>
 
 /*!
@@ -210,9 +211,8 @@ static void add_address1(const char *str, address_entry *ae)
 {
 	completion_entry *ce1;
 	ce1 = g_new0(completion_entry, 1),
-	ce1->string = g_utf8_strdown(str, -1);
 	/* GCompletion list is case sensitive */
-	g_strdown(ce1->string);
+	ce1->string = g_utf8_strdown(str, -1);
 	ce1->ref = ae;
 
 	g_completion_list = g_list_prepend(g_completion_list, ce1);
@@ -710,37 +710,6 @@ static void completion_window_advance_selection(GtkTreeView *list_view, gboolean
 	}
 }
 
-#if 0
-/* completion_window_accept_selection() - accepts the current selection in the
- * clist, and destroys the window */
-static void completion_window_accept_selection(GtkWidget **window,
-					       GtkCList *clist,
-					       GtkEntry *entry)
-{
-	gchar *address = NULL, *text = NULL;
-	gint   cursor_pos, row;
-
-	g_return_if_fail(window != NULL);
-	g_return_if_fail(*window != NULL);
-	g_return_if_fail(clist != NULL);
-	g_return_if_fail(entry != NULL);
-	g_return_if_fail(clist->selection != NULL);
-
-	/* FIXME: I believe it's acceptable to access the selection member directly  */
-	row = GPOINTER_TO_INT(clist->selection->data);
-
-	/* we just need the cursor position */
-	address = get_address_from_edit(entry, &cursor_pos);
-	g_free(address);
-	gtk_clist_get_text(clist, row, 0, &text);
-	replace_address_in_edit(entry, text, cursor_pos);
-
-	clear_completion_cache();
-	gtk_widget_destroy(*window);
-	*window = NULL;
-}
-#endif
-
 /**
  * Resize window to accommodate maximum number of address entries.
  * \param cw Completion window.
@@ -826,6 +795,7 @@ static gboolean addrcompl_idle( gpointer data ) {
 		_displayQueue_ = NULL;
 	}
 	pthread_mutex_unlock( & _completionMutex_ );
+	sylpheed_do_idle();
 
 	return TRUE;
 }
@@ -1144,6 +1114,7 @@ static gboolean address_completion_complete_address_in_entry(GtkEntry *entry,
 		g_free( new );
 	}
 
+#ifndef USE_LDAP
 	/* Select the address if there is only one match */
 	if (ncount == 2) {
 		/* Display selected address in entry field */		
@@ -1158,7 +1129,9 @@ static gboolean address_completion_complete_address_in_entry(GtkEntry *entry,
 		clear_completion_cache();
 	}
 	/* Make sure that drop-down appears uniform! */
-	else if( ncount == 0 ) {
+	else 
+#endif
+	if( ncount == 0 ) {
 		addrcompl_add_queue( g_strdup( searchTerm ) );
 	}
 	g_free( searchTerm );
@@ -1323,21 +1296,6 @@ static gboolean completion_window_key_press(GtkWidget *widget,
 		return FALSE;
 	}		
 
-#if 0	
-	/* also make tab / shift tab go to next previous completion entry. we're
-	 * changing the key value */
-	if (event->keyval == GDK_Tab || event->keyval == GDK_ISO_Left_Tab) {
-		event->keyval = (event->state & GDK_SHIFT_MASK)
-			? GDK_Up : GDK_Down;
-		/* need to reset shift state if going up */
-		if (event->state & GDK_SHIFT_MASK)
-			event->state &= ~GDK_SHIFT_MASK;
-		completion_window_advance_selection(GTK_CLIST(clist), 
-			event->keyval == GDK_Down ? TRUE : FALSE);
-		return FALSE;
-	}
-#endif
-
 	/* make tab move to next field */
 	if( event->keyval == GDK_Tab ) {
 		/* Reference to parent */
