@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2001-2002 Match Grun and the Sylpheed-Claws team
+ * Copyright (C) 2001-2006 Match Grun and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /* General functions for accessing address book files */
@@ -508,6 +508,7 @@ static void addrbook_parse_attribute(XMLFile *file, ItemPerson *person)
 
 	element = xml_get_element(file);
 	addritem_attrib_set_value(uAttr, element);
+	g_free(element);
 
 	if (uAttr) {
 		if (person) {
@@ -1722,29 +1723,6 @@ ItemFolder *addrbook_add_new_folder(AddressBookFile *book, ItemFolder *parent)
 	g_return_val_if_fail(book != NULL, NULL);
 	return addrcache_add_new_folder( book->addressCache, parent );
 }
-#if 0
-{
-	ItemFolder *folder = NULL;
-	ItemFolder *p = parent;
-
-	g_return_val_if_fail(book != NULL, NULL);
-
-	if (!p)
-		p = book->addressCache->rootFolder;
-	folder = addritem_create_item_folder();
-	addrcache_id_folder(book->addressCache, folder);
-	if (addrcache_hash_add_folder(book->addressCache, folder)) {
-		p->listFolder = g_list_append(p->listFolder, folder);
-		ADDRITEM_PARENT(folder) = ADDRITEM_OBJECT(p);
-		addrcache_set_dirty(book->addressCache, TRUE);
-	}
-	else {
-		addritem_free_item_folder(folder);
-		folder = NULL;
-	}
-	return folder;
-}
-#endif
 
 /**
  * Update address book attribute list for specified person. Note: The existing
