@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2001-2004 Match Grun
+ * Copyright (C) 2001-2006 Match Grun and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /*
@@ -55,6 +55,7 @@
 #include "ldapserver.h"
 #include "ldapctrl.h"
 #include "ldapquery.h"
+#include "ldaputil.h"
 #endif
 
 #define TAG_ADDRESS_INDEX    "addressbook"
@@ -110,13 +111,8 @@
 /* New attributes */
 #define ATTAG_LDAP_DEFAULT    "default"
 
-#if 0
-N_("Common address")
-N_("Personal address")
-#endif
-
-#define DISP_NEW_COMMON       _("Common address")
-#define DISP_NEW_PERSONAL     _("Personal address")
+#define DISP_NEW_COMMON       _("Common addresses")
+#define DISP_NEW_PERSONAL     _("Personal addresses")
 
 /* Old address book */
 #define TAG_IF_OLD_COMMON     "common_address"
@@ -1079,37 +1075,6 @@ static void addrindex_write_fragment(
 	}
 }
 
-#if 0
-static void addrindex_print_fragment_r(
-		const AddressIfFragment *fragment, FILE *stream, gint lvl )
-{
-	GList *node;
-	gint i;
-
-	for( i = 0; i < lvl; i++ )
-		fprintf( stream, "  " );
-	fprintf( stream, "Element:%s:\n", fragment->name );
-	node = fragment->attributes;
-	while( node ) {
-		AddressIfAttrib *nv = node->data;
-		for( i = 0; i < lvl; i++ )
-			fprintf( stream, "  " );
-		fprintf( stream, "    %s : %s\n", nv->name, nv->value );
-		node = g_list_next( node );
-	}
-	node = fragment->children;
-	while( node ) {
-		AddressIfFragment *child = node->data;
-		addrindex_print_fragment_r( child, stream, 1+lvl );
-		node = g_list_next( node );
-	}
-}
-
-static void addrindex_print_fragment( const AddressIfFragment *fragment, FILE *stream ) {
-	addrindex_print_fragment_r( fragment, stream, 0 );
-}
-#endif
-
 /**
  * Read/parse address index file, creating a data source for a regular
  * intrinsic XML addressbook.
@@ -1336,6 +1301,7 @@ static void addrindex_parse_ldap_attrlist( XMLFile *file, LdapControl *ctl ) {
 
 }
 
+void ldapsvr_set_control( LdapServer *server, LdapControl *ctl );
 /**
  * Parse LDAP control data from XML file.
  * \param  file Index file.
@@ -1850,6 +1816,7 @@ static AddressCvtNode *addrindex_parse_item( XMLFile *file ) {
 		if( xml_compare_tag( file, ELTAG_IF_OLD_REMARKS ) ) {
 			nn->remarks = g_strdup( element );
 		}
+		g_free(element);
 		xml_parse_next_tag(file);
 	}
 }
@@ -2538,6 +2505,7 @@ LdapQuery *ldapsvr_new_dynamic_search(
 		LdapServer *server, QueryRequest *req );
 LdapQuery *ldapsvr_new_explicit_search(
 		LdapServer *server, QueryRequest *req, ItemFolder *folder );
+void ldapsvr_execute_query( LdapServer *server, LdapQuery *qry );
 
 #endif
 
@@ -2651,12 +2619,21 @@ gint addrindex_setup_explicit_search(
 	QueryRequest *req;
 	gint queryID;
 	gchar *name;
+	gchar *mySearch;
 
 	/* Name the query */
 	name = g_strdup_printf( "Search '%s'", searchTerm );
 
 	/* Set up query request */
-	req = qrymgr_add_request( searchTerm, callBackEnd, callBackEntry );
+	if (!strcmp(searchTerm, "*"))
+		mySearch = g_strdup("*@");
+	else
+		mySearch = g_strdup(searchTerm);
+	
+	req = qrymgr_add_request( mySearch, callBackEnd, callBackEntry );
+
+	g_free(mySearch);
+
 	qryreq_set_search_type( req, ADDRSEARCH_EXPLICIT );
 	queryID = req->queryID;
 
