@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto & The Sylpheed-Claws Team
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto & The Sylpheed-Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,7 +24,12 @@
 #include "defs.h"
 
 #include <glib.h>
+#ifdef ENABLE_NLS
 #include <glib/gi18n.h>
+#else
+#define _(a) (a)
+#define N_(a) (a)
+#endif
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -45,11 +50,15 @@
 #include <dirent.h>
 #include <time.h>
 #include <regex.h>
+
+#ifdef G_OS_UNIX
 #include <sys/utsname.h>
+#endif
 
 #ifdef G_OS_WIN32
 #  include <direct.h>
 #  include <io.h>
+#  include <fcntl.h>
 #endif
 
 #include "utils.h"
@@ -59,6 +68,9 @@
 #define BUFFSIZE	8192
 
 static gboolean debug_mode = FALSE;
+#ifdef G_OS_WIN32
+static GSList *tempfiles=NULL;
+#endif
 
 
 #if !GLIB_CHECK_VERSION(2, 7, 0) && !defined(G_OS_UNIX)
@@ -153,6 +165,38 @@ gint g_chmod(const gchar *path, gint mode)
 }
 #endif /* GLIB_CHECK_VERSION && G_OS_UNIX */
 
+
+#ifdef G_OS_WIN32
+gint mkstemp_name(const gchar *template, gchar **name_used)
+{
+	static gulong count=0; /* W32-_mktemp only supports up to 27
+                                  tempfiles... */
+	int tmpfd;
+
+	*name_used = g_strdup_printf("%s.%ld",_mktemp(template),count++);
+	tmpfd = open (*name_used, (O_CREAT | O_RDWR | O_BINARY),
+                                    (S_IRUSR | S_IWUSR));
+
+	tempfiles=g_slist_append(tempfiles, g_strdup(*name_used));
+	if (tmpfd<0) {
+		perror(g_strdup_printf("cant create %s",*name_used));
+		return -1;
+	}
+	else
+		return tmpfd;
+}
+#endif /* G_OS_WIN32 */
+
+#ifdef G_OS_WIN32
+gint mkstemp(const gchar *template)
+{
+	gchar *dummyname;
+	gint res = mkstemp_name(template, &dummyname);
+	g_free(dummyname);
+	return res;
+}
+#endif /* G_OS_WIN32 */
+
 void list_free_strings(GList *list)
 {
 	list = g_list_first(list);
@@ -177,7 +221,7 @@ GSList *slist_concat_unique (GSList *first, GSList *second)
 	if (first == NULL) {
 		if (second == NULL)
 			return NULL;
-		else 
+		else
 			return second;
 	} else if (second == NULL)
 		return first;
@@ -188,7 +232,7 @@ GSList *slist_concat_unique (GSList *first, GSList *second)
 	}
 	return ret;
 }
- 
+
 static void hash_free_strings_func(gpointer key, gpointer value, gpointer data)
 {
 	g_free(key);
@@ -326,17 +370,35 @@ gchar *strstr2(const gchar *s1, const gchar *s2)
 gint path_cmp(const gchar *s1, const gchar *s2)
 {
 	gint len1, len2;
+        int rc;
+#ifdef G_OS_WIN32
+        gchar *s1buf, *s2buf;
+#endif
 
 	if (s1 == NULL || s2 == NULL) return -1;
 	if (*s1 == '\0' || *s2 == '\0') return -1;
 
+#ifdef G_OS_WIN32
+        s1buf = g_strdup (s1);
+        s2buf = g_strdup (s2);
+        subst_char (s1buf, '/', G_DIR_SEPARATOR);
+        subst_char (s2buf, '/', G_DIR_SEPARATOR);
+        s1 = s1buf;
+        s2 = s2buf;
+#endif /* !G_OS_WIN32 */
+
 	len1 = strlen(s1);
 	len2 = strlen(s2);
 
 	if (s1[len1 - 1] == G_DIR_SEPARATOR) len1--;
 	if (s2[len2 - 1] == G_DIR_SEPARATOR) len2--;
 
-	return strncmp(s1, s2, MAX(len1, len2));
+	rc = strncmp(s1, s2, MAX(len1, len2));
+#ifdef G_OS_WIN32
+        g_free (s1buf);
+        g_free (s2buf);
+#endif /* !G_OS_WIN32 */
+        return rc;
 }
 
 /* remove trailing return code */
@@ -384,14 +446,14 @@ gchar *strcrchomp(gchar *str)
 	return str;
 }
 
-void file_strip_crs(const gchar *file) 
+void file_strip_crs(const gchar *file)
 {
 	FILE *fp = NULL, *outfp = NULL;
 	gchar buf[4096];
 	gchar *out = get_tmp_file();
 	if (file == NULL)
 		goto freeout;
-	
+
 	fp = fopen(file, "rb");
 	if (!fp)
 		goto freeout;
@@ -406,7 +468,7 @@ void file_strip_crs(const gchar *file)
 		strcrchomp(buf);
 		fputs(buf, outfp);
 	}
-	
+
 	fclose(fp);
 	fclose(outfp);
 	rename_force(out, file);
@@ -758,7 +820,7 @@ void trim_subject_for_sort(gchar *str)
 	g_strstrip(str);
 
 	srcp = str + subject_get_prefix_length(str);
-	if (srcp != str)	
+	if (srcp != str)
 		memmove(str, srcp, strlen(srcp) + 1);
 }
 
@@ -767,7 +829,7 @@ void trim_subject(gchar *str)
 	register gchar *srcp;
 	gchar op, cl;
 	gint in_brace;
-	
+
 	g_strstrip(str);
 
 	srcp = str + subject_get_prefix_length(str);
@@ -1137,7 +1199,7 @@ GList *add_history(GList *list, const gchar *str)
 		last = g_list_last(list);
 		if (last) {
 			g_free(last->data);
-			g_list_remove(list, last->data);
+			list = g_list_remove(list, last->data);
 		}
 	}
 
@@ -1218,7 +1280,11 @@ void subst_for_filename(gchar *str)
 {
 	if (!str)
 		return;
+#ifdef G_OS_WIN32
+	subst_chars(str, "\t\r\n\\/*:", '_');
+#else
 	subst_chars(str, "\t\r\n\\/*", '_');
+#endif
 }
 
 void subst_for_shellsafe_filename(gchar *str)
@@ -1285,9 +1351,9 @@ gint get_quote_level(const gchar *str, const gchar *quote_chars)
 			quote_level++;
 		else if (*p != '-' && !g_ascii_isspace(*p) && p <= last_pos) {
 			/* any characters are allowed except '-' and space */
-			while (*p != '-' 
-			       && !strchr(quote_chars, *p) 
-			       && !g_ascii_isspace(*p) 
+			while (*p != '-'
+			       && !strchr(quote_chars, *p)
+			       && !g_ascii_isspace(*p)
 			       && p < last_pos)
 				p++;
 			if (strchr(quote_chars, *p))
@@ -1328,7 +1394,7 @@ gint check_line_length(const gchar *str, gint max_chars, gint *line)
 	return 0;
 }
 
-const gchar * line_has_quote_char(const gchar * str, const gchar *quote_chars) 
+const gchar * line_has_quote_char(const gchar * str, const gchar *quote_chars)
 {
 	gchar * position = NULL;
 	gchar * tmp_pos = NULL;
@@ -1336,17 +1402,17 @@ const gchar * line_has_quote_char(const gchar * str, const gchar *quote_chars)
 
 	if (quote_chars == NULL)
 		return FALSE;
-	
+
 	for (i = 0; i < strlen(quote_chars); i++) {
 		tmp_pos = strchr (str,	quote_chars[i]);
-		if(position == NULL 
+		if(position == NULL
 		   || (tmp_pos != NULL && position >= tmp_pos) )
 			position = tmp_pos;
 	}
-	return position; 
+	return position;
 }
 
-const gchar * line_has_quote_char_last(const gchar * str, const gchar *quote_chars) 
+const gchar * line_has_quote_char_last(const gchar * str, const gchar *quote_chars)
 {
 	gchar * position = NULL;
 	gchar * tmp_pos = NULL;
@@ -1354,14 +1420,14 @@ const gchar * line_has_quote_char_last(const gchar * str, const gchar *quote_cha
 
 	if (quote_chars == NULL)
 		return FALSE;
-	
+
 	for (i = 0; i < strlen(quote_chars); i++) {
 		tmp_pos = strrchr (str,	quote_chars[i]);
-		if(position == NULL 
+		if(position == NULL
 		   || (tmp_pos != NULL && position <= tmp_pos) )
 			position = tmp_pos;
 	}
-	return position; 
+	return position;
 }
 
 gchar *strstr_with_skip_quote(const gchar *haystack, const gchar *needle)
@@ -1657,13 +1723,13 @@ GList *uri_list_extract_filenames(const gchar *uri_list)
 	return result;
 }
 
-/* Converts two-digit hexadecimal to decimal.  Used for unescaping escaped 
+/* Converts two-digit hexadecimal to decimal.  Used for unescaping escaped
  * characters
  */
 static gint axtoi(const gchar *hexstr)
 {
 	gint hi, lo, result;
-       
+
 	hi = hexstr[0];
 	if ('0' <= hi && hi <= '9') {
 		hi -= '0';
@@ -1811,17 +1877,122 @@ gint scan_mailto_url(const gchar *mailto, gchar **to, gchar **cc, gchar **bcc,
 	return 0;
 }
 
+
+#ifdef G_OS_WIN32
+#include <windows.h>
+#ifndef CSIDL_APPDATA
+#define CSIDL_APPDATA 0x001a
+#endif
+#ifndef CSIDL_LOCAL_APPDATA
+#define CSIDL_LOCAL_APPDATA 0x001c
+#endif
+#ifndef CSIDL_FLAG_CREATE
+#define CSIDL_FLAG_CREATE 0x8000
+#endif
+#define DIM(v)		     (sizeof(v)/sizeof((v)[0]))
+
+#define RTLD_LAZY 0
+const char *
+w32_strerror (int w32_errno)
+{
+  static char strerr[256];
+  int ec = (int)GetLastError ();
+
+  if (w32_errno == 0)
+    w32_errno = ec;
+  FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, w32_errno,
+                 MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
+                 strerr, DIM (strerr)-1, NULL);
+  return strerr;
+}
+
+static __inline__ void *
+dlopen (const char * name, int flag)
+{
+  void * hd = LoadLibrary (name);
+  return hd;
+}
+
+static __inline__ void *
+dlsym (void * hd, const char * sym)
+{
+  if (hd && sym)
+    {
+      void * fnc = GetProcAddress (hd, sym);
+      if (!fnc)
+        return NULL;
+      return fnc;
+    }
+  return NULL;
+}
+
+
+static __inline__ const char *
+dlerror (void)
+{
+  return w32_strerror (0);
+}
+
+
+static __inline__ int
+dlclose (void * hd)
+{
+  if (hd)
+    {
+      FreeLibrary (hd);
+      return 0;
+    }
+  return -1;
+}
+
+static HRESULT
+w32_shgetfolderpath (HWND a, int b, HANDLE c, DWORD d, LPSTR e)
+{
+  static int initialized;
+  static HRESULT (WINAPI * func)(HWND,int,HANDLE,DWORD,LPSTR);
+
+  if (!initialized)
+    {
+      static char *dllnames[] = { "shell32.dll", "shfolder.dll", NULL };
+      void *handle;
+      int i;
+
+      initialized = 1;
+
+      for (i=0, handle = NULL; !handle && dllnames[i]; i++)
+        {
+          handle = dlopen (dllnames[i], RTLD_LAZY);
+          if (handle)
+            {
+              func = dlsym (handle, "SHGetFolderPathA");
+              if (!func)
+                {
+                  dlclose (handle);
+                  handle = NULL;
+                }
+            }
+        }
+    }
+
+  if (func)
+    return func (a,b,c,d,e);
+  else
+    return -1;
+}
+#endif
+
 const gchar *get_home_dir(void)
 {
 #ifdef G_OS_WIN32
-	static const gchar *home_dir = NULL;
+	static char home_dir[MAX_PATH] = "";
 
-	if (!home_dir) {
-		home_dir = g_get_home_dir();
-		if (!home_dir)
-			home_dir = "C:\\Sylpheed";
+	if (home_dir[0] == '\0')
+		{
+			if (w32_shgetfolderpath
+			    (NULL, CSIDL_APPDATA|CSIDL_FLAG_CREATE,
+			     NULL, 0, home_dir) < 0)
+				strcpy (home_dir, "C:\\Sylpheed");
 	}
-
 	return home_dir;
 #else
 	return g_get_home_dir();
@@ -1833,14 +2004,8 @@ const gchar *get_rc_dir(void)
 	static gchar *rc_dir = NULL;
 
 	if (!rc_dir)
-#ifdef G_OS_WIN32
-		rc_dir = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
-				     "Application Data", G_DIR_SEPARATOR_S,
-				     RC_DIR, NULL);
-#else
 		rc_dir = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
 				     RC_DIR, NULL);
-#endif
 
 	return rc_dir;
 }
@@ -1926,6 +2091,22 @@ const gchar *get_header_cache_dir(void)
 	return header_dir;
 }
 
+/* Return the default directory for Plugins. */
+const gchar *get_plugin_dir(void)
+{
+#ifdef G_OS_WIN32
+	static gchar *plugin_dir = NULL;
+
+	if (!plugin_dir)
+                plugin_dir = g_strconcat(sylpheed_get_startup_dir(),
+                                         "\\lib\\sylpheed-claws\\plugins\\",
+                                         NULL);
+	return plugin_dir;
+#else
+        return PLUGINDIR;
+#endif
+}
+
 const gchar *get_tmp_dir(void)
 {
 	static gchar *tmp_dir = NULL;
@@ -2061,6 +2242,30 @@ gboolean file_exist(const gchar *file, gboolean allow_fifo)
 	return FALSE;
 }
 
+
+/* Test on whether FILE is a relative file name. This is
+ * straightforward for Unix but more complex for Windows. */
+gboolean is_relative_filename(const gchar *file)
+{
+        if (!file)
+                return TRUE;
+#ifdef G_OS_WIN32
+        if ( *file == '\\' && file[1] == '\\' && strchr (file+2, '\\') )
+                return FALSE; /* Prefixed with a hostname - this can't
+                               * be a relative name. */
+
+        if ( ((*file >= 'a' && *file <= 'z')
+              || (*file >= 'A' && *file <= 'Z'))
+             && file[1] == ':')
+                file += 2;  /* Skip drive letter. */
+
+        return !(*file == '\\' || *file == '/');
+#else
+        return !(*file == G_DIR_SEPARATOR);
+#endif
+}
+
+
 gboolean is_dir_exist(const gchar *dir)
 {
 	if (dir == NULL)
@@ -2446,85 +2651,6 @@ gint rename_force(const gchar *oldpath, const gchar *newpath)
 	return g_rename(oldpath, newpath);
 }
 
-#if 0
-/* this seems to be slower than the stdio version... */
-gint copy_file(const gchar *src, const gchar *dest)
-{
-	gint src_fd, dest_fd;
-	gint n_read;
-	gint n_write;
-	gchar buf[BUFSIZ];
-	gchar *dest_bak = NULL;
-
-	if ((src_fd = open(src, O_RDONLY)) < 0) {
-		FILE_OP_ERROR(src, "open");
-		return -1;
-	}
-
-	if (is_file_exist(dest)) {
-		dest_bak = g_strconcat(dest, ".bak", NULL);
-		if (rename_force(dest, dest_bak) < 0) {
-			FILE_OP_ERROR(dest, "rename");
-			close(src_fd);
-			g_free(dest_bak);
-			return -1;
-		}
-	}
-
-	if ((dest_fd = open(dest, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR)) < 0) {
-		FILE_OP_ERROR(dest, "open");
-		close(src_fd);
-		if (dest_bak) {
-			if (rename(dest_bak, dest) < 0)
-				FILE_OP_ERROR(dest_bak, "rename");
-			g_free(dest_bak);
-		}
-		return -1;
-	}
-
-	while ((n_read = read(src_fd, buf, sizeof(buf))) > 0) {
-		gint len = n_read;
-		gchar *bufp = buf;
-
-		while (len > 0) {
-			n_write = write(dest_fd, bufp, len);
-			if (n_write <= 0) {
-				g_warning("writing to %s failed.\n", dest);
-				close(dest_fd);
-				close(src_fd);
-				g_unlink(dest);
-				if (dest_bak) {
-					if (rename(dest_bak, dest) < 0)
-						FILE_OP_ERROR(dest_bak, "rename");
-					g_free(dest_bak);
-				}
-				return -1;
-			}
-			len -= n_write;
-			bufp += n_write;
-		}
-	}
-
-	close(src_fd);
-	close(dest_fd);
-
-	if (n_read < 0 || get_file_size(src) != get_file_size(dest)) {
-		g_warning("File copy from %s to %s failed.\n", src, dest);
-		g_unlink(dest);
-		if (dest_bak) {
-			if (rename(dest_bak, dest) < 0)
-				FILE_OP_ERROR(dest_bak, "rename");
-			g_free(dest_bak);
-		}
-		return -1;
-	}
-	g_free(dest_bak);
-
-	return 0;
-}
-#endif
-
-
 /*
  * Append src file body to the tail of dest file.
  * Now keep_backup has no effects.
@@ -2541,7 +2667,7 @@ gint append_file(const gchar *src, const gchar *dest, gboolean keep_backup)
 		FILE_OP_ERROR(src, "fopen");
 		return -1;
 	}
-	
+
 	if ((dest_fp = g_fopen(dest, "ab")) == NULL) {
 		FILE_OP_ERROR(dest, "fopen");
 		fclose(src_fp);
@@ -3035,8 +3161,7 @@ gchar *get_outgoing_rfc2822_str(FILE *fp)
  * uniqueness if everything is either quoted-printable or base64
  * encoded (note that conversion is allowed), but because MIME bodies
  * may be nested, it may happen that the same boundary has already
- * been used. We avoid scanning the message for conflicts and hope the
- * best.
+ * been used.
  *
  *   boundary := 0*69<bchars> bcharsnospace
  *   bchars := bcharsnospace / " "
@@ -3052,21 +3177,15 @@ gchar *generate_mime_boundary(const gchar *prefix)
 	static gchar tbl[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 			     "abcdefghijklmnopqrstuvwxyz"
 			     "1234567890+_./=";
-	gchar buf_uniq[17];
-	gchar buf_date[64];
+	gchar buf_uniq[24];
 	gint i;
 
 	for (i = 0; i < sizeof(buf_uniq) - 1; i++)
 		buf_uniq[i] = tbl[g_random_int_range(0, sizeof(tbl) - 1)];
 	buf_uniq[i] = '\0';
 
-	get_rfc822_date(buf_date, sizeof(buf_date));
-	subst_char(buf_date, ' ', '_');
-	subst_char(buf_date, ',', '_');
-	subst_char(buf_date, ':', '_');
-
-	return g_strdup_printf("%s_%s_%s", prefix ? prefix : "Multipart",
-			       buf_date, buf_uniq);
+	return g_strdup_printf("%s_%s", prefix ? prefix : "MP",
+			       buf_uniq);
 }
 
 gint change_file_mode_rw(FILE *fp, const gchar *file)
@@ -3080,7 +3199,7 @@ gint change_file_mode_rw(FILE *fp, const gchar *file)
 
 FILE *my_tmpfile(void)
 {
-#if HAVE_MKSTEMP
+#if HAVE_MKSTEMP || defined(G_OS_WIN32)
 	const gchar suffix[] = ".XXXXXX";
 	const gchar *tmpdir;
 	guint tmplen;
@@ -3108,14 +3227,16 @@ FILE *my_tmpfile(void)
 	if (fd < 0)
 		return tmpfile();
 
+#ifndef G_OS_WIN32
 	g_unlink(fname);
+#endif
 
 	fp = fdopen(fd, "w+b");
 	if (!fp)
 		close(fd);
 	else
 		return fp;
-#endif /* HAVE_MKSTEMP */
+#endif /* HAVE_MKSTEMP || G_OS_WIN32 */
 
 	return tmpfile();
 }
@@ -3123,10 +3244,15 @@ FILE *my_tmpfile(void)
 FILE *get_tmpfile_in_dir(const gchar *dir, gchar **filename)
 {
 	int fd;
-	
+#ifdef G_OS_WIN32
+	char *template = g_strdup_printf ("%s%csylpheed.XXXXXX",
+                                          dir, G_DIR_SEPARATOR);
+	fd = mkstemp_name(template, filename);
+	g_free(template);
+#else
 	*filename = g_strdup_printf("%s%csylpheed.XXXXXX", dir, G_DIR_SEPARATOR);
 	fd = mkstemp(*filename);
-
+#endif
 	return fdopen(fd, "w+");
 }
 
@@ -3349,7 +3475,7 @@ void encode_uri(gchar *encoded_uri, gint bufsize, const gchar *uri)
 		}
 		else {
 			char * hexa = "0123456789ABCDEF";
-			
+
 			if (k + 4 >= bufsize)
 				break;
 			encoded_uri[k++] = '%';
@@ -3365,12 +3491,12 @@ gint open_uri(const gchar *uri, const gchar *cmdline)
 	gchar buf[BUFFSIZE];
 	gchar *p;
 	gchar encoded_uri[BUFFSIZE];
-	
+
 	g_return_val_if_fail(uri != NULL, -1);
 
 	/* an option to choose whether to use encode_uri or not ? */
 	encode_uri(encoded_uri, BUFFSIZE, uri);
-	
+
 	if (cmdline &&
 	    (p = strchr(cmdline, '%')) && *(p + 1) == 's' &&
 	    !strchr(p + 2, '%'))
@@ -3388,6 +3514,30 @@ gint open_uri(const gchar *uri, const gchar *cmdline)
 	return 0;
 }
 
+gint open_txt_editor(const gchar *filepath, const gchar *cmdline)
+{
+	gchar buf[BUFFSIZE];
+	gchar *p;
+
+	g_return_val_if_fail(filepath != NULL, -1);
+
+	if (cmdline &&
+	    (p = strchr(cmdline, '%')) && *(p + 1) == 's' &&
+	    !strchr(p + 2, '%'))
+		g_snprintf(buf, sizeof(buf), cmdline, filepath);
+	else {
+		if (cmdline)
+			g_warning("Open Text Editor command line is invalid "
+				  "(there must be only one '%%s'): %s",
+				  cmdline);
+		g_snprintf(buf, sizeof(buf), DEFAULT_EDITOR_CMD, filepath);
+	}
+
+	execute_command_line(buf, TRUE);
+
+	return 0;
+}
+
 time_t remote_tzoffset_sec(const gchar *zone)
 {
 	static gchar ustzstr[] = "PSTPDTMSTMDTCSTCDTESTEDT";
@@ -3565,6 +3715,14 @@ void debug_print_real(const gchar *format, ...)
 	g_print("%s", buf);
 }
 
+
+const char * debug_srcname(const char *file)
+{
+        const char *s = strrchr (file, '/');
+        return s? s+1:file;
+}
+
+
 void * subject_table_lookup(GHashTable *subject_table, gchar * subject)
 {
 	if (subject == NULL)
@@ -3589,21 +3747,21 @@ void subject_table_remove(GHashTable *subject_table, gchar * subject)
 	if (subject == NULL)
 		return;
 
-	subject += subject_get_prefix_length(subject);	
+	subject += subject_get_prefix_length(subject);
 	g_hash_table_remove(subject_table, subject);
 }
 
 /*!
- *\brief	Check if a string is prefixed with known (combinations) 
- *		of prefixes. The function assumes that each prefix 
+ *\brief	Check if a string is prefixed with known (combinations)
+ *		of prefixes. The function assumes that each prefix
  *		is terminated by zero or exactly _one_ space.
  *
  *\param	str String to check for a prefixes
  *
- *\return	int Number of chars in the prefix that should be skipped 
+ *\return	int Number of chars in the prefix that should be skipped
  *		for a "clean" subject line. If no prefix was found, 0
  *		is returned.
- */		
+ */
 int subject_get_prefix_length(const gchar *subject)
 {
 	/*!< Array with allowable reply prefixes regexps. */
@@ -3631,23 +3789,23 @@ int subject_get_prefix_length(const gchar *subject)
 
 	if (!init_) {
 		GString *s = g_string_new("");
-		
+
 		for (n = 0; n < PREFIXES; n++)
 			/* Terminate each prefix regexpression by a
 			 * "\ ?" (zero or ONE space), and OR them */
 			g_string_append_printf(s, "(%s\\ ?)%s",
 					  prefixes[n],
-					  n < PREFIXES - 1 ? 
+					  n < PREFIXES - 1 ?
 					  "|" : "");
-		
+
 		g_string_prepend(s, "(");
 		g_string_append(s, ")+");	/* match at least once */
 		g_string_prepend(s, "^\\ *");	/* from beginning of line */
-		
 
-		/* We now have something like "^\ *((PREFIX1\ ?)|(PREFIX2\ ?))+" 
+
+		/* We now have something like "^\ *((PREFIX1\ ?)|(PREFIX2\ ?))+"
 		 * TODO: Should this be       "^\ *(((PREFIX1)|(PREFIX2))\ ?)+" ??? */
-		if (regcomp(&regex, s->str, REG_EXTENDED | REG_ICASE)) { 
+		if (regcomp(&regex, s->str, REG_EXTENDED | REG_ICASE)) {
 			debug_print("Error compiling regexp %s\n", s->str);
 			g_string_free(s, TRUE);
 			return 0;
@@ -3656,7 +3814,7 @@ int subject_get_prefix_length(const gchar *subject)
 			g_string_free(s, TRUE);
 		}
 	}
-	
+
 	if (!regexec(&regex, subject, 1, &pos, 0) && pos.rm_so != -1)
 		return pos.rm_eo;
 	else
@@ -3712,9 +3870,9 @@ gchar *generate_msgid(gchar *buf, gint len)
 
 /*
    quote_cmd_argument()
-   
+
    return a quoted string safely usable in argument of a command.
-   
+
    code is extracted and adapted from etPan! project -- DINH V. Ho.
 */
 
@@ -3733,7 +3891,7 @@ gint quote_cmd_argument(gchar * result, guint size,
 		if (isalnum((guchar)*p) || (* p == '/')) {
 			if (remaining > 0) {
 				* result_p = * p;
-				result_p ++; 
+				result_p ++;
 				remaining --;
 			}
 			else {
@@ -3741,12 +3899,12 @@ gint quote_cmd_argument(gchar * result, guint size,
 				return -1;
 			}
 		}
-		else { 
+		else {
 			if (remaining >= 2) {
 				* result_p = '\\';
-				result_p ++; 
+				result_p ++;
 				* result_p = * p;
-				result_p ++; 
+				result_p ++;
 				remaining -= 2;
 			}
 			else {
@@ -3762,11 +3920,11 @@ gint quote_cmd_argument(gchar * result, guint size,
 		result[size - 1] = '\0';
 		return -1;
 	}
-  
+
 	return 0;
 }
 
-typedef struct 
+typedef struct
 {
 	GNode 		*parent;
 	GNodeMapFunc	 func;
@@ -3870,7 +4028,7 @@ void get_hex_str(gchar *out, guchar ch)
  *\brief	Register ref counted pointer. It is based on GBoxed, so should
  *		work with anything that uses the GType system. The semantics
  *		are similar to a C++ auto pointer, with the exception that
- *		C doesn't have automatic closure (calling destructors) when 
+ *		C doesn't have automatic closure (calling destructors) when
  *		exiting a block scope.
  *		Use the \ref G_TYPE_AUTO_POINTER macro instead of calling this
  *		function directly.
@@ -3886,7 +4044,7 @@ GType g_auto_pointer_register(void)
 				("G_TYPE_AUTO_POINTER",
 				 (GBoxedCopyFunc) g_auto_pointer_copy,
 				 (GBoxedFreeFunc) g_auto_pointer_free);
-	return auto_pointer_type;						     
+	return auto_pointer_type;
 }
 
 /*!
@@ -3911,12 +4069,12 @@ typedef struct AutoPointer {
 /*!
  *\brief	Creates an auto pointer for a g_new()ed pointer. Example:
  *
- *\code	
+ *\code
  *
  *		... tell gtk_list_store it should use a G_TYPE_AUTO_POINTER
  *		... when assigning, copying and freeing storage elements
  *
- *		gtk_list_store_new(N_S_COLUMNS, 
+ *		gtk_list_store_new(N_S_COLUMNS,
  *				   G_TYPE_AUTO_POINTER,
  *				   -1);
  *
@@ -3928,16 +4086,16 @@ typedef struct AutoPointer {
  *				   S_DATA, protect,
  *				   -1);
  *
- *		... the gtk_list_store has copied the pointer and 
+ *		... the gtk_list_store has copied the pointer and
  *		... incremented its reference count, we should free
  *		... the auto pointer (in C++ a destructor would do
  *		... this for us when leaving block scope)
- * 
+ *
  *		g_auto_pointer_free(protect);
  *
  *		... gtk_list_store_set() now manages the data. When
- *		... *explicitly* requesting a pointer from the list 
- *		... store, don't forget you get a copy that should be 
+ *		... *explicitly* requesting a pointer from the list
+ *		... store, don't forget you get a copy that should be
  *		... freed with g_auto_pointer_free() eventually.
  *
  *\endcode
@@ -3951,8 +4109,8 @@ GAuto *g_auto_pointer_new(gpointer p)
 {
 	AutoPointerRef *ref;
 	AutoPointer    *ptr;
-	
-	if (p == NULL) 
+
+	if (p == NULL)
 		return NULL;
 
 	ref = g_new0(AutoPointerRef, 1);
@@ -3978,20 +4136,20 @@ GAuto *g_auto_pointer_new(gpointer p)
 GAuto *g_auto_pointer_new_with_free(gpointer p, GFreeFunc free_)
 {
 	AutoPointer *aptr;
-	
+
 	if (p == NULL)
 		return NULL;
 
 	aptr = g_auto_pointer_new(p);
 	aptr->ref->free = free_;
-	return aptr; 
+	return aptr;
 }
 
 gpointer g_auto_pointer_get_ptr(GAuto *auto_ptr)
 {
-	if (auto_ptr == NULL) 
+	if (auto_ptr == NULL)
 		return NULL;
-	return ((AutoPointer *) auto_ptr)->ptr; 
+	return ((AutoPointer *) auto_ptr)->ptr;
 }
 
 /*!
@@ -3999,7 +4157,7 @@ gpointer g_auto_pointer_get_ptr(GAuto *auto_ptr)
  *		to call this function directly, unless you copy/assign
  *		the guarded pointer.
  *
- *\param	auto_ptr Auto pointer returned by previous call to 
+ *\param	auto_ptr Auto pointer returned by previous call to
  *		g_auto_pointer_new_XXX()
  *
  *\return	gpointer An auto pointer
@@ -4010,7 +4168,7 @@ GAuto *g_auto_pointer_copy(GAuto *auto_ptr)
 	AutoPointerRef	*ref;
 	AutoPointer	*newp;
 
-	if (auto_ptr == NULL) 
+	if (auto_ptr == NULL)
 		return NULL;
 
 	ptr = auto_ptr;
@@ -4020,7 +4178,7 @@ GAuto *g_auto_pointer_copy(GAuto *auto_ptr)
 	newp->ref = ref;
 	newp->ptr = ref->pointer;
 	++(ref->cnt);
-	
+
 #ifdef REF_DEBUG
 	G_PRINT_REF ("XXXX COPY(%lx) -- REF (%d)\n", ref->pointer, ref->cnt);
 #endif
@@ -4034,7 +4192,7 @@ void g_auto_pointer_free(GAuto *auto_ptr)
 {
 	AutoPointer	*ptr;
 	AutoPointerRef	*ref;
-	
+
 	if (auto_ptr == NULL)
 		return;
 
@@ -4047,12 +4205,12 @@ void g_auto_pointer_free(GAuto *auto_ptr)
 #endif
 		ref->free(ref->pointer);
 		g_free(ref);
-	} 
+	}
 #ifdef REF_DEBUG
 	else
 		G_PRINT_REF ("XXXX DEREF(%lx) -- REF (%d)\n", ref->pointer, ref->cnt);
 #endif
-	g_free(ptr);		
+	g_free(ptr);
 }
 
 void replace_returns(gchar *str)
@@ -4071,9 +4229,10 @@ void replace_returns(gchar *str)
 /* get_uri_part() - retrieves a URI starting from scanpos.
 		    Returns TRUE if succesful */
 gboolean get_uri_part(const gchar *start, const gchar *scanpos,
-			     const gchar **bp, const gchar **ep)
+			     const gchar **bp, const gchar **ep, gboolean hdr)
 {
 	const gchar *ep_;
+	gint parenthese_cnt = 0;
 
 	g_return_val_if_fail(start != NULL, FALSE);
 	g_return_val_if_fail(scanpos != NULL, FALSE);
@@ -4084,10 +4243,18 @@ gboolean get_uri_part(const gchar *start, const gchar *scanpos,
 
 	/* find end point of URI */
 	for (ep_ = scanpos; *ep_ != '\0'; ep_++) {
-		if (!isgraph(*(const guchar *)ep_) ||
+		if (!g_ascii_isgraph(*(const guchar *)ep_) ||
 		    !IS_ASCII(*(const guchar *)ep_) ||
-		    strchr("[]{}()<>\"", *ep_))
+		    strchr("[]{}<>\"", *ep_)) {
 			break;
+		} else if (strchr("(", *ep_)) {
+			parenthese_cnt++;
+		} else if (strchr(")", *ep_)) {
+			if (parenthese_cnt > 0)
+				parenthese_cnt--;
+			else
+				break;
+		}
 	}
 
 	/* no punctuation at end of string */
@@ -4096,10 +4263,10 @@ gboolean get_uri_part(const gchar *start, const gchar *scanpos,
 	 * should pass some URI type to this function and decide on that whether
 	 * to perform punctuation stripping */
 
-#define IS_REAL_PUNCT(ch)	(ispunct(ch) && ((ch) != '/')) 
+#define IS_REAL_PUNCT(ch)	(g_ascii_ispunct(ch) && !strchr("/?=-)", ch))
 
 	for (; ep_ - 1 > scanpos + 1 &&
-	       IS_REAL_PUNCT(*(const guchar *)(ep_ - 1));
+	       IS_REAL_PUNCT(*(ep_ - 1));
 	     ep_--)
 		;
 
@@ -4107,7 +4274,7 @@ gboolean get_uri_part(const gchar *start, const gchar *scanpos,
 
 	*ep = ep_;
 
-	return TRUE;		
+	return TRUE;
 }
 
 gchar *make_uri_string(const gchar *bp, const gchar *ep)
@@ -4156,13 +4323,13 @@ static GHashTable *create_domain_tab(void)
 	    "tc", "td", "tf", "tg", "th", "tj", "tk", "tm", "tn", "to",
 	    "tp", "tr", "tt", "tv", "tw", "tz", "ua", "ug", "uk", "um",
 	    "us", "uy", "uz", "va", "vc", "ve", "vg", "vi", "vn", "vu",
-            "wf", "ws", "ye", "yt", "yu", "za", "zm", "zw" 
+            "wf", "ws", "ye", "yt", "yu", "za", "zm", "zw"
 	};
 	gint n;
 	GHashTable *htab = g_hash_table_new(g_stricase_hash, g_stricase_equal);
-	
+
 	g_return_val_if_fail(htab, NULL);
-	for (n = 0; n < sizeof toplvl_domains / sizeof toplvl_domains[0]; n++) 
+	for (n = 0; n < sizeof toplvl_domains / sizeof toplvl_domains[0]; n++)
 		g_hash_table_insert(htab, (gpointer) toplvl_domains[n], (gpointer) toplvl_domains[n]);
 	return htab;
 }
@@ -4173,7 +4340,7 @@ static gboolean is_toplvl_domain(GHashTable *tab, const gchar *first, const gcha
 	gchar buf[MAX_LVL_DOM_NAME_LEN + 1];
 	const gchar *m = buf + MAX_LVL_DOM_NAME_LEN + 1;
 	register gchar *p;
-	
+
 	if (last - first > MAX_LVL_DOM_NAME_LEN || first > last)
 		return FALSE;
 
@@ -4186,7 +4353,7 @@ static gboolean is_toplvl_domain(GHashTable *tab, const gchar *first, const gcha
 
 /* get_email_part() - retrieves an email address. Returns TRUE if succesful */
 gboolean get_email_part(const gchar *start, const gchar *scanpos,
-			       const gchar **bp, const gchar **ep)
+			       const gchar **bp, const gchar **ep, gboolean hdr)
 {
 	/* more complex than the uri part because we need to scan back and forward starting from
 	 * the scan position. */
@@ -4197,7 +4364,7 @@ gboolean get_email_part(const gchar *start, const gchar *scanpos,
 	const gchar *last_dot = NULL;
 	const gchar *prelast_dot = NULL;
 	const gchar *last_tld_char = NULL;
-	
+
 	/* the informative part of the email address (describing the name
 	 * of the email address owner) may contain quoted parts. the
 	 * closure stack stores the last encountered quotes. */
@@ -4209,9 +4376,43 @@ gboolean get_email_part(const gchar *start, const gchar *scanpos,
 	g_return_val_if_fail(bp != NULL, FALSE);
 	g_return_val_if_fail(ep != NULL, FALSE);
 
+	if (hdr) {
+search_again:
+		/* go to the real start */
+		if (start[0] == ',')
+			start++;
+		if (start[0] == ';')
+			start++;
+		while (start[0] == ' ' || start[0] == '\t')
+			start++;
+
+		*bp = start;
+
+		/* find end (either , or ; or end of line) */
+		if (strstr(start, ",") && strstr(start, ";"))
+			*ep = strstr(start,",") < strstr(start, ";")
+				? strstr(start, ",") : strstr(start, ";");
+		else if (strstr(start, ","))
+			*ep = strstr(start, ",");
+		else if (strstr(start, ";"))
+			*ep = strstr(start, ";");
+		else
+			*ep = start+strlen(start);
+
+		/* check there's still an @ in that, or search
+		 * further if possible */
+		if (strstr(start, "@") && strstr(start, "@") < *ep)
+			return TRUE;
+		else if (*ep < start+strlen(start)) {
+			start = *ep;
+			goto search_again;
+		} else
+			return FALSE;
+	}
+
 	if (!dom_tab)
 		dom_tab = create_domain_tab();
-	g_return_val_if_fail(dom_tab, FALSE);	
+	g_return_val_if_fail(dom_tab, FALSE);
 
 	/* scan start of address */
 	for (bp_ = scanpos - 1;
@@ -4265,21 +4466,21 @@ gboolean get_email_part(const gchar *start, const gchar *scanpos,
 
 	if (!result) return FALSE;
 
-	if (*ep_ && *(bp_ - 1) == '"' && *(ep_) == '"' 
+	if (*ep_ && *(bp_ - 1) == '"' && *(ep_) == '"'
 	&& *(ep_ + 1) == ' ' && *(ep_ + 2) == '<'
 	&& IS_RFC822_CHAR(*(ep_ + 3))) {
-		/* this informative part with an @ in it is 
+		/* this informative part with an @ in it is
 		 * followed by the email address */
 		ep_ += 3;
-		
+
 		/* go to matching '>' (or next non-rfc822 char, like \n) */
 		for (; *ep_ != '>' && *ep != '\0' && IS_RFC822_CHAR(*ep_); ep_++)
 			;
-			
+
 		/* include the bracket */
 		if (*ep_ == '>') ep_++;
-		
-		/* include the leading quote */		
+
+		/* include the leading quote */
 		bp_--;
 
 		*ep = ep_;
@@ -4319,7 +4520,7 @@ gboolean get_email_part(const gchar *start, const gchar *scanpos,
 		}
 
 		/* if nothing in the closure stack, do the special conditions
-		 * the following if..else expression simply checks whether 
+		 * the following if..else expression simply checks whether
 		 * a token is acceptable. if not acceptable, the clause
 		 * should terminate the loop with a 'break' */
 		if (!PEEK_STACK()) {
@@ -4328,7 +4529,7 @@ gboolean get_email_part(const gchar *start, const gchar *scanpos,
 			&& (((bp_ + 1) < ep_)    && isalnum(*(bp_ + 1)))) {
 				/* hyphens are allowed, but only in
 				   between alnums */
-			} else if (!strchr(",;:=?./+<>!&\r\n\t", *bp_)) {
+			} else if (strchr(" \"'", *bp_)) {
 				/* but anything not being a punctiation
 				   is ok */
 			} else {
@@ -4339,19 +4540,19 @@ gboolean get_email_part(const gchar *start, const gchar *scanpos,
 
 	bp_++;
 
+	/* scan forward (should start with an alnum) */
+	for (; *bp_ != '<' && isspace(*bp_) && *bp_ != '"'; bp_++)
+		;
 #undef PEEK_STACK
 #undef PUSH_STACK
 #undef POP_STACK
 #undef IN_STACK
 #undef FULL_STACK
 
-	/* scan forward (should start with an alnum) */
-	for (; *bp_ != '<' && isspace(*bp_) && *bp_ != '"'; bp_++)
-		;
 
-	*ep = ep_;
 	*bp = bp_;
-	
+	*ep = ep_;
+
 	return result;
 }
 
@@ -4386,3 +4587,117 @@ gchar *make_http_string(const gchar *bp, const gchar *ep)
 	return result;
 }
 
+static gchar *mailcap_get_command_in_file(const gchar *path, const gchar *type)
+{
+	FILE *fp = fopen(path, "rb");
+	gchar buf[BUFFSIZE];
+	gchar *result = NULL;
+	if (!fp)
+		return NULL;
+	while (fgets(buf, sizeof (buf), fp) != NULL) {
+		gchar **parts = g_strsplit(buf, ";", -1);
+		gchar *trimmed = parts[0];
+		while (trimmed[0] == ' ')
+			trimmed++;
+		while (trimmed[strlen(trimmed)-1] == ' ')
+			trimmed[strlen(trimmed)-1] = '\0';
+
+		if (!strcmp(trimmed, type)) {
+			trimmed = parts[1];
+			while (trimmed[0] == ' ')
+				trimmed++;
+			while (trimmed[strlen(trimmed)-1] == ' ')
+				trimmed[strlen(trimmed)-1] = '\0';
+			while (trimmed[strlen(trimmed)-1] == '\n')
+				trimmed[strlen(trimmed)-1] = '\0';
+			while (trimmed[strlen(trimmed)-1] == '\r')
+				trimmed[strlen(trimmed)-1] = '\0';
+			result = g_strdup(trimmed);
+			g_strfreev(parts);
+			fclose(fp);
+			if (strstr(result, "%s") && !strstr(result, "'%s'")) {
+				gchar *start = g_strdup(result);
+				gchar *end = g_strdup(strstr(result, "%s")+2);
+				gchar *tmp;
+				*strstr(start, "%s") = '\0';
+				tmp = g_strconcat(start,"'%s'",end, NULL);
+				g_free(start);
+				g_free(end);
+				g_free(result);
+				result = tmp;
+			}
+			return result;
+		}
+		g_strfreev(parts);
+	}
+	fclose(fp);
+	return NULL;
+}
+gchar *mailcap_get_command_for_type(const gchar *type)
+{
+	gchar *result = NULL;
+	gchar *path = NULL;
+	path = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, ".mailcap", NULL);
+	result = mailcap_get_command_in_file(path, type);
+	g_free(path);
+	if (result)
+		return result;
+	result = mailcap_get_command_in_file("/etc/mailcap", type);
+	return result;
+}
+
+gint copy_dir(const gchar *src, const gchar *dst)
+{
+	GDir *dir;
+	const gchar *name;
+
+	if ((dir = g_dir_open(src, 0, NULL)) == NULL) {
+		g_warning("failed to open directory: %s\n", src);
+		return -1;
+	}
+
+	if (make_dir(dst) < 0)
+		return -1;
+
+	while ((name = g_dir_read_name(dir)) != NULL) {
+		gchar *old_file, *new_file;
+		old_file = g_strconcat(src, G_DIR_SEPARATOR_S, name, NULL);
+		new_file = g_strconcat(dst, G_DIR_SEPARATOR_S, name, NULL);
+		debug_print("copying: %s -> %s\n", old_file, new_file);
+		if (g_file_test(old_file, G_FILE_TEST_IS_REGULAR)) {
+			gint r = copy_file(old_file, new_file, TRUE);
+			if (r < 0)
+				return r;
+		} else if (g_file_test(old_file, G_FILE_TEST_IS_DIR)) {
+			gint r = copy_dir(old_file, new_file);
+			if (r < 0)
+				return r;
+		}
+	}
+	return 0;
+}
+
+/* crude test to see if a file is an email. */
+gboolean file_is_email (const gchar *filename)
+{
+	FILE *fp = NULL;
+	gchar buffer[2048];
+	gint i = 0;
+	gint score = 0;
+	if (filename == NULL)
+		return FALSE;
+	if ((fp = g_fopen(filename, "rb")) == NULL)
+		return FALSE;
+	while (i < 60 && score < 3
+	       && fgets(buffer, sizeof (buffer), fp) > 0) {
+		if (!strncmp(buffer, "From:", strlen("From:")))
+			score++;
+		if (!strncmp(buffer, "To:", strlen("To:")))
+			score++;
+		if (!strncmp(buffer, "Subject:", strlen("Subject:")))
+			score++;
+		i++;
+	}
+	fclose(fp);
+	return (score >= 3);
+}
