@@ -8,7 +8,7 @@
 #include <imap.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#if (defined (__NetBSD__) || defined (__FreeBSD__))
+#if (defined (__NetBSD__) || defined (__FreeBSD__) || defined (__CYGWIN__))
 #include <sys/socket.h>
 #endif
 #include <fcntl.h>
@@ -19,6 +19,7 @@
 #include <log.h>
 #include "etpan-thread-manager.h"
 #include "utils.h"
+#include "ssl_certificate.h"
 
 #define DISABLE_LOG_DURING_LOGIN
 
@@ -42,7 +43,9 @@ static gboolean thread_manager_event(GIOChannel * source,
 void imap_logger(int direction, const char * str, size_t size) 
 {
 	gchar buf[512];
-	strncpy(buf, str, 511);
+
+	memset(buf, 0, 512);
+	strncpy(buf, str, size > 510 ? 510:size);
 	buf[511] = '\0';
 	if (size < 511)
 		buf[size] = '\0';
@@ -282,6 +285,30 @@ int imap_threaded_connect(Folder * folder, const char * server, int port)
 	return result.error;
 }
 
+static int etpan_certificate_check(unsigned char *certificate, int len, void *data)
+{
+#ifdef USE_OPENSSL
+	struct connect_param *param = (struct connect_param *)data;
+	X509 *cert = NULL;
+
+	if (certificate == NULL || len < 0) {
+		g_warning("no cert presented.\n");
+		return 0;
+	}
+	cert = d2i_X509(NULL, &certificate, len);
+	if (cert == NULL) {
+		g_warning("can't get cert\n");
+		return 0;
+	} else if (ssl_certificate_check(cert, 
+		(gchar *)param->server, param->port) == TRUE) {
+		return 0;
+	} else {
+		return -1;
+	}
+#else
+	return 0;
+#endif
+}
 
 static void connect_ssl_run(struct etpan_thread_op * op)
 {
@@ -294,7 +321,6 @@ static void connect_ssl_run(struct etpan_thread_op * op)
 	
 	r = mailimap_ssl_connect(param->imap,
 				 param->server, param->port);
-	
 	result->error = r;
 }
 
@@ -305,7 +331,9 @@ int imap_threaded_connect_ssl(Folder * folder, const char * server, int port)
 	chashdatum key;
 	chashdatum value;
 	mailimap * imap;
-	
+	unsigned char *certificate = NULL;
+	int cert_len;
+
 	imap = mailimap_new(0, NULL);
 	
 	key.data = &folder;
@@ -320,7 +348,14 @@ int imap_threaded_connect_ssl(Folder * folder, const char * server, int port)
 	
 	threaded_run(folder, &param, &result, connect_ssl_run);
 	
-	debug_print("connect ok\n");
+	if (result.error >= 0) {
+		cert_len = mailstream_ssl_get_certificate(imap->imap_stream, &certificate);
+		if (etpan_certificate_check(certificate, cert_len, &param) < 0)
+			return -1;
+		if (certificate) 
+			free(certificate); 
+	}
+	debug_print("connect %d\n", result.error);
 	
 	return result.error;
 }
@@ -347,7 +382,7 @@ static void capability_run(struct etpan_thread_op * op)
 	r = mailimap_capability(param->imap, &caps);
 	
 	result->error = r;
-	result->caps = caps;
+	result->caps = (r == 0 ? caps : NULL);
 }
 
 
@@ -572,7 +607,8 @@ static void status_run(struct etpan_thread_op * op)
 }
 
 int imap_threaded_status(Folder * folder, const char * mb,
-			 struct mailimap_mailbox_data_status ** data_status)
+			 struct mailimap_mailbox_data_status ** data_status,
+			 guint mask)
 {
 	struct status_param param;
 	struct status_result result;
@@ -581,17 +617,26 @@ int imap_threaded_status(Folder * folder, const char * mb,
 	debug_print("imap status - begin\n");
 	
 	status_att_list = mailimap_status_att_list_new_empty();
-	mailimap_status_att_list_add(status_att_list,
+	if (mask & 1 << 0) {
+		mailimap_status_att_list_add(status_att_list,
 				     MAILIMAP_STATUS_ATT_MESSAGES);
-	mailimap_status_att_list_add(status_att_list,
+	}
+	if (mask & 1 << 1) {
+		mailimap_status_att_list_add(status_att_list,
 				     MAILIMAP_STATUS_ATT_RECENT);
-	mailimap_status_att_list_add(status_att_list,
+	}
+	if (mask & 1 << 2) {
+		mailimap_status_att_list_add(status_att_list,
 				     MAILIMAP_STATUS_ATT_UIDNEXT);
-	mailimap_status_att_list_add(status_att_list,
+	}
+	if (mask & 1 << 3) {
+		mailimap_status_att_list_add(status_att_list,
 				     MAILIMAP_STATUS_ATT_UIDVALIDITY);
-	mailimap_status_att_list_add(status_att_list,
+	}
+	if (mask & 1 << 4) {
+		mailimap_status_att_list_add(status_att_list,
 				     MAILIMAP_STATUS_ATT_UNSEEN);
-	
+	}
 	param.imap = get_imap(folder);
 	param.mb = mb;
 	param.status_att_list = status_att_list;
@@ -602,6 +647,8 @@ int imap_threaded_status(Folder * folder, const char * mb,
 	
 	* data_status = result.data_status;
 	
+	mailimap_status_att_list_free(status_att_list);
+	
 	return result.error;
 }
 
@@ -655,20 +702,16 @@ int imap_threaded_noop(Folder * folder, unsigned int * p_exists)
 }
 
 
-struct starttls_param {
-	mailimap * imap;
-};
-
 struct starttls_result {
 	int error;
 };
 
 static void starttls_run(struct etpan_thread_op * op)
 {
-	struct starttls_param * param;
+	struct connect_param * param;
 	struct starttls_result * result;
 	int r;
-	
+
 	param = op->param;
 	r = mailimap_starttls(param->imap);
 	
@@ -689,9 +732,10 @@ static void starttls_run(struct etpan_thread_op * op)
 			result->error = MAILIMAP_ERROR_STREAM;
 			return;
 		}
-		tls_low = mailstream_low_ssl_open(fd);
+
+		tls_low = mailstream_low_tls_open(fd);
 		if (tls_low == NULL) {
-			debug_print("imap starttls run - can't ssl_open\n");
+			debug_print("imap starttls run - can't tls_open\n");
 			result->error = MAILIMAP_ERROR_STREAM;
 			return;
 		}
@@ -700,19 +744,30 @@ static void starttls_run(struct etpan_thread_op * op)
 	}
 }
 
-int imap_threaded_starttls(Folder * folder)
+int imap_threaded_starttls(Folder * folder, const gchar *host, int port)
 {
-	struct starttls_param param;
+	struct connect_param param;
 	struct starttls_result result;
+	int cert_len;
+	unsigned char *certificate;
 	
 	debug_print("imap starttls - begin\n");
 	
 	param.imap = get_imap(folder);
+	param.server = host;
+	param.port = port;
 	
 	threaded_run(folder, &param, &result, starttls_run);
 	
 	debug_print("imap starttls - end\n");
 	
+	if (result.error == 0) {
+		cert_len = mailstream_ssl_get_certificate(param.imap->imap_stream, &certificate);
+		if (etpan_certificate_check(certificate, cert_len, &param) < 0)
+			result.error = MAILIMAP_ERROR_STREAM;
+		if (certificate) 
+			free(certificate); 
+	}	
 	return result.error;
 }
 
@@ -1030,6 +1085,14 @@ static void search_run(struct etpan_thread_op * op)
 							  NULL, NULL, NULL, NULL, NULL,
 							  NULL, 0, NULL, NULL, NULL);
 		break;
+	case IMAP_SEARCH_TYPE_DELETED:
+		search_type_key = mailimap_search_key_new(MAILIMAP_SEARCH_KEY_DELETED,
+							  NULL, NULL, NULL, NULL, NULL,
+							  NULL, NULL, NULL, NULL, NULL,
+							  NULL, NULL, NULL, NULL, 0,
+							  NULL, NULL, NULL, NULL, NULL,
+							  NULL, 0, NULL, NULL, NULL);
+		break;
 	}
 	
 	if (search_type_key != NULL) {
@@ -1057,7 +1120,7 @@ int imap_threaded_search(Folder * folder, int search_type,
 	mailimap * imap;
 	
 	debug_print("imap search - begin\n");
-	
+
 	imap = get_imap(folder);
 	param.imap = imap;
 	param.set = set;
@@ -1703,6 +1766,8 @@ fetch_to_env_info(struct mailimap_msg_att * msg_att)
 	imap_get_msg_att_info(msg_att, &uid, &headers, &size,
 			      &att_dyn);
 	
+	if (!headers)
+		return NULL;
 	info = malloc(sizeof(* info));
 	info->uid = uid;
 	info->headers = strdup(headers);
@@ -1719,7 +1784,6 @@ imap_fetch_result_to_envelop_list(clist * fetch_result,
 	clistiter * cur;
 	unsigned int i;
 	carray * env_list;
-  
 	i = 0;
   
 	env_list = carray_new(16);
@@ -1732,6 +1796,8 @@ imap_fetch_result_to_envelop_list(clist * fetch_result,
 		msg_att = clist_content(cur);
 
 		env_info = fetch_to_env_info(msg_att);
+		if (!env_info)
+			return MAILIMAP_ERROR_MEMORY;
 		carray_add(env_list, env_info, NULL);
 	}
   
@@ -1797,7 +1863,7 @@ imap_get_envelopes_list(mailimap * imap, struct mailimap_set * set,
 	int res;
 	clist * fetch_result;
 	int r;
-	carray * env_list;
+	carray * env_list = NULL;
 	chashdatum key;
 	chashdatum value;
 	
@@ -1831,11 +1897,13 @@ imap_get_envelopes_list(mailimap * imap, struct mailimap_set * set,
 		break;
 	default:
 		mailimap_fetch_type_free(fetch_type);
+		debug_print("uid_fetch: %d\n", r);
 		return r;
 	}
 	
 	if (clist_begin(fetch_result) == NULL) {
 		res = MAILIMAP_ERROR_FETCH;
+		debug_print("clist_begin = NULL\n");
 		goto err;
 	}
 	
@@ -1845,6 +1913,7 @@ imap_get_envelopes_list(mailimap * imap, struct mailimap_set * set,
 	if (r != MAILIMAP_NO_ERROR) {
 		mailimap_fetch_type_free(fetch_type);
 		res = MAILIMAP_ERROR_MEMORY;
+		debug_print("fetch_result_to_envelop_list: %d\n", res);
 		goto err;
 	}
 	
@@ -1957,6 +2026,7 @@ struct append_param {
 
 struct append_result {
 	int error;
+	int uid;
 };
 
 static void append_run(struct etpan_thread_op * op)
@@ -1967,7 +2037,7 @@ static void append_run(struct etpan_thread_op * op)
 	char * data;
 	size_t size;
 	struct stat stat_buf;
-	int fd;
+	int fd, uid = 0;
 	
 	param = op->param;
 	result = op->result;
@@ -1994,19 +2064,20 @@ static void append_run(struct etpan_thread_op * op)
 	
 	r = mailimap_append(param->imap, param->mailbox,
 			    param->flag_list, NULL,
-			    data, size);
+			    data, size/*, &uid */);
 	
 	munmap(data, size);
 	close(fd);
 	
 	result->error = r;
-	
-	debug_print("imap append run - end %i\n", r);
+	result->uid = uid;
+	debug_print("imap append run - end %i uid %d\n", r, uid);
 }
 
 int imap_threaded_append(Folder * folder, const char * mailbox,
 			 const char * filename,
-			 struct mailimap_flag_list * flag_list)
+			 struct mailimap_flag_list * flag_list,
+			 int *uid)
 {
 	struct append_param param;
 	struct append_result result;
@@ -2026,7 +2097,9 @@ int imap_threaded_append(Folder * folder, const char * mailbox,
 		return result.error;
 	
 	debug_print("imap append - end\n");
-	
+	if (uid != NULL)
+		*uid = result.uid;
+
 	return result.error;
 }
 
@@ -2299,12 +2372,13 @@ int socket_connect_cmd(mailimap * imap, const char * command,
 	}
 	
 	r = mailimap_connect(imap, s);
-	if (r != MAILIMAP_NO_ERROR) {
+	if (r != MAILIMAP_NO_ERROR_AUTHENTICATED
+	&&  r != MAILIMAP_NO_ERROR_NON_AUTHENTICATED) {
 		mailstream_close(s);
 		return r;
 	}
 	
-	return MAILIMAP_NO_ERROR;
+	return r;
 }
 
 /* connect cmd */
