@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2002-2003 Match Grun
+ * Copyright (C) 2002-2006 Match Grun and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /*
@@ -46,6 +46,7 @@
 #include "exporthtml.h"
 #include "utils.h"
 #include "manage_window.h"
+#include "filesel.h"
 
 #define PAGE_FILE_INFO             0
 #define PAGE_FORMAT                1
@@ -183,9 +184,9 @@ static gboolean exp_html_move_file( void ) {
 		"does not exist. OK to create new directory?" ),
 		_exportCtl_->dirOutput );
 	aval = alertpanel( _("Create Directory" ),
-		msg, GTK_STOCK_YES, GTK_STOCK_NO, NULL );
+		msg, GTK_STOCK_NO, GTK_STOCK_YES, NULL );
 	g_free( msg );
-	if( aval != G_ALERTDEFAULT ) return FALSE;
+	if( aval != G_ALERTALTERNATE ) return FALSE;
 
 	/* Create directory */
 	if( ! exporthtml_create_dir( _exportCtl_ ) ) {
@@ -299,6 +300,8 @@ static void export_html_next( GtkWidget *widget ) {
 		if( exp_html_move_format() ) {
 			gtk_notebook_set_current_page(
 				GTK_NOTEBOOK(exphtml_dlg.notebook), PAGE_FINISH );
+			gtk_button_set_label(GTK_BUTTON(exphtml_dlg.btnCancel),
+				GTK_STOCK_CLOSE);
 			exp_html_finish_show();
 			exporthtml_save_settings( _exportCtl_ );
 			export_html_message();
@@ -319,75 +322,27 @@ static void export_html_browse( GtkWidget *widget, gpointer data ) {
 	g_free( uri );
 }
 
-/**
- * Callback function to accept HTML file selection.
- * \param widget Widget (button).
- * \param data   User data.
- */
-static void exp_html_file_ok( GtkWidget *widget, gpointer data ) {
-	const gchar *sFile;
-	AddressFileSelection *afs;
-	GtkWidget *fileSel;
-
-	afs = ( AddressFileSelection * ) data;
-	fileSel = afs->fileSelector;
-	sFile = gtk_file_selection_get_filename( GTK_FILE_SELECTION(fileSel) );
-
-	afs->cancelled = FALSE;
-	gtk_entry_set_text( GTK_ENTRY(exphtml_dlg.entryHtml), sFile );
-	gtk_widget_hide( afs->fileSelector );
-	gtk_grab_remove( afs->fileSelector );
-	gtk_widget_grab_focus( exphtml_dlg.entryHtml );
-}
-
-/**
- * Callback function to cancel HTML file selection dialog.
- * \param widget Widget (button).
- * \param data   User data.
- */
-static void exp_html_file_cancel( GtkWidget *widget, gpointer data ) {
-	AddressFileSelection *afs = ( AddressFileSelection * ) data;
-	afs->cancelled = TRUE;
-	gtk_widget_hide( afs->fileSelector );
-	gtk_grab_remove( afs->fileSelector );
-	gtk_widget_grab_focus( exphtml_dlg.entryHtml );
-}
-
 /**
  * Create HTML file selection dialog.
  * \param afs Address file selection data.
  */
 static void exp_html_file_select_create( AddressFileSelection *afs ) {
-	GtkWidget *fileSelector;
-
-	fileSelector = gtk_file_selection_new( _("Select HTML Output File") );
-	gtk_file_selection_hide_fileop_buttons( GTK_FILE_SELECTION(fileSelector) );
-	gtk_file_selection_complete( GTK_FILE_SELECTION(fileSelector), "*.html" );
-	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(fileSelector)->ok_button),
-			 "clicked", 
-			 G_CALLBACK(exp_html_file_ok), (gpointer)afs);
-	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(fileSelector)->cancel_button),
-			 "clicked", 
-			 G_CALLBACK(exp_html_file_cancel), (gpointer)afs);
-	afs->fileSelector = fileSelector;
-	afs->cancelled = TRUE;
+	gchar *file = filesel_select_file_save(_("Select HTML output file"), NULL);
+	
+	if (file == NULL)
+		afs->cancelled = TRUE;
+	else {
+		afs->cancelled = FALSE;
+		gtk_entry_set_text( GTK_ENTRY(exphtml_dlg.entryHtml), file );
+		g_free(file);
+	}
 }
 
 /**
  * Callback function to display HTML file selection dialog.
  */
 static void exp_html_file_select( void ) {
-	gchar *sFile;
-	if( ! _exp_html_file_selector_.fileSelector )
-		exp_html_file_select_create( & _exp_html_file_selector_ );
-
-	sFile = gtk_editable_get_chars( GTK_EDITABLE(exphtml_dlg.entryHtml), 0, -1 );
-	gtk_file_selection_set_filename(
-		GTK_FILE_SELECTION( _exp_html_file_selector_.fileSelector ),
-		sFile );
-	g_free( sFile );
-	gtk_widget_show( _exp_html_file_selector_.fileSelector );
-	gtk_grab_add( _exp_html_file_selector_.fileSelector );
+	exp_html_file_select_create( & _exp_html_file_selector_ );
 }
 
 /**
@@ -445,7 +400,7 @@ static void export_html_page_file( gint pageNum, gchar *pageLbl ) {
 	gtk_table_attach(GTK_TABLE(table), entryHtml, 1, 2, top, (top + 1),
 		GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
 
-	btnFile = gtk_button_new_with_label( _(" ... "));
+	btnFile = gtkut_get_browse_file_btn(_("B_rowse"));
 	gtk_table_attach(GTK_TABLE(table), btnFile, 2, 3, top, (top + 1),
 		GTK_FILL, 0, 3, 0);
 
@@ -587,7 +542,7 @@ static void export_html_page_format( gint pageNum, gchar *pageLbl ) {
 
 	/* Fourth row */
 	top++;
-	checkLinkEMail = gtk_check_button_new_with_label( _( "Format E-Mail Links" ) );
+	checkLinkEMail = gtk_check_button_new_with_label( _( "Format Email Links" ) );
 	gtk_table_attach(GTK_TABLE(table), checkLinkEMail, 1, 2, top, (top + 1),
 		GTK_FILL, 0, 0, 0);
 
@@ -799,6 +754,9 @@ void addressbook_exp_html( AddressCache *cache ) {
 	/* Setup GUI */
 	if( ! exphtml_dlg.window )
 		export_html_create();
+
+	gtk_button_set_label(GTK_BUTTON(exphtml_dlg.btnCancel),
+			     GTK_STOCK_CANCEL);
 	exphtml_dlg.cancelled = FALSE;
 	gtk_widget_show(exphtml_dlg.window);
 	manage_window_set_transient(GTK_WINDOW(exphtml_dlg.window));
