@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2002 Match Grun
+ * Copyright (C) 2002-2006 Match Grun and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,13 +14,17 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /*
  * Export address book to HTML file.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <sys/stat.h>
 #include <dirent.h>
 #include <errno.h>
@@ -34,11 +38,16 @@
 #include "exporthtml.h"
 #include "xmlprops.h"
 
+#ifdef MKDIR_TAKES_ONE_ARG
+#undef mkdir
+#define mkdir(a,b) mkdir(a)
+#endif
+
 #define DFL_DIR_SYLPHEED_OUT  "sylpheed-out"
 #define DFL_FILE_SYLPHEED_OUT "addressbook.html"
 
 #define FMT_BUFSIZE         2048
-#define HTML_SPACE          "&nbsp;"
+#define SC_HTML_SPACE          "&nbsp;"
 #define BORDER_SIZE         2
 #define CELL_PADDING        2
 #define CELL_SPACING        2
@@ -550,7 +559,7 @@ static void exporthtml_fmt_email(
 	gchar *name;
 
 	if( listEMail == NULL ) {
-		fprintf( stream, HTML_SPACE );
+		fprintf( stream, SC_HTML_SPACE );
 		return;
 	}
 
@@ -626,7 +635,7 @@ static gboolean exporthtml_fmt_group(
 	fprintf( stream, "%s", _( "Group Name" ) );
 	fprintf( stream, "</th>\n" );
 	fprintf( stream, "  <th width=\"300\">" );
-	fprintf( stream, "%s", _( "E-Mail Address" ) );
+	fprintf( stream, "%s", _( "Email Address" ) );
 	fprintf( stream, "</th>\n" );
 	fprintf( stream, "</tr>\n" );
 	list = exporthtml_sort_name( folder->listGroup );
@@ -681,7 +690,7 @@ static void exporthtml_fmt_attribs(
 	GList *list;
 
 	if( listAttr == NULL ) {
-		fprintf( stream, HTML_SPACE );
+		fprintf( stream, SC_HTML_SPACE );
 		return;
 	}
 
@@ -790,7 +799,7 @@ static gboolean exporthtml_fmt_person(
 	fprintf( stream, "%s", _( "Display Name" ) );
 	fprintf( stream, "</th>\n" );
 	fprintf( stream, "  <th width=\"300\">" );
-	fprintf( stream, "%s", _( "E-Mail Address" ) );
+	fprintf( stream, "%s", _( "Email Address" ) );
 	fprintf( stream, "</th>\n" );
 	fprintf( stream, "  <th width=\"200\">" );
 	fprintf( stream, "%s", _( "Full Name" ) );
@@ -839,7 +848,7 @@ static gboolean exporthtml_fmt_person(
 				fprintf( stream, "%s", buf );
 			}
 			else {
-				fprintf( stream, "%s", HTML_SPACE );
+				fprintf( stream, "%s", SC_HTML_SPACE );
 			}
 			fprintf( stream, "</td>\n" );
 
@@ -998,7 +1007,7 @@ void exporthtml_process(
 		return;
 	}
 
-	title = _( "Sylpheed Address Book" );
+	title = _( "Sylpheed-Claws Address Book" );
 	rootFolder = cache->rootFolder;
 	dsName = cache->name;
 
