@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2003 Match Grun
+ * Copyright (C) 2003-2006 Match Grun and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,12 +14,15 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /*
  * Export address book to LDIF file.
  */
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <sys/stat.h>
 #include <dirent.h>
@@ -35,6 +38,12 @@
 #include "xmlprops.h"
 #include "ldif.h"
 
+
+#ifdef MKDIR_TAKES_ONE_ARG
+#undef mkdir
+#define mkdir(a,b) mkdir(a)
+#endif
+
 #define DFL_DIR_SYLPHEED_OUT  "sylpheed-out"
 #define DFL_FILE_SYLPHEED_OUT "addressbook.ldif"
 
@@ -172,7 +181,7 @@ void exportldif_set_suffix( ExportLdifCtl *ctl, const char *value ) {
  * <ul>
  * <li><code>EXPORT_LDIF_ID_UID</code> - Use Sylpheed UID.</li>
  * <li><code>EXPORT_LDIF_ID_DNAME</code> - Use Sylpheed display name.</li>
- * <li><code>EXPORT_LDIF_ID_EMAIL</code> - Use first E-Mail address.</li>
+ * <li><code>EXPORT_LDIF_ID_EMAIL</code> - Use first Email address.</li>
  * </ul>
  */
 void exportldif_set_rdn( ExportLdifCtl *ctl, const gint value ) {
