@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "defs.h"
@@ -205,6 +205,15 @@ FolderItem *foldersel_folder_sel(Folder *cur_folder, FolderSelectionType type,
 		return NULL;
 }
 
+static void foldersel_size_allocate_cb(GtkWidget *widget,
+					 GtkAllocation *allocation)
+{
+	g_return_if_fail(allocation != NULL);
+
+	prefs_common.folderselwin_width = allocation->width;
+	prefs_common.folderselwin_height = allocation->height;
+}
+
 static void foldersel_create(void)
 {
 	GtkWidget *vbox;
@@ -213,6 +222,7 @@ static void foldersel_create(void)
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 	GtkTreeSelection *selection;
+	static GdkGeometry geometry;
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(window), _("Select folder"));
@@ -221,19 +231,20 @@ static void foldersel_create(void)
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, FALSE);
 	gtk_window_set_wmclass
-		(GTK_WINDOW(window), "folder_selection", "Sylpheed");
+		(GTK_WINDOW(window), "folder_selection", "Sylpheed-Claws");
 	gtk_widget_realize(window);
 	g_signal_connect(G_OBJECT(window), "delete_event",
 			 G_CALLBACK(delete_event), NULL);
 	g_signal_connect(G_OBJECT(window), "key_press_event",
 			 G_CALLBACK(key_pressed), NULL);
+	g_signal_connect(G_OBJECT(window), "size_allocate",
+			 G_CALLBACK(foldersel_size_allocate_cb), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
 
 	vbox = gtk_vbox_new(FALSE, 4);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
 	scrolledwin = gtk_scrolled_window_new(NULL, NULL);
-	gtk_widget_set_size_request(scrolledwin, 300, 360);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin),
 				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwin),
@@ -267,7 +278,6 @@ static void foldersel_create(void)
 
 	g_signal_connect(G_OBJECT(treeview), "row-activated",
 			 G_CALLBACK(foldersel_tree_activated), NULL);
-
 	gtk_container_add(GTK_CONTAINER(scrolledwin), treeview);
 
 	column = gtk_tree_view_column_new();
@@ -302,9 +312,9 @@ static void foldersel_create(void)
 			 G_CALLBACK(foldersel_entry_activated), NULL);
 
 	gtkut_stock_button_set_create(&confirm_area,
-				      &ok_button,     GTK_STOCK_OK,
+				      &new_button,    GTK_STOCK_NEW,
 				      &cancel_button, GTK_STOCK_CANCEL,
-				      &new_button,    GTK_STOCK_NEW);
+				      &ok_button,     GTK_STOCK_OK);
 
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_button);
@@ -316,6 +326,16 @@ static void foldersel_create(void)
 	g_signal_connect(G_OBJECT(new_button), "clicked",
 			 G_CALLBACK(foldersel_new_folder), NULL);
 
+	if (!geometry.min_height) {
+		geometry.min_width = 300;
+		geometry.min_height = 360;
+	}
+
+	gtk_window_set_geometry_hints(GTK_WINDOW(window), NULL, &geometry,
+				      GDK_HINT_MIN_SIZE);
+	gtk_widget_set_size_request(window, prefs_common.folderselwin_width,
+				    prefs_common.folderselwin_height);
+
 	gtk_widget_show_all(vbox);
 }
 
