@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "defs.h"
@@ -64,6 +64,7 @@
 #include "folderutils.h"
 #include "partial_download.h"
 #include "prefs_folder_column.h"
+#include "filtering.h"
 
 #define COL_FOLDER_WIDTH	150
 #define COL_NUM_WIDTH		32
@@ -291,7 +292,8 @@ static GtkItemFactoryEntry folder_view_trash_popup_entries[] = {
 
 GtkTargetEntry folderview_drag_types[] =
 {
-	{"text/plain", GTK_TARGET_SAME_APP, TARGET_DUMMY}
+	{"sylpheed-claws/internal", GTK_TARGET_SAME_APP, TARGET_DUMMY},
+	{"text/uri-list", 0, TARGET_MAIL_URI_LIST}
 };
 
 void folderview_initialize(void)
@@ -489,7 +491,7 @@ GtkWidget *folderview_ctree_create(FolderView *folderview)
 
         /* drop callback */
 	gtk_drag_dest_set(ctree, GTK_DEST_DEFAULT_ALL & ~GTK_DEST_DEFAULT_HIGHLIGHT,
-			  summary_drag_types, 1,
+			  folderview_drag_types, 2,
 			  GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_DEFAULT);
 	g_signal_connect(G_OBJECT(ctree), "drag_motion",
 			 G_CALLBACK(folderview_drag_motion_cb),
@@ -565,7 +567,7 @@ FolderView *folderview_create(void)
 
 	gtk_widget_show_all(scrolledwin);
 	
-	folderview->target_list = gtk_target_list_new(folderview_drag_types, 1);
+	folderview->target_list = gtk_target_list_new(folderview_drag_types, 2);
 	folderview_list = g_list_append(folderview_list, folderview);
 
 	return folderview;
@@ -574,6 +576,7 @@ FolderView *folderview_create(void)
 void folderview_init(FolderView *folderview)
 {
 	GtkWidget *ctree = folderview->ctree;
+	GdkColor gdk_color;
 
 	stock_pixmap_gdk(ctree, STOCK_PIXMAP_INBOX_CLOSE, &inboxxpm, &inboxxpmmask);
 	stock_pixmap_gdk(ctree, STOCK_PIXMAP_INBOX_CLOSE_HRM, &inboxhrmxpm, &inboxhrmxpmmask);
@@ -632,19 +635,20 @@ void folderview_init(FolderView *folderview)
 					(normal_style->font_desc);
 			normal_style->font_desc = font_desc;
 		}
+		gtkut_convert_int_to_gdk_color(prefs_common.color_new, &gdk_color);
 		normal_color_style = gtk_style_copy(normal_style);
-		normal_color_style->fg[GTK_STATE_NORMAL] = folderview->color_new;
+		normal_color_style->fg[GTK_STATE_NORMAL] = gdk_color;
 
 		gtk_widget_set_style(ctree, normal_style);
 	}
 
 	if (!bold_style) {
-		PangoFontDescription *font_desc;
+		gtkut_convert_int_to_gdk_color(prefs_common.color_new, &gdk_color);
 		bold_style = gtk_style_copy(gtk_widget_get_style(ctree));
 		pango_font_description_set_weight
 			(bold_style->font_desc, PANGO_WEIGHT_BOLD);
 		bold_color_style = gtk_style_copy(bold_style);
-		bold_color_style->fg[GTK_STATE_NORMAL] = folderview->color_new;
+		bold_color_style->fg[GTK_STATE_NORMAL] = gdk_color;
 
 		bold_tgtfold_style = gtk_style_copy(bold_style);
 		bold_tgtfold_style->fg[GTK_STATE_NORMAL] = folderview->color_op;
@@ -656,6 +660,9 @@ void folderview_set(FolderView *folderview)
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	MainWindow *mainwin = folderview->mainwin;
 
+	if (!mainwin)
+		return;
+
 	debug_print("Setting folder info...\n");
 	STATUSBAR_PUSH(mainwin, _("Setting folder info..."));
 
@@ -703,11 +710,25 @@ static void mark_all_read_cb(FolderView *folderview, guint action,
                              GtkWidget *widget)
 {
 	FolderItem *item;
-
+	AlertValue val;
+	
 	item = folderview_get_selected_item(folderview);
 	if (item == NULL)
 		return;
 
+	if (prefs_common.ask_mark_all_read) {
+		val = alertpanel_full(_("Mark all as read"),
+			_("Do you really want to mark all mails in this "
+			  "folder as read ?"), GTK_STOCK_NO, GTK_STOCK_YES, NULL,
+			  TRUE, NULL, ALERT_QUESTION, G_ALERTALTERNATE);
+
+		if (val == G_ALERTDEFAULT ||
+		    val == (G_ALERTDEFAULT|G_ALERTDISABLE))
+			return;
+		else if (val == (G_ALERTALTERNATE|G_ALERTDISABLE))
+			prefs_common.ask_mark_all_read = FALSE;
+	}
+	
 	summary_lock(folderview->summaryview);
 	folder_item_update_freeze();
 	if (folderview->summaryview->folder_item == item)
@@ -881,7 +902,7 @@ void folderview_append_item(FolderItem *item)
 				gtk_clist_freeze(GTK_CLIST(ctree));
 
 				text[col_pos[F_COL_FOLDER]] = item->name;
-				child = gtk_ctree_insert_node
+				child = gtk_sctree_insert_node
 					(ctree, node, NULL, text,
 					 FOLDER_SPACING,
 					 folderxpm, folderxpmmask,
@@ -902,11 +923,29 @@ void folderview_append_item(FolderItem *item)
 static void folderview_set_folders(FolderView *folderview)
 {
 	GList *list;
-
+	static gboolean missing_imap_warning = TRUE;
 	list = folder_get_list();
 
-	for (; list != NULL; list = list->next)
+	for (; list != NULL; list = list->next) {
+#ifndef HAVE_LIBETPAN
+		if ((FOLDER(list->data))
+		&&  (FOLDER(list->data))->klass
+		&&  (FOLDER(list->data))->klass->type == F_IMAP
+		&&  missing_imap_warning) {
+			missing_imap_warning = FALSE;
+			alertpanel_error(
+				_("You have one or more IMAP accounts "
+				  "defined. However this version of "
+				  "Sylpheed-Claws has been built without "
+				  "IMAP support; your IMAP account(s) are "
+				  "disabled.\n\n"
+				  "You probably need to "
+				  "install libetpan and recompile "
+				  "Sylpheed-Claws."));
+		}
+#endif
 		folderview_append_folder(folderview, FOLDER(list->data));
+	}
 }
 
 static void folderview_scan_tree_func(Folder *folder, FolderItem *item,
@@ -933,30 +972,7 @@ static void folderview_scan_tree_func(Folder *folder, FolderItem *item,
 	}
 }
 
-static GtkWidget *label_window_create(const gchar *str)
-{
-	GtkWidget *window;
-	GtkWidget *label;
-
-	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_widget_set_size_request(window, 380, 60);
-	gtk_container_set_border_width(GTK_CONTAINER(window), 8);
-	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
-	gtk_window_set_title(GTK_WINDOW(window), str);
-	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
-	gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
-	manage_window_set_transient(GTK_WINDOW(window));
-
-	label = gtk_label_new(str);
-	gtk_container_add(GTK_CONTAINER(window), label);
-	gtk_widget_show(label);
-
-	gtk_widget_show_now(window);
-
-	return window;
-}
-
-void folderview_rescan_tree(Folder *folder)
+void folderview_rescan_tree(Folder *folder, gboolean rebuild)
 {
 	GtkWidget *window;
 
@@ -964,11 +980,24 @@ void folderview_rescan_tree(Folder *folder)
 
 	if (!folder->klass->scan_tree) return;
 
+	if (rebuild && 
+	    alertpanel_full(_("Rebuild folder tree"), 
+	    		 _("Rebuilding the folder tree will remove "
+			   "local caches. Do you want to continue?"),
+		       	 GTK_STOCK_NO, GTK_STOCK_YES, NULL, FALSE,
+		       	 NULL, ALERT_WARNING, G_ALERTDEFAULT) 
+		!= G_ALERTALTERNATE) {
+		return;
+	}
+
 	inc_lock();
-	window = label_window_create(_("Rebuilding folder tree..."));
+	if (rebuild)
+		window = label_window_create(_("Rebuilding folder tree..."));
+	else 
+		window = label_window_create(_("Scanning folder tree..."));
 
 	folder_set_ui_func(folder, folderview_scan_tree_func, NULL);
-	folder_scan_tree(folder);
+	folder_scan_tree(folder, rebuild);
 	folder_set_ui_func(folder, NULL, NULL);
 
 	folderview_set_all();
@@ -992,7 +1021,7 @@ gint folderview_check_new(Folder *folder)
 	GtkCTreeNode *node;
 	gint new_msgs = 0;
 	gint former_new_msgs = 0;
-	gint former_new = 0;
+	gint former_new = 0, former_unread = 0, former_total;
 
 	for (list = folderview_list; list != NULL; list = list->next) {
 		folderview = (FolderView *)list->data;
@@ -1000,7 +1029,6 @@ gint folderview_check_new(Folder *folder)
 
 		inc_lock();
 		main_window_lock(folderview->mainwin);
-		gtk_widget_set_sensitive(folderview->ctree, FALSE);
 
 		for (node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
 		     node != NULL; node = gtkut_ctree_node_next(ctree, node)) {
@@ -1012,7 +1040,9 @@ gint folderview_check_new(Folder *folder)
 			if (!item->prefs->newmailcheck) continue;
 
 			folderview_scan_tree_func(item->folder, item, NULL);
-			former_new = item->new_msgs;
+			former_new    = item->new_msgs;
+			former_unread = item->unread_msgs;
+			former_total  = item->total_msgs;
 
 			if (folder_item_scan(item) < 0) {
 				summaryview_unlock(folderview->summaryview, item);
@@ -1020,12 +1050,15 @@ gint folderview_check_new(Folder *folder)
 					break;
 			}
 
-			folderview_update_node(folderview, node);
+			if (former_new    != item->new_msgs ||
+			    former_unread != item->unread_msgs ||
+			    former_total  != item->total_msgs)
+				folderview_update_node(folderview, node);
+
 			new_msgs += item->new_msgs;
 			former_new_msgs += former_new;
 		}
 
-		gtk_widget_set_sensitive(folderview->ctree, TRUE);
 		main_window_unlock(folderview->mainwin);
 		inc_unlock();
 	}
@@ -1185,6 +1218,43 @@ static gboolean folderview_have_matching_children(FolderView *folderview,
 	return folderview_have_matching_children_sub(folderview, item, FALSE);
 }
 
+static gboolean folderview_have_marked_children_sub(FolderView *folderview,
+						    FolderItem *item,
+						    gboolean in_sub)
+{
+	GNode *node = NULL;
+	
+	if (!item || !item->folder || !item->folder->node)
+		return FALSE;
+		
+	node = item->folder->node;
+	
+	node = g_node_find(node, G_PRE_ORDER, G_TRAVERSE_ALL, item);
+	node = node->children;
+
+	if (item->marked_msgs != 0) {
+		return TRUE;
+	}
+
+	while (node != NULL) {
+		if (node && node->data) {
+			FolderItem *next_item = (FolderItem*) node->data;
+			node = node->next;
+			if (folderview_have_marked_children_sub(folderview,
+							        next_item, TRUE))
+				return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+static gboolean folderview_have_marked_children(FolderView *folderview,
+					     FolderItem *item)
+{
+	return folderview_have_marked_children_sub(folderview, item, FALSE);
+}
+
 static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
@@ -1202,13 +1272,28 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 	gboolean add_sub_match_mark;
 	gboolean use_bold, use_color;
 	gint *col_pos = folderview->col_pos;
-
+	SpecialFolderItemType stype;
+	
 	item = gtk_ctree_node_get_row_data(ctree, node);
 	g_return_if_fail(item != NULL);
 
-	mark = (item->marked_msgs != 0);
+	if (!GTK_CTREE_ROW(node)->expanded)
+		mark = folderview_have_marked_children(folderview, item);
+	else
+		mark = (item->marked_msgs != 0);
 
-	switch (item->stype) {
+	stype = item->stype;
+	if (stype == F_NORMAL) {
+		if (folder_has_parent_of_type(item, F_TRASH))
+			stype = F_TRASH;
+		else if (folder_has_parent_of_type(item, F_DRAFT))
+			stype = F_DRAFT;
+		else if (folder_has_parent_of_type(item, F_OUTBOX))
+			stype = F_OUTBOX;
+		else if (folder_has_parent_of_type(item, F_QUEUE))
+			stype = F_QUEUE;
+	}
+	switch (stype) {
 	case F_INBOX:
 		if (item->hide_read_msgs) {
 			xpm = mark?m_inboxhrmxpm:inboxhrmxpm;
@@ -1311,7 +1396,7 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 	    prefs_common.display_folder_unread) {
 		str = g_strdup_printf("%s (%d%s)", name, item->total_msgs,
 				      add_unread_mark ? "+" : "");
-		gtk_ctree_set_node_info(ctree, node, str, FOLDER_SPACING,
+		gtk_sctree_set_node_info(ctree, node, str, FOLDER_SPACING,
 					xpm, mask, openxpm, openmask,
 					FALSE, GTK_CTREE_ROW(node)->expanded);
 		g_free(str);
@@ -1325,7 +1410,7 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 				              item->unreadmarked_msgs > 0 ? "!":"");
 		else
 			str = g_strdup_printf("%s (+)", name);
-		gtk_ctree_set_node_info(ctree, node, str, FOLDER_SPACING,
+		gtk_sctree_set_node_info(ctree, node, str, FOLDER_SPACING,
 					xpm, mask, openxpm, openmask,
 					FALSE, GTK_CTREE_ROW(node)->expanded);
 		g_free(str);
@@ -1333,7 +1418,7 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 		str = g_strdup_printf("%s%s", name, 
 			              item->unreadmarked_msgs > 0 ? " (!)":"");
 	
-		gtk_ctree_set_node_info(ctree, node, str, FOLDER_SPACING,
+		gtk_sctree_set_node_info(ctree, node, str, FOLDER_SPACING,
 					xpm, mask, openxpm, openmask,
 					FALSE, GTK_CTREE_ROW(node)->expanded);
 		g_free(str);
@@ -1371,22 +1456,29 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 	gtk_ctree_node_set_foreground(ctree, node, NULL);
 
 	if (use_bold) {
-		if (item->prefs->color > 0 && !use_color) {
-			GdkColor gdk_color;
+		GdkColor gdk_color;
 
+		if (item->prefs->color > 0 && !use_color) {
 			gtkut_convert_int_to_gdk_color(item->prefs->color, &gdk_color);
 			color_style = gtk_style_copy(bold_style);
 			color_style->fg[GTK_STATE_NORMAL] = gdk_color;
 			style = color_style;
-		} else if (use_color)
+		} else if (use_color) {
+			gtkut_convert_int_to_gdk_color(prefs_common.color_new, &gdk_color);
+			bold_color_style = gtk_style_copy(bold_style);
+			bold_color_style->fg[GTK_STATE_NORMAL] = gdk_color;
 			style = bold_color_style;
-		else
+		} else
 			style = bold_style;
 		if (item->op_count > 0) {
 			style = bold_tgtfold_style;
 		}
 	} else if (use_color) {
-		style = normal_color_style;
+		GdkColor gdk_color;
+
+		gtkut_convert_int_to_gdk_color(prefs_common.color_new, &gdk_color);
+		style = gtk_style_copy(normal_color_style);
+		style->fg[GTK_STATE_NORMAL] = gdk_color;
 		gtk_ctree_node_set_foreground(ctree, node,
 					      &folderview->color_new);
 	} else if (item->op_count > 0) {
@@ -1609,7 +1701,7 @@ static gboolean folderview_button_pressed(GtkWidget *ctree, GdkEventButton *even
 
 	if (!event) return FALSE;
 
-	if (event->button == 1) {
+	if (event->button == 1 || event->button == 2) {
 		folderview->open_folder = TRUE;
 
 	        if (event->type == GDK_2BUTTON_PRESS) {
@@ -1673,11 +1765,13 @@ static gboolean folderview_button_pressed(GtkWidget *ctree, GdkEventButton *even
 	if (NULL != (ac = account_find_from_item(item)))
 		special_trash = account_get_special_folder(ac, F_TRASH);
 
-	if ((item == folder->trash || item == special_trash) &&
+	if ((item == folder->trash || item == special_trash
+	     || folder_has_parent_of_type(item, F_TRASH)) &&
 	    gtk_item_factory_get_item(fpopup_factory, "/Empty trash...") == NULL) {
 		gtk_item_factory_create_item(fpopup_factory, &folder_view_trash_popup_entries[0], folderview, 1);
 		gtk_item_factory_create_item(fpopup_factory, &folder_view_trash_popup_entries[1], folderview, 1);
-	} else if (item != folder->trash && (special_trash == NULL || item != special_trash)) {
+	} else if (item != folder->trash && (special_trash == NULL || item != special_trash)
+	        && !folder_has_parent_of_type(item, F_TRASH)) {
 		gtk_item_factory_delete_entry(fpopup_factory, &folder_view_trash_popup_entries[0]);
 		gtk_item_factory_delete_entry(fpopup_factory, &folder_view_trash_popup_entries[1]);
 	}
@@ -1690,7 +1784,8 @@ static gboolean folderview_button_pressed(GtkWidget *ctree, GdkEventButton *even
 		 folderview->selected == folderview->opened);
 	SET_SENS("/Properties...", item->node->parent != NULL);
 	SET_SENS("/Processing...", item->node->parent != NULL);
-	if (item == folder->trash || item == special_trash) {
+	if (item == folder->trash || item == special_trash
+	    || folder_has_parent_of_type(item, F_TRASH)) {
 		GSList *msglist = folder_item_get_msg_list(item);
 		SET_SENS("/Empty trash...", msglist != NULL);
 		procmsg_msg_list_free(msglist);
@@ -1789,6 +1884,7 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 		if (olditem) {
 			/* will be null if we just moved the previously opened folder */
 			summary_save_prefs_to_folderitem(folderview->summaryview, olditem);
+			summary_show(folderview->summaryview, NULL);
 			folder_item_close(olditem);
 		}
 	}
@@ -1837,7 +1933,7 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 
 		return;
         }
-
+	
 	main_window_cursor_normal(folderview->mainwin);
 
 	/* Show messages */
@@ -1927,6 +2023,7 @@ void folderview_create_folder_node(FolderView *folderview, FolderItem *item)
 	gchar *text[N_FOLDER_COLS] = {NULL, "0", "0", "0"};
 	GtkCTreeNode *node, *parent_node;
 	gint *col_pos = folderview->col_pos;
+	FolderItemUpdateData hookdata;
 
 	parent_node = gtk_ctree_find_by_row_data(ctree, NULL, folder_item_parent(item));
 	if (parent_node == NULL)
@@ -1935,7 +2032,7 @@ void folderview_create_folder_node(FolderView *folderview, FolderItem *item)
 	gtk_clist_freeze(GTK_CLIST(ctree));
 
 	text[col_pos[F_COL_FOLDER]] = item->name;
-	node = gtk_ctree_insert_node(ctree, parent_node, NULL, text,
+	node = gtk_sctree_insert_node(ctree, parent_node, NULL, text,
 				     FOLDER_SPACING,
 				     folderxpm, folderxpmmask,
 				     folderopenxpm, folderopenxpmmask,
@@ -1946,6 +2043,11 @@ void folderview_create_folder_node(FolderView *folderview, FolderItem *item)
 		gtk_ctree_node_set_row_style(ctree, node, normal_style);
 	folderview_sort_folders(folderview, parent_node, item->folder);
 
+	hookdata.item = item;
+	hookdata.update_flags = F_ITEM_UPDATE_NAME;
+	hookdata.msg = NULL;
+	hooks_invoke(FOLDER_ITEM_UPDATE_HOOKLIST, &hookdata);
+
 	gtk_clist_thaw(GTK_CLIST(ctree));
 }
 
@@ -1956,10 +2058,10 @@ static void folderview_empty_trash_cb(FolderView *folderview, guint action,
 	FolderItem *item;
 	GSList *mlist = NULL;
 	GSList *cur = NULL;
-	if (!folderview->selected) return;
 	FolderItem *special_trash = NULL;
 	PrefsAccount *ac;
 
+	if (!folderview->selected) return;
 	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
@@ -1967,12 +2069,13 @@ static void folderview_empty_trash_cb(FolderView *folderview, guint action,
 	if (NULL != (ac = account_find_from_item(item)))
 		special_trash = account_get_special_folder(ac, F_TRASH);
 
-	if (item != item->folder->trash && item != special_trash) return;
+	if (item != item->folder->trash && item != special_trash
+	&&  !folder_has_parent_of_type(item, F_TRASH)) return;
 	
 	if (prefs_common.ask_on_clean) {
 		if (alertpanel(_("Empty trash"),
 			       _("Delete all messages in trash?"),
-			       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
+			       GTK_STOCK_CANCEL, _("+_Empty trash"), NULL) != G_ALERTALTERNATE)
 			return;
 	}
 	
@@ -2057,12 +2160,12 @@ void folderview_move_folder(FolderView *folderview, FolderItem *from_folder,
 					"sub-folder of '%s' ?"), from_folder->name,
 					to_folder->name);
 		status = alertpanel_full(_("Move folder"), buf,
-				       	 GTK_STOCK_YES, GTK_STOCK_NO, NULL, TRUE,
-				       	 NULL, ALERT_QUESTION, G_ALERTALTERNATE);
+				       	 GTK_STOCK_NO, GTK_STOCK_YES, NULL, TRUE,
+				       	 NULL, ALERT_QUESTION, G_ALERTDEFAULT);
 		g_free(buf);
 
-		if (status != G_ALERTDEFAULT
-		 && status != (G_ALERTDEFAULT | G_ALERTDISABLE))
+		if (status != G_ALERTALTERNATE
+		 && status != (G_ALERTALTERNATE | G_ALERTDISABLE))
 			return;
 		if (status & G_ALERTDISABLE)
 			prefs_common.warn_dnd = FALSE;
@@ -2166,21 +2269,63 @@ void folderview_set_target_folder_color(gint color_op)
 	}
 }
 
+static gchar *last_font = NULL;
 void folderview_reflect_prefs_pixmap_theme(FolderView *folderview)
 {
+	/* force reinit */
+	if (last_font) 
+		g_free(last_font);
+	last_font = NULL;
+	
 }
 
 void folderview_reflect_prefs(void)
 {
+	gboolean update_font = TRUE;
 	FolderView *folderview = mainwindow_get_mainwindow()->folderview;
-	normal_style = normal_color_style = bold_style = 
-		bold_color_style = bold_tgtfold_style = NULL;
 	FolderItem *item = folderview_get_selected_item(folderview);
-	folderview_init(folderview);
+	GtkAdjustment *pos = gtk_scrolled_window_get_vadjustment(
+				GTK_SCROLLED_WINDOW(folderview->scrolledwin));
+	gint height = pos->value;
+
+	if (last_font && !strcmp(last_font, NORMAL_FONT))
+		update_font = FALSE;
+
+	if (last_font)
+		g_free(last_font);
+	
+	last_font = g_strdup(NORMAL_FONT);
+
+	if (update_font) {		
+		normal_style = normal_color_style = bold_style = 
+			bold_color_style = bold_tgtfold_style = NULL;
+
+		folderview_init(folderview);
+	}
+	gtk_clist_freeze(GTK_CLIST(folderview->ctree));
 	folderview_column_set_titles(folderview);
 	folderview_set_all();
-	if (item)
+
+	g_signal_handlers_block_by_func
+		(G_OBJECT(folderview->ctree),
+		 G_CALLBACK(folderview_selected), folderview);
+
+	if (item) {
+		GtkCTreeNode *node = gtk_ctree_find_by_row_data(
+			GTK_CTREE(folderview->ctree), NULL, item);
+
 		folderview_select(folderview, item);
+		folderview->selected = node;
+	}
+
+	g_signal_handlers_unblock_by_func
+		(G_OBJECT(folderview->ctree),
+		 G_CALLBACK(folderview_selected), folderview);
+
+	pos = gtk_scrolled_window_get_vadjustment(
+				GTK_SCROLLED_WINDOW(folderview->scrolledwin));
+	gtk_adjustment_set_value(pos, height);
+	gtk_clist_thaw(GTK_CLIST(folderview->ctree));
 }
 
 static void drag_state_stop(FolderView *folderview)
@@ -2247,20 +2392,23 @@ static void folderview_drag_data_get(GtkWidget        *widget,
 	FolderItem *item;
 	GList *cur;
 	gchar *source = NULL;
-
-	for (cur = GTK_CLIST(folderview->ctree)->selection;
-	     cur != NULL; cur = cur->next) {
-		item = gtk_ctree_node_get_row_data
-			(GTK_CTREE(folderview->ctree), 
-			 GTK_CTREE_NODE(cur->data));
-		if (item) {
-			source = g_strdup_printf ("FROM_OTHER_FOLDER%s", folder_item_get_identifier(item));
-			gtk_selection_data_set(selection_data,
-					       selection_data->target, 8,
-					       source, strlen(source));
-			break;
-		} else
-			return;
+	if (info == TARGET_DUMMY) {
+		for (cur = GTK_CLIST(folderview->ctree)->selection;
+		     cur != NULL; cur = cur->next) {
+			item = gtk_ctree_node_get_row_data
+				(GTK_CTREE(folderview->ctree), 
+				 GTK_CTREE_NODE(cur->data));
+			if (item) {
+				source = g_strdup_printf ("FROM_OTHER_FOLDER%s", folder_item_get_identifier(item));
+				gtk_selection_data_set(selection_data,
+						       selection_data->target, 8,
+						       source, strlen(source));
+				break;
+			} else
+				return;
+		}
+	} else {
+		g_warning("unknown info %d\n", info);
 	}
 }
 
@@ -2280,7 +2428,11 @@ gboolean folderview_update_folder(gpointer source, gpointer userdata)
 
 	if (hookdata->update_flags & FOLDER_ADD_FOLDERITEM)
 		folderview_create_folder_node(folderview, hookdata->item);
-	else if (hookdata->update_flags & FOLDER_REMOVE_FOLDERITEM) {
+	else if (hookdata->update_flags & FOLDER_RENAME_FOLDERITEM) {
+		GtkCTreeNode *node = gtk_ctree_find_by_row_data(GTK_CTREE(ctree),
+				NULL, folder_item_parent(hookdata->item));
+		folderview_sort_folders(folderview, node, hookdata->folder);
+	} else if (hookdata->update_flags & FOLDER_REMOVE_FOLDERITEM) {
 		GtkCTreeNode *node;
 
 		node = gtk_ctree_find_by_row_data(GTK_CTREE(ctree), NULL, hookdata->item);
@@ -2339,6 +2491,13 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 			    FOLDER_CLASS(item->folder)->copy_msg != NULL &&
 			    FOLDER_CLASS(item->folder)->create_folder != NULL)
 				acceptable = TRUE;
+		} else {
+			/* comes from another app */
+			/* we are adding messages, so only accept folder items that are 
+			   no root items and can copy messages */
+			if (item && item->folder && folder_item_parent(item) != NULL
+			    && FOLDER_CLASS(item->folder)->add_msg != NULL)
+				acceptable = TRUE;
 		}
 	}
 
@@ -2357,7 +2516,8 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 					(context->actions == GDK_ACTION_COPY ?
 					GDK_ACTION_COPY : GDK_ACTION_MOVE) , time);
 	} else {
-		gtk_ctree_select(GTK_CTREE(widget), folderview->opened);
+		if (folderview->opened)
+			gtk_ctree_select(GTK_CTREE(widget), folderview->opened);
 		gdk_drag_status(context, 0, time);
 	}
 
@@ -2373,6 +2533,50 @@ static void folderview_drag_leave_cb(GtkWidget      *widget,
 	gtk_ctree_select(GTK_CTREE(widget), folderview->opened);
 }
 
+static void free_info (gpointer stuff, gpointer data)
+{
+	g_free(stuff);
+}
+
+void folderview_finish_dnd(const gchar *data, GdkDragContext *drag_context,
+			   guint time, FolderItem *item)
+{
+	GList *list, *tmp;
+	GSList *msglist = NULL;
+	list = uri_list_extract_filenames(data);
+	if (!(item && item->folder && folder_item_parent(item) != NULL
+		    && FOLDER_CLASS(item->folder)->add_msg != NULL))
+	{
+		gtk_drag_finish(drag_context, FALSE, FALSE, time);			
+		return;
+	}	
+	if (!list) {
+		gtk_drag_finish(drag_context, FALSE, FALSE, time);			
+		return;
+	}
+	for (tmp = list; tmp != NULL; tmp = tmp->next) {
+		MsgFileInfo *info = NULL;
+
+		if (file_is_email((gchar *)tmp->data)) {
+			info = g_new0(MsgFileInfo, 1);
+			info->msginfo = NULL;
+			info->file = (gchar *)tmp->data;
+			msglist = g_slist_prepend(msglist, info);
+		}
+	}
+	if (msglist) {
+		msglist = g_slist_reverse(msglist);
+		folder_item_add_msgs(item, msglist, FALSE);
+		g_slist_foreach(msglist, free_info, NULL);
+		g_slist_free(msglist);
+		gtk_drag_finish(drag_context, TRUE, FALSE, time);
+	} else {
+		gtk_drag_finish(drag_context, FALSE, FALSE, time);			
+	}
+	list_free_strings(list);
+	g_list_free(list);
+}
+
 static void folderview_drag_received_cb(GtkWidget        *widget,
 					GdkDragContext   *drag_context,
 					gint              x,
@@ -2383,75 +2587,93 @@ static void folderview_drag_received_cb(GtkWidget        *widget,
 					FolderView       *folderview)
 {
 	gint row, column;
-	FolderItem *item, *src_item;
+	FolderItem *item = NULL, *src_item;
 	GtkCTreeNode *node;
 
-	drag_state_stop(folderview);
-	if ((void *)strstr(data->data, "FROM_OTHER_FOLDER") != (void *)data->data) {
-		/* comes from summaryview */
+	if (info == TARGET_DUMMY) {
+		drag_state_stop(folderview);
+		if ((void *)strstr(data->data, "FROM_OTHER_FOLDER") != (void *)data->data) {
+			/* comes from summaryview */
+			if (gtk_clist_get_selection_info
+				(GTK_CLIST(widget), x - 24, y - 24, &row, &column) == 0)
+				return;
+
+			node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
+			item = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
+			src_item = folderview->summaryview->folder_item;
+
+			/* re-check (due to acceptable possibly set for folder moves */
+			if (!(item && item->folder && item->path && !item->no_select && 
+			      src_item && src_item != item && FOLDER_CLASS(item->folder)->copy_msg != NULL)) {
+				return;
+			}
+			if (item && src_item) {
+				switch (drag_context->action) {
+				case GDK_ACTION_COPY:
+					summary_copy_selected_to(folderview->summaryview, item);
+					gtk_drag_finish(drag_context, TRUE, FALSE, time);
+					break;
+				case GDK_ACTION_MOVE:
+				case GDK_ACTION_DEFAULT:
+				default:
+					if (FOLDER_CLASS(src_item->folder)->remove_msg == NULL)
+				        	summary_copy_selected_to(folderview->summaryview, item);
+					else
+						summary_move_selected_to(folderview->summaryview, item);
+					gtk_drag_finish(drag_context, TRUE, TRUE, time);
+				}
+			} else
+				gtk_drag_finish(drag_context, FALSE, FALSE, time);
+		} else {
+			/* comes from folderview */
+			char *source;
+			gboolean folder_is_normal = TRUE;
+
+			source = data->data + 17;
+			if (gtk_clist_get_selection_info
+			    (GTK_CLIST(widget), x - 24, y - 24, &row, &column) == 0
+			    || *source == 0) {
+				gtk_drag_finish(drag_context, FALSE, FALSE, time);			
+				return;
+			}
+			node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
+			item = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
+			src_item = folder_find_item_from_identifier(source);
+
+			folder_is_normal = 
+				src_item != NULL &&
+				src_item->stype == F_NORMAL &&
+				!folder_has_parent_of_type(src_item, F_OUTBOX) &&
+				!folder_has_parent_of_type(src_item, F_DRAFT) &&
+				!folder_has_parent_of_type(src_item, F_QUEUE) &&
+				!folder_has_parent_of_type(src_item, F_TRASH);
+			if (!item || item->no_select || !src_item 
+			||  !folder_is_normal) {
+				gtk_drag_finish(drag_context, FALSE, FALSE, time);			
+				return;
+			}
+
+			folderview_move_folder(folderview, src_item, item);
+			gtk_drag_finish(drag_context, TRUE, TRUE, time);
+		}
+		folderview->nodes_to_recollapse = NULL;
+	} else if (info == TARGET_MAIL_URI_LIST) {
 		if (gtk_clist_get_selection_info
 			(GTK_CLIST(widget), x - 24, y - 24, &row, &column) == 0)
 			return;
 
 		node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
-		item = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
-		src_item = folderview->summaryview->folder_item;
-		
-		/* re-check (due to acceptable possibly set for folder moves */
-		if (!(item && item->folder && item->path && !item->no_select && 
-		      src_item && src_item != item && FOLDER_CLASS(item->folder)->copy_msg != NULL)) {
-			return;
-		}
-		if (item && src_item) {
-			switch (drag_context->action) {
-			case GDK_ACTION_COPY:
-				summary_copy_selected_to(folderview->summaryview, item);
-				gtk_drag_finish(drag_context, TRUE, FALSE, time);
-				break;
-			case GDK_ACTION_MOVE:
-			case GDK_ACTION_DEFAULT:
-			default:
-				if (FOLDER_CLASS(src_item->folder)->remove_msg == NULL)
-				        summary_copy_selected_to(folderview->summaryview, item);
-				else
-					summary_move_selected_to(folderview->summaryview, item);
-				gtk_drag_finish(drag_context, TRUE, TRUE, time);
-			}
-		} else
-			gtk_drag_finish(drag_context, FALSE, FALSE, time);
-	} else {
-		/* comes from folderview */
-		char *source;
-		gboolean folder_is_normal = TRUE;
-		
-		source = data->data + 17;
-		if (gtk_clist_get_selection_info
-		    (GTK_CLIST(widget), x - 24, y - 24, &row, &column) == 0
-		    || *source == 0) {
+		if (!node) {
 			gtk_drag_finish(drag_context, FALSE, FALSE, time);			
 			return;
 		}
-		node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
 		item = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
-		src_item = folder_find_item_from_identifier(source);
-
-		folder_is_normal = 
-			src_item != NULL &&
-			src_item->stype == F_NORMAL &&
-			!folder_has_parent_of_type(src_item, F_OUTBOX) &&
-			!folder_has_parent_of_type(src_item, F_DRAFT) &&
-			!folder_has_parent_of_type(src_item, F_QUEUE) &&
-			!folder_has_parent_of_type(src_item, F_TRASH);
-		if (!item || item->no_select || !src_item 
-		||  !folder_is_normal) {
+		if (!item) {
 			gtk_drag_finish(drag_context, FALSE, FALSE, time);			
 			return;
 		}
-
-		folderview_move_folder(folderview, src_item, item);
-		gtk_drag_finish(drag_context, TRUE, TRUE, time);
+		folderview_finish_dnd(data->data, drag_context, time, item);
 	}
-	folderview->nodes_to_recollapse = NULL;
 }
 
 static void folderview_drag_end_cb(GtkWidget	    *widget, 
