@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef __FILESEL_H__
@@ -30,5 +30,8 @@ gchar *filesel_select_file_open_folder(const gchar *title, const gchar *path);
 gchar *filesel_select_file_save_folder(const gchar *title, const gchar *path);
 
 GList *filesel_select_multiple_files_open(const gchar *title);
+GList *filesel_select_multiple_files_open_with_filter(	const gchar *title,
+							const gchar *path,
+							const gchar *filter);
 
 #endif /* __FILESEL_H__ */
