@@ -121,13 +121,11 @@ struct _GtkAspell
 	gint		 end_check_pos;
 	gboolean	 misspelled;
 	gboolean	 check_while_typing;
+	gboolean	 recheck_when_changing_dict;
 	gboolean	 use_alternate;
 
 	ContCheckFunc 	 continue_check; 
 
-	GtkWidget	*config_menu;
-	GtkWidget	*popup_config_menu;
-	GtkWidget	*sug_menu;
 	GtkWidget	*replace_entry;
 	GtkWidget 	*parent_window;
 
@@ -159,10 +157,13 @@ static void entry_delete_cb			(GtkTextBuffer	*textbuf,
 						 GtkTextIter	*startiter,
 						 GtkTextIter	*enditer,
 						 GtkAspell	*gtkaspell);
-static gint button_press_intercept_cb		(GtkTextView	*gtktext,
+/*static gint button_press_intercept_cb		(GtkTextView	*gtktext,
 						 GdkEvent	*e, 
 						 GtkAspell	*gtkaspell);
-
+*/
+static void button_press_intercept_cb(GtkTextView *gtktext,
+   			GtkMenu *menu, GtkAspell *gtkaspell);
+			
 /* Checker creation */
 static GtkAspeller* gtkaspeller_new		(Dictionary	*dict);
 static GtkAspeller* gtkaspeller_real_new	(Dictionary	*dict);
@@ -203,20 +204,18 @@ static void toggle_check_while_typing_cb	(GtkWidget	*w,
 						 gpointer	 data);
 
 /* Menu creation */
-static void popup_menu				(GtkAspell	*gtkaspell, 
-						 GdkEventButton	*eb);
-static GtkMenu*	make_sug_menu			(GtkAspell	*gtkaspell);
-static void populate_submenu			(GtkAspell	*gtkaspell, 
-						 GtkWidget	*menu);
-static GtkMenu*	make_config_menu		(GtkAspell	*gtkaspell);
+static GSList*	make_sug_menu			(GtkAspell	*gtkaspell);
+static GSList * populate_submenu		(GtkAspell	*gtkaspell);
+static GSList*	make_config_menu		(GtkAspell	*gtkaspell);
 static void set_menu_pos			(GtkMenu	*menu, 
 						 gint		*x, 
 						 gint		*y, 
 						 gboolean	*push_in,
 						 gpointer	 data);
 /* Other menu callbacks */
-static gboolean cancel_menu_cb			(GtkMenuShell	*w,
-						 gpointer	 data);
+static gboolean aspell_key_pressed		(GtkWidget *widget,
+				  		 GdkEventKey *event,
+				  		 GtkAspell *gtkaspell);
 static void change_dict_cb			(GtkWidget	*w, 
 						 GtkAspell	*gtkaspell);
 static void switch_to_alternate_cb		(GtkWidget	*w, 
@@ -325,6 +324,7 @@ void gtkaspell_checkers_quit(void)
 
 		g_slist_foreach(checkers, free_checkers, NULL);
 		g_slist_free(checkers);
+		gtkaspellcheckers->checkers = NULL;
 	}
 
 	if ((dict_list = gtkaspellcheckers->dictionary_list)) {
@@ -336,7 +336,7 @@ void gtkaspell_checkers_quit(void)
 	}
 
 	g_free(gtkaspellcheckers->error_message);
-
+	gtkaspellcheckers->error_message = NULL;
 	return;
 }
 
@@ -374,6 +374,7 @@ GtkAspell *gtkaspell_new(const gchar *dictionary_path,
 			 const gchar *encoding,
 			 gint  misspelled_color,
 			 gboolean check_while_typing,
+			 gboolean recheck_when_changing_dict,
 			 gboolean use_alternate,
 			 GtkTextView *gtktext,
 			 GtkWindow *parent_win)
@@ -384,6 +385,11 @@ GtkAspell *gtkaspell_new(const gchar *dictionary_path,
 	GtkTextBuffer *buffer;
 
 	g_return_val_if_fail(gtktext, NULL);
+	g_return_val_if_fail(dictionary && strlen(dictionary) > 0, 
+			NULL);
+	g_return_val_if_fail(dictionary_path && strlen(dictionary_path) > 0, 
+			NULL);
+
 	buffer = gtk_text_view_get_buffer(gtktext);
 	
 	dict 	       = g_new0(Dictionary, 1);
@@ -409,10 +415,8 @@ GtkAspell *gtkaspell_new(const gchar *dictionary_path,
 	gtkaspell->end_check_pos      = -1;
 	gtkaspell->misspelled	      = -1;
 	gtkaspell->check_while_typing = check_while_typing;
+	gtkaspell->recheck_when_changing_dict = recheck_when_changing_dict;
 	gtkaspell->continue_check     = NULL;
-	gtkaspell->config_menu        = NULL;
-	gtkaspell->popup_config_menu  = NULL;
-	gtkaspell->sug_menu	      = NULL;
 	gtkaspell->replace_entry      = NULL;
 	gtkaspell->gtktext	      = gtktext;
 	gtkaspell->default_sug_mode   = ASPELL_FASTMODE;
@@ -427,28 +431,30 @@ GtkAspell *gtkaspell_new(const gchar *dictionary_path,
 			       G_CALLBACK(entry_insert_cb), gtkaspell);
 	g_signal_connect_after(G_OBJECT(buffer), "delete-range",
 		               G_CALLBACK(entry_delete_cb), gtkaspell);
-	g_signal_connect(G_OBJECT(gtktext), "button-press-event",
+	/*g_signal_connect(G_OBJECT(gtktext), "button-press-event",
 			 G_CALLBACK(button_press_intercept_cb),
-			 gtkaspell);
+			 gtkaspell);*/
+	g_signal_connect(G_OBJECT(gtktext), "populate-popup",
+			 G_CALLBACK(button_press_intercept_cb), gtkaspell);
 	
 	debug_print("Aspell: created gtkaspell %0x\n", (guint) gtkaspell);
 
 	return gtkaspell;
 }
 
-void gtkaspell_delete(GtkAspell * gtkaspell) 
+void gtkaspell_delete(GtkAspell *gtkaspell) 
 {
 	GtkTextView *gtktext = gtkaspell->gtktext;
 	
         g_signal_handlers_disconnect_by_func(G_OBJECT(gtktext),
 					     G_CALLBACK(entry_insert_cb),
-				      gtkaspell);
+					     gtkaspell);
 	g_signal_handlers_disconnect_by_func(G_OBJECT(gtktext),
 					     G_CALLBACK(entry_delete_cb),
-				      	     gtkaspell);
+					     gtkaspell);
 	g_signal_handlers_disconnect_by_func(G_OBJECT(gtktext),
-                                      	     G_CALLBACK(button_press_intercept_cb),
-				      	     gtkaspell);
+					     G_CALLBACK(button_press_intercept_cb),
+					     gtkaspell);
 
 	gtkaspell_uncheck_all(gtkaspell);
 	
@@ -457,19 +463,11 @@ void gtkaspell_delete(GtkAspell * gtkaspell)
 	if (gtkaspell->use_alternate && gtkaspell->alternate_speller)
 		gtkaspeller_delete(gtkaspell->alternate_speller);
 
-	if (gtkaspell->sug_menu)
-		gtk_widget_destroy(gtkaspell->sug_menu);
-
-	if (gtkaspell->popup_config_menu)
-		gtk_widget_destroy(gtkaspell->popup_config_menu);
-
-	if (gtkaspell->config_menu)
-		gtk_widget_destroy(gtkaspell->config_menu);
-
 	if (gtkaspell->suggestions_list)
 		free_suggestions_list(gtkaspell);
 
 	g_free((gchar *)gtkaspell->dictionary_path);
+	gtkaspell->dictionary_path = NULL;
 
 	debug_print("Aspell: deleting gtkaspell %0x\n", (guint) gtkaspell);
 
@@ -540,54 +538,51 @@ static void entry_delete_cb(GtkTextBuffer *textbuf,
 	/* gtk_editable_select_region(GTK_EDITABLE(gtktext), origpos, origpos); */
 }
 
-/* ok, this is pretty wacky:
- * we need to let the right-mouse-click go through, so it moves the cursor,
- * but we *can't* let it go through, because GtkText interprets rightclicks as
- * weird selection modifiers.
- *
- * so what do we do?  forge rightclicks as leftclicks, then popup the menu.
- * HACK HACK HACK.
- */
-static gint button_press_intercept_cb(GtkTextView *gtktext,
-				      GdkEvent *e, 
-				      GtkAspell *gtkaspell)
+static void button_press_intercept_cb(GtkTextView *gtktext,
+   			GtkMenu *menu, GtkAspell *gtkaspell)
 {
-	GdkEventButton *eb;
-	gboolean retval;
+	GtkMenuItem *menuitem;
+	GSList *spell_menu = NULL;
+	GSList *items;
+	gboolean suggest = FALSE;
 
-	g_return_val_if_fail(gtkaspell->gtkaspeller->checker, FALSE);
+	menuitem = GTK_MENU_ITEM(gtk_separator_menu_item_new());
+	gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), GTK_WIDGET(menuitem));
+	gtk_widget_show(GTK_WIDGET(menuitem));
 
-	if (e->type != GDK_BUTTON_PRESS) 
-		return FALSE;
-	eb = (GdkEventButton*) e;
+	gtktext = gtkaspell->gtktext;
 
-	if (eb->button != 3) 
-		return FALSE;
+	gtkaspell->orig_pos = get_textview_buffer_offset(gtktext);
 
-        g_signal_handlers_block_by_func(G_OBJECT(gtktext),
-					G_CALLBACK(button_press_intercept_cb), 
-					gtkaspell);
-	g_signal_emit_by_name(G_OBJECT(gtktext), "button-release-event",
-			      e, &retval);
+	if (check_at(gtkaspell, gtkaspell->orig_pos)) {
 
-	/* forge the leftclick */
-	eb->button = 1;
+		if (misspelled_suggest(gtkaspell, gtkaspell->theword)) {
+			spell_menu = make_sug_menu(gtkaspell);
+			suggest = TRUE;
+		}
+	} 
+	if (!spell_menu) 
+		spell_menu = make_config_menu(gtkaspell);
 	
-	g_signal_emit_by_name(G_OBJECT(gtktext), "button-press-event",
-			      e, &retval);
-	g_signal_emit_by_name(G_OBJECT(gtktext), "button-release-event",
-			      e, &retval);
-	g_signal_handlers_unblock_by_func(G_OBJECT(gtktext),
-					  G_CALLBACK(button_press_intercept_cb), 
-					   gtkaspell);
-	g_signal_stop_emission_by_name(G_OBJECT(gtktext), "button-press-event");
-    
-	/* now do the menu wackiness */
-	popup_menu(gtkaspell, eb);
-	gtk_grab_remove(GTK_WIDGET(gtktext));
-	return FALSE;
-}
+	spell_menu = g_slist_reverse(spell_menu);
+	for (items = spell_menu;
+	     items; items = items->next) {
+		menuitem = GTK_MENU_ITEM(items->data);
+		gtk_menu_shell_prepend(GTK_MENU_SHELL(menu), GTK_WIDGET(menuitem));
+		gtk_widget_show(GTK_WIDGET(menuitem));
+	}
+	g_slist_free(spell_menu);
+	
+	g_signal_connect(G_OBJECT(menu), "deactivate",
+				 G_CALLBACK(destroy_menu), 
+				 gtkaspell);
+	if (suggest)
+		g_signal_connect(G_OBJECT(menu),
+			"key_press_event",
+		       	G_CALLBACK(aspell_key_pressed), gtkaspell);
 
+
+}
 /* Checker creation */
 static GtkAspeller *gtkaspeller_new(Dictionary *dictionary)
 {
@@ -808,12 +803,11 @@ static void set_sug_mode_cb(GtkMenuItem *w, GtkAspell *gtkaspell)
 {
 	char *themode;
 	
-	gtk_label_get(GTK_LABEL(GTK_BIN(w)->child), (gchar **) &themode);
+	themode = (char *) gtk_label_get_text(GTK_LABEL(GTK_BIN(w)->child));
+	themode = g_strdup(themode);
 	
 	set_real_sug_mode(gtkaspell, themode);
-
-	if (gtkaspell->config_menu)
-		populate_submenu(gtkaspell, gtkaspell->config_menu);
+	g_free(themode);
 }
 
 static void set_real_sug_mode(GtkAspell *gtkaspell, const char *themode)
@@ -894,7 +888,7 @@ static GList *misspelled_suggest(GtkAspell *gtkaspell, guchar *word)
 		elements    = aspell_word_list_elements(suggestions);
 		list        = g_list_append(list, g_strdup(word)); 
 		
-		while (newword = aspell_string_enumeration_next(elements))
+		while ((newword = aspell_string_enumeration_next(elements)) != NULL)
 			list = g_list_append(list, g_strdup(newword));
 
 		gtkaspell->max_sug          = g_list_length(list) - 1;
@@ -918,7 +912,7 @@ static int misspelled_test(GtkAspell *gtkaspell, unsigned char *word)
 
 static gboolean iswordsep(unsigned char c) 
 {
-	return !isalpha(c) && c != '\'';
+	return (isspace(c) || ispunct(c)) && c != '\'';
 }
 
 static guchar get_text_index_whar(GtkAspell *gtkaspell, int pos) 
@@ -926,7 +920,7 @@ static guchar get_text_index_whar(GtkAspell *gtkaspell, int pos)
 	GtkTextView *view = gtkaspell->gtktext;
 	GtkTextBuffer *buffer = gtk_text_view_get_buffer(view);
 	GtkTextIter start, end;
-	const gchar *utf8chars;
+	gchar *utf8chars;
 	guchar a = '\0';
 
 	gtk_text_buffer_get_iter_at_offset(buffer, &start, pos);
@@ -943,6 +937,7 @@ static guchar get_text_index_whar(GtkAspell *gtkaspell, int pos)
 			g_free(tr);
 		}
 	}
+	g_free(utf8chars);
 
 	return a;
 }
@@ -978,7 +973,11 @@ static gboolean get_word_from_pos(GtkAspell *gtkaspell, gint pos,
 		c = get_text_index_whar(gtkaspell, start);
 		if (c == '\'') {
 			if (start > 0) {
-				if (!isalpha(get_text_index_whar(gtkaspell,
+				if (isspace(get_text_index_whar(gtkaspell,
+								 start - 1))
+				||  ispunct(get_text_index_whar(gtkaspell,
+								 start - 1))
+				||  isdigit(get_text_index_whar(gtkaspell,
 								 start - 1))) {
 					/* start_quote = TRUE; */
 					break;
@@ -989,7 +988,7 @@ static gboolean get_word_from_pos(GtkAspell *gtkaspell, gint pos,
 				break;
 			}
 		}
-		else if (!isalpha(c))
+		else if (isspace(c) || ispunct(c) || isdigit(c))
 				break;
 	}
 
@@ -999,7 +998,11 @@ static gboolean get_word_from_pos(GtkAspell *gtkaspell, gint pos,
 		c = get_text_index_whar(gtkaspell, end); 
 		if (c == '\'') {
 			if (end < get_textview_buffer_charcount(gtktext)) {
-				if (!isalpha(get_text_index_whar(gtkaspell,
+				if (isspace(get_text_index_whar(gtkaspell,
+								 end + 1))
+				||  ispunct(get_text_index_whar(gtkaspell,
+								 end + 1))
+				||  isdigit(get_text_index_whar(gtkaspell,
 								 end + 1))) {
 					/* end_quote = TRUE; */
 					break;
@@ -1010,7 +1013,7 @@ static gboolean get_word_from_pos(GtkAspell *gtkaspell, gint pos,
 				break;
 			}
 		}
-		else if(!isalpha(c))
+		else if (isspace(c) || ispunct(c) || isdigit(c))
 				break;
 	}
 						
@@ -1021,11 +1024,18 @@ static gboolean get_word_from_pos(GtkAspell *gtkaspell, gint pos,
 
 	if (buf) {
 		if (end - start < buflen) {
-			for (pos = start; pos < end; pos++) {
-				buf[pos - start] =
-					get_text_index_whar(gtkaspell, pos);
-			}
-			buf[pos - start] = 0;
+			GtkTextIter iterstart, iterend;
+			gchar *tmp, *conv;
+			GtkTextBuffer *buffer = gtk_text_view_get_buffer(gtktext);
+			gtk_text_buffer_get_iter_at_offset(buffer, &iterstart, start);
+			gtk_text_buffer_get_iter_at_offset(buffer, &iterend, end);
+			tmp = gtk_text_buffer_get_text(buffer, &iterstart, &iterend, FALSE);
+			conv = conv_iconv_strdup(tmp, CS_UTF_8, 
+				gtkaspell->gtkaspeller->dictionary->encoding);
+			g_free(tmp);
+			strncpy(buf, conv, buflen-1);
+			buf[buflen-1]='\0';
+			g_free(conv);
 		} else
 			return FALSE;
 	}
@@ -1096,7 +1106,8 @@ static gboolean check_next_prev(GtkAspell *gtkaspell, gboolean forward)
 			pos += direc;
 	}
 	if (misspelled) {
-		GtkMenu *menu = NULL;
+		GSList *list, *cur;
+		GtkWidget *menu;
 		misspelled_suggest(gtkaspell, gtkaspell->theword);
 
 		if (forward)
@@ -1113,12 +1124,20 @@ static gboolean check_next_prev(GtkAspell *gtkaspell, gboolean forward)
 		while (gtk_events_pending ())
 			gtk_main_iteration ();
 
-		menu = make_sug_menu(gtkaspell);
-		gtk_menu_popup(menu, NULL, NULL,
+		list = make_sug_menu(gtkaspell);
+		menu = gtk_menu_new();
+		for (cur = list; cur; cur = cur->next)
+			gtk_menu_append(menu, GTK_WIDGET(cur->data));
+		g_slist_free(list);
+		gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
 				set_menu_pos, gtkaspell, 0, GDK_CURRENT_TIME);
 		g_signal_connect(G_OBJECT(menu), "deactivate",
 					 G_CALLBACK(destroy_menu), 
 					 gtkaspell);
+		g_signal_connect(G_OBJECT(menu),
+			"key_press_event",
+		       	G_CALLBACK(aspell_key_pressed), gtkaspell);
+
 
 	} else {
 		reset_theword_data(gtkaspell);
@@ -1258,7 +1277,8 @@ static void replace_word_cb(GtkWidget *w, gpointer data)
 	GtkAspell *gtkaspell = (GtkAspell *) data;
 	GdkEvent *e= (GdkEvent *) gtk_get_current_event();
 
-	gtk_label_get(GTK_LABEL(GTK_BIN(w)->child), (gchar**) &newword);
+	newword = (unsigned char *) gtk_label_get_text(GTK_LABEL(GTK_BIN(w)->child));
+	newword = g_strdup(newword);
 
 	replace_real_word(gtkaspell, newword);
 
@@ -1275,6 +1295,7 @@ static void replace_word_cb(GtkWidget *w, gpointer data)
 	gtk_menu_shell_deactivate(GTK_MENU_SHELL(w->parent));
 
 	set_point_continue(gtkaspell);
+	g_free(newword);
 }
 
 static void replace_real_word(GtkAspell *gtkaspell, gchar *newword)
@@ -1402,19 +1423,27 @@ static void check_with_alternate_cb(GtkWidget *w, gpointer data)
 		gtkaspell->misspelled = misspelled;
 
 		if (gtkaspell->misspelled) {
-			GtkMenu *menu;
+			GtkWidget *menu;
+			GSList *list, *cur;
 			misspelled_suggest(gtkaspell, gtkaspell->theword);
 
 			set_textview_buffer_offset(gtkaspell->gtktext,
 					    gtkaspell->end_pos);
 
-			menu = make_sug_menu(gtkaspell);
-			gtk_menu_popup(menu, NULL, NULL,
+			list = make_sug_menu(gtkaspell);
+			menu = gtk_menu_new();
+			for (cur = list; cur; cur = cur->next)
+				gtk_menu_append(menu, GTK_WIDGET(cur->data));
+			g_slist_free(list);
+			gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
 				       set_menu_pos, gtkaspell, 0,
 				       GDK_CURRENT_TIME);
 			g_signal_connect(G_OBJECT(menu), "deactivate",
 					 G_CALLBACK(destroy_menu), 
 					 gtkaspell);
+			g_signal_connect(G_OBJECT(menu),
+				"key_press_event",
+			       	G_CALLBACK(aspell_key_pressed), gtkaspell);
 			return;
 		}
 	} else
@@ -1531,8 +1560,8 @@ static void replace_with_create_dialog_cb(GtkWidget *w, gpointer data)
 	hbox = gtk_hbox_new(TRUE, 0);
 
 	gtkut_stock_button_set_create(&confirm_area,
-				      &ok_button, GTK_STOCK_OK,
 				      &cancel_button, GTK_STOCK_CANCEL,
+				      &ok_button, GTK_STOCK_OK,
 				      NULL, NULL);
 
 	gtk_box_pack_end(GTK_BOX(GTK_DIALOG(dialog)->action_area),
@@ -1581,9 +1610,6 @@ static void toggle_check_while_typing_cb(GtkWidget *w, gpointer data)
 
 	if (!gtkaspell->check_while_typing)
 		gtkaspell_uncheck_all(gtkaspell);
-
-	if (gtkaspell->config_menu)
-		populate_submenu(gtkaspell, gtkaspell->config_menu);
 }
 
 static GSList *create_empty_dictionary_list(void)
@@ -1682,6 +1708,7 @@ void gtkaspell_free_dictionary_list(GSList *list)
 	g_slist_free(list);
 }
 
+/* FIXME */
 GtkWidget *gtkaspell_dictionary_option_menu_new(const gchar *aspell_path)
 {
 	GSList *dict_list, *tmp;
@@ -1759,6 +1786,7 @@ gint gtkaspell_set_dictionary_menu_active_item(GtkWidget *menu,
 	return 0;
 }
 
+/* FIXME */
 GtkWidget *gtkaspell_sugmode_option_menu_new(gint sugmode)
 {
 	GtkWidget *menu;
@@ -1821,9 +1849,6 @@ static void use_alternate_dict(GtkAspell *gtkaspell)
 	tmp = gtkaspell->gtkaspeller;
 	gtkaspell->gtkaspeller = gtkaspell->alternate_speller;
 	gtkaspell->alternate_speller = tmp;
-
-	if (gtkaspell->config_menu)
-		populate_submenu(gtkaspell, gtkaspell->config_menu);
 }
 
 static void destroy_menu(GtkWidget *widget,
@@ -1838,43 +1863,6 @@ static void destroy_menu(GtkWidget *widget,
 	}
 }
 
-static void popup_menu(GtkAspell *gtkaspell, GdkEventButton *eb) 
-{
-	GtkTextView * gtktext;
-	GtkMenu *menu = NULL;
-	
-	gtktext = gtkaspell->gtktext;
-
-	gtkaspell->orig_pos = get_textview_buffer_offset(gtktext);
-
-	if (!(eb->state & GDK_SHIFT_MASK)) {
-		if (check_at(gtkaspell, gtkaspell->orig_pos)) {
-
-			set_textview_buffer_offset(gtktext, gtkaspell->orig_pos);
-
-			if (misspelled_suggest(gtkaspell, gtkaspell->theword)) {
-				menu = make_sug_menu(gtkaspell);
-				gtk_menu_popup(menu,
-					       NULL, NULL, NULL, NULL,
-					       eb->button, eb->time);
-
-				g_signal_connect(G_OBJECT(menu), "deactivate",
-							 G_CALLBACK(destroy_menu), 
-							 gtkaspell);
-				return;
-			}
-		} else
-			set_textview_buffer_offset(gtktext, gtkaspell->orig_pos);
-	}
-	menu = make_config_menu(gtkaspell);
-	gtk_menu_popup(menu, NULL, NULL, NULL, NULL,
-		       eb->button, eb->time);
-
-	g_signal_connect(G_OBJECT(menu), "deactivate",
-				 G_CALLBACK(destroy_menu), 
-				 gtkaspell);
-}
-
 static gboolean aspell_key_pressed(GtkWidget *widget,
 				   GdkEventKey *event,
 				   GtkAspell *gtkaspell)
@@ -1892,22 +1880,18 @@ static gboolean aspell_key_pressed(GtkWidget *widget,
 /* make_sug_menu() - Add menus to accept this word for this session 
  * and to add it to personal dictionary 
  */
-static GtkMenu *make_sug_menu(GtkAspell *gtkaspell) 
+static GSList *make_sug_menu(GtkAspell *gtkaspell) 
 {
-	GtkWidget 	*menu, *item;
+	GtkWidget 	*item;
 	unsigned char	*caption;
 	GtkTextView 	*gtktext;
 	GtkAccelGroup 	*accel;
 	GList 		*l = gtkaspell->suggestions_list;
 	gchar		*utf8buf;
-
+	GSList *list = NULL;
 	gtktext = gtkaspell->gtktext;
 
 	accel = gtk_accel_group_new();
-	menu = gtk_menu_new(); 
-
-	if (gtkaspell->sug_menu)
-		gtk_widget_destroy(gtkaspell->sug_menu);
 
 	if (gtkaspell->accel_group) {
 		gtk_window_remove_accel_group(GTK_WINDOW(gtkaspell->parent_window), 
@@ -1915,27 +1899,26 @@ static GtkMenu *make_sug_menu(GtkAspell *gtkaspell)
 		gtkaspell->accel_group = NULL;
 	}
 
-	gtkaspell->sug_menu = menu;	
-
-	g_signal_connect(G_OBJECT(menu), "cancel",
-			 G_CALLBACK(cancel_menu_cb), gtkaspell);
-
+	utf8buf  = conv_codeset_strdup((unsigned char*)l->data,
+				conv_get_locale_charset_str(),
+				CS_UTF_8);
 	caption = g_strdup_printf(_("\"%s\" unknown in %s"), 
-				  (unsigned char*) l->data, 
+				  utf8buf, 
 				  gtkaspell->gtkaspeller->dictionary->dictname);
 	item = gtk_menu_item_new_with_label(caption);
+	g_free(utf8buf);
 	gtk_widget_show(item);
-	gtk_menu_append(GTK_MENU(menu), item);
+	list = g_slist_append(list, item);
 	gtk_misc_set_alignment(GTK_MISC(GTK_BIN(item)->child), 0.5, 0.5);
 	g_free(caption);
 
 	item = gtk_menu_item_new();
 	gtk_widget_show(item);
-	gtk_menu_append(GTK_MENU(menu), item);
+	list = g_slist_append(list, item);
 
 	item = gtk_menu_item_new_with_label(_("Accept in this session"));
 	gtk_widget_show(item);
-	gtk_menu_append(GTK_MENU(menu), item);
+	list = g_slist_append(list, item);
         g_signal_connect(G_OBJECT(item), "activate",
 			 G_CALLBACK(add_word_to_session_cb), 
 			 gtkaspell);
@@ -1945,7 +1928,7 @@ static GtkMenu *make_sug_menu(GtkAspell *gtkaspell)
 
 	item = gtk_menu_item_new_with_label(_("Add to personal dictionary"));
 	gtk_widget_show(item);
-	gtk_menu_append(GTK_MENU(menu), item);
+	list = g_slist_append(list, item);
         g_signal_connect(G_OBJECT(item), "activate",
 			 G_CALLBACK(add_word_to_personal_cb), 
 			 gtkaspell);
@@ -1955,7 +1938,7 @@ static GtkMenu *make_sug_menu(GtkAspell *gtkaspell)
 
         item = gtk_menu_item_new_with_label(_("Replace with..."));
 	gtk_widget_show(item);
-	gtk_menu_append(GTK_MENU(menu), item);
+	list = g_slist_append(list, item);
         g_signal_connect(G_OBJECT(item), "activate",
 			 G_CALLBACK(replace_with_create_dialog_cb), 
 			 gtkaspell);
@@ -1971,7 +1954,7 @@ static GtkMenu *make_sug_menu(GtkAspell *gtkaspell)
 		item = gtk_menu_item_new_with_label(caption);
 		g_free(caption);
 		gtk_widget_show(item);
-		gtk_menu_append(GTK_MENU(menu), item);
+		list = g_slist_append(list, item);
 		g_signal_connect(G_OBJECT(item), "activate",
 				 G_CALLBACK(check_with_alternate_cb),
 				 gtkaspell);
@@ -1984,15 +1967,15 @@ static GtkMenu *make_sug_menu(GtkAspell *gtkaspell)
 
 	item = gtk_menu_item_new();
         gtk_widget_show(item);
-        gtk_menu_append(GTK_MENU(menu), item);
+        list = g_slist_append(list, item);
 
 	l = l->next;
         if (l == NULL) {
 		item = gtk_menu_item_new_with_label(_("(no suggestions)"));
 		gtk_widget_show(item);
-		gtk_menu_append(GTK_MENU(menu), item);
+		list = g_slist_append(list, item);
         } else {
-		GtkWidget *curmenu = menu;
+		GtkWidget *curmenu = NULL;
 		gint count = 0;
 		
 		do {
@@ -2001,7 +1984,10 @@ static GtkMenu *make_sug_menu(GtkAspell *gtkaspell)
 
 				item = gtk_menu_item_new_with_label(_("More..."));
 				gtk_widget_show(item);
-				gtk_menu_append(GTK_MENU(curmenu), item);
+				if (curmenu)
+					gtk_menu_append(GTK_MENU(curmenu), item);
+				else 
+					list = g_slist_append(list, item);
 
 				curmenu = gtk_menu_new();
 				gtk_menu_item_set_submenu(GTK_MENU_ITEM(item),
@@ -2012,13 +1998,18 @@ static GtkMenu *make_sug_menu(GtkAspell *gtkaspell)
 							conv_get_locale_charset_str(),
 							CS_UTF_8);
 			item = gtk_menu_item_new_with_label(utf8buf);
+			g_free(utf8buf);
 			gtk_widget_show(item);
-			gtk_menu_append(GTK_MENU(curmenu), item);
+			if (curmenu == NULL) {
+				list = g_slist_append(list, item);
+			} else {
+				gtk_menu_append(GTK_MENU(curmenu), item);
+			}
 			g_signal_connect(G_OBJECT(item), "activate",
 					 G_CALLBACK(replace_word_cb),
 					 gtkaspell);
 
-			if (curmenu == menu && count < MENUCOUNT) {
+			if (curmenu == NULL && count < MENUCOUNT) {
 				gtk_widget_add_accelerator(item, "activate",
 							   accel,
 							   GDK_A + count, 0,
@@ -2041,39 +2032,27 @@ static GtkMenu *make_sug_menu(GtkAspell *gtkaspell)
 		 accel);
 	gtkaspell->accel_group = accel;
 
-	g_signal_connect(G_OBJECT(menu),
-			"key_press_event",
-		       	G_CALLBACK(aspell_key_pressed), gtkaspell);
-	
-	return GTK_MENU(menu);
+	return list;
 }
 
-static void populate_submenu(GtkAspell *gtkaspell, GtkWidget *menu)
+static GSList *populate_submenu(GtkAspell *gtkaspell)
 {
 	GtkWidget *item, *submenu;
 	gchar *dictname;
 	GtkAspeller *gtkaspeller = gtkaspell->gtkaspeller;
+	GSList *list = NULL;
 
-	if (GTK_MENU_SHELL(menu)->children) {
-		GList *amenu, *alist;
-		for (amenu = (GTK_MENU_SHELL(menu)->children); amenu; ) {
-			alist = amenu->next;
-			gtk_widget_destroy(GTK_WIDGET(amenu->data));
-			amenu = alist;
-		}
-	}
-	
 	dictname = g_strdup_printf(_("Dictionary: %s"),
 				   gtkaspeller->dictionary->dictname);
 	item = gtk_menu_item_new_with_label(dictname);
 	gtk_misc_set_alignment(GTK_MISC(GTK_BIN(item)->child), 0.5, 0.5);
 	g_free(dictname);
 	gtk_widget_show(item);
-	gtk_menu_append(GTK_MENU(menu), item);
+	list = g_slist_append(list, item);
 
 	item = gtk_menu_item_new();
         gtk_widget_show(item);
-        gtk_menu_append(GTK_MENU(menu), item);
+        list = g_slist_append(list, item);
 		
 	if (gtkaspell->use_alternate && gtkaspell->alternate_speller) {
 		dictname = g_strdup_printf(_("Use alternate (%s)"), 
@@ -2084,7 +2063,7 @@ static void populate_submenu(GtkAspell *gtkaspell, GtkWidget *menu)
 				 G_CALLBACK(switch_to_alternate_cb),
 				 gtkaspell);
 		gtk_widget_show(item);
-		gtk_menu_append(GTK_MENU(menu), item);
+		list = g_slist_append(list, item);
 	}
 
       	item = gtk_check_menu_item_new_with_label(_("Fast Mode"));
@@ -2096,7 +2075,7 @@ static void populate_submenu(GtkAspell *gtkaspell, GtkWidget *menu)
 				 G_CALLBACK(set_sug_mode_cb),
 				 gtkaspell);
 	gtk_widget_show(item);
-	gtk_menu_append(GTK_MENU(menu), item);
+	list = g_slist_append(list, item);
 
 	item = gtk_check_menu_item_new_with_label(_("Normal Mode"));
 	if (gtkaspell->gtkaspeller->sug_mode == ASPELL_NORMALMODE) {
@@ -2107,7 +2086,7 @@ static void populate_submenu(GtkAspell *gtkaspell, GtkWidget *menu)
 				 G_CALLBACK(set_sug_mode_cb),
 				 gtkaspell);
 	gtk_widget_show(item);
-	gtk_menu_append(GTK_MENU(menu),item);
+	list = g_slist_append(list, item);
 
 	item = gtk_check_menu_item_new_with_label(_("Bad Spellers Mode"));
 	if (gtkaspell->gtkaspeller->sug_mode == ASPELL_BADSPELLERMODE) {
@@ -2118,11 +2097,11 @@ static void populate_submenu(GtkAspell *gtkaspell, GtkWidget *menu)
 				 G_CALLBACK(set_sug_mode_cb),
 				 gtkaspell);
 	gtk_widget_show(item);
-	gtk_menu_append(GTK_MENU(menu), item);
+	list = g_slist_append(list, item);
 	
 	item = gtk_menu_item_new();
         gtk_widget_show(item);
-        gtk_menu_append(GTK_MENU(menu), item);
+        list = g_slist_append(list, item);
 	
 	item = gtk_check_menu_item_new_with_label(_("Check while typing"));
 	if (gtkaspell->check_while_typing)
@@ -2133,17 +2112,17 @@ static void populate_submenu(GtkAspell *gtkaspell, GtkWidget *menu)
 			 G_CALLBACK(toggle_check_while_typing_cb),
 			 gtkaspell);
 	gtk_widget_show(item);
-	gtk_menu_append(GTK_MENU(menu), item);
+	list = g_slist_append(list, item);
 
 	item = gtk_menu_item_new();
         gtk_widget_show(item);
-        gtk_menu_append(GTK_MENU(menu), item);
+        list = g_slist_append(list, item);
 
 	submenu = gtk_menu_new();
         item = gtk_menu_item_new_with_label(_("Change dictionary"));
         gtk_menu_item_set_submenu(GTK_MENU_ITEM(item),submenu);
         gtk_widget_show(item);
-        gtk_menu_append(GTK_MENU(menu), item);
+        list = g_slist_append(list, item);
 
 	/* Dict list */
         if (gtkaspellcheckers->dictionary_list == NULL)
@@ -2191,32 +2170,12 @@ static void populate_submenu(GtkAspell *gtkaspell, GtkWidget *menu)
 			count++;
 		}
         }  
+	return list;
 }
 
-static GtkMenu *make_config_menu(GtkAspell *gtkaspell)
+static GSList *make_config_menu(GtkAspell *gtkaspell)
 {
-	if (!gtkaspell->popup_config_menu)
-		gtkaspell->popup_config_menu = gtk_menu_new();
-
-	debug_print("Aspell: creating/using popup_config_menu %0x\n", 
-			(guint) gtkaspell->popup_config_menu);
-	populate_submenu(gtkaspell, gtkaspell->popup_config_menu);
-
-        return GTK_MENU(gtkaspell->popup_config_menu);
-}
-
-void gtkaspell_populate_submenu(GtkAspell *gtkaspell, GtkWidget *menuitem)
-{
-	GtkWidget *menu;
-
-	menu = GTK_WIDGET(GTK_MENU_ITEM(menuitem)->submenu);
-	
-	debug_print("Aspell: using config menu %0x\n", 
-			(guint) gtkaspell->popup_config_menu);
-	populate_submenu(gtkaspell, menu);
-	
-	gtkaspell->config_menu = menu;
-	
+	return populate_submenu(gtkaspell);
 }
 
 static void set_menu_pos(GtkMenu *menu, gint *x, gint *y, 
@@ -2266,16 +2225,6 @@ static void set_menu_pos(GtkMenu *menu, gint *x, gint *y,
 
 /* Menu call backs */
 
-static gboolean cancel_menu_cb(GtkMenuShell *w, gpointer data)
-{
-	GtkAspell *gtkaspell = (GtkAspell *) data;
-
-	gtkaspell->continue_check = NULL;
-	set_point_continue(gtkaspell);
-
-	return FALSE;
-}
-
 gboolean gtkaspell_change_dict(GtkAspell *gtkaspell, const gchar *dictionary)
 {
 	Dictionary 	*dict;       
@@ -2295,6 +2244,7 @@ gboolean gtkaspell_change_dict(GtkAspell *gtkaspell, const gchar *dictionary)
 	    dict == gtkaspell->alternate_speller->dictionary) {
 		use_alternate_dict(gtkaspell);
 		dictionary_delete(dict);
+		gtkaspell->alternate_speller->dictionary = NULL;
 		return TRUE;
 	}
 	
@@ -2321,9 +2271,6 @@ gboolean gtkaspell_change_dict(GtkAspell *gtkaspell, const gchar *dictionary)
 	
 	dictionary_delete(dict);
 
-	if (gtkaspell->config_menu)
-		populate_submenu(gtkaspell, gtkaspell->config_menu);
-
 	return TRUE;	
 }
 
@@ -2338,6 +2285,9 @@ static void change_dict_cb(GtkWidget *w, GtkAspell *gtkaspell)
 		return;
 
 	gtkaspell_change_dict(gtkaspell, fullname);
+	if (gtkaspell->recheck_when_changing_dict) {
+		gtkaspell_highlight_all(gtkaspell);
+	}
 }
 
 static void switch_to_alternate_cb(GtkWidget *w,
@@ -2375,8 +2325,13 @@ static void allocate_color(GtkAspell *gtkaspell, gint rgbvalue)
 	color->blue  = (int) (((gdouble) (rgbvalue & 0x0000ff)        / 255.0)
 			* 65535.0);
 
-	gtk_text_buffer_create_tag(buffer, "misspelled",
+	if (rgbvalue != 0)
+		gtk_text_buffer_create_tag(buffer, "misspelled",
 				   "foreground-gdk", color, NULL);
+	else
+		gtk_text_buffer_create_tag(buffer, "misspelled",
+				   "underline", PANGO_UNDERLINE_ERROR, NULL);
+
 }
 
 static void change_color(GtkAspell * gtkaspell, 
@@ -2388,7 +2343,8 @@ static void change_color(GtkAspell * gtkaspell,
 	GtkTextBuffer *buffer;
 	GtkTextIter startiter, enditer;
 
-	g_return_if_fail(start < end);
+	if (start > end)
+		return;
     
 	gtktext = gtkaspell->gtktext;
     
@@ -2398,9 +2354,11 @@ static void change_color(GtkAspell * gtkaspell,
 	if (color)
 		gtk_text_buffer_apply_tag_by_name(buffer, "misspelled",
 						  &startiter, &enditer);
-	else
+	else {
+		gtk_text_iter_forward_char(&enditer);
 		gtk_text_buffer_remove_tag_by_name(buffer, "misspelled",
 						   &startiter, &enditer);
+	}
 }
 
 /* convert_to_aspell_encoding () - converts ISO-8859-* strings to iso8859-* 
@@ -2476,7 +2434,7 @@ static void free_suggestions_list(GtkAspell *gtkaspell)
 	     list = list->next)
 		g_free(list->data);
 
-	g_list_free(list);
+	g_list_free(gtkaspell->suggestions_list);
 	
 	gtkaspell->max_sug          = -1;
 	gtkaspell->suggestions_list = NULL;
