@@ -6,7 +6,7 @@
 
 #include "gtksctree.h"
 #include "sylpheed-marshal.h"
-
+#include "stock_pixmap.h"
 
 enum {
 	ROW_POPUP_MENU,
@@ -16,6 +16,8 @@ enum {
 	LAST_SIGNAL
 };
 
+static GdkPixmap *emptyxpm = NULL;
+static GdkBitmap *emptyxpmmask = NULL;
 
 static void gtk_sctree_class_init (GtkSCTreeClass *class);
 static void gtk_sctree_init (GtkSCTree *sctree);
@@ -243,8 +245,11 @@ select_row (GtkSCTree *sctree, gint row, gint col, guint state)
 
 	GTK_CLIST_GET_CLASS(sctree)->refresh(GTK_CLIST(sctree));
 
-	if (!additive)
+	if (!additive) {
+		sctree->selecting_range = TRUE;
 		gtk_clist_unselect_all (GTK_CLIST (sctree));
+		sctree->selecting_range = FALSE;
+	}
 
 	if (!range) {
 		GtkCTreeNode *node;
@@ -328,8 +333,11 @@ gtk_sctree_button_press (GtkWidget *widget, GdkEventButton *event)
 					sctree->dnd_select_pending_row = row;
 				} else
 					select_row (sctree, row, col, event->state);
-			} else
+			} else {
+				sctree->selecting_range = TRUE;
 				gtk_clist_unselect_all (clist);
+				sctree->selecting_range = FALSE;
+			}
 
 			retval = TRUE;
 		} else if (event->button == 3) {
@@ -341,7 +349,9 @@ gtk_sctree_button_press (GtkWidget *widget, GdkEventButton *event)
 						 sctree_signals[ROW_POPUP_MENU],
 						 0, event);
 			} else {
+				sctree->selecting_range = TRUE;
 				gtk_clist_unselect_all(clist);
+				sctree->selecting_range = FALSE;
 				g_signal_emit (G_OBJECT (sctree),
 						 sctree_signals[EMPTY_POPUP_MENU],
 						 0, event);
@@ -556,16 +566,6 @@ gtk_sctree_collapse (GtkCTree *ctree, GtkCTreeNode *node)
 GtkWidget *gtk_sctree_new_with_titles (gint columns, gint tree_column, 
 				       gchar *titles[])
 {
-#if 0
-	GtkSCTree* sctree;
-
-	sctree = gtk_type_new (gtk_sctree_get_type ());
-	gtk_ctree_construct (GTK_CTREE (sctree), columns, tree_column, titles);
-	
-	gtk_clist_set_selection_mode(GTK_CLIST(sctree), GTK_SELECTION_EXTENDED);
-
-	return GTK_WIDGET (sctree);
-#else
 	GtkWidget *widget;
                                                                                                             
 	g_return_val_if_fail (columns > 0, NULL);
@@ -585,7 +585,6 @@ GtkWidget *gtk_sctree_new_with_titles (gint columns, gint tree_column,
 	}
 
 	return widget;
-#endif
 }
 
 void gtk_sctree_select (GtkSCTree *sctree, GtkCTreeNode *node)
@@ -604,7 +603,9 @@ void gtk_sctree_select_with_state (GtkSCTree *sctree, GtkCTreeNode *node, int st
 
 void gtk_sctree_unselect_all (GtkSCTree *sctree)
 {
+	sctree->selecting_range = TRUE;
 	gtk_clist_unselect_all(GTK_CLIST(sctree));
+	sctree->selecting_range = FALSE;
 	sctree->anchor_row = NULL;
 }
 
@@ -613,6 +614,13 @@ void gtk_sctree_set_anchor_row (GtkSCTree *sctree, GtkCTreeNode *node)
 	sctree->anchor_row = node;
 }
 
+void gtk_sctree_remove_node (GtkSCTree *sctree, GtkCTreeNode *node)
+{
+	if (sctree->anchor_row == node)
+		sctree->anchor_row = NULL;
+	gtk_ctree_remove_node(GTK_CTREE(sctree), node);
+}
+
 /***********************************************************
  *             Tree sorting functions                      *
  ***********************************************************/
@@ -1084,3 +1092,71 @@ gtk_ctree_last_visible (GtkCTree     *ctree,
 
 	return gtk_ctree_last_visible (ctree, work);
 }
+
+/* this wrapper simply replaces NULL pixmaps 
+ * with a transparent, 1x1 pixmap. This works
+ * around a memory problem deep inside gtk, 
+ * revealed by valgrind. 
+ */
+GtkCTreeNode* gtk_sctree_insert_node        (GtkCTree *ctree,
+                                             GtkCTreeNode *parent,
+                                             GtkCTreeNode *sibling,
+                                             gchar *text[],
+                                             guint8 spacing,
+                                             GdkPixmap *pixmap_closed,
+                                             GdkBitmap *mask_closed,
+                                             GdkPixmap *pixmap_opened,
+                                             GdkBitmap *mask_opened,
+                                             gboolean is_leaf,
+                                             gboolean expanded)
+{
+	if (!emptyxpm) {
+		stock_pixmap_gdk(GTK_WIDGET(ctree), STOCK_PIXMAP_EMPTY,
+			 &emptyxpm, &emptyxpmmask);
+	}
+	if (!pixmap_closed) {
+		pixmap_closed = emptyxpm;
+		mask_closed = emptyxpmmask;
+	}
+	if (!pixmap_opened) {
+		pixmap_opened = emptyxpm;
+		mask_opened = emptyxpmmask;
+	}
+	return gtk_ctree_insert_node(ctree, parent, sibling, text,spacing,
+		pixmap_closed, mask_closed, pixmap_opened, mask_opened,
+		is_leaf, expanded);
+}
+
+/* this wrapper simply replaces NULL pixmaps 
+ * with a transparent, 1x1 pixmap. This works
+ * around a memory problem deep inside gtk, 
+ * revealed by valgrind. 
+ */
+void        gtk_sctree_set_node_info        (GtkCTree *ctree,
+                                             GtkCTreeNode *node,
+                                             const gchar *text,
+                                             guint8 spacing,
+                                             GdkPixmap *pixmap_closed,
+                                             GdkBitmap *mask_closed,
+                                             GdkPixmap *pixmap_opened,
+                                             GdkBitmap *mask_opened,
+                                             gboolean is_leaf,
+                                             gboolean expanded)
+{
+	if (!emptyxpm) {
+		stock_pixmap_gdk(GTK_WIDGET(ctree), STOCK_PIXMAP_EMPTY,
+			 &emptyxpm, &emptyxpmmask);
+	}
+	if (!pixmap_closed) {
+		pixmap_closed = emptyxpm;
+		mask_closed = emptyxpmmask;
+	}
+	if (!pixmap_opened) {
+		pixmap_opened = emptyxpm;
+		mask_opened = emptyxpmmask;
+	}
+	gtk_ctree_set_node_info(ctree, node, text, spacing,
+		pixmap_closed, mask_closed, pixmap_opened, mask_opened,
+		is_leaf, expanded);
+
+}
