@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +36,14 @@
 #include <stdarg.h>
 #include <sys/stat.h>
 
+#if HAVE_LIBCOMPFACE
+#  include <compface.h>
+#endif
+
+#if HAVE_LIBCOMPFACE
+#define XPM_XFACE_HEIGHT	(HEIGHT + 3)  /* 3 = 1 header + 2 colors */
+#endif
+
 #if (HAVE_WCTYPE_H && HAVE_WCHAR_H)
 #  include <wchar.h>
 #  include <wctype.h>
@@ -50,6 +58,8 @@
 #include "menu.h"
 #include "prefs_account.h"
 #include "prefs_common.h"
+#include "manage_window.h"
+#include "base64.h"
 
 gboolean gtkut_get_font_size(GtkWidget *widget,
 			     gint *width, gint *height)
@@ -580,6 +590,86 @@ gchar *gtkut_text_view_get_selection(GtkTextView *textview)
 		return NULL;
 }
 
+
+void gtkut_text_view_set_position(GtkTextView *text, gint pos)
+{
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+
+	g_return_if_fail(text != NULL);
+
+	buffer = gtk_text_view_get_buffer(text);
+
+	gtk_text_buffer_get_iter_at_offset(buffer, &iter, pos);
+	gtk_text_buffer_place_cursor(buffer, &iter);
+	gtk_text_view_scroll_to_iter(text, &iter, 0.0, FALSE, 0.0, 0.0);
+}
+
+gboolean gtkut_text_view_search_string(GtkTextView *text, const gchar *str,
+					gboolean case_sens)
+{
+	GtkTextBuffer *buffer;
+	GtkTextIter iter, match_pos;
+	GtkTextMark *mark;
+	gint len;
+
+	g_return_val_if_fail(text != NULL, FALSE);
+	g_return_val_if_fail(str != NULL, FALSE);
+
+	buffer = gtk_text_view_get_buffer(text);
+
+	len = g_utf8_strlen(str, -1);
+	g_return_val_if_fail(len >= 0, FALSE);
+
+	mark = gtk_text_buffer_get_insert(buffer);
+	gtk_text_buffer_get_iter_at_mark(buffer, &iter, mark);
+
+	if (gtkut_text_buffer_find(buffer, &iter, str, case_sens,
+				   &match_pos)) {
+		GtkTextIter end = match_pos;
+
+		gtk_text_iter_forward_chars(&end, len);
+		/* place "insert" at the last character */
+		gtk_text_buffer_select_range(buffer, &end, &match_pos);
+		gtk_text_view_scroll_to_mark(text, mark, 0.0, FALSE, 0.0, 0.0);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+gboolean gtkut_text_view_search_string_backward(GtkTextView *text, const gchar *str,
+					gboolean case_sens)
+{
+	GtkTextBuffer *buffer;
+	GtkTextIter iter, match_pos;
+	GtkTextMark *mark;
+	gint len;
+
+	g_return_val_if_fail(text != NULL, FALSE);
+	g_return_val_if_fail(str != NULL, FALSE);
+
+	buffer = gtk_text_view_get_buffer(text);
+
+	len = g_utf8_strlen(str, -1);
+	g_return_val_if_fail(len >= 0, FALSE);
+
+	mark = gtk_text_buffer_get_insert(buffer);
+	gtk_text_buffer_get_iter_at_mark(buffer, &iter, mark);
+
+	if (gtkut_text_buffer_find_backward(buffer, &iter, str, case_sens,
+					    &match_pos)) {
+		GtkTextIter end = match_pos;
+
+		gtk_text_iter_forward_chars(&end, len);
+		gtk_text_buffer_select_range(buffer, &match_pos, &end);
+		gtk_text_view_scroll_to_mark(text, mark, 0.0, FALSE, 0.0, 0.0);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 void gtkut_window_popup(GtkWidget *window)
 {
 	gint x, y, sx, sy, new_x, new_y;
@@ -692,6 +782,29 @@ void gtkut_widget_set_composer_icon(GtkWidget *widget)
 	gdk_window_set_icon(widget->window, NULL, xpm, bmp);	
 }
 
+GtkWidget *label_window_create(const gchar *str)
+{
+	GtkWidget *window;
+	GtkWidget *label;
+
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_widget_set_size_request(window, 380, 60);
+	gtk_container_set_border_width(GTK_CONTAINER(window), 8);
+	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
+	gtk_window_set_title(GTK_WINDOW(window), str);
+	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
+	gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
+	manage_window_set_transient(GTK_WINDOW(window));
+
+	label = gtk_label_new(str);
+	gtk_container_add(GTK_CONTAINER(window), label);
+	gtk_widget_show(label);
+
+	gtk_widget_show_now(window);
+
+	return window;
+}
+
 GtkWidget *gtkut_account_menu_new(GList			*ac_list,
 				  GCallback		 callback,
 				  gpointer		 data)
@@ -790,3 +903,351 @@ GtkWidget *gtkut_get_focused_child(GtkContainer *parent)
 		
 	return result;
 }
+
+/*!
+ *\brief	Create a Browse (file) button based on GTK+ stock
+ */
+GtkWidget *gtkut_get_browse_file_btn(const gchar *button_label)
+{
+	GtkWidget *button;
+
+#if GTK_CHECK_VERSION(2, 6, 0)
+	button = gtk_button_new_with_mnemonic(button_label);
+	gtk_button_set_image(GTK_BUTTON(button),
+		gtk_image_new_from_stock(GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_BUTTON));
+#else
+	GtkWidget* image;
+	GtkWidget* box;
+	GtkWidget* label;
+
+	button = gtk_button_new();
+	box = gtk_hbox_new(FALSE, 0);
+
+	image = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON);
+	label = gtk_label_new(NULL);
+	gtk_label_set_text_with_mnemonic(GTK_LABEL(label), button_label);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), button);
+
+	gtk_box_pack_start(GTK_BOX(box), image, FALSE, FALSE, 1);
+	gtk_box_pack_end(GTK_BOX(box), label, FALSE, FALSE, 1);
+	gtk_widget_show(label);
+	gtk_widget_show(image);
+	gtk_widget_show(box);
+	gtk_container_add(GTK_CONTAINER(button), box);
+#endif
+	return button;
+}
+
+/*!
+ *\brief	Create a Browse (directory) button based on GTK+ stock
+ */
+GtkWidget *gtkut_get_browse_directory_btn(const gchar *button_label)
+{
+	GtkWidget *button;
+
+#if GTK_CHECK_VERSION(2, 6, 0)
+	button = gtk_button_new_with_mnemonic(button_label);
+	gtk_button_set_image(GTK_BUTTON(button),
+		gtk_image_new_from_stock(GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_BUTTON));
+#else
+	GtkWidget* image;
+	GtkWidget* box;
+	GtkWidget* label;
+
+	button = gtk_button_new();
+	box = gtk_hbox_new(FALSE, 0);
+
+	image = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON);
+	label = gtk_label_new(NULL);
+	gtk_label_set_text_with_mnemonic(GTK_LABEL(label), button_label);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), button);
+
+	gtk_box_pack_start(GTK_BOX(box), image, FALSE, FALSE, 1);
+	gtk_box_pack_end(GTK_BOX(box), label, FALSE, FALSE, 1);
+	gtk_widget_show(label);
+	gtk_widget_show(image);
+	gtk_widget_show(box);
+	gtk_container_add(GTK_CONTAINER(button), box);
+#endif
+	return button;
+}
+
+#if HAVE_LIBCOMPFACE
+gint create_xpm_from_xface(gchar *xpm[], const gchar *xface)
+{
+	static gchar *bit_pattern[] = {
+		"....",
+		"...#",
+		"..#.",
+		"..##",
+		".#..",
+		".#.#",
+		".##.",
+		".###",
+		"#...",
+		"#..#",
+		"#.#.",
+		"#.##",
+		"##..",
+		"##.#",
+		"###.",
+		"####"
+	};
+
+	static gchar *xface_header = "48 48 2 1";
+	static gchar *xface_black  = "# c #000000";
+	static gchar *xface_white  = ". c #ffffff";
+
+	gint i, line = 0;
+	const guchar *p;
+	gchar buf[WIDTH * 4 + 1];  /* 4 = strlen("0x0000") */
+
+	p = xface;
+
+	strcpy(xpm[line++], xface_header);
+	strcpy(xpm[line++], xface_black);
+	strcpy(xpm[line++], xface_white);
+
+	for (i = 0; i < HEIGHT; i++) {
+		gint col;
+
+		buf[0] = '\0';
+     
+		for (col = 0; col < 3; col++) {
+			gint figure;
+
+			p += 2;  /* skip '0x' */
+
+			for (figure = 0; figure < 4; figure++) {
+				gint n = 0;
+
+				if ('0' <= *p && *p <= '9') {
+					n = *p - '0';
+				} else if ('a' <= *p && *p <= 'f') {
+					n = *p - 'a' + 10;
+				} else if ('A' <= *p && *p <= 'F') {
+					n = *p - 'A' + 10;
+				}
+
+				strcat(buf, bit_pattern[n]);
+				p++;  /* skip ',' */
+			}
+
+			p++;  /* skip '\n' */
+		}
+
+		strcpy(xpm[line++], buf);
+		p++;
+	}
+
+	return 0;
+}
+#endif
+
+gboolean get_tag_range(GtkTextIter *iter,
+				       GtkTextTag *tag,
+				       GtkTextIter *start_iter,
+				       GtkTextIter *end_iter)
+{
+	GtkTextIter _start_iter, _end_iter;
+
+	_end_iter = *iter;
+	if (!gtk_text_iter_forward_to_tag_toggle(&_end_iter, tag)) {
+		debug_print("Can't find end");
+		return FALSE;
+	}
+
+	_start_iter = _end_iter;
+	if (!gtk_text_iter_backward_to_tag_toggle(&_start_iter, tag)) {
+		debug_print("Can't find start.");
+		return FALSE;
+	}
+
+	*start_iter = _start_iter;
+	*end_iter = _end_iter;
+
+	return TRUE;
+}
+
+#if HAVE_LIBCOMPFACE
+GtkWidget *xface_get_from_header(const gchar *o_xface, GdkColor *background,
+				 GdkWindow *window)
+{
+	static gchar *xpm_xface[XPM_XFACE_HEIGHT];
+	static gboolean xpm_xface_init = TRUE;
+	GdkPixmap *pixmap;
+	GdkBitmap *mask;
+	gchar xface[2048];
+	strncpy(xface, o_xface, sizeof(xface));
+
+	if (uncompface(xface) < 0) {
+		g_warning("uncompface failed\n");
+		return NULL;
+	}
+
+	if (xpm_xface_init) {
+		gint i;
+
+		for (i = 0; i < XPM_XFACE_HEIGHT; i++) {
+			xpm_xface[i] = g_malloc(WIDTH + 1);
+			*xpm_xface[i] = '\0';
+		}
+		xpm_xface_init = FALSE;
+	}
+
+	create_xpm_from_xface(xpm_xface, xface);
+
+	pixmap = gdk_pixmap_create_from_xpm_d
+		(window, &mask, 
+		 background, xpm_xface);
+	return gtk_image_new_from_pixmap(pixmap, mask);
+}
+#endif
+
+GtkWidget *face_get_from_header(const gchar *o_face)
+{
+	gchar face[2048];
+	gchar face_png[2048];
+	gint pngsize;
+	GdkPixbuf *pixbuf;
+	GError *error = NULL;
+	GdkPixbufLoader *loader = gdk_pixbuf_loader_new ();
+	GtkWidget *image;
+	
+	strncpy2(face, o_face, sizeof(face));
+
+	unfold_line(face); /* strip all whitespace and linebreaks */
+	remove_space(face);
+
+	pngsize = base64_decode(face_png, face, strlen(face));
+
+	if (!gdk_pixbuf_loader_write (loader, face_png, pngsize, &error) ||
+	    !gdk_pixbuf_loader_close (loader, &error)) {
+		g_warning("loading face failed\n");
+		g_object_unref(loader);
+		return NULL;
+	}
+
+	pixbuf = g_object_ref(gdk_pixbuf_loader_get_pixbuf(loader));
+
+	g_object_unref(loader);
+
+	if ((gdk_pixbuf_get_width(pixbuf) != 48) || (gdk_pixbuf_get_height(pixbuf) != 48)) {
+		g_object_unref(pixbuf);
+		g_warning("wrong_size");
+		return NULL;
+	}
+
+	image = gtk_image_new_from_pixbuf(pixbuf);
+	g_object_unref(pixbuf);
+	return image;
+}
+
+static GdkCursor *hand_cursor = NULL;
+
+static void link_btn_enter(GtkButton *button, gpointer data)
+{
+	GtkWidget *window = (GtkWidget *)data;
+
+	if (!hand_cursor)
+		hand_cursor = gdk_cursor_new(GDK_HAND2);
+	if (window && window->window)
+		gdk_window_set_cursor(window->window, hand_cursor);
+
+	gtk_button_set_relief(button, GTK_RELIEF_NONE);
+	gtk_widget_set_state(GTK_WIDGET(button), GTK_STATE_NORMAL);
+	
+}
+
+static void link_btn_leave(GtkButton *button, gpointer data)
+{
+	GtkWidget *window = (GtkWidget *)data;
+
+	if (window && window->window)
+		gdk_window_set_cursor(window->window, NULL);
+
+	gtk_button_set_relief(button, GTK_RELIEF_NONE);
+	gtk_widget_set_state(GTK_WIDGET(button), GTK_STATE_NORMAL);
+}
+
+static void link_btn_pressed(GtkButton *button, gpointer data)
+{
+	gtk_button_set_relief(button, GTK_RELIEF_NONE);
+	gtk_widget_set_state(GTK_WIDGET(button), GTK_STATE_NORMAL);
+}
+
+static void link_btn_released(GtkButton *button, gpointer data)
+{
+	gtk_button_set_relief(button, GTK_RELIEF_NONE);
+	gtk_widget_set_state(GTK_WIDGET(button), GTK_STATE_NORMAL);
+}
+
+static void link_btn_clicked(GtkButton *button, gpointer data)
+{
+	gchar *url = (gchar *)data;
+	gtk_button_set_relief(button, GTK_RELIEF_NONE);
+	gtk_widget_set_state(GTK_WIDGET(button), GTK_STATE_NORMAL);
+	open_uri(url, prefs_common.uri_cmd);
+}
+
+static void link_btn_unrealize(GtkButton *button, gpointer data)
+{
+	gchar *url = (gchar *)data;
+	g_signal_handlers_disconnect_by_func(G_OBJECT(button), 
+			 G_CALLBACK(link_btn_clicked), url);
+	g_free(url);
+}
+
+GtkWidget *gtkut_get_link_btn(GtkWidget *window, const gchar *url, const gchar *label)
+{
+	GtkWidget *btn;
+	GtkWidget *btn_label;
+	GdkColormap *cmap;
+	GdkColor uri_color[2] = {{0, 0, 0, 0xffff}, {0, 0xffff, 0, 0}};
+	gboolean success[2];
+	gchar *underlined = NULL;
+	gchar *local_url = NULL;
+	if (!url)
+		return NULL;
+
+	gtkut_convert_int_to_gdk_color(prefs_common.uri_col,
+					       &uri_color[0]);
+	gtkut_convert_int_to_gdk_color(prefs_common.uri_col,
+					       &uri_color[1]);
+
+	underlined = g_markup_printf_escaped("<u>%s</u>", label?label:url);
+	btn = gtk_button_new_with_label(underlined);
+	g_free(underlined);
+	gtk_button_set_relief(GTK_BUTTON(btn), GTK_RELIEF_NONE);
+	btn_label = GTK_BIN(btn)->child;
+	gtk_label_set_use_markup(GTK_LABEL(btn_label), TRUE);
+	cmap = gdk_drawable_get_colormap(window->window);
+	gdk_colormap_alloc_colors(cmap, uri_color, 2, FALSE, TRUE, success);
+	if (success[0] == TRUE && success[1] == TRUE) {
+		GtkStyle *style;
+		gtk_widget_ensure_style(btn_label);
+		style = gtk_style_copy
+			(gtk_widget_get_style(btn_label));
+		style->fg[GTK_STATE_NORMAL]   = uri_color[0];
+		style->fg[GTK_STATE_ACTIVE]   = uri_color[1];
+		style->fg[GTK_STATE_PRELIGHT] = uri_color[0];
+		gtk_widget_set_style(btn_label, style);
+	} else
+		g_warning("about_create(): color allocation failed.\n");
+
+	g_signal_connect(G_OBJECT(btn), "enter",
+			 G_CALLBACK(link_btn_enter), window);
+	g_signal_connect(G_OBJECT(btn), "leave",
+			 G_CALLBACK(link_btn_leave), window);
+	g_signal_connect(G_OBJECT(btn), "pressed",
+			 G_CALLBACK(link_btn_pressed), window);
+	g_signal_connect(G_OBJECT(btn), "released",
+			 G_CALLBACK(link_btn_released), window);
+			 
+	local_url = g_strdup(url);
+	g_signal_connect(G_OBJECT(btn), "clicked",
+			 G_CALLBACK(link_btn_clicked), local_url);
+	g_signal_connect(G_OBJECT(btn), "unrealize",
+			 G_CALLBACK(link_btn_unrealize), local_url);
+	return btn;
+}
