@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto and the Sylpheed-Claws Team
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,6 +50,7 @@ struct _PrefsWindow
 	GtkWidget *window;
 	GtkWidget *table1;
 	GtkWidget *scrolledwindow1;
+	GtkWidget *scrolledwindow2;
 	GtkWidget *tree_view;
 	GtkWidget *table2;
 	GtkWidget *pagelabel;
@@ -60,6 +61,8 @@ struct _PrefsWindow
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
 	GtkWidget *apply_btn;
+	gint *save_width;
+	gint *save_height;
 
 	GtkWidget *empty_page;
 
@@ -74,6 +77,8 @@ struct _PrefsTreeNode
 	gfloat     treeweight; /* GTK2: not used */
 };
 
+static void prefs_size_allocate_cb(GtkWidget *widget,
+							 GtkAllocation *allocation, gpointer *user_data);
 static GtkTreeStore *prefswindow_create_data_store	(void);
 static GtkWidget *prefswindow_tree_view_create		(PrefsWindow* prefswindow);
 static void prefs_filtering_create_tree_view_columns	(GtkWidget *tree_view);
@@ -331,19 +336,25 @@ static void prefswindow_build_tree(GtkWidget *tree_view, GSList *prefs_pages)
 		gtk_tree_selection_select_iter(selection, &iter);
 }
 
-void prefswindow_open_full(const gchar *title, GSList *prefs_pages, gpointer data, GtkDestroyNotify func)
+void prefswindow_open_full(const gchar *title, GSList *prefs_pages, gpointer data, GtkDestroyNotify func,
+							 gint *save_width, gint *save_height)
 {
 	PrefsWindow *prefswindow;
+	gint x = gdk_screen_width();
+	gint y = gdk_screen_height();
+	static GdkGeometry geometry;
 
 	prefswindow = g_new0(PrefsWindow, 1);
 
 	prefswindow->data = data;
 	prefswindow->func = func;
 	prefswindow->prefs_pages = g_slist_copy(prefs_pages);
+	prefswindow->save_width = save_width;
+	prefswindow->save_height = save_height;
 
 	prefswindow->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(prefswindow->window), title);
-	gtk_window_set_default_size(GTK_WINDOW(prefswindow->window), 600, 440);
+
 	gtk_window_set_position (GTK_WINDOW(prefswindow->window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal (GTK_WINDOW (prefswindow->window), TRUE);
 	gtk_window_set_resizable (GTK_WINDOW(prefswindow->window), TRUE);
@@ -383,12 +394,18 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages, gpointer dat
 	gtk_misc_set_alignment(GTK_MISC(prefswindow->pagelabel), 0, 0.0);
 	gtk_container_add(GTK_CONTAINER(prefswindow->labelframe), prefswindow->pagelabel);
 
+	prefswindow->scrolledwindow2 = gtk_scrolled_window_new(NULL, NULL);
+	gtk_widget_show(prefswindow->scrolledwindow2);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+
 	prefswindow->notebook = gtk_notebook_new();
 	gtk_widget_show(prefswindow->notebook);
 	gtk_notebook_set_scrollable(GTK_NOTEBOOK(prefswindow->notebook), TRUE);
 	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(prefswindow->notebook), FALSE);
 	gtk_notebook_set_show_border(GTK_NOTEBOOK(prefswindow->notebook), FALSE);
-	gtk_table_attach(GTK_TABLE(prefswindow->table2), prefswindow->notebook, 0, 1, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 8, 8);
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2),
+					prefswindow->notebook);
+	gtk_table_attach(GTK_TABLE(prefswindow->table2), prefswindow->scrolledwindow2, 0, 1, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 8, 8);
 
 	prefswindow->empty_page = gtk_label_new("");
 	gtk_widget_show(prefswindow->empty_page);
@@ -399,9 +416,9 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages, gpointer dat
 	gtk_widget_grab_focus(prefswindow->tree_view);
 
 	gtkut_stock_button_set_create(&prefswindow->confirm_area,
-				      &prefswindow->ok_btn,	GTK_STOCK_OK,
+				      &prefswindow->apply_btn,	GTK_STOCK_APPLY,
 				      &prefswindow->cancel_btn,	GTK_STOCK_CANCEL,
-				      &prefswindow->apply_btn,	GTK_STOCK_APPLY);
+				      &prefswindow->ok_btn,	GTK_STOCK_OK);
 
 	gtk_widget_show_all(prefswindow->confirm_area);
 
@@ -417,13 +434,54 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages, gpointer dat
 			 G_CALLBACK(window_closed), prefswindow);
 	g_signal_connect(G_OBJECT(prefswindow->window), "key_press_event",
 			   G_CALLBACK(prefswindow_key_pressed), &(prefswindow->window));
+	/* connect to callback only if we hhave non-NULL pointers to store size to */
+	if (prefswindow->save_width && prefswindow->save_height) {
+		g_signal_connect(G_OBJECT(prefswindow->window), "size_allocate",
+				 G_CALLBACK(prefs_size_allocate_cb), prefswindow);
+	}
+
+	if (!geometry.min_height) {
+		
+		if (x < 800 && y < 600) {
+			geometry.min_width = 600;
+			geometry.min_height = 440;
+		} else {
+			geometry.min_width = 700;
+			geometry.min_height = 550;
+		}
+	}
+
+	gtk_window_set_geometry_hints(GTK_WINDOW(prefswindow->window), NULL, &geometry,
+				      GDK_HINT_MIN_SIZE);
+	if (prefswindow->save_width && prefswindow->save_height) {
+		gtk_widget_set_size_request(prefswindow->window, *(prefswindow->save_width),
+					    *(prefswindow->save_height));
+	}
 
 	gtk_widget_show(prefswindow->window);
 }
 
-void prefswindow_open(const gchar *title, GSList *prefs_pages, gpointer data)
+void prefswindow_open(const gchar *title, GSList *prefs_pages, gpointer data,
+					 gint *save_width, gint *save_height)
 {
-	prefswindow_open_full(title, prefs_pages, data, NULL);
+	prefswindow_open_full(title, prefs_pages, data, NULL, save_width, save_height);
+}
+
+/*!
+ *\brief	Save Gtk object size to prefs dataset
+ */
+static void prefs_size_allocate_cb(GtkWidget *widget,
+					 GtkAllocation *allocation, gpointer *user_data)
+{
+	PrefsWindow *prefswindow = (PrefsWindow *) user_data;
+
+	g_return_if_fail(allocation != NULL);
+
+	/* don't try to save size to NULL pointers */
+	if (prefswindow && prefswindow->save_width && prefswindow->save_height) {
+		*(prefswindow->save_width) = allocation->width;
+		*(prefswindow->save_height) = allocation->height;
+	}
 }
 
 static GtkTreeStore *prefswindow_create_data_store(void)
