@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,6 +37,9 @@
 #include "quicksearch.h"
 #include "folderview.h"
 #include "folder.h"
+#include "prefs_matcher.h"
+#include "sylpheed.h"
+#include "statusbar.h"
 
 struct _QuickSearch
 {
@@ -44,7 +47,9 @@ struct _QuickSearch
 	GtkWidget			*search_type;
 	GtkWidget			*search_type_opt;
 	GtkWidget			*search_string_entry;
+	GtkWidget			*search_condition_expression;
 	GtkWidget			*search_description;
+	GtkWidget			*clear_search;
 
 	gboolean			 active;
 	gchar				*search_string;
@@ -54,6 +59,8 @@ struct _QuickSearch
 	gpointer			 callback_data;
 	gboolean			 running;
 	gboolean			 has_focus;
+	gboolean			 matching;
+	gboolean			 deferred_free;
 	FolderItem			*root_folder_item;
 };
 
@@ -65,13 +72,21 @@ static void prepare_matcher(QuickSearch *quicksearch)
 {
 	const gchar *search_string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(quicksearch->search_string_entry)->entry));
 
+	if (search_string == NULL || search_string[0] == '\0') {
+		quicksearch_set_active(quicksearch, FALSE);
+	}
+
 	if (quicksearch->matcher_list != NULL) {
+		if (quicksearch->matching) {
+			quicksearch->deferred_free = TRUE;
+			return;
+		}
+		quicksearch->deferred_free = FALSE;
 		matcherlist_free(quicksearch->matcher_list);
 		quicksearch->matcher_list = NULL;
 	}
 
 	if (search_string == NULL || search_string[0] == '\0') {
-		quicksearch_set_active(quicksearch, FALSE);
 		return;
 	}
 
@@ -97,19 +112,21 @@ static void prepare_matcher(QuickSearch *quicksearch)
 	quicksearch_set_active(quicksearch, TRUE);
 }
 
-static void update_extended_button (QuickSearch *quicksearch)
+static void update_extended_buttons (QuickSearch *quicksearch)
 {
-	GtkWidget *btn = quicksearch->search_description;
-	
-	g_return_if_fail(btn != NULL);
-		
+	GtkWidget *expr_btn = quicksearch->search_condition_expression;
+	GtkWidget *ext_btn = quicksearch->search_description;
+
+	g_return_if_fail(expr_btn != NULL);
+	g_return_if_fail(ext_btn != NULL);
+
 	if (prefs_common.summary_quicksearch_type == QUICK_SEARCH_EXTENDED) {
-		gtk_button_set_label(GTK_BUTTON(btn), _("Extended symbols"));
-		gtk_widget_show(btn);
+		gtk_widget_show(expr_btn);
+		gtk_widget_show(ext_btn);
 	} else {
-		gtk_widget_hide(btn);
+		gtk_widget_hide(expr_btn);
+		gtk_widget_hide(ext_btn);
 	}
-	
 }
 
 static gboolean searchbar_focus_evt(GtkWidget *widget, GdkEventFocus *event,
@@ -124,51 +141,60 @@ gboolean quicksearch_has_focus(QuickSearch *quicksearch)
 	return quicksearch->has_focus;
 }
 
+static void searchbar_run(QuickSearch *quicksearch)
+{
+	const gchar *search_string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(quicksearch->search_string_entry)->entry));
+
+	if (search_string && strlen(search_string) != 0) {
+		prefs_common.summary_quicksearch_history =
+			add_history(prefs_common.summary_quicksearch_history,
+					search_string);
+		gtk_combo_set_popdown_strings(GTK_COMBO(quicksearch->search_string_entry),
+			prefs_common.summary_quicksearch_history);
+	}
+
+	prepare_matcher(quicksearch);
+
+	quicksearch_set_running(quicksearch, TRUE);
+	if (quicksearch->callback != NULL)
+		quicksearch->callback(quicksearch, quicksearch->callback_data);
+	quicksearch_set_running(quicksearch, FALSE);
+}
+
 static gboolean searchbar_pressed(GtkWidget *widget, GdkEventKey *event,
 			      	  QuickSearch *quicksearch)
 {
 	if (event != NULL && event->keyval == GDK_Escape) {
 		quicksearch_set(quicksearch, prefs_common.summary_quicksearch_type, "");
-		gtk_widget_grab_focus(GTK_WIDGET(GTK_COMBO(quicksearch->search_string_entry)->entry));
 		return TRUE;
 	}
 
 	if (event != NULL && event->keyval == GDK_Return) {
-		const gchar *search_string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(quicksearch->search_string_entry)->entry));
+		/* add expression to history list and exec quicksearch */
+		searchbar_run(quicksearch);
 
-		if (search_string && strlen(search_string) != 0) {
-			prefs_common.summary_quicksearch_history =
-				add_history(prefs_common.summary_quicksearch_history,
-					    search_string);
-			gtk_combo_set_popdown_strings(GTK_COMBO(quicksearch->search_string_entry), 
-				prefs_common.summary_quicksearch_history);			
-		}
-
-		prepare_matcher(quicksearch);
-
-		quicksearch_set_running(quicksearch, TRUE);
-		if (quicksearch->callback != NULL)
-			quicksearch->callback(quicksearch, quicksearch->callback_data);
-		quicksearch_set_running(quicksearch, FALSE);
-	 	g_signal_stop_emission_by_name(G_OBJECT(widget), "key_press_event");
-		gtk_widget_grab_focus(GTK_WIDGET(GTK_COMBO(
-			quicksearch->search_string_entry)->entry));
+		g_signal_stop_emission_by_name(G_OBJECT(widget), "key_press_event");
 		return TRUE;
 	}
 
-	return FALSE; 		
+	return FALSE;
 }
 
 static gboolean searchtype_changed(GtkMenuItem *widget, gpointer data)
 {
 	QuickSearch *quicksearch = (QuickSearch *)data;
+	const gchar *search_string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(quicksearch->search_string_entry)->entry));
 
 	prefs_common.summary_quicksearch_type = GPOINTER_TO_INT(g_object_get_data(
 				   G_OBJECT(GTK_MENU_ITEM(gtk_menu_get_active(
 				   GTK_MENU(quicksearch->search_type)))), MENU_VAL_ID));
 
 	/* Show extended search description button, only when Extended is selected */
-	update_extended_button(quicksearch);
+	update_extended_buttons(quicksearch);
+
+	if (!search_string || strlen(search_string) == 0) {
+		return TRUE;
+	}
 
 	prepare_matcher(quicksearch);
 
@@ -183,13 +209,18 @@ static gboolean searchtype_recursive_changed(GtkMenuItem *widget, gpointer data)
 {
 	QuickSearch *quicksearch = (QuickSearch *)data;
 	gboolean checked = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
-	
-	prefs_common.summary_quicksearch_recurse = checked; 
+	const gchar *search_string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(quicksearch->search_string_entry)->entry));
+
+	prefs_common.summary_quicksearch_recurse = checked;
 
 	/* reselect the search type */
-	gtk_option_menu_set_history(GTK_OPTION_MENU(quicksearch->search_type_opt), 
+	gtk_option_menu_set_history(GTK_OPTION_MENU(quicksearch->search_type_opt),
 				    prefs_common.summary_quicksearch_type);
 
+	if (!search_string || strlen(search_string) == 0) {
+		return TRUE;
+	}
+
 	prepare_matcher(quicksearch);
 
 	quicksearch_set_running(quicksearch, TRUE);
@@ -199,9 +230,23 @@ static gboolean searchtype_recursive_changed(GtkMenuItem *widget, gpointer data)
 	return TRUE;
 }
 
+static gboolean searchtype_sticky_changed(GtkMenuItem *widget, gpointer data)
+{
+	QuickSearch *quicksearch = (QuickSearch *)data;
+	gboolean checked = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
+
+	prefs_common.summary_quicksearch_sticky = checked;
+
+	/* reselect the search type */
+	gtk_option_menu_set_history(GTK_OPTION_MENU(quicksearch->search_type_opt),
+				    prefs_common.summary_quicksearch_type);
+
+	return TRUE;
+}
+
 /*
  * Strings describing how to use Extended Search
- * 
+ *
  * When adding new lines, remember to put 2 strings for each line
  */
 static gchar *search_descr_strings[] = {
@@ -241,22 +286,26 @@ static gchar *search_descr_strings[] = {
 	"X cmd", N_("messages returning 0 when passed to command"),
 	"y S",	 N_("messages which contain S in X-Label header"),
 	"",	 "" ,
-	"&",	 N_("logical AND operator"),
+	"&amp;",	 N_("logical AND operator"),
 	"|",	 N_("logical OR operator"),
 	"! or ~",	N_("logical NOT operator"),
 	"%",	 N_("case sensitive search"),
 	"",	 "" ,
 	" ",	 N_("all filtering expressions are allowed"),
-	NULL,	 NULL 
+	NULL,	 NULL
 };
- 
+
 static DescriptionWindow search_descr = {
-	NULL, 
+	NULL,
+	NULL,
 	2,
-	N_("Extended Search symbols"),
+	N_("Extended Search"),
+	N_("Extended Search allows the user to define criteria that messages must "
+           "have in order to match and be displayed in the message list.\n\n"
+	   "The following symbols can be used:"),
 	search_descr_strings
 };
-	
+
 static void search_description_cb(GtkWidget *widget)
 {
 	description_window_create(&search_descr);
@@ -265,9 +314,60 @@ static void search_description_cb(GtkWidget *widget)
 static gboolean clear_search_cb(GtkMenuItem *widget, gpointer data)
 {
 	QuickSearch *quicksearch = (QuickSearch *)data;
-	
+
+	if (!quicksearch->active)
+		return TRUE;
+
 	quicksearch_set(quicksearch, prefs_common.summary_quicksearch_type, "");
-	
+
+	return TRUE;
+};
+
+static void search_condition_expr_done(MatcherList * matchers)
+{
+	gchar *str;
+
+	g_return_if_fail(
+			mainwindow_get_mainwindow()->summaryview->quicksearch != NULL);
+
+	if (matchers == NULL)
+		return;
+
+	str = matcherlist_to_string(matchers);
+
+	if (str != NULL) {
+		quicksearch_set(mainwindow_get_mainwindow()->summaryview->quicksearch,
+				prefs_common.summary_quicksearch_type, str);
+		g_free(str);
+
+		/* add expression to history list and exec quicksearch */
+		searchbar_run(mainwindow_get_mainwindow()->summaryview->quicksearch);
+	}
+}
+
+static gboolean search_condition_expr(GtkMenuItem *widget, gpointer data)
+{
+	const gchar * cond_str;
+	MatcherList * matchers = NULL;
+
+	g_return_val_if_fail(
+			mainwindow_get_mainwindow()->summaryview->quicksearch != NULL,
+			FALSE);
+
+	/* re-use it the current quicksearch value if it's a condition expression,
+	   otherwise ignore it silently */
+	cond_str = gtk_entry_get_text(
+			GTK_ENTRY(GTK_COMBO(mainwindow_get_mainwindow()->summaryview->quicksearch->
+			search_string_entry)->entry));
+	if (*cond_str != '\0') {
+		matchers = matcher_parser_get_cond((gchar*)cond_str);
+	}
+
+	prefs_matcher_open(matchers, search_condition_expr_done);
+
+	if (matchers != NULL)
+		matcherlist_free(matchers);
+
 	return TRUE;
 };
 
@@ -279,10 +379,12 @@ QuickSearch *quicksearch_new()
 	GtkWidget *search_type_opt;
 	GtkWidget *search_type;
 	GtkWidget *search_string_entry;
-	GtkWidget *search_hbbox;
+	GtkWidget *search_hbox;
 	GtkWidget *search_description;
 	GtkWidget *clear_search;
+	GtkWidget *search_condition_expression;
 	GtkWidget *menuitem;
+	GtkTooltips *search_cond_expr_tip;
 
 	quicksearch = g_new0(QuickSearch, 1);
 
@@ -312,92 +414,100 @@ QuickSearch *quicksearch_new()
 			 quicksearch);
 
 	gtk_menu_shell_append(GTK_MENU_SHELL(search_type), gtk_separator_menu_item_new());
-	
+
 	menuitem = gtk_check_menu_item_new_with_label(_("Recursive"));
 	gtk_menu_shell_append(GTK_MENU_SHELL(search_type), menuitem);
-	
+
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
 					prefs_common.summary_quicksearch_recurse);
-	
+
 	g_signal_connect(G_OBJECT(menuitem), "activate",
 			 G_CALLBACK(searchtype_recursive_changed),
 			 quicksearch);
 
+	menuitem = gtk_check_menu_item_new_with_label(_("Sticky"));
+	gtk_menu_shell_append(GTK_MENU_SHELL(search_type), menuitem);
+
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
+					prefs_common.summary_quicksearch_sticky);
+
+	g_signal_connect(G_OBJECT(menuitem), "activate",
+			 G_CALLBACK(searchtype_sticky_changed),
+			 quicksearch);
+
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(search_type_opt), search_type);
-	
+
 	gtk_option_menu_set_history(GTK_OPTION_MENU(search_type_opt), prefs_common.summary_quicksearch_type);
-	
+
 	gtk_widget_show(search_type);
-	
+
 	search_string_entry = gtk_combo_new();
 	gtk_box_pack_start(GTK_BOX(hbox_search), search_string_entry, FALSE, FALSE, 2);
 	gtk_combo_set_value_in_list(GTK_COMBO(search_string_entry), FALSE, TRUE);
 	gtk_combo_set_case_sensitive(GTK_COMBO(search_string_entry), TRUE);
-	if (prefs_common.summary_quicksearch_history) 
-		gtk_combo_set_popdown_strings(GTK_COMBO(search_string_entry), 
+	if (prefs_common.summary_quicksearch_history)
+		gtk_combo_set_popdown_strings(GTK_COMBO(search_string_entry),
 			prefs_common.summary_quicksearch_history);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(search_string_entry)->entry), "");
 	gtk_widget_show(search_string_entry);
 
-	search_hbbox = gtk_hbutton_box_new();
-	gtk_button_box_set_layout(GTK_BUTTON_BOX(search_hbbox), 	
-				  GTK_BUTTONBOX_START);
+	search_hbox = gtk_hbox_new(FALSE, 5);
 
-	gtk_box_set_spacing(GTK_BOX(search_hbbox), 5);
-		
-	if (prefs_common.summary_quicksearch_sticky) {
-		clear_search = gtk_button_new_with_label(_("Clear"));
-		gtk_box_pack_start(GTK_BOX(search_hbbox), clear_search,
-				   FALSE, FALSE, 0);
-		gtk_widget_set_size_request(clear_search, 120, -1);
-		g_signal_connect(G_OBJECT(clear_search), "clicked",
-				 G_CALLBACK(clear_search_cb), quicksearch);
-		gtk_widget_show(clear_search);
-	}
+	clear_search = gtk_button_new_with_label(_(" Clear "));
+	gtk_box_pack_start(GTK_BOX(search_hbox), clear_search,
+			   FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(clear_search), "clicked",
+			 G_CALLBACK(clear_search_cb), quicksearch);
+	gtk_widget_show(clear_search);
 
-	search_description = gtk_button_new_with_label(_("Extended Symbols"));
-	gtk_box_pack_start(GTK_BOX(search_hbbox), search_description,
-			   TRUE, TRUE, 0);
-	gtk_widget_show(search_description);
-		
+	search_condition_expression = gtk_button_new_with_label(" ... ");
+	gtk_box_pack_start(GTK_BOX(search_hbox), search_condition_expression,
+			   FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT (search_condition_expression), "clicked",
+			 G_CALLBACK(search_condition_expr),
+			 quicksearch);
+	search_cond_expr_tip = gtk_tooltips_new();
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(search_cond_expr_tip),
+			     search_condition_expression,
+			     _("Edit search criteria"), NULL);
+	gtk_widget_show(search_condition_expression);
+
+	search_description = gtk_button_new_with_label(_(" Extended Symbols... "));
+	gtk_box_pack_start(GTK_BOX(search_hbox), search_description,
+			   FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(search_description), "clicked",
 			 G_CALLBACK(search_description_cb), NULL);
+	gtk_widget_show(search_description);
 
-	gtk_box_pack_start(GTK_BOX(hbox_search), search_hbbox, FALSE, FALSE, 2);				
-	gtk_widget_show(search_hbbox);
-	if (prefs_common.summary_quicksearch_type == QUICK_SEARCH_EXTENDED)
-		gtk_widget_show(search_description);
-	else
-		gtk_widget_hide(search_description);
-	
-	g_signal_connect(G_OBJECT(GTK_COMBO(search_string_entry)->entry), 
+	gtk_box_pack_start(GTK_BOX(hbox_search), search_hbox, FALSE, FALSE, 2);
+	gtk_widget_show(search_hbox);
+
+	g_signal_connect(G_OBJECT(GTK_COMBO(search_string_entry)->entry),
 			   "key_press_event",
 			   G_CALLBACK(searchbar_pressed),
 			   quicksearch);
-
-	
 	g_signal_connect(G_OBJECT(GTK_COMBO(search_string_entry)->entry),
 			 "focus_in_event",
 			 G_CALLBACK(searchbar_focus_evt),
 			 quicksearch);
-
 	g_signal_connect(G_OBJECT(GTK_COMBO(search_string_entry)->entry),
 			 "focus_out_event",
 			 G_CALLBACK(searchbar_focus_evt),
 			 quicksearch);
-	
 
 	quicksearch->hbox_search = hbox_search;
 	quicksearch->search_type = search_type;
 	quicksearch->search_type_opt = search_type_opt;
 	quicksearch->search_string_entry = search_string_entry;
+	quicksearch->search_condition_expression = search_condition_expression;
 	quicksearch->search_description = search_description;
 	quicksearch->matcher_list = NULL;
 	quicksearch->active = FALSE;
 	quicksearch->running = FALSE;
-	
-	update_extended_button(quicksearch);
-	
+	quicksearch->clear_search = clear_search;
+
+	update_extended_buttons(quicksearch);
+
 	return quicksearch;
 }
 
@@ -410,13 +520,14 @@ void quicksearch_show(QuickSearch *quicksearch)
 {
 	prepare_matcher(quicksearch);
 	gtk_widget_show(quicksearch->hbox_search);
-	update_extended_button(quicksearch);
+	update_extended_buttons(quicksearch);
 	gtk_widget_grab_focus(
 		GTK_WIDGET(GTK_COMBO(quicksearch->search_string_entry)->entry));
 }
 
 void quicksearch_hide(QuickSearch *quicksearch)
 {
+	quicksearch_set(quicksearch, prefs_common.summary_quicksearch_type, "");
 	quicksearch_set_active(quicksearch, FALSE);
 	gtk_widget_hide(quicksearch->hbox_search);
 }
@@ -434,7 +545,7 @@ void quicksearch_set(QuickSearch *quicksearch, QuickSearchType type,
 
 	quicksearch_set_running(quicksearch, TRUE);
 	if (quicksearch->callback != NULL)
-		quicksearch->callback(quicksearch, quicksearch->callback_data);	
+		quicksearch->callback(quicksearch, quicksearch->callback_data);
 	quicksearch_set_running(quicksearch, FALSE);
 }
 
@@ -445,7 +556,41 @@ gboolean quicksearch_is_active(QuickSearch *quicksearch)
 
 static void quicksearch_set_active(QuickSearch *quicksearch, gboolean active)
 {
+	static GdkColor yellow;
+	static GdkColor black;
+	static gboolean yellow_initialised = FALSE;
+
+	if (!yellow_initialised) {
+		gdk_color_parse("#f5f6be", &yellow);
+		gdk_color_parse("#000000", &black);
+		yellow_initialised = gdk_colormap_alloc_color(
+			gdk_colormap_get_system(), &yellow, FALSE, TRUE);
+		yellow_initialised &= gdk_colormap_alloc_color(
+			gdk_colormap_get_system(), &black, FALSE, TRUE);
+
+	}
 	quicksearch->active = active;
+
+	if (active) {
+		gtk_widget_set_sensitive(quicksearch->clear_search, TRUE);
+		if (yellow_initialised)
+			gtk_widget_modify_base(
+				GTK_COMBO(quicksearch->search_string_entry)->entry,
+				GTK_STATE_NORMAL, &yellow);
+			gtk_widget_modify_text(
+				GTK_COMBO(quicksearch->search_string_entry)->entry,
+				GTK_STATE_NORMAL, &black);
+	} else {
+		gtk_widget_set_sensitive(quicksearch->clear_search, FALSE);
+		if (yellow_initialised)
+			gtk_widget_modify_base(
+				GTK_COMBO(quicksearch->search_string_entry)->entry,
+				GTK_STATE_NORMAL, NULL);
+			gtk_widget_modify_text(
+				GTK_COMBO(quicksearch->search_string_entry)->entry,
+				GTK_STATE_NORMAL, NULL);
+	}
+
 	if (!active) {
 		quicksearch_reset_cur_folder_item(quicksearch);
 	}
@@ -462,6 +607,7 @@ void quicksearch_set_execute_callback(QuickSearch *quicksearch,
 gboolean quicksearch_match(QuickSearch *quicksearch, MsgInfo *msginfo)
 {
 	gchar *searched_header = NULL;
+	gboolean result = FALSE;
 
 	if (!quicksearch->active)
 		return TRUE;
@@ -482,16 +628,21 @@ gboolean quicksearch_match(QuickSearch *quicksearch, MsgInfo *msginfo)
 		debug_print("unknown search type (%d)\n", prefs_common.summary_quicksearch_type);
 		break;
 	}
-
-	if (prefs_common.summary_quicksearch_type != QUICK_SEARCH_EXTENDED && 
+	quicksearch->matching = TRUE;
+	if (prefs_common.summary_quicksearch_type != QUICK_SEARCH_EXTENDED &&
 	    quicksearch->search_string &&
             searched_header && strcasestr(searched_header, quicksearch->search_string) != NULL)
-		return TRUE;
-	else if ((quicksearch->matcher_list != NULL) && 
+		result = TRUE;
+	else if ((quicksearch->matcher_list != NULL) &&
 	         matcherlist_match(quicksearch->matcher_list, msginfo))
-		return TRUE;
+		result = TRUE;
 
-	return FALSE;
+	quicksearch->matching = FALSE;
+	if (quicksearch->deferred_free) {
+		prepare_matcher(quicksearch);
+	}
+
+	return result;
 }
 
 /* allow Mutt-like patterns in quick search */
@@ -507,7 +658,7 @@ gchar *expand_search_string(const gchar *search_string)
 	/* list of allowed pattern abbreviations */
 	struct {
 		gchar		*abbreviated;	/* abbreviation */
-		gchar		*command;	/* actual matcher command */ 
+		gchar		*command;	/* actual matcher command */
 		gint		numparams;	/* number of params for cmd */
 		gboolean	qualifier;	/* do we append regexpcase */
 		gboolean	quotes;		/* do we need quotes */
@@ -545,7 +696,7 @@ gchar *expand_search_string(const gchar *search_string)
 		{ "T",	"marked",			0,	FALSE,	FALSE },
 		{ "U",	"unread",			0,	FALSE,	FALSE },
 		{ "x",	"header \"References\"",	1,	TRUE,	TRUE  },
-		{ "X",  "test",				1,	FALSE,  FALSE }, 
+		{ "X",  "test",				1,	FALSE,  FALSE },
 		{ "y",	"header \"X-Label\"",		1,	TRUE,	TRUE  },
 		{ "&",	"&",				0,	FALSE,	FALSE },
 		{ "|",	"|",				0,	FALSE,	FALSE },
@@ -663,7 +814,7 @@ gchar *expand_search_string(const gchar *search_string)
 
 	g_free(copy_str);
 
-	/* return search string if no match is found to allow 
+	/* return search string if no match is found to allow
 	   all available filtering expressions in quicksearch */
 	if (matcherstr->len > 0) returnstr = matcherstr->str;
 	else returnstr = g_strdup(search_string);
@@ -677,7 +828,7 @@ static void quicksearch_set_running(QuickSearch *quicksearch, gboolean run)
 	quicksearch->running = run;
 }
 
-gboolean quicksearch_is_running(QuickSearch *quicksearch) 
+gboolean quicksearch_is_running(QuickSearch *quicksearch)
 {
 	return quicksearch->running;
 }
@@ -685,59 +836,74 @@ gboolean quicksearch_is_running(QuickSearch *quicksearch)
 void quicksearch_pass_key(QuickSearch *quicksearch, guint val, GdkModifierType mod)
 {
 	GtkEntry *entry = GTK_ENTRY(GTK_COMBO(quicksearch->search_string_entry)->entry);
-	gint curpos = gtk_editable_get_position(GTK_EDITABLE(entry));
+	glong curpos = gtk_editable_get_position(GTK_EDITABLE(entry));
+	guint32 c;
 	char *str = g_strdup(gtk_entry_get_text(entry));
 	char *begin = str;
 	char *end = NULL;
 	char *new = NULL;
-	
-	if (mod == GDK_SHIFT_MASK)
-		val = toupper(val);
-	
-	if (curpos < strlen(str)-1) {
-		end = g_strdup(str+curpos);
-		*(str+curpos) = '\0';
-		new = g_strdup_printf("%s%c%s", begin, val, end);
+	char key[7] = "";
+	guint char_len = 0;
+
+	if (!(c = gdk_keyval_to_unicode(val))) {
+		g_free(str);
+		return;
+	}
+	char_len = g_unichar_to_utf8(c, key);
+	if (char_len < 0)
+		return;
+	key[char_len] = '\0';
+	if (curpos < g_utf8_strlen(str, -1)) {
+		end = g_utf8_offset_to_pointer(str, curpos+1);
+		new = g_strdup_printf("%s%s%s", begin, key, end);
 		gtk_entry_set_text(entry, new);
 		g_free(end);
 	} else {
-		new = g_strdup_printf("%s%c", begin, val);
+		new = g_strdup_printf("%s%s", begin, key);
 		gtk_entry_set_text(entry, new);
 	}
 	g_free(str);
 	g_free(new);
 	gtk_editable_set_position(GTK_EDITABLE(entry), curpos+1);
-	
+
 }
 
-static gboolean quicksearch_match_subfolder(QuickSearch *quicksearch, 
+static gboolean quicksearch_match_subfolder(QuickSearch *quicksearch,
 				 FolderItem *src)
 {
 	GSList *msglist = folder_item_get_msg_list(src);
 	GSList *cur;
 	gboolean result = FALSE;
-	
+	gint num = 0, total = src->total_msgs;
+	statusbar_print_all(_("Searching in %s... \n"),
+		src->path ? src->path : "(null)");
 	for (cur = msglist; cur != NULL; cur = cur->next) {
 		MsgInfo *msg = (MsgInfo *)cur->data;
+		statusbar_progress_all(num++,total, 50);
 		if (quicksearch_match(quicksearch, msg)) {
 			procmsg_msginfo_free(msg);
 			result = TRUE;
 			break;
 		}
 		procmsg_msginfo_free(msg);
+		GTK_EVENTS_FLUSH();
+		if (!quicksearch_is_active(quicksearch))
+			break;
 	}
+	statusbar_progress_all(0,0,0);
+	statusbar_pop_all();
 
 	g_slist_free(msglist);
 	return result;
 }
 
-void quicksearch_search_subfolders(QuickSearch *quicksearch, 
+void quicksearch_search_subfolders(QuickSearch *quicksearch,
 				   FolderView *folderview,
 				   FolderItem *folder_item)
 {
 	FolderItem *cur = NULL;
 	GNode *node = folder_item->node->children;
-	
+
 	if (!prefs_common.summary_quicksearch_recurse)
 		return;
 
@@ -754,15 +920,17 @@ void quicksearch_search_subfolders(QuickSearch *quicksearch,
 						      cur);
 	}
 	quicksearch->root_folder_item = folder_item;
+	if (!quicksearch_is_active(quicksearch))
+		quicksearch_reset_cur_folder_item(quicksearch);
 }
 
 static void quicksearch_reset_folder_items(QuickSearch *quicksearch,
 				    FolderItem *folder_item)
 {
 	FolderItem *cur = NULL;
-	GNode *node = (folder_item && folder_item->node) ? 
+	GNode *node = (folder_item && folder_item->node) ?
 			folder_item->node->children : NULL;
-	
+
 	for (; node != NULL; node = node->next) {
 		cur = FOLDER_ITEM(node->data);
 		folderview_update_search_icon(cur, FALSE);
@@ -775,8 +943,8 @@ static void quicksearch_reset_folder_items(QuickSearch *quicksearch,
 void quicksearch_reset_cur_folder_item(QuickSearch *quicksearch)
 {
 	if (quicksearch->root_folder_item)
-		quicksearch_reset_folder_items(quicksearch, 
+		quicksearch_reset_folder_items(quicksearch,
 					       quicksearch->root_folder_item);
-	
+
 	quicksearch->root_folder_item = NULL;
 }
