@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2001-2003 Match Grun
+ * Copyright (C) 2001-2006 Match Grun and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /*
@@ -1097,12 +1097,13 @@ static void jpilot_parse_label( JPilotFile *pilotFile, gchar *labelEntry, ItemPe
 		strcpy( buffer, labelEntry );
 		node = list = jpilot_parse_email( buffer );
 		while( node ) {
-			gchar convertBuff[JPILOT_LEN_LABEL];
 			email = addritem_create_item_email();
 			addritem_email_set_address( email, node->data );
 			if (convert_charcode) {
 				gchar *convertBuff;
-				convertBuff = conv_codeset_strdup( labelEntry, conv_get_locale_charset_str(), CS_INTERNAL );
+				convertBuff = conv_codeset_strdup( labelEntry, 
+						conv_get_locale_charset_str_no_utf8(), 
+						CS_INTERNAL );
 				addritem_email_set_remarks( email, convertBuff );
 				g_free( convertBuff );
 			}
@@ -1184,7 +1185,9 @@ static void jpilot_load_address(
 
 		if( convert_charcode ) {
 			gchar *nameConv;
-			nameConv = conv_codeset_strdup( fullName, conv_get_locale_charset_str(), CS_INTERNAL );
+			nameConv = conv_codeset_strdup( fullName, 
+					conv_get_locale_charset_str_no_utf8(), 
+					CS_INTERNAL );
 			strncpy2( fullName, nameConv, FULLNAME_BUFSIZE );
 			g_free( nameConv );
 		}
@@ -1254,10 +1257,9 @@ static void jpilot_load_address(
 			addritem_free_item_person( person );
 			person = NULL;
 		}
+		/* Free up pointer allocated inside address */
+		free_Address( & addr );
 	}
-
-	/* Free up pointer allocated inside address */
-	free_Address( & addr );
 }
 
 /**
@@ -1351,7 +1353,9 @@ static gboolean jpilot_setup_labels( JPilotFile *pilotFile ) {
 				gchar convertBuff[ JPILOT_LEN_LABEL ];
 
 				if( convert_charcode ) {
-					labelName = conv_codeset_strdup( labelName, conv_get_locale_charset_str(), CS_INTERNAL );
+					labelName = conv_codeset_strdup( labelName, 
+							conv_get_locale_charset_str_no_utf8(), 
+							CS_INTERNAL );
 					strncpy2( convertBuff, labelName, JPILOT_LEN_LABEL );
 					g_free( labelName );
 					labelName = convertBuff;
@@ -1389,7 +1393,9 @@ GList *jpilot_load_label( JPilotFile *pilotFile, GList *labelList ) {
 
 			if( labelName ) {
 				if( convert_charcode ) {
-					labelName = conv_codeset_strdup( labelName, conv_get_locale_charset_str(), CS_INTERNAL );
+					labelName = conv_codeset_strdup( labelName, 
+							conv_get_locale_charset_str_no_utf8(), 
+							CS_INTERNAL );
 				}
 				else {
 					labelName = g_strdup( labelName );
@@ -1479,7 +1485,9 @@ GList *jpilot_load_custom_label( JPilotFile *pilotFile, GList *labelList ) {
 				g_strchug( labelName );
 				if( *labelName != '\0' ) {
 					if( convert_charcode ) {
-						labelName = conv_codeset_strdup( labelName, conv_get_locale_charset_str(), CS_INTERNAL );
+						labelName = conv_codeset_strdup( labelName, 
+								conv_get_locale_charset_str_no_utf8(), 
+								CS_INTERNAL );
 					}
 					else {
 						labelName = g_strdup( labelName );
@@ -1535,7 +1543,9 @@ static void jpilot_build_category_list( JPilotFile *pilotFile ) {
 
 		if( convert_charcode ) {
 			gchar *catName;
-			catName = conv_codeset_strdup( cat->name[i], conv_get_locale_charset_str(), CS_INTERNAL );
+			catName = conv_codeset_strdup( cat->name[i], 
+					conv_get_locale_charset_str_no_utf8(), 
+					CS_INTERNAL );
 			addritem_folder_set_name( folder, catName );
 			g_free( catName );
 		}
