@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -138,6 +138,9 @@ static struct RemoteCmd {
 	gboolean crash;
 	int online_mode;
 	gchar   *crash_params;
+	gboolean exit;
+	gboolean subscribe;
+	const gchar *subscribe_uri;
 } cmd;
 
 static void parse_cmd_opt(int argc, char *argv[]);
@@ -213,19 +216,64 @@ static void startup_notification_complete(gboolean with_window)
 	sn_context = sn_launchee_context_new_from_environment(sn_display,
 						 DefaultScreen(xdisplay));
 
-	if (sn_context != NULL)
-	{
+	if (sn_context != NULL)	{
 		sn_launchee_context_complete(sn_context);
 		sn_launchee_context_unref(sn_context);
 		sn_display_unref(sn_display);
 	}
+	if (with_window) {
+		gtk_widget_destroy(hack);
+	}
 }
 #endif /* HAVE_STARTUP_NOTIFICATION */
 
 void sylpheed_gtk_idle(void) 
 {
-	while(gtk_events_pending())
+	while(gtk_events_pending()) {
 		gtk_main_iteration();
+	}
+	g_usleep(50000);
+}
+
+gboolean defer_check_all(void *data)
+{
+	gboolean autochk = GPOINTER_TO_INT(data);
+
+	inc_all_account_mail(static_mainwindow, autochk, 
+			prefs_common.newmail_notify_manu);
+
+	return FALSE;
+}
+
+gboolean defer_check(void *data)
+{
+	inc_mail(static_mainwindow, prefs_common.newmail_notify_manu);
+
+	return FALSE;
+}
+
+static gboolean migrate_old_config(const gchar *old_cfg_dir, const gchar *new_cfg_dir)
+{
+	gchar *message = g_strdup_printf(_("Configuration for Sylpheed-Claws %s found.\n"
+			 "Do you want to migrate this configuration?"),
+			 !strcmp(old_cfg_dir, OLD_GTK1_RC_DIR)?
+			 	_("1.0.5 or previous"):_("1.9.15 or previous"));
+	gint r = 0;
+	GtkWidget *window = NULL;
+	if (alertpanel(_("Migration of configuration"),
+		       message,
+		       GTK_STOCK_NO, GTK_STOCK_YES, NULL) != G_ALERTALTERNATE) {
+		return FALSE;
+	}
+	
+	window = label_window_create(_("Copying configuration..."));
+	GTK_EVENTS_FLUSH();
+	r = copy_dir(old_cfg_dir, new_cfg_dir);
+	gtk_widget_destroy(window);
+	if (r != 0) {
+		alertpanel_error(_("Migration failed!"));
+	}
+	return (r == 0);
 }
 
 int main(int argc, char *argv[])
@@ -271,10 +319,13 @@ int main(int argc, char *argv[])
 	}
 
 	if (cmd.status || cmd.status_full) {
-		puts("0 Sylpheed not running.");
+		puts("0 Sylpheed-Claws not running.");
 		lock_socket_remove();
 		return 0;
 	}
+	
+	if (cmd.exit)
+		return 0;
 #endif
 	g_thread_init(NULL);
 	/* gdk_threads_init(); */
@@ -286,8 +337,9 @@ int main(int argc, char *argv[])
 	gtk_widget_set_default_colormap(gdk_rgb_get_colormap());
 	gtk_widget_set_default_visual(gdk_rgb_get_visual());
 
-	if (!g_thread_supported())
+	if (!g_thread_supported()) {
 		g_error(_("g_thread is not supported by glib.\n"));
+	}
 
 	/* parse gtkrc files */
 	userrc = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, ".gtkrc-2.0",
@@ -298,6 +350,18 @@ int main(int argc, char *argv[])
 			     G_DIR_SEPARATOR_S, "gtkrc-2.0", NULL);
 	gtk_rc_parse(userrc);
 	g_free(userrc);
+
+	CHDIR_RETURN_VAL_IF_FAIL(get_home_dir(), 1);
+	if (!is_dir_exist(RC_DIR)) {
+		gboolean r = FALSE;
+		if (is_dir_exist(OLD_GTK2_RC_DIR))
+			r = migrate_old_config(OLD_GTK2_RC_DIR, RC_DIR);
+		else if (is_dir_exist(OLD_GTK1_RC_DIR))
+			r = migrate_old_config(OLD_GTK1_RC_DIR, RC_DIR);
+		if (r == FALSE && !is_dir_exist(RC_DIR) && make_dir(RC_DIR) < 0)
+			exit(1);
+	}
+
 	userrc = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "gtkrc-2.0", NULL);
 	gtk_rc_parse(userrc);
 	g_free(userrc);
@@ -310,12 +374,9 @@ int main(int argc, char *argv[])
 				       "gtk-can-change-accels",
 				       (glong)TRUE, "XProperty");
 
-	CHDIR_RETURN_VAL_IF_FAIL(get_home_dir(), 1);
-
-	MAKE_DIR_IF_NOT_EXIST(RC_DIR);
-
 	CHDIR_RETURN_VAL_IF_FAIL(get_rc_dir(), 1);
 
+	MAKE_DIR_IF_NOT_EXIST(get_mail_base_dir());
 	MAKE_DIR_IF_NOT_EXIST(get_imap_cache_dir());
 	MAKE_DIR_IF_NOT_EXIST(get_news_cache_dir());
 	MAKE_DIR_IF_NOT_EXIST(get_mime_tmp_dir());
@@ -372,6 +433,7 @@ int main(int argc, char *argv[])
 	gtk_window_set_default_icon(icon);
 
 	folderview_initialize();
+
 	mh_gtk_init();
 	imap_gtk_init();
 	news_gtk_init();
@@ -380,6 +442,9 @@ int main(int argc, char *argv[])
 		(prefs_common.sep_folder | prefs_common.sep_msg << 1);
 	folderview = mainwin->folderview;
 
+	gtk_clist_freeze(GTK_CLIST(mainwin->folderview->ctree));
+	folder_item_update_freeze();
+
 	/* register the callback of unix domain socket input */
 #ifdef G_OS_UNIX
 	lock_socket_tag = gdk_input_add(lock_socket,
@@ -405,6 +470,8 @@ int main(int argc, char *argv[])
 			exit_sylpheed(mainwin);
 	}
 
+	main_window_popup(mainwin);
+
 #ifdef HAVE_LIBETPAN
 	imap_main_init();
 #endif	
@@ -417,29 +484,30 @@ int main(int argc, char *argv[])
 	/* make one all-folder processing before using sylpheed */
 	main_window_cursor_wait(mainwin);
 	folder_func_to_all_folders(initial_processing, (gpointer *)mainwin);
-	main_window_cursor_normal(mainwin);
 
 	/* if Sylpheed crashed, rebuild caches */
 	if (!cmd.crash && crash_file_present) {
-		debug_print("Sylpheed crashed, checking for new messages in local folders\n");
+		debug_print("Sylpheed-Claws crashed, checking for new messages in local folders\n");
+		folder_item_update_thaw();
 		folderview_check_new(NULL);
 		folder_clean_cache_memory_force();
+		folder_item_update_freeze();
 	}
 	/* make the crash-indicator file */
 	str_write_to_file("foo", get_crashfile_name());
 
-	addressbook_read_file();
-
 	inc_autocheck_timer_init(mainwin);
 
 	/* ignore SIGPIPE signal for preventing sudden death of program */
 #ifdef G_OS_UNIX
 	signal(SIGPIPE, SIG_IGN);
 #endif
-	if (cmd.online_mode == ONLINE_MODE_OFFLINE)
-		main_window_toggle_work_offline(mainwin, TRUE);
-	if (cmd.online_mode == ONLINE_MODE_ONLINE)
-		main_window_toggle_work_offline(mainwin, FALSE);
+	if (cmd.online_mode == ONLINE_MODE_OFFLINE) {
+		main_window_toggle_work_offline(mainwin, TRUE, FALSE);
+	}
+	if (cmd.online_mode == ONLINE_MODE_ONLINE) {
+		main_window_toggle_work_offline(mainwin, FALSE, FALSE);
+	}
 
 	if (cmd.status_folders) {
 		g_ptr_array_free(cmd.status_folders, TRUE);
@@ -456,32 +524,55 @@ int main(int argc, char *argv[])
 
 	plugin_load_all("GTK2");
 	
+	if (plugin_get_unloaded_list() != NULL) {
+		alertpanel_warning(_("Some plugin(s) failed to load. "
+				     "Check the Plugins configuration "
+				     "for more information."));
+	}
+	
+	if (!folder_have_mailbox()) {
+		alertpanel_error(_("Sylpheed-Claws has detected a configured "
+				   "mailbox, but could not load it. It is "
+				   "probably provided by an out-of-date "
+				   "external plugin. Please reinstall the "
+				   "plugin and try again."));
+		exit(1);
+	}
+	
 	static_mainwindow = mainwin;
 
 #ifdef HAVE_STARTUP_NOTIFICATION
 	startup_notification_complete(FALSE);
 #endif
+	folder_item_update_thaw();
+	gtk_clist_thaw(GTK_CLIST(mainwin->folderview->ctree));
+	main_window_cursor_normal(mainwin);
 
-	if (cmd.receive_all)
-		inc_all_account_mail(mainwin, FALSE, 
-				     prefs_common.newmail_notify_manu);
-	else if (prefs_common.chk_on_startup)
-		inc_all_account_mail(mainwin, TRUE, 
-				     prefs_common.newmail_notify_manu);
-	else if (cmd.receive)
-		inc_mail(mainwin, prefs_common.newmail_notify_manu);
-	else
+	if (cmd.receive_all) {
+		g_timeout_add(1000, defer_check_all, GINT_TO_POINTER(FALSE));
+	} else if (prefs_common.chk_on_startup) {
+		g_timeout_add(1000, defer_check_all, GINT_TO_POINTER(TRUE));
+	} else if (cmd.receive) {
+		g_timeout_add(1000, defer_check, NULL);
+	} else {
 		gtk_widget_grab_focus(folderview->ctree);
+	}
 
-	if (cmd.compose)
+	if (cmd.compose) {
 		open_compose_new(cmd.compose_mailto, cmd.attach_files);
+	}
 	if (cmd.attach_files) {
 		ptr_array_free_strings(cmd.attach_files);
 		g_ptr_array_free(cmd.attach_files, TRUE);
 		cmd.attach_files = NULL;
 	}
-	if (cmd.send)
+	if (cmd.subscribe) {
+		folder_subscribe(cmd.subscribe_uri);
+	}
+
+	if (cmd.send) {
 		send_queue();
+	}
 	
 	gtk_main();
 
@@ -492,8 +583,9 @@ int main(int argc, char *argv[])
 
 static void save_all_caches(FolderItem *item, gpointer data)
 {
-	if (!item->cache)
+	if (!item->cache) {
 		return;
+	}
 	folder_item_write_cache(item);
 }
 
@@ -504,12 +596,12 @@ static void exit_sylpheed(MainWindow *mainwin)
 	debug_print("shutting down\n");
 	inc_autocheck_timer_remove();
 
-	if (prefs_common.clean_on_exit)
+	if (prefs_common.clean_on_exit) {
 		main_window_empty_trash(mainwin, prefs_common.ask_on_clean);
+	}
 
 	/* save prefs for opened folder */
-	if(mainwin->folderview->opened)
-	{
+	if(mainwin->folderview->opened) {
 		FolderItem *item;
 
 		item = gtk_ctree_node_get_row_data(GTK_CTREE(mainwin->folderview->ctree), mainwin->folderview->opened);
@@ -545,7 +637,7 @@ static void exit_sylpheed(MainWindow *mainwin)
 
 	lock_socket_remove();
 
-	main_window_destroy(mainwin);
+	main_window_destroy_all();
 	
 	plugin_unload_all("GTK2");
 
@@ -580,35 +672,44 @@ static void parse_cmd_opt(int argc, char *argv[])
 	gint i;
 
 	for (i = 1; i < argc; i++) {
-		if (!strncmp(argv[i], "--receive-all", 13))
+		if (!strncmp(argv[i], "--receive-all", 13)) {
 			cmd.receive_all = TRUE;
-		else if (!strncmp(argv[i], "--receive", 9))
+		} else if (!strncmp(argv[i], "--receive", 9)) {
 			cmd.receive = TRUE;
-		else if (!strncmp(argv[i], "--compose", 9)) {
+		} else if (!strncmp(argv[i], "--compose", 9)) {
 			const gchar *p = argv[i + 1];
 
 			cmd.compose = TRUE;
 			cmd.compose_mailto = NULL;
 			if (p && *p != '\0' && *p != '-') {
-				if (!strncmp(p, "mailto:", 7))
+				if (!strncmp(p, "mailto:", 7)) {
 					cmd.compose_mailto = p + 7;
-				else
+				} else {
 					cmd.compose_mailto = p;
+				}
 				i++;
 			}
+		} else if (!strncmp(argv[i], "--subscribe", 11)) {
+			const gchar *p = argv[i + 1];
+			if (p && *p != '\0' && *p != '-') {
+				cmd.subscribe = TRUE;
+				cmd.subscribe_uri = p;
+			}
 		} else if (!strncmp(argv[i], "--attach", 8)) {
 			const gchar *p = argv[i + 1];
 			gchar *file;
 
 			while (p && *p != '\0' && *p != '-') {
-				if (!cmd.attach_files)
+				if (!cmd.attach_files) {
 					cmd.attach_files = g_ptr_array_new();
-				if (*p != G_DIR_SEPARATOR)
+				}
+				if (*p != G_DIR_SEPARATOR) {
 					file = g_strconcat(sylpheed_get_startup_dir(),
 							   G_DIR_SEPARATOR_S,
 							   p, NULL);
-				else
+				} else {
 					file = g_strdup(p);
+				}
 				g_ptr_array_add(cmd.attach_files, file);
 				i++;
 				p = argv[i + 1];
@@ -616,16 +717,17 @@ static void parse_cmd_opt(int argc, char *argv[])
 		} else if (!strncmp(argv[i], "--send", 6)) {
 			cmd.send = TRUE;
 		} else if (!strncmp(argv[i], "--version", 9)) {
-			puts("Sylpheed version " VERSION);
+			puts("Sylpheed-Claws version " VERSION);
 			exit(0);
  		} else if (!strncmp(argv[i], "--status-full", 13)) {
  			const gchar *p = argv[i + 1];
  
  			cmd.status_full = TRUE;
  			while (p && *p != '\0' && *p != '-') {
- 				if (!cmd.status_full_folders)
+ 				if (!cmd.status_full_folders) {
  					cmd.status_full_folders =
  						g_ptr_array_new();
+				}
  				g_ptr_array_add(cmd.status_full_folders,
  						g_strdup(p));
  				i++;
@@ -652,6 +754,7 @@ static void parse_cmd_opt(int argc, char *argv[])
 			g_print(_("Usage: %s [OPTION]...\n"), base);
 
 			g_print("%s\n", _("  --compose [address]    open composition window"));
+			g_print("%s\n", _("  --subscribe [uri]      subscribe to the given URI if possible"));
 			g_print("%s\n", _("  --attach file1 [file2]...\n"
 			          "                         open composition window with specified files\n"
 			          "                         attached"));
@@ -663,6 +766,7 @@ static void parse_cmd_opt(int argc, char *argv[])
  			       "                         show the status of each folder"));
 			g_print("%s\n", _("  --online               switch to online mode"));
 			g_print("%s\n", _("  --offline              switch to offline mode"));
+			g_print("%s\n", _("  --exit                 exit Sylpheed-Claws"));
 			g_print("%s\n", _("  --debug                debug mode"));
 			g_print("%s\n", _("  --help                 display this help and exit"));
 			g_print("%s\n", _("  --version              output version information and exit"));
@@ -677,6 +781,29 @@ static void parse_cmd_opt(int argc, char *argv[])
 		} else if (!strncmp(argv[i], "--config-dir", sizeof "--config-dir" - 1)) {
 			puts(RC_DIR);
 			exit(0);
+		} else if (!strncmp(argv[i], "--exit", 6)) {
+			cmd.exit = TRUE;
+		} else if (i == 1 && argc == 2) {
+			/* only one parameter. Do something intelligent about it */
+			if (strstr(argv[i], "@") && !strstr(argv[i], "://")) {
+				const gchar *p = argv[i];
+
+				cmd.compose = TRUE;
+				cmd.compose_mailto = NULL;
+				if (p && *p != '\0' && *p != '-') {
+					if (!strncmp(p, "mailto:", 7)) {
+						cmd.compose_mailto = p + 7;
+					} else {
+						cmd.compose_mailto = p;
+					}
+				}
+			} else if (strstr(argv[i], "://")) {
+				const gchar *p = argv[i];
+				if (p && *p != '\0' && *p != '-') {
+					cmd.subscribe = TRUE;
+					cmd.subscribe_uri = p;
+				}
+			}
 		}
 		
 	}
@@ -692,7 +819,9 @@ static gint get_queued_message_num(void)
 	FolderItem *queue;
 
 	queue = folder_get_default_queue();
-	if (!queue) return -1;
+	if (!queue) {
+		return -1;
+	}
 
 	folder_item_scan(queue);
 	return queue->total_msgs;
@@ -712,8 +841,9 @@ static void initial_processing(FolderItem *item, gpointer data)
 	g_free(buf);
 
 	
-        if (item->prefs->enable_processing)
-                folder_item_apply_processing(item);
+	if (item->prefs->enable_processing) {
+		folder_item_apply_processing(item);
+	}
 
 	debug_print("done.\n");
 	STATUSBAR_POP(mainwin);
@@ -733,8 +863,9 @@ gboolean clean_quit(gpointer data)
 {
 	static gboolean firstrun = TRUE;
 
-	if (!firstrun)
+	if (!firstrun) {
 		return FALSE;
+	}
 	firstrun = FALSE;
 
 	/*!< Good idea to have the main window stored in a 
@@ -749,8 +880,9 @@ gboolean clean_quit(gpointer data)
 	/* FIXME: Use something else to signal that we're
 	 * in the original spawner, and not in a spawned
 	 * child. */
-	if (!static_mainwindow) 
+	if (!static_mainwindow) {
 		return FALSE;
+	}
 		
 	draft_all_messages();
 
@@ -763,11 +895,11 @@ gboolean clean_quit(gpointer data)
 void app_will_exit(GtkWidget *widget, gpointer data)
 {
 	MainWindow *mainwin = data;
-	
+
 	if (compose_get_compose_list()) {
 		gint val = alertpanel(_("Really quit?"),
 			       _("Composing message exists."),
-			       _("Draft them"), _("Discard them"), _("Don't quit"));
+			       _("_Save to Draft"), _("_Discard them"), _("Do_n't quit"));
 		switch (val) {
 			case G_ALERTOTHER:
 				return;
@@ -776,16 +908,17 @@ void app_will_exit(GtkWidget *widget, gpointer data)
 			default:
 				draft_all_messages();
 		}
-		
+
 		manage_window_focus_in(mainwin->window, NULL, NULL);
 	}
 
 	if (prefs_common.warn_queued_on_exit && get_queued_message_num() > 0) {
 		if (alertpanel(_("Queued messages"),
 			       _("Some unsent messages are queued. Exit now?"),
-			       GTK_STOCK_OK, GTK_STOCK_CANCEL, NULL)
-		    != G_ALERTDEFAULT)
+			       GTK_STOCK_CANCEL, GTK_STOCK_OK, NULL)
+		    != G_ALERTALTERNATE) {
 			return;
+		}
 		manage_window_focus_in(mainwin->window, NULL, NULL);
 	}
 
@@ -806,7 +939,7 @@ gchar *get_socket_name(void)
 	static gchar *filename = NULL;
 
 	if (filename == NULL) {
-		filename = g_strdup_printf("%s%csylpheed-%d",
+		filename = g_strdup_printf("%s%csylpheed-claws-%d",
 					   g_get_tmp_dir(), G_DIR_SEPARATOR,
 #if HAVE_GETUID
 					   getuid());
@@ -844,21 +977,22 @@ static gint prohibit_duplicate_launch(void)
 
 	/* remote command mode */
 
-	debug_print("another Sylpheed is already running.\n");
+	debug_print("another Sylpheed-Claws instance is already running.\n");
 
-	if (cmd.receive_all)
+	if (cmd.receive_all) {
 		fd_write_all(uxsock, "receive_all\n", 12);
-	else if (cmd.receive)
+	} else if (cmd.receive) {
 		fd_write_all(uxsock, "receive\n", 8);
-	else if (cmd.compose && cmd.attach_files) {
+	} else if (cmd.compose && cmd.attach_files) {
 		gchar *str, *compose_str;
 		gint i;
 
-		if (cmd.compose_mailto)
+		if (cmd.compose_mailto) {
 			compose_str = g_strdup_printf("compose_attach %s\n",
 						      cmd.compose_mailto);
-		else
+		} else {
 			compose_str = g_strdup("compose_attach\n");
+		}
 
 		fd_write_all(uxsock, compose_str, strlen(compose_str));
 		g_free(compose_str);
@@ -873,14 +1007,19 @@ static gint prohibit_duplicate_launch(void)
 	} else if (cmd.compose) {
 		gchar *compose_str;
 
-		if (cmd.compose_mailto)
+		if (cmd.compose_mailto) {
 			compose_str = g_strdup_printf
 				("compose %s\n", cmd.compose_mailto);
-		else
+		} else {
 			compose_str = g_strdup("compose\n");
+		}
 
 		fd_write_all(uxsock, compose_str, strlen(compose_str));
 		g_free(compose_str);
+	} else if (cmd.subscribe) {
+		gchar *str = g_strdup_printf("subscribe %s\n", cmd.subscribe_uri);
+		fd_write_all(uxsock, str, strlen(str));
+		g_free(str);
 	} else if (cmd.send) {
 		fd_write_all(uxsock, "send\n", 5);
 	} else if (cmd.online_mode == ONLINE_MODE_ONLINE) {
@@ -910,6 +1049,8 @@ static gint prohibit_duplicate_launch(void)
  			if (!strncmp(buf, ".\n", 2)) break;
  			fputs(buf, stdout);
  		}
+	} else if (cmd.exit) {
+		fd_write_all(uxsock, "exit\n", 5);
 	} else
 		fd_write_all(uxsock, "popup\n", 6);
 
@@ -922,10 +1063,13 @@ static gint lock_socket_remove(void)
 #ifdef G_OS_UNIX
 	gchar *filename;
 
-	if (lock_socket < 0) return -1;
+	if (lock_socket < 0) {
+		return -1;
+	}
 
-	if (lock_socket_tag > 0)
+	if (lock_socket_tag > 0) {
 		gdk_input_remove(lock_socket_tag);
+	}
 	fd_close(lock_socket);
 	filename = get_socket_name();
 	g_unlink(filename);
@@ -942,14 +1086,19 @@ static GPtrArray *get_folder_item_list(gint sock)
 
 	for (;;) {
 		fd_gets(sock, buf, sizeof(buf));
-		if (!strncmp(buf, ".\n", 2)) break;
+		if (!strncmp(buf, ".\n", 2)) {
+			break;
+		}
 		strretchomp(buf);
-		if (!folders) folders = g_ptr_array_new();
+		if (!folders) {
+			folders = g_ptr_array_new();
+		}
 		item = folder_find_item_from_identifier(buf);
-		if (item)
+		if (item) {
 			g_ptr_array_add(folders, item);
-		else
+		} else {
 			g_warning("no such folder: %s\n", buf);
+		}
 	}
 
 	return folders;
@@ -980,7 +1129,9 @@ static void lock_socket_input_cb(gpointer data,
 		mailto = g_strdup(buf + strlen("compose_attach") + 1);
 		files = g_ptr_array_new();
 		while (fd_gets(sock, buf, sizeof(buf)) > 0) {
-			if (buf[0] == '.' && buf[1] == '\n') break;
+			if (buf[0] == '.' && buf[1] == '\n') {
+				break;
+			}
 			strretchomp(buf);
 			g_ptr_array_add(files, g_strdup(buf));
 		}
@@ -990,12 +1141,15 @@ static void lock_socket_input_cb(gpointer data,
 		g_free(mailto);
 	} else if (!strncmp(buf, "compose", 7)) {
 		open_compose_new(buf + strlen("compose") + 1, NULL);
+	} else if (!strncmp(buf, "subscribe", 9)) {
+		main_window_popup(mainwin);
+		folder_subscribe(buf + strlen("subscribe") + 1);
 	} else if (!strncmp(buf, "send", 4)) {
 		send_queue();
 	} else if (!strncmp(buf, "online", 6)) {
-		main_window_toggle_work_offline(mainwin, FALSE);
+		main_window_toggle_work_offline(mainwin, FALSE, FALSE);
 	} else if (!strncmp(buf, "offline", 7)) {
-		main_window_toggle_work_offline(mainwin, TRUE);
+		main_window_toggle_work_offline(mainwin, TRUE, FALSE);
  	} else if (!strncmp(buf, "status-full", 11) ||
  		   !strncmp(buf, "status", 6)) {
  		gchar *status;
@@ -1008,6 +1162,8 @@ static void lock_socket_input_cb(gpointer data,
  		fd_write_all(sock, ".\n", 2);
  		g_free(status);
  		if (folders) g_ptr_array_free(folders, TRUE);
+	} else if (!strncmp(buf, "exit", 4)) {
+		app_will_exit(NULL, mainwin);
 	}
 
 	fd_close(sock);
@@ -1036,10 +1192,12 @@ static void send_queue(void)
 			gint res = procmsg_send_queue
 				(folder->queue, prefs_common.savemsg);
 
-			if (res < 0)	
+			if (res < 0) {
 				alertpanel_error(_("Some errors occurred while sending queued messages."));
-			if (res) 	
+			}
+			if (res) {
 				folder_item_scan(folder->queue);
+			}
 		}
 	}
 }
@@ -1053,6 +1211,7 @@ static void quit_signal_handler(int sig)
 
 static void install_basic_sighandlers()
 {
+#ifndef G_OS_WIN32
 	sigset_t    mask;
 	struct sigaction act;
 
@@ -1083,4 +1242,5 @@ static void install_basic_sighandlers()
 #endif	
 
 	sigprocmask(SIG_UNBLOCK, &mask, 0);
+#endif /* !G_OS_WIN32 */
 }
