@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "defs.h"
@@ -62,6 +62,8 @@
 #include "filtering.h"
 #include "partial_download.h"
 #include "gedit-print.h"
+#include "inc.h"
+#include "log.h"
 
 static GList *messageview_list = NULL;
 
@@ -153,17 +155,17 @@ static GList *msgview_list = NULL;
 static GtkItemFactoryEntry msgview_entries[] =
 {
 	{N_("/_File"),			NULL, NULL, 0, "<Branch>"},
-	{N_("/_File/_Save as..."),	NULL, save_as_cb, 0, NULL},
-	{N_("/_File/_Print..."),	NULL, print_cb, 0, NULL},
+	{N_("/_File/_Save as..."),	"<control>S", save_as_cb, 0, NULL},
+	{N_("/_File/_Print..."),	"<control>P", print_cb, 0, NULL},
 	{N_("/_File/---"),		NULL, NULL, 0, "<Separator>"},
-	{N_("/_File/_Close"),		NULL, close_cb, 0, NULL},
+	{N_("/_File/_Close"),		"<control>W", close_cb, 0, NULL},
 
 	{N_("/_Edit"),			NULL, NULL, 0, "<Branch>"},
-	{N_("/_Edit/_Copy"),		NULL, copy_cb, 0, NULL},
-	{N_("/_Edit/Select _all"),	NULL, allsel_cb, 0, NULL},
+	{N_("/_Edit/_Copy"),		"<control>C", copy_cb, 0, NULL},
+	{N_("/_Edit/Select _all"),	"<control>A", allsel_cb, 0, NULL},
 	{N_("/_Edit/---"),		NULL, NULL, 0, "<Separator>"},
 	{N_("/_Edit/_Find in current message..."),
-					NULL, search_cb, 0, NULL},
+					"<control>F", search_cb, 0, NULL},
 
 	{N_("/_View"),			NULL, NULL, 0, "<Branch>"},
 
@@ -186,6 +188,8 @@ static GtkItemFactoryEntry msgview_entries[] =
 	 ENC_ACTION(C_ISO_8859_1)},
 	{N_("/_View/Character _encoding/Western European (ISO-8859-15)"),
 	 ENC_ACTION(C_ISO_8859_15)},
+	{N_("/_View/Character _encoding/Western European (Windows-1252)"),
+	 ENC_ACTION(C_WINDOWS_1252)},
 	ENC_SEPARATOR,
 	{N_("/_View/Character _encoding/Central European (ISO-8859-_2)"),
 	 ENC_ACTION(C_ISO_8859_2)},
@@ -203,6 +207,11 @@ static GtkItemFactoryEntry msgview_entries[] =
 	{N_("/_View/Character _encoding/Hebrew (Windows-1255)"),
 	 ENC_ACTION(C_CP1255)},
 	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/Arabic (ISO-8859-_6)"),
+	 ENC_ACTION(C_ISO_8859_6)},
+	{N_("/_View/Character _encoding/Arabic (Windows-1256)"),
+	 ENC_ACTION(C_CP1256)},
+	ENC_SEPARATOR,
 	{N_("/_View/Character _encoding/Turkish (ISO-8859-_9)"),
 	 ENC_ACTION(C_ISO_8859_9)},
 	ENC_SEPARATOR,
@@ -265,22 +274,22 @@ static GtkItemFactoryEntry msgview_entries[] =
 #undef DEC_ACTION
 
 	{N_("/_View/---"),		NULL, NULL, 0, "<Separator>"},
-	{N_("/_View/Mess_age source"),	NULL, view_source_cb, 0, NULL},
-	{N_("/_View/Show all _headers"),NULL, show_all_header_cb, 0, "<ToggleItem>"},
+	{N_("/_View/Mess_age source"),	"<control>U", view_source_cb, 0, NULL},
+	{N_("/_View/Show all _headers"),"<control>H", show_all_header_cb, 0, "<ToggleItem>"},
 
 	{N_("/_Message"),		NULL, NULL, 0, "<Branch>"},
 	{N_("/_Message/Compose _new message"),
-					NULL, compose_cb, 0, NULL},
+					"<control>M", compose_cb, 0, NULL},
 	{N_("/_Message/---"),		NULL, NULL, 0, "<Separator>"},
-	{N_("/_Message/_Reply"),	NULL, reply_cb, COMPOSE_REPLY, NULL},
+	{N_("/_Message/_Reply"),	"<control>R", reply_cb, COMPOSE_REPLY, NULL},
 	{N_("/_Message/Repl_y to/_all"),
-					NULL, reply_cb, COMPOSE_REPLY_TO_ALL, NULL},
+					"<control><shift>R", reply_cb, COMPOSE_REPLY_TO_ALL, NULL},
 	{N_("/_Message/Repl_y to/_sender"),
 					NULL, reply_cb, COMPOSE_REPLY_TO_SENDER, NULL},
 	{N_("/_Message/Repl_y to/mailing _list"),
-					NULL, reply_cb, COMPOSE_REPLY_TO_LIST, NULL},
+					"<control>L", reply_cb, COMPOSE_REPLY_TO_LIST, NULL},
 	{N_("/_Message/---"),		NULL, NULL, 0, "<Separator>"},
-	{N_("/_Message/_Forward"),	NULL, reply_cb, COMPOSE_FORWARD, NULL},
+	{N_("/_Message/_Forward"),	"<control><alt>F", reply_cb, COMPOSE_FORWARD_INLINE, NULL},
 	{N_("/_Message/For_ward as attachment"),
 					NULL, reply_cb, COMPOSE_FORWARD_AS_ATTACH, NULL},
 	{N_("/_Message/Redirec_t"),	NULL, reply_cb, COMPOSE_REDIRECT, NULL},
@@ -288,7 +297,7 @@ static GtkItemFactoryEntry msgview_entries[] =
 	{N_("/_Message/Re-_edit"),	NULL, reedit_cb, 0, NULL},
 
 	{N_("/_Tools"),			NULL, NULL, 0, "<Branch>"},
-	{N_("/_Tools/_Address book"),	NULL, addressbook_open_cb, 0, NULL},
+	{N_("/_Tools/_Address book"),	"<control><shift>A", addressbook_open_cb, 0, NULL},
 	{N_("/_Tools/Add sender to address boo_k"),
 					NULL, add_address_cb, 0, NULL},
 	{N_("/_Tools/---"),		NULL, NULL, 0, "<Separator>"},
@@ -547,15 +556,14 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 		AlertValue val;
 		gchar *message;
 		message = g_strdup_printf(
-				 _("The notification address to which the "
-				   "return receipt is to be sent\n"
-				   "does not correspond to the return path:\n"
-				   "Notification address: %s\n"
-				   "Return path: %s\n"
-				   "It is advised to not to send the return "
-				   "receipt."), to, buf);
-		val = alertpanel_full(_("Warning"), message,
-				       	_("Send"), _("+Don't Send"), NULL, FALSE,
+		  _("The notification address to which the return receipt is\n"
+		    "to be sent does not correspond to the return path:\n"
+		    "Notification address: %s\n"
+		    "Return path: %s\n"
+		    "It is advised to not to send the return receipt."),
+		  to, buf);
+		val = alertpanel_full(_("Warning"), message, _("_Send"),
+					_("+_Don't Send"), NULL, FALSE,
 				       	NULL, ALERT_WARNING, G_ALERTALTERNATE);
 		g_free(message);				
 		if (val != G_ALERTDEFAULT)
@@ -568,21 +576,20 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 	if (ac_list == NULL) {
 		AlertValue val = 
 		alertpanel_full(_("Warning"),
-				_("This message is asking for a return "
-				"receipt notification\n"
-				"but according to its 'To:' and 'CC:' "
-				"headers it was not\nofficially addressed "
-				"to you.\n"
-				"It is advised to not to send the return ."
-				"receipt."),
-				_("Send"), _("+Don't Send"),NULL, FALSE,
-				NULL, ALERT_WARNING, G_ALERTALTERNATE);
+		  _("This message is asking for a return receipt notification\n"
+		    "but according to its 'To:' and 'CC:' headers it was not\n"
+		    "officially addressed to you.\n"
+		    "It is advised to not to send the return receipt."),
+		  _("_Send"), _("+_Don't Send"),NULL, FALSE,
+		  NULL, ALERT_WARNING, G_ALERTALTERNATE);
 		if (val != G_ALERTDEFAULT)
 			return -1;
 	}
 
-	if (g_list_length(ac_list) > 1)
-		account = select_account_from_list(ac_list);
+	if (g_list_length(ac_list) > 1) {
+		if ((account = select_account_from_list(ac_list)) == NULL)
+			return -1;
+	}
 	else if (ac_list != NULL)
 		account = (PrefsAccount *) ac_list->data;
 	g_list_free(ac_list);
@@ -693,11 +700,11 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 		return -1;
 	}
 		
-	if (prefs_common.work_offline)
-		if (alertpanel(_("Offline warning"), 
-			       _("You're working offline. Override?"),
-			       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
-			return 0;
+	if (prefs_common.work_offline && 
+	    !inc_offline_should_override(
+		_("Sylpheed-Claws needs network access in order "
+		  "to send this email.")))
+		return 0;
 
 	/* send it */
 	path = folder_item_fetch_msg(queue, num);
@@ -741,10 +748,37 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 {
 	gchar *file;
 	MimeInfo *mimeinfo, *encinfo;
-
+	gchar *subject = NULL;
 	g_return_val_if_fail(msginfo != NULL, -1);
 
+	if (messageview->toolbar)
+		toolbar_set_learn_button
+			(messageview->toolbar,
+			 MSG_IS_SPAM(msginfo->flags)?LEARN_HAM:LEARN_SPAM);
+	else
+		toolbar_set_learn_button
+			(messageview->mainwin->toolbar,
+			 MSG_IS_SPAM(msginfo->flags)?LEARN_HAM:LEARN_SPAM);
+
+	if (messageview->toolbar) {
+		if (messageview->toolbar->learn_ham_btn)
+			gtk_widget_set_sensitive(
+				messageview->toolbar->learn_ham_btn, 
+				procmsg_spam_can_learn());
+		if (messageview->toolbar->learn_spam_btn)
+			gtk_widget_set_sensitive(
+				messageview->toolbar->learn_spam_btn, 
+				procmsg_spam_can_learn());
+	}
+	messageview->updating = TRUE;
 	mimeinfo = procmime_scan_message(msginfo);
+	messageview->updating = FALSE;
+	
+	if (messageview->deferred_destroy) {
+		messageview_destroy(messageview);
+		return 0;
+	}
+
 	if (!mimeinfo) {
 		textview_show_error(messageview->mimeview->textview);
 		return -1;
@@ -756,14 +790,23 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 			break;
 	}
 	
+	messageview->updating = TRUE;
 	file = procmsg_get_message_file_path(msginfo);
+	messageview->updating = FALSE;
+	
+	if (messageview->deferred_destroy) {
+		g_free(file);
+		messageview_destroy(messageview);
+		return 0;
+	}
+
 	if (!file) {
 		g_warning("can't get message file path.\n");
 		procmime_mimeinfo_free_all(mimeinfo);
 		textview_show_error(messageview->mimeview->textview);
 		return -1;
 	}
-
+	
 	if (messageview->msginfo != msginfo) {
 		procmsg_msginfo_free(messageview->msginfo);
 		messageview->msginfo = procmsg_msginfo_get_full_info(msginfo);
@@ -776,6 +819,24 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 			messageview->all_headers);
 
 	mimeview_show_message(messageview->mimeview, mimeinfo, file);
+	
+	if (messageview->window)
+		gtk_window_set_title(GTK_WINDOW(messageview->window), 
+				_("Sylpheed-Claws - Message View"));
+	if (messageview->window && msginfo->subject) {
+		subject = g_strdup(msginfo->subject);
+		if (!g_utf8_validate(subject, -1, NULL)) {
+			g_free(subject);
+			subject = g_malloc(strlen(msginfo->subject)*2 +1);
+			conv_localetodisp(subject, strlen(msginfo->subject)*2 +1, 
+				msginfo->subject);
+		}
+		if (g_utf8_validate(subject, -1, NULL))
+			gtk_window_set_title(GTK_WINDOW(messageview->window), 
+				subject);
+		g_free(subject);
+	}
+
 	messageview_set_position(messageview, 0);
 
 	if (messageview->msginfo->partial_recv)
@@ -791,9 +852,11 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 
 	mimeinfo = procmime_mimeinfo_next(mimeinfo);
 	if (mimeinfo && (mimeinfo->type != MIMETYPE_TEXT || 
-	    (strcmp(mimeinfo->subtype, "plain") &&
-	     strcmp(mimeinfo->subtype, "html")))) {
-		mimeview_show_part(messageview->mimeview,mimeinfo);
+	    strcmp(mimeinfo->subtype, "plain"))) {
+	    	if (strcmp(mimeinfo->subtype, "html"))
+			mimeview_show_part(messageview->mimeview,mimeinfo);
+		else if (prefs_common.invoke_plugin_on_html)
+			mimeview_show_part(messageview->mimeview,mimeinfo);
 	}
 
 	g_free(file);
@@ -828,8 +891,17 @@ void messageview_destroy(MessageView *messageview)
 	debug_print("destroy messageview\n");
 	messageview_list = g_list_remove(messageview_list, messageview);
 
-	hooks_unregister_hook(MSGINFO_UPDATE_HOOKLIST,
+	if (!messageview->deferred_destroy) {
+		hooks_unregister_hook(MSGINFO_UPDATE_HOOKLIST,
 			      messageview->msginfo_update_callback_id);
+	}
+
+	if (messageview->updating) {
+		debug_print("uh oh, better not touch that now\n");
+		messageview->deferred_destroy = TRUE;
+		gtk_widget_hide(messageview->window);
+		return;
+	}
 
 	headerview_destroy(messageview->headerview);
 	mimeview_destroy(messageview->mimeview);
@@ -1005,6 +1077,8 @@ gboolean messageview_search_string_backward(MessageView *messageview,
 
 gboolean messageview_is_visible(MessageView *messageview)
 {
+	if (messageview == NULL)
+		return FALSE;
 	return messageview->visible;
 }
 
@@ -1028,8 +1102,8 @@ void messageview_save_as(MessageView *messageview)
 
 		aval = alertpanel(_("Overwrite"),
 				  _("Overwrite existing file?"),
-				  GTK_STOCK_OK, GTK_STOCK_CANCEL, NULL);
-		if (G_ALERTDEFAULT != aval) return;
+				  GTK_STOCK_CANCEL, GTK_STOCK_OK, NULL);
+		if (G_ALERTALTERNATE != aval) return;
 	}
 
 	src = procmsg_get_message_file(msginfo);
@@ -1065,6 +1139,13 @@ static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event,
 		messageview_destroy(messageview);
 		return TRUE;
 	}
+
+	if ((event->state & (GDK_MOD1_MASK|GDK_CONTROL_MASK)) != 0)
+		return FALSE;
+
+	g_signal_stop_emission_by_name(G_OBJECT(widget),
+					"key_press_event");
+	mimeview_pass_key_press_event(messageview->mimeview, event);
 	return FALSE;
 }
 
@@ -1106,6 +1187,11 @@ void messageview_toggle_view_real(MessageView *messageview)
 
 static void return_receipt_show(NoticeView *noticeview, MsgInfo *msginfo)
 {
+	if (msginfo->folder 
+		&& (folder_has_parent_of_type(msginfo->folder, F_QUEUE)
+		 || folder_has_parent_of_type(msginfo->folder, F_DRAFT)))
+		return;
+
 	noticeview_set_text(noticeview, _("This message asks for a return receipt."));
 	noticeview_set_button_text(noticeview, _("Send receipt"));
 	noticeview_set_button_press_callback(noticeview,
@@ -1255,7 +1341,7 @@ static PrefsAccount *select_account_from_list(GList *ac_list)
 				  "accounts.\n"
 				  "Please choose which account do you want to "
 				  "use for sending the receipt notification:"),
-			        _("Send Notification"), _("+Cancel"), NULL,
+			        _("_Send Notification"), _("+_Cancel"), NULL,
 			        optmenu) != G_ALERTDEFAULT)
 		return NULL;
 	return account_find_from_id(account_id);
@@ -1555,3 +1641,27 @@ void messageview_set_menu_sensitive(MessageView *messageview)
 			 messageview->mimeview->textview->show_all_headers);
 	}
 }
+
+void messageview_learn (MessageView *msgview, gboolean is_spam)
+{
+	if (is_spam) {
+		if (procmsg_spam_learner_learn(msgview->msginfo, NULL, TRUE) == 0)
+			procmsg_msginfo_set_flags(msgview->msginfo, MSG_SPAM, 0);
+		else
+			log_error(_("An error happened while learning.\n"));
+		
+	} else {
+		if (procmsg_spam_learner_learn(msgview->msginfo, NULL, FALSE) == 0)
+			procmsg_msginfo_unset_flags(msgview->msginfo, MSG_SPAM, 0);
+		else
+			log_error(_("An error happened while learning.\n"));
+	}
+	if (msgview->toolbar)
+		toolbar_set_learn_button
+			(msgview->toolbar,
+			 MSG_IS_SPAM(msgview->msginfo->flags)?LEARN_HAM:LEARN_SPAM);
+	else
+		toolbar_set_learn_button
+			(msgview->mainwin->toolbar,
+			 MSG_IS_SPAM(msgview->msginfo->flags)?LEARN_HAM:LEARN_SPAM);
+}
