@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,6 +44,19 @@
 #include "utils.h"
 #include "codeconv.h"
 
+/* Define possible missing constants for Windows. */
+#ifdef G_OS_WIN32
+# ifndef S_IRGRP
+# define S_IRGRP 0
+# define S_IWGRP 0
+# endif
+# ifndef S_IROTH
+# define S_IROTH 0
+# define S_IWOTH 0
+# endif
+#endif
+
+
 static void	mh_folder_init		(Folder		*folder,
 					 const gchar	*name,
 					 const gchar	*path);
@@ -68,6 +81,10 @@ static gint     mh_add_msgs		(Folder		*folder,
 static gint     mh_copy_msg		(Folder		*folder,
 					 FolderItem	*dest,
 					 MsgInfo	*msginfo);
+static gint	mh_copy_msgs		(Folder 	*folder, 
+					 FolderItem 	*dest, 
+					 MsgInfoList 	*msglist, 
+			 		 GRelation 	*relation);
 static gint     mh_remove_msg		(Folder		*folder,
 					 FolderItem	*item,
 					 gint 		 num);
@@ -141,6 +158,7 @@ FolderClass *mh_get_class(void)
 		mh_class.add_msg = mh_add_msg;
 		mh_class.add_msgs = mh_add_msgs;
 		mh_class.copy_msg = mh_copy_msg;
+		mh_class.copy_msgs = mh_copy_msgs;
 		mh_class.remove_msg = mh_remove_msg;
 		mh_class.remove_all_msg = mh_remove_all_msg;
 		mh_class.is_msg_changed = mh_is_msg_changed;
@@ -189,16 +207,16 @@ gboolean mh_scan_required(Folder *folder, FolderItem *item)
 		(s.st_mtime - 3600 != item->mtime)) {
 		debug_print("MH scan required, folder updated: %s (%ld > %ld)\n",
 			    path,
-			    s.st_mtime,
-			    item->mtime);
+			    (long int) s.st_mtime,
+			    (long int) item->mtime);
 		g_free(path);
 		return TRUE;
 	}
 
 	debug_print("MH scan not required: %s (%ld <= %ld)\n",
 		    path,
-		    s.st_mtime,
-		    item->mtime);
+		    (long int) s.st_mtime,
+		    (long int) item->mtime);
 	g_free(path);
 	return FALSE;
 }
@@ -408,14 +426,33 @@ static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 }
 
 static gint mh_copy_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
+{
+	GSList msglist;
+
+	g_return_val_if_fail(msginfo != NULL, -1);
+
+	msglist.data = msginfo;
+	msglist.next = NULL;
+
+	return mh_copy_msgs(folder, dest, &msglist, NULL);	
+}
+
+static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist, 
+			 GRelation *relation)
 {
 	gboolean dest_need_scan = FALSE;
 	gchar *srcfile;
 	gchar *destfile;
 	gint filemode = 0;
 	FolderItemPrefs *prefs;
-
+	MsgInfo *msginfo = NULL;
+	gboolean remove_special_headers = FALSE;
+	MsgInfoList *cur = NULL;
 	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(msglist != NULL, -1);
+	
+	msginfo = (MsgInfo *)msglist->data;
+
 	g_return_val_if_fail(msginfo != NULL, -1);
 
 	if (msginfo->folder == dest) {
@@ -428,60 +465,69 @@ static gint mh_copy_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
 		if (dest->last_num < 0) return -1;
 	}
 
-	prefs = dest->prefs;
-
-	srcfile = procmsg_get_message_file(msginfo);
-	destfile = mh_get_new_msg_filename(dest);
-	if (!destfile) {
-		g_free(srcfile);
-		return -1;
-	}
-	
-	debug_print("Copying message %s%c%d to %s ...\n",
-		    msginfo->folder->path, G_DIR_SEPARATOR,
-		    msginfo->msgnum, dest->path);
-	
-
 	if ((MSG_IS_QUEUED(msginfo->flags) || MSG_IS_DRAFT(msginfo->flags))
 	&& !folder_has_parent_of_type(dest, F_QUEUE)
 	&& !folder_has_parent_of_type(dest, F_DRAFT)) {
-		if (procmsg_remove_special_headers(srcfile, destfile) !=0) {
-			g_free(srcfile);
-			g_free(destfile);
-			return -1;
-		}
+		/* as every msginfo comes from the same folder, it is supposed they
+		 * will either match the preceding condition either all or none.
+		 */
+		remove_special_headers = TRUE;
 	} else if (!(MSG_IS_QUEUED(msginfo->flags) || MSG_IS_DRAFT(msginfo->flags))
 	&& (folder_has_parent_of_type(dest, F_QUEUE)
 	 || folder_has_parent_of_type(dest, F_DRAFT))) {
+		return -1;
+	} 
+
+	prefs = dest->prefs;
+
+	for (cur = msglist; cur; cur = cur->next) {
+		msginfo = (MsgInfo *)cur->data;
+		if (!msginfo)
+			continue;
+		srcfile = procmsg_get_message_file(msginfo);
+		destfile = mh_get_new_msg_filename(dest);
+		if (!destfile) {
+			g_free(srcfile);
+			continue;
+		}
+
+		debug_print("Copying message %s%c%d to %s ...\n",
+			    msginfo->folder->path, G_DIR_SEPARATOR,
+			    msginfo->msgnum, dest->path);
+
+
+		if (remove_special_headers) {
+			if (procmsg_remove_special_headers(srcfile, destfile) !=0) {
+				g_free(srcfile);
+				g_free(destfile);
+				continue;
+			}
+		} else if (copy_file(srcfile, destfile, TRUE) < 0) {
+			FILE_OP_ERROR(srcfile, "copy");
+			g_free(srcfile);
+			g_free(destfile);
+			continue;
+		}
+		if (prefs && prefs->enable_folder_chmod && prefs->folder_chmod) {
+			if (chmod(destfile, prefs->folder_chmod) < 0)
+				FILE_OP_ERROR(destfile, "chmod");
+
+			/* for mark file */
+			filemode = prefs->folder_chmod;
+			if (filemode & S_IRGRP) filemode |= S_IWGRP;
+			if (filemode & S_IROTH) filemode |= S_IWOTH;
+		}
+		if (relation)
+			g_relation_insert(relation, msginfo, GINT_TO_POINTER(dest->last_num+1));
 		g_free(srcfile);
 		g_free(destfile);
-		return -1;
-	} else if (copy_file(srcfile, destfile, TRUE) < 0) {
-		FILE_OP_ERROR(srcfile, "copy");
-		g_free(srcfile);
-		g_free(destfile);
-		return -1;
+		dest->last_num++;
 	}
 
 	dest_need_scan = mh_scan_required(dest->folder, dest);
 	if (!dest_need_scan)
 		dest->mtime = time(NULL);
 
-
-	if (prefs && prefs->enable_folder_chmod && prefs->folder_chmod) {
-		if (chmod(destfile, prefs->folder_chmod) < 0)
-			FILE_OP_ERROR(destfile, "chmod");
-
-		/* for mark file */
-		filemode = prefs->folder_chmod;
-		if (filemode & S_IRGRP) filemode |= S_IWGRP;
-		if (filemode & S_IROTH) filemode |= S_IWOTH;
-	}
-
-	g_free(srcfile);
-	g_free(destfile);
-	dest->last_num++;
-
 	return dest->last_num;
 }
 
@@ -612,7 +658,10 @@ static gchar *mh_item_get_path(Folder *folder, FolderItem *item)
 	folder_path = g_strdup(LOCAL_FOLDER(folder)->rootpath);
 	g_return_val_if_fail(folder_path != NULL, NULL);
 
-        if (folder_path[0] == G_DIR_SEPARATOR) {
+        /* FIXME: [W32] The code below does not correctly merge
+           relative filenames; there should be a function to handle
+           this.  */
+        if ( !is_relative_filename (folder_path) ) {
                 if (item->path)
                         path = g_strconcat(folder_path, G_DIR_SEPARATOR_S,
                                            item->path, NULL);
@@ -792,32 +841,6 @@ static MsgInfo *mh_parse_msg(const gchar *file, FolderItem *item)
 	return msginfo;
 }
 
-#if 0
-static gboolean mh_is_maildir_one(const gchar *path, const gchar *dir)
-{
-	gchar *entry;
-	gboolean result;
-
-	entry = g_strconcat(path, G_DIR_SEPARATOR_S, dir, NULL);
-	result = is_dir_exist(entry);
-	g_free(entry);
-
-	return result;
-}
-
-/*
- * check whether PATH is a Maildir style mailbox.
- * This is the case if the 3 subdir: new, cur, tmp are existing.
- * This functon assumes that entry is an directory
- */
-static gboolean mh_is_maildir(const gchar *path)
-{
-	return mh_is_maildir_one(path, "new") &&
-	       mh_is_maildir_one(path, "cur") &&
-	       mh_is_maildir_one(path, "tmp");
-}
-#endif
-
 static gboolean mh_remove_missing_folder_items_func(GNode *node, gpointer data)
 {
 	FolderItem *item;
@@ -921,15 +944,6 @@ static void mh_scan_tree_recursive(FolderItem *item)
 			FolderItem *new_item = NULL;
 			GNode *node;
 
-#if 0
-			if (mh_is_maildir(entry)) {
-				g_free(entry);
-				g_free(utf8entry);
-				g_free(utf8name);
-				continue;
-			}
-#endif
-
 			node = item->node;
 			for (node = node->children; node != NULL; node = node->next) {
 				FolderItem *cur_item = FOLDER_ITEM(node->data);
