@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,6 +60,7 @@
 #include "utils.h"
 #include "gtkutils.h"
 #include "prefs_common.h"
+#include "procheader.h"
 #include "stock_pixmap.h"
 #include "gtk/gtkvscrollbutton.h"
 
@@ -118,7 +119,8 @@ static void mimeview_launch		(MimeView	*mimeview);
 static void mimeview_open_with		(MimeView	*mimeview);
 static void mimeview_view_file		(const gchar	*filename,
 					 MimeInfo	*partinfo,
-					 const gchar	*cmdline);
+					 const gchar	*cmdline,
+					 MimeView  	*mimeview);
 static gboolean icon_clicked_cb		(GtkWidget 	*button, 
 					 GdkEventButton	*event, 
 					 MimeView 	*mimeview);
@@ -271,6 +273,7 @@ MimeView *mimeview_create(MainWindow *mainwin)
 	gtk_widget_show(vbox);
 	siginfoview = noticeview_create(mainwin);
 	noticeview_hide(siginfoview);
+	noticeview_set_icon_clickable(siginfoview, TRUE);
 	gtk_box_pack_start(GTK_BOX(vbox), mime_notebook, TRUE, TRUE, 0);
 	gtk_box_pack_end(GTK_BOX(vbox), GTK_WIDGET_PTR(siginfoview), FALSE, FALSE, 0);
 
@@ -354,24 +357,52 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 	}
 }
 
+#ifdef USE_PTHREAD
+static void mimeview_check_sig_cancel_now(MimeView *mimeview);
+#endif
+
+static void mimeview_free_mimeinfo(MimeView *mimeview)
+{
+	gboolean defer = FALSE;
+#ifdef USE_PTHREAD
+	defer = (mimeview->check_data != NULL);
+	if (defer)
+		mimeview->check_data->free_after_use = TRUE;
+#endif
+	if (mimeview->mimeinfo != NULL && !defer) {
+		procmime_mimeinfo_free_all(mimeview->mimeinfo);
+		mimeview->mimeinfo = NULL;
+	} else if (defer) {
+#ifdef USE_PTHREAD
+		debug_print("deferring free(mimeinfo) and cancelling check\n");
+		mimeview_check_sig_cancel_now(mimeview);
+#endif
+	}
+}
+
 void mimeview_destroy(MimeView *mimeview)
 {
 	GSList *cur;
 	
 	for (cur = mimeview->viewers; cur != NULL; cur = g_slist_next(cur)) {
 		MimeViewer *viewer = (MimeViewer *) cur->data;
-		gtk_container_remove(GTK_CONTAINER(mimeview->mime_notebook),
-			GTK_WIDGET(viewer->get_widget(viewer)));
 		viewer->destroy_viewer(viewer);
 	}
 	g_slist_free(mimeview->viewers);
 	gtk_target_list_unref(mimeview->target_list);
 
-	procmime_mimeinfo_free_all(mimeview->mimeinfo);
-	g_free(mimeview->file);
-	g_free(mimeview);
-
-	mimeviews = g_slist_remove(mimeviews, mimeview);
+	mimeview_free_mimeinfo(mimeview);
+#ifdef USE_PTHREAD
+	if (mimeview->check_data) {
+		mimeview->check_data->destroy_mimeview = TRUE;
+		debug_print("deferring destroy\n");
+	} else 
+#endif
+	{
+		g_free(mimeview->file);
+		g_free(mimeview);
+		mimeviews = g_slist_remove(mimeviews, mimeview);
+	}
 	
 }
 
@@ -441,7 +472,7 @@ static GtkCTreeNode *mimeview_append_part(MimeView *mimeview,
 	else
 		str[COL_NAME] = (gchar *) get_part_name(partinfo);
 
-	node = gtk_ctree_insert_node(ctree, parent, NULL, str, 0,
+	node = gtk_sctree_insert_node(ctree, parent, NULL, str, 0,
 				     NULL, NULL, NULL, NULL,
 				     FALSE, TRUE);
 	gtk_ctree_node_set_row_data(ctree, node, partinfo);
@@ -571,6 +602,7 @@ gboolean mimeview_show_part(MimeView *mimeview, MimeInfo *partinfo)
 		mimeview->mimeviewer = viewer;
 		mimeview_change_view_type(mimeview, MIMEVIEW_VIEWER);
 	}
+	viewer->mimeview = mimeview;
 	viewer->show_mimepart(viewer, mimeview->file, partinfo);
 
 	return TRUE;
@@ -603,8 +635,16 @@ static void mimeview_change_view_type(MimeView *mimeview, MimeViewType type)
 
 void mimeview_clear(MimeView *mimeview)
 {
-	GtkCList *clist = GTK_CLIST(mimeview->ctree);
+	GtkCList *clist = NULL;
+	
+	if (!mimeview)
+		return;
 
+	if (g_slist_find(mimeviews, mimeview) == NULL)
+		return;
+	
+	clist = GTK_CLIST(mimeview->ctree);
+	
 	noticeview_hide(mimeview->siginfoview);
 
 	gtk_clist_clear(clist);
@@ -612,8 +652,8 @@ void mimeview_clear(MimeView *mimeview)
 	if (mimeview->mimeviewer != NULL)
 		mimeview->mimeviewer->clear_viewer(mimeview->mimeviewer);
 
-	if (mimeview->mimeinfo != NULL)
-		procmime_mimeinfo_free_all(mimeview->mimeinfo);
+	mimeview_free_mimeinfo(mimeview);
+
 	mimeview->mimeinfo = NULL;
 
 	mimeview->opened = NULL;
@@ -622,38 +662,47 @@ void mimeview_clear(MimeView *mimeview)
 	mimeview->file = NULL;
 
 	icon_list_clear(mimeview);
+	mimeview_change_view_type(mimeview, MIMEVIEW_TEXT);
 }
 
 static void check_signature_cb(GtkWidget *widget, gpointer user_data);
+void mimeview_check_signature(MimeView *mimeview);
 static void display_full_info_cb(GtkWidget *widget, gpointer user_data);
 
-static void update_signature_noticeview(MimeView *mimeview, MimeInfo *mimeinfo)
+static void update_signature_noticeview(MimeView *mimeview, MimeInfo *mimeinfo, 
+					gboolean special, SignatureStatus code)
 {
 	gchar *text = NULL, *button_text = NULL;
 	void  *func = NULL;
 	StockPixmap icon = STOCK_PIXMAP_PRIVACY_SIGNED;
-
+	SignatureStatus mycode = SIGNATURE_UNCHECKED;
+	
 	g_return_if_fail(mimeview != NULL);
 	g_return_if_fail(mimeinfo != NULL);
 	
-	switch (privacy_mimeinfo_get_sig_status(mimeinfo)) {
+	if (special)
+		mycode = code;
+	else 
+		mycode = privacy_mimeinfo_get_sig_status(mimeinfo);
+
+	switch (mycode) {
 	case SIGNATURE_UNCHECKED:
-		button_text = _("Check");
+		button_text = _("Check signature");
 		func = check_signature_cb;
 		icon = STOCK_PIXMAP_PRIVACY_SIGNED;
 		break;
 	case SIGNATURE_OK:
-		button_text = _("Full info");
+		button_text = _("View full information");
 		func = display_full_info_cb;
 		icon = STOCK_PIXMAP_PRIVACY_PASSED;
 		break;
 	case SIGNATURE_WARN:
-		button_text = _("Full info");
+		button_text = _("View full information");
 		func = display_full_info_cb;
 		icon = STOCK_PIXMAP_PRIVACY_WARN;
 		break;
 	case SIGNATURE_INVALID:
-		button_text = _("Full info");
+		button_text = _("View full information");
 		func = display_full_info_cb;
 		icon = STOCK_PIXMAP_PRIVACY_FAILED;
 		break;
@@ -661,29 +710,258 @@ static void update_signature_noticeview(MimeView *mimeview, MimeInfo *mimeinfo)
 		button_text = _("Check again");
 		func = check_signature_cb;
 		icon = STOCK_PIXMAP_PRIVACY_UNKNOWN;
+	case SIGNATURE_CHECK_TIMEOUT:
+		button_text = _("Check again");
+		func = check_signature_cb;
+		icon = STOCK_PIXMAP_PRIVACY_UNKNOWN;
 	default:
 		break;
 	}
-	text = privacy_mimeinfo_sig_info_short(mimeinfo);
+	if (mycode == SIGNATURE_UNCHECKED) {
+		gchar *tmp = privacy_mimeinfo_sig_info_short(mimeinfo);
+		text = g_strdup_printf("%s %s",
+			tmp, _("Click the icon or hit 'C' to check it."));
+		g_free(tmp);
+	} else if (mycode != SIGNATURE_CHECK_TIMEOUT) {
+		text = privacy_mimeinfo_sig_info_short(mimeinfo);
+	} else if (mycode == SIGNATURE_CHECK_TIMEOUT) {
+		text = g_strdup(_("Timeout checking the signature. Click the icon or hit 'C' to try again."));
+	}
+
 	noticeview_set_text(mimeview->siginfoview, text);
+	gtk_label_set_selectable(GTK_LABEL(mimeview->siginfoview->text), TRUE);
+
 	g_free(text);
-	noticeview_set_button_text(mimeview->siginfoview, button_text);
+	noticeview_set_button_text(mimeview->siginfoview, NULL);
 	noticeview_set_button_press_callback(
 		mimeview->siginfoview,
 		G_CALLBACK(func),
 		(gpointer) mimeview);
 	noticeview_set_icon(mimeview->siginfoview, icon);
+	noticeview_set_tooltip(mimeview->siginfoview, button_text);
 }
 
+#ifdef USE_PTHREAD
+
+/* reset all thread stuff, and do the cleanups we've been left to do */
+static void mimeview_check_data_reset(MimeView *mimeview)
+{
+	if (!mimeview->check_data)
+		return;
+
+	if (mimeview->check_data->free_after_use) {
+		debug_print("freeing deferred mimeinfo\n");
+		procmime_mimeinfo_free_all(mimeview->check_data->siginfo);
+	}
+	if (mimeview->check_data->destroy_mimeview) {
+		debug_print("freeing deferred mimeview\n");
+		g_free(mimeview->file);
+		g_free(mimeview);
+		mimeviews = g_slist_remove(mimeviews, mimeview);
+	}
+
+	g_free(mimeview->check_data);
+	mimeview->check_data = NULL;
+}
+
+/* GUI update once the checker thread is done or killed */
+static gboolean mimeview_check_sig_thread_cb(void *data)
+{
+	MimeView *mimeview = (MimeView *) data;
+	MimeInfo *mimeinfo = mimeview->siginfo;
+
+	debug_print("mimeview_check_sig_thread_cb\n");
+	
+	if (mimeinfo == NULL) {
+		/* message changed !? */
+		g_warning("no more siginfo!\n");
+		goto end;
+	}
+	
+	if (!mimeview->check_data) {
+		g_warning("nothing to check\n");
+		return FALSE;
+	}
+
+	if (mimeview->check_data->siginfo != mimeinfo) {
+		/* message changed !? */
+		g_warning("different siginfo!\n");
+		goto end;
+	}
+
+	if (mimeview->check_data->destroy_mimeview ||
+	    mimeview->check_data->free_after_use) {
+	    	debug_print("not bothering, we're changing message\n"); 
+		goto end;
+	}
+	
+	/* update status */
+	if (mimeview->check_data->timeout) 
+		update_signature_noticeview(mimeview, mimeview->siginfo, 
+			TRUE, SIGNATURE_CHECK_TIMEOUT);
+	else
+		update_signature_noticeview(mimeview, mimeview->siginfo, 
+			FALSE, 0);
+	icon_list_clear(mimeview);
+	icon_list_create(mimeview, mimeview->mimeinfo);
+	
+end:
+	mimeview_check_data_reset(mimeview);
+	return FALSE;
+}
+
+/* sig checker thread */
+static void *mimeview_check_sig_worker_thread(void *data)
+{
+	MimeView *mimeview = (MimeView *)data;
+	MimeInfo *mimeinfo = mimeview->siginfo;
+	
+	debug_print("checking...\n");
+
+	if (!mimeview->check_data)
+		return NULL;
+
+	if (mimeinfo && mimeinfo == mimeview->check_data->siginfo)
+		privacy_mimeinfo_check_signature(mimeinfo);
+	else {
+		/* that's strange! we changed message without 
+		 * getting killed. */
+		g_warning("different siginfo!\n");
+		mimeview_check_data_reset(mimeview);
+		return NULL;
+	}
+
+	/* use g_timeout so that GUI updates is done from the
+	 * correct thread */
+	g_timeout_add(0,mimeview_check_sig_thread_cb,mimeview);
+	
+	return NULL;
+}
+
+/* killer thread - acts when the checker didn't work fast
+ * enough. */
+static void *mimeview_check_sig_cancel_thread(void *data)
+{
+	MimeView *mimeview = (MimeView *)data;
+	
+	if (!mimeview->check_data)
+		return NULL; /* nothing to kill ! */
+
+	/* wait for a few seconds... */
+	debug_print("waiting a while\n");
+
+	sleep(5);
+	
+	if (!mimeview->check_data)
+		return NULL; /* nothing to kill, it's done in time :) */
+	
+	/* too late, go away checker thread */
+	debug_print("killing checker thread\n");
+	pthread_cancel(mimeview->check_data->th);
+	
+	/* tell upstream it was a timeout */
+	mimeview->check_data->timeout = TRUE;
+	/* use g_timeout so that GUI updates is done from the
+	 * correct thread */
+	g_timeout_add(0,mimeview_check_sig_thread_cb,mimeview);
+
+	return NULL;
+}
+
+/* get rid of the checker thread right now - used when changing the
+ * displayed message for example. */
+static void mimeview_check_sig_cancel_now(MimeView *mimeview)
+{
+	if (!mimeview->check_data)
+		return;
+	debug_print("killing checker thread NOW\n");
+	pthread_cancel(mimeview->check_data->th);
+
+	/* tell upstream it was a timeout */
+	mimeview->check_data->timeout = TRUE;
+	mimeview_check_sig_thread_cb(mimeview);
+	return;
+}
+
+/* creates a thread to check the signature, and a second one
+ * to kill the first one after a timeout */
+static void mimeview_check_sig_in_thread(MimeView *mimeview)
+{
+	pthread_t th, th2;
+	pthread_attr_t detach, detach2;
+	
+	if (mimeview->check_data) {
+		g_warning("already checking it");
+		return;
+	}
+	
+	mimeview->check_data = g_new0(SigCheckData, 1);
+	mimeview->check_data->siginfo = mimeview->siginfo;
+	debug_print("creating thread\n");
+
+	pthread_attr_init(&detach);
+	pthread_attr_setdetachstate(&detach, TRUE);
+
+	pthread_attr_init(&detach2);
+	pthread_attr_setdetachstate(&detach2, TRUE);
+
+	/* create the checker thread */
+	if (pthread_create(&th, &detach, 
+			mimeview_check_sig_worker_thread, 
+			mimeview) != 0) {
+		/* arh. We'll do it synchronously. */
+		g_warning("can't create thread");
+		g_free(mimeview->check_data);
+		mimeview->check_data = NULL;
+		return;
+	} else 
+		mimeview->check_data->th = th;
+
+	/* create the killer thread */
+	pthread_create(&th2, &detach2, 
+			mimeview_check_sig_cancel_thread, 
+			mimeview);
+}
+#endif
+
 static void check_signature_cb(GtkWidget *widget, gpointer user_data)
 {
 	MimeView *mimeview = (MimeView *) user_data;
 	MimeInfo *mimeinfo = mimeview->siginfo;
 	
-	privacy_mimeinfo_check_signature(mimeinfo);
-	update_signature_noticeview(mimeview, mimeview->siginfo);
-	icon_list_clear(mimeview);
-	icon_list_create(mimeview, mimeview->mimeinfo);
+	if (mimeinfo == NULL)
+		return;
+#ifdef USE_PTHREAD
+	if (mimeview->check_data)
+		return;
+#endif
+	noticeview_set_text(mimeview->siginfoview, _("Checking signature..."));
+	GTK_EVENTS_FLUSH();
+#if (defined USE_PTHREAD && defined __GLIBC__ && (__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 3)))
+	/* let's do it non-blocking */
+	mimeview_check_sig_in_thread(mimeview);
+	if (!mimeview->check_data) /* let's check syncronously */
+#endif
+	{
+		debug_print("checking without thread\n");
+		privacy_mimeinfo_check_signature(mimeinfo);
+		update_signature_noticeview(mimeview, mimeview->siginfo, FALSE, 0);
+		icon_list_clear(mimeview);
+		icon_list_create(mimeview, mimeview->mimeinfo);
+	}
+}
+
+void mimeview_check_signature(MimeView *mimeview)
+{
+	check_signature_cb(NULL, mimeview);	
+}
+
+static void redisplay_email(GtkWidget *widget, gpointer user_data)
+{
+	MimeView *mimeview = (MimeView *) user_data;
+	GtkCTreeNode *node = mimeview->opened;
+	mimeview->opened = NULL;
+	mimeview_selected(GTK_CTREE(mimeview->ctree), node, 0, mimeview);
 }
 
 static void display_full_info_cb(GtkWidget *widget, gpointer user_data)
@@ -695,15 +973,34 @@ static void display_full_info_cb(GtkWidget *widget, gpointer user_data)
 	textview_set_text(mimeview->textview, siginfo);
 	g_free(siginfo);
 	noticeview_set_button_text(mimeview->siginfoview, NULL);
+	noticeview_set_button_press_callback(
+		mimeview->siginfoview,
+		G_CALLBACK(redisplay_email),
+		(gpointer) mimeview);
+	noticeview_set_tooltip(mimeview->siginfoview, _("Go back to email"));
 }
 
 static void update_signature_info(MimeView *mimeview, MimeInfo *selected)
 {
 	MimeInfo *siginfo;
-
+	MimeInfo *first_text;
+	
 	g_return_if_fail(mimeview != NULL);
 	g_return_if_fail(selected != NULL);
 	
+	if (selected->type == MIMETYPE_MESSAGE 
+	&&  !g_ascii_strcasecmp(selected->subtype, "rfc822")) {
+		/* if the first text part is signed, check that */
+		first_text = selected;
+		while (first_text && first_text->type != MIMETYPE_TEXT) {
+			first_text = procmime_mimeinfo_next(first_text);
+		}
+		if (first_text) {
+			update_signature_info(mimeview, first_text);
+			return;
+		}	
+	}
+
 	siginfo = selected;
 	while (siginfo != NULL) {
 		if (privacy_mimeinfo_is_signed(siginfo))
@@ -717,7 +1014,7 @@ static void update_signature_info(MimeView *mimeview, MimeInfo *selected)
 		return;
 	}
 	
-	update_signature_noticeview(mimeview, siginfo);
+	update_signature_noticeview(mimeview, siginfo, FALSE, 0);
 	noticeview_show(mimeview->siginfoview);
 }
 
@@ -725,7 +1022,6 @@ static void mimeview_selected(GtkCTree *ctree, GtkCTreeNode *node, gint column,
 			      MimeView *mimeview)
 {
 	MimeInfo *partinfo;
-
 	if (mimeview->opened == node) return;
 	mimeview->opened = node;
 	gtk_ctree_node_moveto(ctree, node, -1, 0.5, 0);
@@ -753,9 +1049,30 @@ static void mimeview_selected(GtkCTree *ctree, GtkCTreeNode *node, gint column,
 		
 			break;
 		default:
-			mimeview->textview->default_text = TRUE;	
+			mimeview->textview->default_text = TRUE;
 			mimeview_change_view_type(mimeview, MIMEVIEW_TEXT);
+			textview_clear(mimeview->textview);
 			textview_show_mime_part(mimeview->textview, partinfo);
+			
+#if 0 /* this sucks. I'll do better. */
+			val = alertpanel_full(_("Unknown part type"), 
+					      _("The type of this part is unknown. What would you like "
+			        		"to do with it?"),
+					      GTK_STOCK_SAVE, GTK_STOCK_OPEN, _("Display as text"),
+					      FALSE, NULL, ALERT_WARNING, G_ALERTALTERNATE);
+			switch (val) {
+				case G_ALERTDEFAULT:
+					mimeview_save_as(mimeview);
+					break;
+				case G_ALERTALTERNATE:
+					mimeview_open_with(mimeview);
+					break;
+				case G_ALERTOTHER:
+					mimeview_display_as_text(mimeview);
+					break;
+				default:
+			}
+#endif
 			break;
 		}
 	}
@@ -928,6 +1245,11 @@ static gint mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		KEY_PRESS_EVENT_STOP();
 		mimeview_open_with(mimeview);
 		return TRUE;
+	case GDK_c:
+		BREAK_ON_MODIFIER_KEY();
+		KEY_PRESS_EVENT_STOP();
+		mimeview_check_signature(mimeview);
+		return TRUE;
 	default:
 		break;
 	}
@@ -945,7 +1267,7 @@ static void mimeview_drag_data_get(GtkWidget	    *widget,
 				   guint	     time,
 				   MimeView	    *mimeview)
 {
-	gchar *filename, *uriname, *tmp;
+	gchar *filename = NULL, *uriname, *tmp;
 	MimeInfo *partinfo;
 
 	if (!mimeview->opened) return;
@@ -954,21 +1276,51 @@ static void mimeview_drag_data_get(GtkWidget	    *widget,
 	partinfo = mimeview_get_selected_part(mimeview);
 	if (!partinfo) return;
 
-	filename = g_path_get_basename(get_part_name(partinfo));
-	if (*filename == '\0') return;
+	if (strlen(get_part_name(partinfo)) > 0) {
+		filename = g_path_get_basename(get_part_name(partinfo));
+		if (*filename == '\0') return;
+	} else if (partinfo->type == MIMETYPE_MESSAGE 
+		   && !g_ascii_strcasecmp(partinfo->subtype, "rfc822")) {
+		gchar *name = NULL;
+		GPtrArray *headers = NULL;
+		FILE *fp;
 
-	tmp = filename;
+		fp = g_fopen(partinfo->data.filename, "rb");
+		fseek(fp, partinfo->offset, SEEK_SET);
+		headers = procheader_get_header_array_asis(fp);
+		if (headers) {
+			gint i;
+			for (i = 0; i < headers->len; i++) {
+				Header *header = g_ptr_array_index(headers, i);
+				if (procheader_headername_equal(header->name, "Subject")) {
+					unfold_line(header->body);
+					name = g_strconcat(header->body, ".txt", NULL);
+					subst_for_filename(name);
+				}
+			}
+			procheader_header_array_destroy(headers);
+		}
+		fclose(fp);
+		if (name)
+			filename = g_path_get_basename(name);
+		g_free(name);
+	}
+	if (filename == NULL)
+		filename = g_path_get_basename("Unnamed part");
+		
+
+	tmp = g_filename_from_utf8(filename, -1, NULL, NULL, NULL);
 	
 	filename = g_strconcat(get_mime_tmp_dir(), G_DIR_SEPARATOR_S,
-			       filename, NULL);
+			       tmp, NULL);
 
 	g_free(tmp);
-	
+
 	if (procmime_get_part(filename, partinfo) < 0)
 		alertpanel_error
 			(_("Can't save the part of multipart message."));
+	uriname = g_strconcat("file://", filename, "\r\n", NULL);
 
-	uriname = g_strconcat("file://", filename, NULL);
 	gtk_selection_data_set(selection_data, selection_data->target, 8,
 			       uriname, strlen(uriname));
 
@@ -1006,7 +1358,9 @@ gchar *mimeview_get_filename_for_part(MimeInfo *partinfo,
 		 ? &filename[1] : filename, NULL);
 
 	g_free(filename);
-	return fullname;
+	filename = conv_filename_from_utf8(fullname);
+	g_free(fullname);
+	return filename;
 }
 
 /**
@@ -1027,13 +1381,20 @@ static gboolean mimeview_write_part(const gchar *filename,
 	if (is_file_exist(filename)) {
 		AlertValue aval;
 		gchar *res;
+		gchar *tmp;
+		
+		if (!g_utf8_validate(filename, -1, NULL))
+			tmp = conv_filename_to_utf8(filename);
+		else 
+			tmp = g_strdup(filename);
 		
 		res = g_strdup_printf(_("Overwrite existing file '%s'?"),
-				      filename);
-		aval = alertpanel(_("Overwrite"), res, GTK_STOCK_OK, 
-				  GTK_STOCK_CANCEL, NULL);
+				      tmp);
+		g_free(tmp);
+		aval = alertpanel(_("Overwrite"), res, GTK_STOCK_CANCEL, 
+				  GTK_STOCK_OK, NULL);
 		g_free(res);					  
-		if (G_ALERTDEFAULT != aval) return FALSE;
+		if (G_ALERTALTERNATE != aval) return FALSE;
 	}
 
 	if (procmime_get_part(filename, partinfo) < 0) {
@@ -1219,7 +1580,7 @@ static void mimeview_launch(MimeView *mimeview)
 		alertpanel_error
 			(_("Can't save the part of multipart message."));
 	else
-		mimeview_view_file(filename, partinfo, NULL);
+		mimeview_view_file(filename, partinfo, NULL, mimeview);
 
 	g_free(filename);
 }
@@ -1229,7 +1590,10 @@ static void mimeview_open_with(MimeView *mimeview)
 	MimeInfo *partinfo;
 	gchar *filename;
 	gchar *cmd;
+	gchar *mime_command = NULL;
+	gchar *content_type = NULL;
 
+	if (!mimeview) return;
 	if (!mimeview->opened) return;
 	if (!mimeview->file) return;
 
@@ -1256,15 +1620,27 @@ static void mimeview_open_with(MimeView *mimeview)
 		prefs_common.mime_open_cmd_history =
 			add_history(NULL, prefs_common.mime_open_cmd);
 
+	if ((partinfo->type == MIMETYPE_APPLICATION) &&
+            (!g_ascii_strcasecmp(partinfo->subtype, "octet-stream"))) {
+	    	/* guess content-type from filename */
+	    	content_type = procmime_get_mime_type(filename);
+	} 
+	if (content_type == NULL) {
+		content_type = procmime_get_content_type_str(partinfo->type,
+			partinfo->subtype);
+	}
+	mime_command = mailcap_get_command_for_type(content_type);
+	g_free(content_type);
 	cmd = input_dialog_combo
 		(_("Open with"),
 		 _("Enter the command line to open file:\n"
 		   "('%s' will be replaced with file name)"),
-		 prefs_common.mime_open_cmd,
+		 mime_command ? mime_command : prefs_common.mime_open_cmd,
 		 prefs_common.mime_open_cmd_history,
 		 TRUE);
+	g_free(mime_command);
 	if (cmd) {
-		mimeview_view_file(filename, partinfo, cmd);
+		mimeview_view_file(filename, partinfo, cmd, mimeview);
 		g_free(prefs_common.mime_open_cmd);
 		prefs_common.mime_open_cmd = cmd;
 		prefs_common.mime_open_cmd_history =
@@ -1275,7 +1651,7 @@ static void mimeview_open_with(MimeView *mimeview)
 }
 
 static void mimeview_view_file(const gchar *filename, MimeInfo *partinfo,
-			       const gchar *cmdline)
+			       const gchar *cmdline, MimeView *mimeview)
 {
 	static gchar *default_image_cmdline = DEFAULT_IMAGE_VIEWER_CMD;
 	static gchar *default_audio_cmdline = DEFAULT_AUDIO_PLAYER_CMD;
@@ -1292,6 +1668,7 @@ static void mimeview_view_file(const gchar *filename, MimeInfo *partinfo,
 		def_cmd = NULL;
 	} else if (MIMETYPE_APPLICATION == partinfo->type &&
 		   !g_ascii_strcasecmp(partinfo->subtype, "octet-stream")) {
+		mimeview_open_with(mimeview);
 		return;
 	} else if (MIMETYPE_IMAGE == partinfo->type) {
 		cmd = prefs_common.mime_image_viewer;
@@ -1321,11 +1698,14 @@ static void mimeview_view_file(const gchar *filename, MimeInfo *partinfo,
 			g_warning("MIME viewer command line is invalid: '%s'", cmd);
 		if (def_cmd)
 			g_snprintf(buf, sizeof(buf), def_cmd, filename);
-		else
+		else {
+			mimeview_open_with(mimeview);
 			return;
+		}
 	}
 
-	execute_command_line(buf, TRUE);
+	if (execute_command_line(buf, TRUE) != 0)
+		mimeview_open_with(mimeview);
 }
 
 void mimeview_register_viewer_factory(MimeViewerFactory *factory)
@@ -1481,6 +1861,11 @@ static gint icon_key_pressed(GtkWidget *button, GdkEventKey *event,
 		KEY_PRESS_EVENT_STOP();
 		mimeview_open_with(mimeview);
 		return TRUE;
+	case GDK_c:
+		BREAK_ON_MODIFIER_KEY();
+		KEY_PRESS_EVENT_STOP();
+		mimeview_check_signature(mimeview);
+		return TRUE;
 	default:
 		break;
 	}
@@ -1574,6 +1959,7 @@ static void icon_list_append_icon (MimeView *mimeview, MimeInfo *mimeinfo)
 		switch (privacy_mimeinfo_get_sig_status(siginfo)) {
 		case SIGNATURE_UNCHECKED:
 		case SIGNATURE_CHECK_FAILED:
+		case SIGNATURE_CHECK_TIMEOUT:
 			pixmap = stock_pixmap_widget_with_overlay(mimeview->mainwin->window, stockp,
 			    STOCK_PIXMAP_PRIVACY_EMBLEM_SIGNED, OVERLAY_BOTTOM_RIGHT, 6, 3);
 			break;
@@ -1635,10 +2021,14 @@ static void icon_list_append_icon (MimeView *mimeview, MimeInfo *mimeinfo)
 	gtk_tooltips_set_tip(mimeview->tooltips, button, tip, NULL);
 	g_free(tip);
 	gtk_widget_show_all(button);
+	gtk_drag_source_set(button, GDK_BUTTON1_MASK|GDK_BUTTON3_MASK, 
+			    mimeview_mime_types, 1, GDK_ACTION_COPY);
 	g_signal_connect(G_OBJECT(button), "button_release_event", 
 			 G_CALLBACK(icon_clicked_cb), mimeview);
 	g_signal_connect(G_OBJECT(button), "key_press_event", 
 			 G_CALLBACK(icon_key_pressed), mimeview);
+	g_signal_connect(G_OBJECT(button), "drag_data_get",
+			 G_CALLBACK(mimeview_drag_data_get), mimeview);
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
 
 }
