@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto & The Sylpheed Claws Team
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto & The Sylpheed Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "defs.h"
@@ -422,12 +422,7 @@ MsgCache *msgcache_read_cache(FolderItem *item, const gchar *cache_file)
 	gchar *ref = NULL;
 	guint memusage = 0;
 	guint tmp_len = 0;
-#if 0
-	struct timeval start;
-	struct timeval end;
-	struct timeval diff;
-	gettimeofday(&start, NULL);
-#endif
+	
 	g_return_val_if_fail(cache_file != NULL, NULL);
 	g_return_val_if_fail(item != NULL, NULL);
 
@@ -461,15 +456,9 @@ MsgCache *msgcache_read_cache(FolderItem *item, const gchar *cache_file)
 	if (srccharset == NULL || dstcharset == NULL) {
 		conv = NULL;
 	} else if (strcmp(srccharset, dstcharset) == 0) {
-		StrdupConverter *strdupconv;
+		debug_print("using Noop Converter\n");
 
-		debug_print("using StrdupConverter\n");
-
-		strdupconv = g_new0(StrdupConverter, 1);
-		strdupconv->converter.convert = strconv_strdup_convert;
-		strdupconv->converter.free = NULL;
-
-		conv = (StringConverter *) strdupconv;
+		conv = NULL;
 	} else {
 		CharsetConverter *charsetconv;
 
@@ -554,11 +543,6 @@ MsgCache *msgcache_read_cache(FolderItem *item, const gchar *cache_file)
 
 	debug_print("done. (%d items read)\n", g_hash_table_size(cache->msgnum_table));
 	debug_print("Cache size: %d messages, %d byte\n", g_hash_table_size(cache->msgnum_table), cache->memusage);
-#if 0
-	gettimeofday(&end, NULL);
-	timersub(&end, &start, &diff);
-	printf("spent %d seconds %d usages %d;%d\n", diff.tv_sec, diff.tv_usec, cache->memusage, memusage);
-#endif
 	return cache;
 }
 
