@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -51,6 +51,8 @@
 #include "progressindicator.h"
 #include "remotefolder.h"
 #include "alertpanel.h"
+#include "inc.h"
+
 #if USE_OPENSSL
 #  include "ssl.h"
 #endif
@@ -69,6 +71,7 @@ struct _NewsFolder
 	RemoteFolder rfolder;
 
 	gboolean use_auth;
+	gboolean lock_count;
 };
 
 static void news_folder_init		 (Folder	*folder,
@@ -131,7 +134,12 @@ gint news_post_stream			 (Folder 	*folder,
 static gchar *news_folder_get_path	 (Folder	*folder);
 gchar *news_item_get_path		 (Folder	*folder,
 					  FolderItem	*item);
-
+static void news_synchronise		 (FolderItem	*item);
+static int news_dummy_remove		 (Folder 	*folder, 
+					  FolderItem 	*item, 
+					  gint 		 msgnum);
+static gint news_remove_folder		 (Folder	*folder,
+					  FolderItem	*item);
 static FolderClass news_class;
 
 FolderClass *news_get_class(void)
@@ -149,16 +157,46 @@ FolderClass *news_get_class(void)
 		news_class.item_get_path = news_item_get_path;
 		news_class.get_num_list = news_get_num_list;
 		news_class.scan_required = news_scan_required;
+		news_class.remove_folder = news_remove_folder;
 
 		/* Message functions */
 		news_class.get_msginfo = news_get_msginfo;
 		news_class.get_msginfos = news_get_msginfos;
 		news_class.fetch_msg = news_fetch_msg;
+		news_class.synchronise = news_synchronise;
+		news_class.remove_msg = news_dummy_remove;
 	};
 
 	return &news_class;
 }
 
+static int news_dummy_remove		 (Folder 	*folder, 
+					  FolderItem 	*item, 
+					  gint 		 msgnum)
+{
+	debug_print("doing nothing on purpose\n");
+	return 0;
+}
+
+static void news_folder_lock(NewsFolder *folder)
+{
+	folder->lock_count++;
+}
+
+static void news_folder_unlock(NewsFolder *folder)
+{
+	if (folder->lock_count > 0)
+		folder->lock_count--;
+}
+
+int news_folder_locked(Folder *folder)
+{
+	if (folder == NULL)
+		return 0;
+
+	return NEWS_FOLDER(folder)->lock_count;
+}
+
 static Folder *news_folder_new(const gchar *name, const gchar *path)
 {
 	Folder *folder;
@@ -241,6 +279,20 @@ static Session *news_session_new_for_folder(Folder *folder)
 	session = news_session_new(ac->nntp_server, port, userid, passwd,
 				   ac->ssl_nntp);
 #else
+	if (ac->ssl_nntp != SSL_NONE) {
+		if (alertpanel_full(_("Insecure connection"),
+			_("This connection is configured to be secured "
+			  "using SSL, but SSL is not available in this "
+			  "build of Sylpheed-Claws. \n\n"
+			  "Do you want to continue connecting to this "
+			  "server? The communication would not be "
+			  "secure."),
+			  _("Con_tinue connecting"), 
+			  GTK_STOCK_CANCEL, NULL,
+			  FALSE, NULL, ALERT_WARNING,
+			  G_ALERTALTERNATE) != G_ALERTDEFAULT)
+			return NULL;
+	}
 	port = ac->set_nntpport ? ac->nntpport : NNTP_PORT;
 	session = news_session_new(ac->nntp_server, port, userid, passwd);
 #endif
@@ -260,7 +312,10 @@ static NNTPSession *news_session_get(Folder *folder)
 	g_return_val_if_fail(FOLDER_CLASS(folder) == &news_class, NULL);
 	g_return_val_if_fail(folder->account != NULL, NULL);
 
-	if (prefs_common.work_offline && !news_gtk_should_override()) {
+	if (prefs_common.work_offline && 
+	    !inc_offline_should_override(
+		_("Sylpheed-Claws needs network access in order "
+		  "to access the News server."))) {
 		return NULL;
 	}
 
@@ -339,7 +394,7 @@ static gchar *news_fetch_msg(Folder *folder, FolderItem *item, gint num)
 		g_free(filename);
 		return NULL;
 	}
-
+	GTK_EVENTS_FLUSH();
 	return filename;
 }
 
@@ -672,7 +727,7 @@ static MsgInfo *news_parse_xover(const gchar *xover_str)
 		while (ref_tokens[i]) {
 			gchar *cur_ref = ref_tokens[i];
 			msginfo->references = references_list_append(msginfo->references, 
-					g_strdup(cur_ref));
+					cur_ref);
 			i++;
 		}
 		g_strfreev(ref_tokens);
@@ -819,10 +874,13 @@ gint news_get_num_list(Folder *folder, FolderItem *item, GSList **msgnum_list, g
 	g_return_val_if_fail(session != NULL, -1);
 
 	*old_uids_valid = TRUE;
+	
+	news_folder_lock(NEWS_FOLDER(item->folder));
 
 	ok = news_select_group(session, item->path, &num, &first, &last);
 	if (ok != NN_SUCCESS) {
 		log_warning(_("can't set group: %s\n"), item->path);
+		news_folder_unlock(NEWS_FOLDER(item->folder));
 		return -1;
 	}
 
@@ -843,7 +901,7 @@ gint news_get_num_list(Folder *folder, FolderItem *item, GSList **msgnum_list, g
 		remove_numbered_files(dir, 1, first - 1);
 	}
 	g_free(dir);
-
+	news_folder_unlock(NEWS_FOLDER(item->folder));
 	return nummsgs;
 }
 
@@ -871,17 +929,22 @@ MsgInfo *news_get_msginfo(Folder *folder, FolderItem *item, gint num)
 	log_message(_("getting xover %d in %s...\n"),
 		    num, item->path);
 	ok = nntp_xover(session, num, num);
+	
+	news_folder_lock(NEWS_FOLDER(item->folder));
+	
 	if (ok != NN_SUCCESS) {
 		log_warning(_("can't get xover\n"));
 		if (ok == NN_SOCKET) {
 			session_destroy(SESSION(session));
 			REMOTE_FOLDER(item->folder)->session = NULL;
 		}
+		news_folder_unlock(NEWS_FOLDER(item->folder));
 		return NULL;
 	}
 	
 	if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
 		log_warning(_("error occurred while getting xover.\n"));
+		news_folder_unlock(NEWS_FOLDER(item->folder));
 		return NULL;
 	}
 	
@@ -892,9 +955,11 @@ MsgInfo *news_get_msginfo(Folder *folder, FolderItem *item, gint num)
 
 	READ_TO_LISTEND("xover");
 
-	if(!msginfo)
+	if(!msginfo) {
+		news_folder_unlock(NEWS_FOLDER(item->folder));
 		return NULL;
-	
+	}
+
 	msginfo->folder = item;
 	msginfo->flags.perm_flags = MSG_NEW|MSG_UNREAD;
 	msginfo->flags.tmp_flags = MSG_NEWS;
@@ -907,11 +972,13 @@ MsgInfo *news_get_msginfo(Folder *folder, FolderItem *item, gint num)
 			session_destroy(SESSION(session));
 			REMOTE_FOLDER(item->folder)->session = NULL;
 		}
+		news_folder_unlock(NEWS_FOLDER(item->folder));
 		return msginfo;
 	}
 
 	if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
 		log_warning(_("error occurred while getting xhdr.\n"));
+		news_folder_unlock(NEWS_FOLDER(item->folder));
 		return msginfo;
 	}
 
@@ -926,18 +993,20 @@ MsgInfo *news_get_msginfo(Folder *folder, FolderItem *item, gint num)
 			session_destroy(SESSION(session));
 			REMOTE_FOLDER(item->folder)->session = NULL;
 		}
+		news_folder_unlock(NEWS_FOLDER(item->folder));
 		return msginfo;
 	}
 
 	if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
 		log_warning(_("error occurred while getting xhdr.\n"));
+		news_folder_unlock(NEWS_FOLDER(item->folder));
 		return msginfo;
 	}
 
 	msginfo->cc = news_parse_xhdr(buf, msginfo);
 
 	READ_TO_LISTEND("xhdr (cc)");
-
+	news_folder_unlock(NEWS_FOLDER(item->folder));
 	return msginfo;
 }
 
@@ -965,9 +1034,12 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 		return NULL;
 	}
 
+	news_folder_lock(NEWS_FOLDER(item->folder));
+	
 	for (;;) {
 		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
 			log_warning(_("error occurred while getting xover.\n"));
+			news_folder_unlock(NEWS_FOLDER(item->folder));
 			return newlist;
 		}
 		count++;
@@ -1004,6 +1076,7 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 			session_destroy(SESSION(session));
 			REMOTE_FOLDER(item->folder)->session = NULL;
 		}
+		news_folder_unlock(NEWS_FOLDER(item->folder));
 		return newlist;
 	}
 
@@ -1012,6 +1085,7 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 	for (;;) {
 		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
 			log_warning(_("error occurred while getting xhdr.\n"));
+			news_folder_unlock(NEWS_FOLDER(item->folder));
 			return newlist;
 		}
 		count++;
@@ -1039,6 +1113,7 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 			session_destroy(SESSION(session));
 			REMOTE_FOLDER(item->folder)->session = NULL;
 		}
+		news_folder_unlock(NEWS_FOLDER(item->folder));
 		return newlist;
 	}
 
@@ -1047,6 +1122,7 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 	for (;;) {
 		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
 			log_warning(_("error occurred while getting xhdr.\n"));
+			news_folder_unlock(NEWS_FOLDER(item->folder));
 			return newlist;
 		}
 		count++;
@@ -1066,6 +1142,7 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 
 		llast = llast->next;
 	}
+	news_folder_unlock(NEWS_FOLDER(item->folder));
 
 	session_set_access_time(SESSION(session));
 
@@ -1096,6 +1173,9 @@ GSList *news_get_msginfos(Folder *folder, FolderItem *item, GSList *msgnum_list)
 
 	first = GPOINTER_TO_INT(tmp_msgnum_list->data);
 	last = first;
+	
+	news_folder_lock(NEWS_FOLDER(item->folder));
+	
 	for(elem = g_slist_next(tmp_msgnum_list); elem != NULL; elem = g_slist_next(elem)) {
 		next = GPOINTER_TO_INT(elem->data);
 		if(next != (last + 1)) {
@@ -1108,6 +1188,9 @@ GSList *news_get_msginfos(Folder *folder, FolderItem *item, GSList *msgnum_list)
 		}
 		last = next;
 	}
+	
+	news_folder_unlock(NEWS_FOLDER(item->folder));
+	
 	session->fetch_base_percentage = ((gfloat) fetched) / ((gfloat) tofetch);
 	session->fetch_total_percentage = ((gfloat) (last - first + 1)) / ((gfloat) tofetch);
 	tmp_msginfo_list = news_get_msginfos_for_range(session, item, first, last);
@@ -1124,3 +1207,28 @@ gboolean news_scan_required(Folder *folder, FolderItem *item)
 {
 	return TRUE;
 }
+
+void news_synchronise(FolderItem *item) 
+{
+	news_gtk_synchronise(item);
+}
+
+static gint news_remove_folder(Folder *folder, FolderItem *item)
+{
+	gchar *path;
+
+	g_return_val_if_fail(folder != NULL, -1);
+	g_return_val_if_fail(item != NULL, -1);
+	g_return_val_if_fail(item->path != NULL, -1);
+
+	path = folder_item_get_path(item);
+	if (remove_dir_recursive(path) < 0) {
+		g_warning("can't remove directory `%s'\n", path);
+		g_free(path);
+		return -1;
+	}
+
+	g_free(path);
+	folder_item_remove(item);
+	return 0;
+}
