@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto and the Sylpheed-Claws Team
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef CLAMAV_PLUGIN_H
@@ -38,5 +38,7 @@ struct _ClamAvConfig
 ClamAvConfig *clamav_get_config		  (void);
 void	      clamav_save_config	  (void);
 void 	      clamav_set_message_callback (MessageCallback callback);
+gint	      clamav_gtk_init(void);
+void 	      clamav_gtk_done(void);
 
 #endif
