@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -271,7 +271,8 @@ static gint pop3_getrange_uidl_recv(Pop3Session *session, const gchar *data,
 			session->msg[num].received = 
 				(partial_recv != POP3_MUST_COMPLETE_RECV);
 			session->msg[num].partial_recv = partial_recv;
-
+			if (partial_recv == POP3_MUST_COMPLETE_RECV)
+				session->new_msg_exist = TRUE;
 		}
 		if (!session->new_msg_exist &&
 		    (session->ac_prefs->getall || recv_time == RECV_TIME_NONE ||
@@ -599,8 +600,8 @@ void pop3_get_uidl_table(PrefsAccount *ac_prefs, Pop3Session *session)
 		recv_time = RECV_TIME_NONE;
 		partial_recv = POP3_TOTALLY_RECEIVED;
 		
-		if (sscanf(buf, "%s\t%ld\t%s", uidl, &recv_time, tmp) < 3) {
-			if (sscanf(buf, "%s\t%ld", uidl, &recv_time) != 2) {
+		if (sscanf(buf, "%s\t%ld\t%s", uidl, (long int *) &recv_time, tmp) < 3) {
+			if (sscanf(buf, "%s\t%ld", uidl, (long int *) &recv_time) != 2) {
 				if (sscanf(buf, "%s", uidl) != 1)
 					continue;
 				else {
@@ -663,7 +664,7 @@ gint pop3_write_uidl_list(Pop3Session *session)
 		if (msg->uidl && msg->received &&
 		    (!msg->deleted || session->state != POP3_DONE))
 			fprintf(fp, "%s\t%ld\t%d\n", 
-				msg->uidl, msg->recv_time, msg->partial_recv);
+				msg->uidl, (long int) msg->recv_time, msg->partial_recv);
 	}
 
 	if (fclose(fp) == EOF) FILE_OP_ERROR(path, "fclose");
