@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto & The Sylpheed Claws Team
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto & The Sylpheed Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -77,7 +77,7 @@ static gint prefs_actions_clist_set_row	(GtkTreeIter *row);
 
 /* callback functions */
 static void prefs_actions_help_cb	(GtkWidget	*w,
-					 gpointer	 data);
+					 GtkWidget	*window);
 static void prefs_actions_register_cb	(GtkWidget	*w,
 					 gpointer	 data);
 static void prefs_actions_substitute_cb	(GtkWidget	*w,
@@ -129,6 +129,18 @@ void prefs_actions_open(MainWindow *mainwin)
 	gtk_widget_show(actions.window);
 }
 
+/*!
+ *\brief	Save Gtk object size to prefs dataset
+ */
+static void prefs_actions_size_allocate_cb(GtkWidget *widget,
+					 GtkAllocation *allocation)
+{
+	g_return_if_fail(allocation != NULL);
+
+	prefs_common.actionswin_width = allocation->width;
+	prefs_common.actionswin_height = allocation->height;
+}
+
 static void prefs_actions_create(MainWindow *mainwin)
 {
 	GtkWidget *window;
@@ -162,6 +174,7 @@ static void prefs_actions_create(MainWindow *mainwin)
 	GtkWidget *btn_vbox;
 	GtkWidget *up_btn;
 	GtkWidget *down_btn;
+	static GdkGeometry geometry;
 
 	debug_print("Creating actions configuration window...\n");
 
@@ -171,14 +184,13 @@ static void prefs_actions_create(MainWindow *mainwin)
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
-	gtk_window_set_default_size(GTK_WINDOW(window), 400, -1);
 
 	vbox = gtk_vbox_new(FALSE, 6);
 	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
-	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
-				      &cancel_btn, GTK_STOCK_CANCEL,
+	gtkut_stock_button_set_create(&confirm_area, &cancel_btn, GTK_STOCK_CANCEL,
+				      &ok_btn, GTK_STOCK_OK,
 				      NULL, NULL);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
@@ -187,6 +199,8 @@ static void prefs_actions_create(MainWindow *mainwin)
 	gtk_window_set_title(GTK_WINDOW(window), _("Actions configuration"));
 	g_signal_connect(G_OBJECT(window), "delete_event",
 			 G_CALLBACK(prefs_actions_deleted), NULL);
+	g_signal_connect(G_OBJECT(window), "size_allocate",
+			 G_CALLBACK(prefs_actions_size_allocate_cb), NULL);
 	g_signal_connect(G_OBJECT(window), "key_press_event",
 			 G_CALLBACK(prefs_actions_key_pressed), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
@@ -257,11 +271,11 @@ static void prefs_actions_create(MainWindow *mainwin)
 	g_signal_connect(G_OBJECT(del_btn), "clicked",
 			 G_CALLBACK(prefs_actions_delete_cb), NULL);
 
-	help_button = gtk_button_new_with_label(_(" Syntax help "));
+	help_button = gtk_button_new_from_stock(GTK_STOCK_HELP);
 	gtk_widget_show(help_button);
 	gtk_box_pack_end(GTK_BOX(reg_hbox), help_button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(help_button), "clicked",
-			 G_CALLBACK(prefs_actions_help_cb), NULL);
+			 G_CALLBACK(prefs_actions_help_cb), GTK_WINDOW(window));
 
 	cond_hbox = gtk_hbox_new(FALSE, 8);
 	gtk_widget_show(cond_hbox);
@@ -296,6 +310,16 @@ static void prefs_actions_create(MainWindow *mainwin)
 	g_signal_connect(G_OBJECT(down_btn), "clicked",
 			 G_CALLBACK(prefs_actions_down), NULL);
 
+	if (!geometry.min_height) {
+		geometry.min_width = 486;
+		geometry.min_height = 322;
+	}
+
+	gtk_window_set_geometry_hints(GTK_WINDOW(window), NULL, &geometry,
+				      GDK_HINT_MIN_SIZE);
+	gtk_widget_set_size_request(window, prefs_common.actionswin_width,
+				    prefs_common.actionswin_height);
+
 	gtk_widget_show(window);
 
 	actions.window = window;
@@ -482,13 +506,18 @@ static gint prefs_actions_clist_set_row(GtkTreeIter *row)
 		return -1;
 	}
 
+	if (entry_text[0] == '/') {
+		alertpanel_error(_("A leading '/' is not allowed in the menu name."));
+		return -1;
+	}
+
 	if (strchr(entry_text, ':')) {
 		alertpanel_error(_("Colon ':' is not allowed in the menu name."));
 		return -1;
 	}
 
 	strncpy(action, entry_text, PREFSBUFSIZE - 1);
-	
+
 	while (strstr(action, "//")) {
 		char *to_move = strstr(action, "//")+1;
 		char *where = strstr(action, "//");
@@ -583,7 +612,7 @@ static void prefs_actions_delete_cb(GtkWidget *w, gpointer data)
 
 	if (alertpanel(_("Delete action"),
 		       _("Do you really want to delete this action?"),
-		       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
+		       GTK_STOCK_CANCEL, GTK_STOCK_DELETE, NULL) != G_ALERTALTERNATE)
 		return;
 
 	/* XXX: Here's the reason why we need to store the original 
@@ -601,16 +630,17 @@ static void prefs_actions_up(GtkWidget *w, gpointer data)
 {
 	GtkTreePath *prev, *sel, *try;
 	GtkTreeIter isel;
-	GtkListStore *store;
+	GtkListStore *store = NULL;
+	GtkTreeModel *model = NULL;
 	GtkTreeIter iprev;
 	
 	if (!gtk_tree_selection_get_selected
 		(gtk_tree_view_get_selection
 			(GTK_TREE_VIEW(actions.actions_list_view)),
-		 (GtkTreeModel **) &store,	
+		 &model,	
 		 &isel))
 		return;
-
+	store = (GtkListStore *)model;
 	sel = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &isel);
 	if (!sel)
 		return;
@@ -643,17 +673,18 @@ static void prefs_actions_up(GtkWidget *w, gpointer data)
 
 static void prefs_actions_down(GtkWidget *w, gpointer data)
 {
-	GtkListStore *store;
+	GtkListStore *store = NULL;
+	GtkTreeModel *model = NULL;
 	GtkTreeIter next, sel;
 	GtkTreePath *try;
 	
 	if (!gtk_tree_selection_get_selected
 		(gtk_tree_view_get_selection
 			(GTK_TREE_VIEW(actions.actions_list_view)),
-		 (GtkTreeModel **) &store,
+		 &model,
 		 &sel))
 		return;
-
+	store = (GtkListStore *)model;
 	try = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &sel);
 	if (!try) 
 		return;
@@ -699,7 +730,8 @@ static void prefs_actions_cancel(GtkWidget *w, gpointer data)
 {
 	if (modified && alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
-				 GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT) {
+				 GTK_STOCK_CLOSE, _("_Continue editing"),
+				 NULL) != G_ALERTDEFAULT) {
 		return;
 	}
 	modified = FALSE;
@@ -718,10 +750,12 @@ static void prefs_actions_ok(GtkWidget *widget, gpointer data)
 
 	if (modified && alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
-				 GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT) {
+				 GTK_STOCK_CLOSE, _("_Continue editing"),
+				 NULL) != G_ALERTDEFAULT) {
 		return;
 	}
 	modified = FALSE;
+	prefs_actions_set_list();
 	prefs_actions_write_config();
 
 	/* Update mainwindow actions menu */
@@ -751,19 +785,19 @@ static void prefs_actions_ok(GtkWidget *widget, gpointer data)
  * When adding new lines, remember to put one string for each line
  */
 static gchar *actions_desc_strings[] = {
-	N_("MENU NAME:"), NULL,
-	"      ",   N_("Use / in menu name to make submenus."),
+	N_("<span weight=\"bold\" underline=\"single\">Menu name:</span>"), NULL,
+	N_("Use / in menu name to make submenus."), NULL,
 	"", NULL,
-	N_("COMMAND LINE:"), NULL,
-	N_("Begin with:"), NULL,
+	N_("<span weight=\"bold\" underline=\"single\">Command line:</span>"), NULL,
+	N_("<span weight=\"bold\">Begin with:</span>"), NULL,
 	"     |",   N_("to send message body or selection to command's standard input"),
-	"     >",   N_("to send user provided text to command's standard input"),
+	"     &gt;",   N_("to send user provided text to command's standard input"),
 	"     *",   N_("to send user provided hidden text to command's standard input"),
-	N_("End with:"), NULL, 
+	N_("<span weight=\"bold\">End with:</span>"), NULL,
 	"     |",   N_("to replace message body or selection with command's standard output"),
-	"     >",   N_("to insert command's standard output without replacing old text"),
-	"     &",   N_("to run command asynchronously"),
-	N_("Use:"), NULL, 
+	"     &gt;",   N_("to insert command's standard output without replacing old text"),
+	"     &amp;",   N_("to run command asynchronously"),
+	N_("<span weight=\"bold\">Use:</span>"), NULL, 
 	"     %f",  N_("for the file of the selected message in RFC822/2822 format "),
 	"     %F",  N_("for the list of the files of the selected messages in RFC822/2822 format"),
 	"     %p",  N_("for the file of the selected decoded message MIME part"),
@@ -771,20 +805,25 @@ static gchar *actions_desc_strings[] = {
 	"     %h",  N_("for a user provided hidden argument (e.g. password)"),
 	"     %s",  N_("for the text selection"),
 	"  %as{}",  N_("apply filtering actions between {} to selected messages"),
-	NULL
+	NULL, NULL
 };
 
 
 static DescriptionWindow actions_desc_win = { 
-        NULL, 
-        2,
-        N_("Description of symbols"),
+	NULL,
+	NULL,
+	2,
+	N_("Actions"),
+  	N_("The Actions feature is a way for the user to launch "
+	   "external commands to process a complete message file or just "
+	   "one of its parts."),
         actions_desc_strings
 };
 
 
-static void prefs_actions_help_cb(GtkWidget *w, gpointer data)
+static void prefs_actions_help_cb(GtkWidget *w, GtkWidget *window)
 {
+	actions_desc_win.parent = window;
 	description_window_create(&actions_desc_win);
 }
 
@@ -841,7 +880,8 @@ static GtkWidget *prefs_actions_list_view_create(void)
 	g_object_unref(model);	
 	
 	gtk_tree_view_set_rules_hint(list_view, prefs_common.enable_rules_hint);
-	
+	gtk_tree_view_set_reorderable(list_view, TRUE);
+
 	selector = gtk_tree_view_get_selection(list_view);
 	gtk_tree_selection_set_mode(selector, GTK_SELECTION_BROWSE);
 	gtk_tree_selection_set_select_function(selector, prefs_actions_selected,
