@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -58,6 +58,7 @@
 #include "folderview.h"
 #include "stock_pixmap.h"
 #include "prefswindow.h"
+#include "colorlabel.h"
 
 enum {
 	DATEFMT_FMT,
@@ -69,6 +70,68 @@ PrefsCommon prefs_common;
 
 GtkWidget *notebook;
 
+#ifdef G_OS_WIN32
+/*
+ * In the Windows version prefs_common contains
+ *   - the non-OS-specific settings of the "Common" section and
+ *   - the OS-specific settings of the "CommonWin32" section
+ * The OS-specific settings of the "Common" section are not used
+ * but saved in prefs_unix.
+ */
+
+#  define SPECIFIC_PREFS prefs_unix
+
+static PrefsCommon prefs_unix;
+
+static PrefParam param_os_specific[] = {
+	/* Receive */
+	{"ext_inc_path", "",
+	 &prefs_common.extinc_cmd, P_STRING, NULL, NULL, NULL},
+	{"newmail_notify_cmd", "",
+	 &prefs_common.newmail_notify_cmd, P_STRING, NULL, NULL, NULL},
+
+	/* new fonts */
+	{"widget_font_gtk2",	NULL,
+	  &prefs_common.widgetfont,		P_STRING, NULL, NULL, NULL},
+	{"message_font_gtk2",	"Monospace 9",
+	 &prefs_common.textfont,		P_STRING, NULL, NULL, NULL},
+	{"small_font_gtk2",	"Sans 9",
+	  &prefs_common.smallfont,		P_STRING, NULL, NULL, NULL},
+	{"normal_font_gtk2",	"Sans 9",
+	  &prefs_common.normalfont,		P_STRING, NULL, NULL, NULL},
+
+	/* Message */
+	{"attach_save_directory", NULL,
+	 &prefs_common.attach_save_dir, P_STRING, NULL, NULL, NULL},
+	{"attach_load_directory", NULL,
+	 &prefs_common.attach_load_dir, P_STRING, NULL, NULL, NULL},
+
+	/* MIME viewer */
+	{"mime_image_viewer", NULL,
+	 &prefs_common.mime_image_viewer, P_STRING, NULL, NULL, NULL},
+	{"mime_audio_player", NULL,
+	 &prefs_common.mime_audio_player, P_STRING, NULL, NULL, NULL},
+	{"mime_open_command", "notepad '%s'",
+	 &prefs_common.mime_open_cmd, P_STRING, NULL, NULL, NULL},
+
+	/* Interface */
+	{"pixmap_theme_path", DEFAULT_PIXMAP_THEME, 
+	 &prefs_common.pixmap_theme_path, P_STRING, NULL, NULL, NULL},
+
+	/* Other */
+	{"uri_open_command", NULL,
+	 &prefs_common.uri_cmd, P_STRING, NULL, NULL, NULL},
+	{"print_command", "notepad /p %s",
+	 &prefs_common.print_cmd, P_STRING, NULL, NULL, NULL},
+	{"ext_editor_command", "notepad %s",
+	 &prefs_common.ext_editor_cmd, P_STRING, NULL, NULL, NULL},
+
+	{NULL, NULL, NULL, P_OTHER, NULL, NULL, NULL}
+};
+#else
+#  define SPECIFIC_PREFS prefs_common
+#endif
+
 /*
    parameter name, default value, pointer to the prefs variable, data type,
    pointer to the widget pointer,
@@ -80,7 +143,7 @@ static PrefParam param[] = {
 	/* Receive */
 	{"use_ext_inc", "FALSE", &prefs_common.use_extinc, P_BOOL,
 	 NULL, NULL, NULL},
-	{"ext_inc_path", DEFAULT_INC_PATH, &prefs_common.extinc_cmd, P_STRING,
+	{"ext_inc_path", DEFAULT_INC_PATH, &SPECIFIC_PREFS.extinc_cmd, P_STRING,
 	 NULL, NULL, NULL},
 
 	{"autochk_newmail", "FALSE", &prefs_common.autochk_newmail, P_BOOL,
@@ -89,13 +152,15 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"check_on_startup", "FALSE", &prefs_common.chk_on_startup, P_BOOL,
 	 NULL, NULL, NULL},
+	{"open_inbox_on_inc", "FALSE", &prefs_common.open_inbox_on_inc,
+	 P_BOOL, NULL, NULL, NULL},
 	{"scan_all_after_inc", "FALSE", &prefs_common.scan_all_after_inc,
 	 P_BOOL, NULL, NULL, NULL},
 	{"newmail_notify_manu", "FALSE", &prefs_common.newmail_notify_manu,
 	 P_BOOL, NULL, NULL, NULL},
  	{"newmail_notify_auto", "FALSE", &prefs_common.newmail_notify_auto,
 	P_BOOL, NULL, NULL, NULL},
- 	{"newmail_notify_cmd", "", &prefs_common.newmail_notify_cmd, P_STRING,
+ 	{"newmail_notify_cmd", "", &SPECIFIC_PREFS.newmail_notify_cmd, P_STRING,
  	 NULL, NULL, NULL},
 	{"receive_dialog_mode", "1", &prefs_common.recv_dialog_mode, P_ENUM,
 	 NULL, NULL, NULL},
@@ -139,9 +204,7 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"linewrap_auto", "TRUE", &prefs_common.autowrap, P_BOOL,
 	 NULL, NULL, NULL},
-	{"linewrap_before_sending", "FALSE", &prefs_common.linewrap_at_send, P_BOOL, 
-	 NULL, NULL, NULL},
-        {"autosave", "FALSE", &prefs_common.autosave,
+        {"autosave", "TRUE", &prefs_common.autosave,
 	 P_BOOL, NULL, NULL, NULL},
         {"autosave_length", "50", &prefs_common.autosave_length, P_INT,
 	 NULL, NULL, NULL},
@@ -158,6 +221,8 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"check_while_typing", "TRUE", &prefs_common.check_while_typing, P_BOOL,
 	 NULL, NULL, NULL},
+	{"recheck_when_changing_dict", "TRUE", &prefs_common.recheck_when_changing_dict,
+	 P_BOOL, NULL, NULL, NULL},
 	{"misspelled_color", "16711680", &prefs_common.misspelled_col, P_COLOR,
 	 NULL, NULL, NULL},
 #endif
@@ -184,15 +249,15 @@ static PrefParam param[] = {
 	/* Quote */
 	{"reply_quote_mark", "> ", &prefs_common.quotemark, P_STRING,
 	 NULL, NULL, NULL},
-	{"reply_quote_format", "On %d\\n%f wrote:\\n\\n%q",
+	{"reply_quote_format", N_("On %d\\n%f wrote:\\n\\n%q"),
 	 &prefs_common.quotefmt, P_STRING, NULL, NULL, NULL},
 
 	{"forward_quote_mark", "> ", &prefs_common.fw_quotemark, P_STRING,
 	 NULL, NULL, NULL},
 	{"forward_quote_format",
-	 "\\n\\nBegin forwarded message:\\n\\n"
+	 N_("\\n\\nBegin forwarded message:\\n\\n"
 	 "?d{Date: %d\\n}?f{From: %f\\n}?t{To: %t\\n}?c{Cc: %c\\n}"
-	 "?n{Newsgroups: %n\\n}?s{Subject: %s\\n}\\n\\n%M",
+	 "?n{Newsgroups: %n\\n}?s{Subject: %s\\n}\\n\\n%M"),
 	 &prefs_common.fw_quotefmt, P_STRING, NULL, NULL, NULL},
 	{"quote_chars", ">", &prefs_common.quote_chars, P_STRING,
 	 NULL, NULL, NULL},
@@ -212,13 +277,43 @@ static PrefParam param[] = {
 
 	/* new fonts */
 	{"widget_font_gtk2",	NULL,
-	  &prefs_common.widgetfont,		P_STRING, NULL, NULL, NULL},
+	  &SPECIFIC_PREFS.widgetfont,		P_STRING, NULL, NULL, NULL},
 	{"message_font_gtk2",	"Monospace 9",
-	 &prefs_common.textfont,		P_STRING, NULL, NULL, NULL},
+	 &SPECIFIC_PREFS.textfont,			P_STRING, NULL, NULL, NULL},
 	{"small_font_gtk2",	"Sans 9",
-	  &prefs_common.smallfont,		P_STRING, NULL, NULL, NULL},
+	  &SPECIFIC_PREFS.smallfont,		P_STRING, NULL, NULL, NULL},
 	{"normal_font_gtk2",	"Sans 9",
-	  &prefs_common.normalfont,		P_STRING, NULL, NULL, NULL},
+	  &SPECIFIC_PREFS.normalfont,		P_STRING, NULL, NULL, NULL},
+
+	/* custom colors */
+	{"custom_color1", "#ff9900", &prefs_common.custom_colorlabel[0].color, P_COLOR,
+	 NULL, NULL, NULL},
+	{"custom_colorlabel1", N_("Orange"), &prefs_common.custom_colorlabel[0].label, P_STRING,
+	 NULL, NULL, NULL},
+	{"custom_color2", "#ff0000", &prefs_common.custom_colorlabel[1].color, P_COLOR,
+	 NULL, NULL, NULL},
+	{"custom_colorlabel2", N_("Red"), &prefs_common.custom_colorlabel[1].label, P_STRING,
+	 NULL, NULL, NULL},
+	{"custom_color3", "#ff66ff", &prefs_common.custom_colorlabel[2].color, P_COLOR,
+	 NULL, NULL, NULL},
+	{"custom_colorlabel3", N_("Pink"), &prefs_common.custom_colorlabel[2].label, P_STRING,
+	 NULL, NULL, NULL},
+	{"custom_color4", "#00ccff", &prefs_common.custom_colorlabel[3].color, P_COLOR,
+	 NULL, NULL, NULL},
+	{"custom_colorlabel4", N_("Sky blue"), &prefs_common.custom_colorlabel[3].label, P_STRING,
+	 NULL, NULL, NULL},
+	{"custom_color5", "#0000ff", &prefs_common.custom_colorlabel[4].color, P_COLOR,
+	 NULL, NULL, NULL},
+	{"custom_colorlabel5", N_("Blue"), &prefs_common.custom_colorlabel[4].label, P_STRING,
+	 NULL, NULL, NULL},
+	{"custom_color6", "#009900", &prefs_common.custom_colorlabel[5].color, P_COLOR,
+	 NULL, NULL, NULL},
+	{"custom_colorlabel6", N_("Green"), &prefs_common.custom_colorlabel[5].label, P_STRING,
+	 NULL, NULL, NULL},
+	{"custom_color7", "#663366", &prefs_common.custom_colorlabel[6].color, P_COLOR,
+	 NULL, NULL, NULL},
+	{"custom_colorlabel7", N_("Brown"), &prefs_common.custom_colorlabel[6].label, P_STRING,
+	 NULL, NULL, NULL},
 
 	/* image viewer */
 	{"display_image", "TRUE", &prefs_common.display_img, P_BOOL,
@@ -228,49 +323,38 @@ static PrefParam param[] = {
 	{"inline_image", "TRUE", &prefs_common.inline_img, P_BOOL,
 	 NULL, NULL, NULL},
 
-	{"display_folder_unread_num", "TRUE",
+	{"display_folder_unread_num", "FALSE",
 	 &prefs_common.display_folder_unread, P_BOOL,
 	 NULL, NULL, NULL},
 	{"newsgroup_abbrev_len", "16",
 	 &prefs_common.ng_abbrev_len, P_INT,
 	 NULL, NULL, NULL},
 
-	{"translate_header", "TRUE", &prefs_common.trans_hdr, P_BOOL,
+	{"translate_header", "FALSE", &prefs_common.trans_hdr, P_BOOL,
 	 NULL, NULL, NULL},
 
 	/* Display: Summary View */
-	{"enable_swap_from", "FALSE", &prefs_common.swap_from, P_BOOL,
-	 NULL, NULL, NULL},
 	{"use_address_book", "FALSE", &prefs_common.use_addr_book, P_BOOL,
 	 NULL, NULL, NULL},
 	{"thread_by_subject", "TRUE", &prefs_common.thread_by_subject, P_BOOL,
 	 NULL, NULL, NULL},
-	{"date_format", "%y/%m/%d(%a) %H:%M", &prefs_common.date_format,
+	{"date_format", N_("%y/%m/%d(%a) %H:%M"), &prefs_common.date_format,
 	 P_STRING, NULL, NULL, NULL},
 
-	{"enable_rules_hint", "TRUE", &prefs_common.enable_rules_hint, P_BOOL,
-	 NULL, NULL, NULL},
-	{"enable_hscrollbar", "TRUE", &prefs_common.enable_hscrollbar, P_BOOL,
-	 NULL, NULL, NULL},
 	{"bold_unread", "TRUE", &prefs_common.bold_unread, P_BOOL,
 	 NULL, NULL, NULL},
-	{"thread_by_subject_max_age", "10", &prefs_common.thread_by_subject_max_age,
-	P_INT, NULL, NULL, NULL },
 
 	{"enable_thread", "TRUE", &prefs_common.enable_thread, P_BOOL,
 	 NULL, NULL, NULL},
 	{"toolbar_style", "3", &prefs_common.toolbar_style, P_ENUM,
 	 NULL, NULL, NULL},
-	{"toolbar_detachable", "TRUE", &prefs_common.toolbar_detachable, P_BOOL,
+	{"toolbar_detachable", "FALSE", &prefs_common.toolbar_detachable, P_BOOL,
 	 NULL, NULL, NULL},
 	{"show_statusbar", "TRUE", &prefs_common.show_statusbar, P_BOOL,
 	 NULL, NULL, NULL},
 	{"show_searchbar", "TRUE", &prefs_common.show_searchbar, P_BOOL,
 	 NULL, NULL, NULL},
 
-	{"folderview_vscrollbar_policy", "0",
-	 &prefs_common.folderview_vscrollbar_policy, P_ENUM,
-	 NULL, NULL, NULL},
 
 	{"summary_col_show_mark", "TRUE",
 	 &prefs_common.summary_col_visible[S_COL_MARK], P_BOOL, NULL, NULL, NULL},
@@ -346,7 +430,7 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"folderwin_y", "16", &prefs_common.folderwin_y, P_INT,
 	 NULL, NULL, NULL},
-	{"folderview_width", "179", &prefs_common.folderview_width, P_INT,
+	{"folderview_width", "270", &prefs_common.folderview_width, P_INT,
 	 NULL, NULL, NULL},
 	{"folderview_height", "450", &prefs_common.folderview_height, P_INT,
 	 NULL, NULL, NULL},
@@ -371,7 +455,7 @@ static PrefParam param[] = {
 	{"folder_col_pos_total", "3",
 	 &prefs_common.folder_col_pos[F_COL_TOTAL], P_INT, NULL, NULL, NULL},
 
-	{"folder_col_size_folder", "150",
+	{"folder_col_size_folder", "120",
 	 &prefs_common.folder_col_size[F_COL_FOLDER], P_INT, NULL, NULL, NULL},
 	{"folder_col_size_new", "32",
 	 &prefs_common.folder_col_size[F_COL_NEW], P_INT, NULL, NULL, NULL},
@@ -380,18 +464,18 @@ static PrefParam param[] = {
 	{"folder_col_size_total", "32",
 	 &prefs_common.folder_col_size[F_COL_TOTAL], P_INT, NULL, NULL, NULL},
 
-	{"summaryview_width", "600", &prefs_common.summaryview_width, P_INT,
+	{"summaryview_width", "500", &prefs_common.summaryview_width, P_INT,
 	 NULL, NULL, NULL},
-	{"summaryview_height", "157", &prefs_common.summaryview_height, P_INT,
+	{"summaryview_height", "244", &prefs_common.summaryview_height, P_INT,
 	 NULL, NULL, NULL},
 
 	{"main_messagewin_x", "256", &prefs_common.main_msgwin_x, P_INT,
 	 NULL, NULL, NULL},
 	{"main_messagewin_y", "210", &prefs_common.main_msgwin_y, P_INT,
 	 NULL, NULL, NULL},
-	{"messageview_width", "600", &prefs_common.msgview_width, P_INT,
+	{"messageview_width", "500", &prefs_common.msgview_width, P_INT,
 	 NULL, NULL, NULL},
-	{"messageview_height", "300", &prefs_common.msgview_height, P_INT,
+	{"messageview_height", "213", &prefs_common.msgview_height, P_INT,
 	 NULL, NULL, NULL},
 	{"messageview_visible", "TRUE", &prefs_common.msgview_visible, P_BOOL,
 	 NULL, NULL, NULL},
@@ -400,9 +484,9 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"mainview_y", "64", &prefs_common.mainview_y, P_INT,
 	 NULL, NULL, NULL},
-	{"mainview_width", "600", &prefs_common.mainview_width, P_INT,
+	{"mainview_width", "500", &prefs_common.mainview_width, P_INT,
 	 NULL, NULL, NULL},
-	{"mainview_height", "600", &prefs_common.mainview_height, P_INT,
+	{"mainview_height", "400", &prefs_common.mainview_height, P_INT,
 	 NULL, NULL, NULL},
 	{"mainwin_x", "64", &prefs_common.mainwin_x, P_INT,
 	 NULL, NULL, NULL},
@@ -447,16 +531,15 @@ static PrefParam param[] = {
 	{"recycle_quote_colors", "FALSE", &prefs_common.recycle_quote_colors,
 	 P_BOOL, NULL, NULL, NULL},
 
-	{"convert_mb_alnum", "FALSE", &prefs_common.conv_mb_alnum, P_BOOL,
-	 NULL, NULL, NULL},
-	{"display_header_pane", "TRUE", &prefs_common.display_header_pane,
+	{"display_header_pane", "FALSE", &prefs_common.display_header_pane,
 	 P_BOOL, NULL, NULL, NULL},
 	{"display_header", "TRUE", &prefs_common.display_header, P_BOOL,
 	 NULL, NULL, NULL},
+	{"display_xface", "TRUE", &prefs_common.display_xface,
+	 P_BOOL, NULL, NULL, NULL},
 	{"render_html", "TRUE", &prefs_common.render_html, P_BOOL,
 	 NULL, NULL, NULL},
-	{"textview_cursor_visible", "FALSE",
-	 &prefs_common.textview_cursor_visible, P_BOOL,
+	{"invoke_plugin_on_html", "FALSE", &prefs_common.invoke_plugin_on_html, P_BOOL,
 	 NULL, NULL, NULL},
 	{"line_space", "2", &prefs_common.line_space, P_INT,
 	 NULL, NULL, NULL},
@@ -475,30 +558,30 @@ static PrefParam param[] = {
 	{"attach_desc", "TRUE", &prefs_common.attach_desc, P_BOOL,
 	 NULL, NULL, NULL},
 	{"attach_save_directory", NULL,
-	 &prefs_common.attach_save_dir, P_STRING, NULL, NULL, NULL},
+	 &SPECIFIC_PREFS.attach_save_dir, P_STRING, NULL, NULL, NULL},
+	{"attach_load_directory", NULL,
+	 &SPECIFIC_PREFS.attach_load_dir, P_STRING, NULL, NULL, NULL},
 
 	/* MIME viewer */
 	{"mime_image_viewer", DEFAULT_IMAGE_VIEWER_CMD,
-	 &prefs_common.mime_image_viewer, P_STRING, NULL, NULL, NULL},
+	 &SPECIFIC_PREFS.mime_image_viewer,	P_STRING, NULL, NULL, NULL},
 	{"mime_audio_player", DEFAULT_AUDIO_PLAYER_CMD,
-	 &prefs_common.mime_audio_player, P_STRING, NULL, NULL, NULL},
+	 &SPECIFIC_PREFS.mime_audio_player, P_STRING, NULL, NULL, NULL},
 	{"mime_open_command", "gedit '%s'",
-	 &prefs_common.mime_open_cmd, P_STRING, NULL, NULL, NULL},
+	 &SPECIFIC_PREFS.mime_open_cmd, P_STRING, NULL, NULL, NULL},
 
 	/* Interface */
 	{"separate_folder", "FALSE", &prefs_common.sep_folder, P_BOOL,
 	 NULL, NULL, NULL},
 	{"separate_message", "FALSE", &prefs_common.sep_msg, P_BOOL,
 	 NULL, NULL, NULL},
-	{"statusbar_update_step", "10", &prefs_common.statusbar_update_step, P_INT,
-	 NULL, NULL, NULL},
 
 	/* {"emulate_emacs", "FALSE", &prefs_common.emulate_emacs, P_BOOL,
 	 NULL, NULL, NULL}, */
 	{"always_show_message_when_selected", "FALSE",
 	 &prefs_common.always_show_msg,
 	 P_BOOL, NULL, NULL, NULL},
-	{"select_on_entry", NULL, &prefs_common.select_on_entry,
+	{"select_on_entry", "2", &prefs_common.select_on_entry,
 	 P_ENUM, NULL, NULL, NULL},
 	{"mark_as_read_on_new_window", "FALSE",
 	 &prefs_common.mark_as_read_on_new_window,
@@ -506,27 +589,25 @@ static PrefParam param[] = {
 	{"mark_as_read_delay", "0",
 	 &prefs_common.mark_as_read_delay, P_INT, 
 	 NULL, NULL, NULL},
-	{"open_inbox_on_inc", "FALSE", &prefs_common.open_inbox_on_inc,
-	 P_BOOL, NULL, NULL, NULL},
 	{"immediate_execution", "TRUE", &prefs_common.immediate_exec, P_BOOL,
 	 NULL, NULL, NULL},
-	{"nextunreadmsg_dialog", NULL, &prefs_common.next_unread_msg_dialog, P_ENUM,
+	{"nextunreadmsg_dialog", "1", &prefs_common.next_unread_msg_dialog, P_ENUM,
 	 NULL, NULL, NULL},
 
 	{"pixmap_theme_path", DEFAULT_PIXMAP_THEME, 
-	 &prefs_common.pixmap_theme_path, P_STRING,
+	 &SPECIFIC_PREFS.pixmap_theme_path, P_STRING,
 	 NULL, NULL, NULL},
 
-	{"hover_timeout", "500", &prefs_common.hover_timeout, P_INT,
+	{"ask_mark_all_read", "TRUE", &prefs_common.ask_mark_all_read, P_BOOL,
 	 NULL, NULL, NULL},
-	
+
 	/* Other */
 	{"uri_open_command", DEFAULT_BROWSER_CMD,
-	 &prefs_common.uri_cmd, P_STRING, NULL, NULL, NULL},
-	{"print_command", "lpr %s", &prefs_common.print_cmd, P_STRING,
-	 NULL, NULL, NULL},
-	{"ext_editor_command", "gedit %s",
-	 &prefs_common.ext_editor_cmd, P_STRING, NULL, NULL, NULL},
+	 &SPECIFIC_PREFS.uri_cmd, P_STRING, NULL, NULL, NULL},
+	{"print_command", "lpr %s",
+	 &SPECIFIC_PREFS.print_cmd, P_STRING, NULL, NULL, NULL},
+	{"ext_editor_command", DEFAULT_EDITOR_CMD,
+	 &SPECIFIC_PREFS.ext_editor_cmd, P_STRING, NULL, NULL, NULL},
 
 	{"add_address_by_click", "FALSE", &prefs_common.add_address_by_click,
 	 P_BOOL, NULL, NULL, NULL},
@@ -542,8 +623,6 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"summary_quicksearch_type", "0", &prefs_common.summary_quicksearch_type, P_INT,
 	 NULL, NULL, NULL},
-	{"summary_quicksearch_sticky", "1", &prefs_common.summary_quicksearch_sticky, P_INT,
-	 NULL, NULL, NULL},
 	{"summary_quicksearch_recurse", "1", &prefs_common.summary_quicksearch_recurse, P_INT,
 	 NULL, NULL, NULL},
 
@@ -558,20 +637,115 @@ static PrefParam param[] = {
 	{"log_length", "500", &prefs_common.loglength, P_INT,
 	 NULL, NULL, NULL},
 
-	{"cache_max_mem_usage", "4096", &prefs_common.cache_max_mem_usage, P_INT,
-	 NULL, NULL, NULL},
-	{"cache_min_keep_time", "15", &prefs_common.cache_min_keep_time, P_INT,
-	 NULL, NULL, NULL},
-
 	{"color_new", "179", &prefs_common.color_new, P_COLOR,
 	 NULL, NULL, NULL},
 
+	/* Some windows' sizes */
 	{"filteringwin_width", "500", &prefs_common.filteringwin_width, P_INT,
 	 NULL, NULL, NULL},
 	{"filteringwin_height", "-1", &prefs_common.filteringwin_height, P_INT,
 	 NULL, NULL, NULL},
+
+	{"filteringactionwin_width", "490", &prefs_common.filteringactionwin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"filteringactionwin_height", "-1", &prefs_common.filteringactionwin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"matcherwin_width", "520", &prefs_common.matcherwin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"matcherwin_height", "-1", &prefs_common.matcherwin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"templateswin_width", "440", &prefs_common.templateswin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"templateswin_height", "-1", &prefs_common.templateswin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"actionswin_width", "486", &prefs_common.actionswin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"actionswin_height", "-1", &prefs_common.actionswin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"addressbookwin_width", "520", &prefs_common.addressbookwin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"addressbookwin_height", "-1", &prefs_common.addressbookwin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"addressbookeditpersonwin_width", "640", &prefs_common.addressbookeditpersonwin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"addressbookeditpersonwin_height", "-1", &prefs_common.addressbookeditpersonwin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"pluginswin_width", "480", &prefs_common.pluginswin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"pluginswin_height", "-1", &prefs_common.pluginswin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"prefswin_width", "600", &prefs_common.prefswin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"prefswin_height", "-1", &prefs_common.prefswin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"folderitemwin_width", "500", &prefs_common.folderitemwin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"folderitemwin_height", "-1", &prefs_common.folderitemwin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"editaccountwin_width", "500", &prefs_common.editaccountwin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"editaccountwin_height", "-1", &prefs_common.editaccountwin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"accountswin_width", "500", &prefs_common.accountswin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"accountswin_height", "-1", &prefs_common.accountswin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"logwin_width", "520", &prefs_common.logwin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"logwin_height", "-1", &prefs_common.logwin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"folderselwin_width", "300", &prefs_common.folderselwin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"folderselwin_height", "-1", &prefs_common.folderselwin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	{"addressaddwin_width", "300", &prefs_common.addressaddwin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"addressaddwin_height", "-1", &prefs_common.addressaddwin_height, P_INT,
+	 NULL, NULL, NULL},
+
+	/* Hidden */
 	{"warn_dnd", "1", &prefs_common.warn_dnd, P_INT,
 	 NULL, NULL, NULL},
+	{"utf8_instead_of_locale_for_broken_mail", "0", 
+	 &prefs_common.broken_are_utf8, P_INT,
+	 NULL, NULL, NULL},
+	{"enable_swap_from", "FALSE", &prefs_common.swap_from, P_BOOL,
+	 NULL, NULL, NULL},
+	{"enable_rules_hint", "TRUE", &prefs_common.enable_rules_hint, P_BOOL,
+	 NULL, NULL, NULL},
+	{"enable_hscrollbar", "TRUE", &prefs_common.enable_hscrollbar, P_BOOL,
+	 NULL, NULL, NULL},
+	{"folderview_vscrollbar_policy", "0",
+	 &prefs_common.folderview_vscrollbar_policy, P_ENUM,
+	 NULL, NULL, NULL},
+	{"textview_cursor_visible", "FALSE",
+	 &prefs_common.textview_cursor_visible, P_BOOL,
+	 NULL, NULL, NULL},
+	{"hover_timeout", "500", &prefs_common.hover_timeout, P_INT,
+	 NULL, NULL, NULL},
+	{"cache_max_mem_usage", "4096", &prefs_common.cache_max_mem_usage, P_INT,
+	 NULL, NULL, NULL},
+	{"cache_min_keep_time", "15", &prefs_common.cache_min_keep_time, P_INT,
+	 NULL, NULL, NULL},
+	{"thread_by_subject_max_age", "10", &prefs_common.thread_by_subject_max_age,
+	P_INT, NULL, NULL, NULL },
+	{"summary_quicksearch_sticky", "1", &prefs_common.summary_quicksearch_sticky, P_INT,
+	 NULL, NULL, NULL},
+	{"statusbar_update_step", "10", &prefs_common.statusbar_update_step, P_INT,
+	 NULL, NULL, NULL},
 
 	{NULL, NULL, NULL, P_OTHER, NULL, NULL, NULL}
 };
@@ -614,15 +788,34 @@ GList *prefs_common_read_history(const gchar *history)
 void prefs_common_read_config(void)
 {
 	gchar *rcpath;
-	
+	gchar *tmp;
+
 	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, COMMON_RC, NULL);
 	prefs_read_config(param, "Common", rcpath, NULL);
+#ifdef G_OS_WIN32
+	prefs_read_config(param_os_specific, "CommonWin32", rcpath, NULL);
+#endif
+
 	g_free(rcpath);
+
+	tmp = g_strdup(gettext(prefs_common.quotefmt));
+	g_free(prefs_common.quotefmt);
+	prefs_common.quotefmt = tmp;
+
+	tmp = g_strdup(gettext(prefs_common.fw_quotefmt));
+	g_free(prefs_common.fw_quotefmt);
+	prefs_common.fw_quotefmt = tmp;
 	
+	tmp = g_strdup(gettext(prefs_common.date_format));
+	g_free(prefs_common.date_format);
+	prefs_common.date_format = tmp;
+
 	prefs_common.mime_open_cmd_history =
 		prefs_common_read_history(COMMAND_HISTORY);
 	prefs_common.summary_quicksearch_history =
 		prefs_common_read_history(QUICKSEARCH_HISTORY);
+
+	colorlabel_update_colortable_from_prefs();
 }
 
 /*
@@ -654,6 +847,9 @@ void prefs_common_save_history(const gchar *history, GList *list)
 void prefs_common_write_config(void)
 {
 	prefs_write_config(param, "Common", COMMON_RC);
+#ifdef G_OS_WIN32
+	prefs_write_config(param_os_specific, "CommonWin32", COMMON_RC);
+#endif
 
 	prefs_common_save_history(COMMAND_HISTORY, 
 		prefs_common.mime_open_cmd_history);
