@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -166,8 +166,8 @@ static void prefs_custom_header_create(void)
 	gtk_widget_show (vbox);
 	gtk_container_add (GTK_CONTAINER (window), vbox);
 
-	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
-				      &cancel_btn, GTK_STOCK_CANCEL,
+	gtkut_stock_button_set_create(&confirm_area, &cancel_btn, GTK_STOCK_CANCEL,
+				      &ok_btn, GTK_STOCK_OK,
 				      NULL, NULL);
 	gtk_widget_show (confirm_area);
 	gtk_box_pack_end (GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
@@ -326,8 +326,8 @@ void prefs_custom_header_read_config(PrefsAccount *ac)
 	/* remove all previous headers list */
 	while (ac->customhdr_list != NULL) {
 		ch = (CustomHeader *)ac->customhdr_list->data;
-		custom_header_free(ch);
 		ac->customhdr_list = g_slist_remove(ac->customhdr_list, ch);
+		custom_header_free(ch);
 	}
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
@@ -421,8 +421,8 @@ void prefs_custom_header_write_config(PrefsAccount *ac)
 
  	while (all_hdrs != NULL) {
  		ch = (CustomHeader *)all_hdrs->data;
- 		custom_header_free(ch);
  		all_hdrs = g_slist_remove(all_hdrs, ch);
+ 		custom_header_free(ch);
  	}
 
 	if (prefs_file_close(pfile) < 0) {
@@ -544,7 +544,7 @@ static void prefs_custom_header_delete_cb(void)
 
 	if (alertpanel(_("Delete header"),
 		       _("Do you really want to delete this header?"),
-		       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
+		       GTK_STOCK_CANCEL, GTK_STOCK_DELETE, NULL) != G_ALERTALTERNATE)
 		return;
 
 	gtk_tree_model_get(model, &sel,
@@ -561,16 +561,17 @@ static void prefs_custom_header_up(void)
 {
 	GtkTreePath *prev, *sel;
 	GtkTreeIter isel;
-	GtkListStore *store;
+	GtkListStore *store = NULL;
+	GtkTreeModel *model = NULL;
 	GtkTreeIter iprev;
 	
 	if (!gtk_tree_selection_get_selected
 		(gtk_tree_view_get_selection
 			(GTK_TREE_VIEW(customhdr.list_view)),
-		 (GtkTreeModel **) &store,
+		 &model,
 		 &isel))
 		return;
-
+	store = (GtkListStore *)model;
 	sel = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &isel);
 	if (!sel)
 		return;
@@ -594,16 +595,17 @@ static void prefs_custom_header_up(void)
 
 static void prefs_custom_header_down(void)
 {
-	GtkListStore *store;
+	GtkListStore *store = NULL;
+	GtkTreeModel *model = NULL;
 	GtkTreeIter next, sel;
 	
 	if (!gtk_tree_selection_get_selected
 		(gtk_tree_view_get_selection
 			(GTK_TREE_VIEW(customhdr.list_view)),
-		 (GtkTreeModel **) &store,
+		 &model,
 		 &sel))
 		return;
-
+	store = (GtkListStore *)model;
 	next = sel;
 	if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &next)) 
 		return;
