@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -54,7 +54,6 @@ static struct DisplayHeader {
 	GtkWidget *cancel_btn;
 
 	GtkWidget *hdr_combo;
-	GtkWidget *hdr_entry;
 	GtkWidget *key_check;
 	GtkWidget *headers_list_view;
 	GtkWidget *hidden_headers_list_view;
@@ -117,16 +116,16 @@ static gchar *defaults[] =
 	"From",
 	"To",
 	"Cc",
+	"Subject",
+	"Date",	
 	"Reply-To",
+	"Sender",
+	"User-Agent",
+	"X-Mailer",	
 	"Newsgroups",
 	"Followup-To",
-	"Subject",
-	"Date",
-	"Sender",
 	"Organization",
-	"X-Mailer",
 	"X-Newsreader",
-	"User-Agent",
 	"-Received",
 	"-Message-ID",
 	"-In-Reply-To",
@@ -199,7 +198,8 @@ static void prefs_display_header_create(void)
 	GtkWidget *hidden_headers_list_view;
 
 	GtkWidget *checkbtn_other_headers;
-
+	gint i;
+	
 	debug_print("Creating display header setting window...\n");
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -216,8 +216,8 @@ static void prefs_display_header_create(void)
 	gtk_widget_show (btn_hbox);
 	gtk_box_pack_end (GTK_BOX (vbox), btn_hbox, FALSE, FALSE, 0);
 
-	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
-				      &cancel_btn, GTK_STOCK_CANCEL,
+	gtkut_stock_button_set_create(&confirm_area, &cancel_btn, GTK_STOCK_CANCEL,
+				      &ok_btn, GTK_STOCK_OK,
 				      NULL, NULL);
 	gtk_widget_show (confirm_area);
 	gtk_box_pack_end (GTK_BOX(btn_hbox), confirm_area, FALSE, FALSE, 0);
@@ -252,14 +252,14 @@ static void prefs_display_header_create(void)
 	gtk_widget_show (hdr_label);
 	gtk_box_pack_start (GTK_BOX (hbox1), hdr_label, FALSE, FALSE, 0);
 
-	hdr_combo = gtk_combo_new ();
+	hdr_combo = gtk_combo_box_new_text();
+	for(i=0; i < 9 ; i++)
+		gtk_combo_box_append_text(GTK_COMBO_BOX (hdr_combo),
+			prefs_common.trans_hdr ? gettext(defaults[i]) : defaults[i]);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(hdr_combo), 0);
 	gtk_widget_show (hdr_combo);
 	gtk_box_pack_start (GTK_BOX (hbox1), hdr_combo, TRUE, TRUE, 0);
 	gtk_widget_set_size_request (hdr_combo, 150, -1);
-	gtkut_combo_set_items (GTK_COMBO (hdr_combo),
-			       "From", "To", "Cc", "Subject", "Date",
-			       "Reply-To", "Sender", "User-Agent", "X-Mailer",
-			       NULL);
 
 	list_view_hbox = gtk_hbox_new (FALSE, 10);
 	gtk_widget_show (list_view_hbox);
@@ -279,6 +279,8 @@ static void prefs_display_header_create(void)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (list_view_scrolledwin),
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(list_view_scrolledwin),
+					    GTK_SHADOW_IN);
 
 	headers_list_view = prefs_display_header_list_view_create
 				(_("Displayed Headers"));
@@ -343,6 +345,8 @@ static void prefs_display_header_create(void)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (list_view_scrolledwin),
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(list_view_scrolledwin),
+					    GTK_SHADOW_IN);
 
 	hidden_headers_list_view = prefs_display_header_list_view_create
 					(_("Hidden headers"));
@@ -368,7 +372,9 @@ static void prefs_display_header_create(void)
 			    G_CALLBACK (prefs_display_header_delete_cb),
 			    hidden_headers_list_view);
 
-	PACK_CHECK_BUTTON (btn_hbox, checkbtn_other_headers,
+	
+
+	PACK_CHECK_BUTTON (vbox, checkbtn_other_headers,
 			   _("Show all unspecified headers"));
 	SET_TOGGLE_SENSITIVITY (checkbtn_other_headers, list_view_hbox2);
 
@@ -379,7 +385,6 @@ static void prefs_display_header_create(void)
 	dispheader.cancel_btn    = cancel_btn;
 
 	dispheader.hdr_combo     = hdr_combo;
-	dispheader.hdr_entry     = GTK_COMBO (hdr_combo)->entry;
 
 	dispheader.headers_list_view        = headers_list_view;
 	dispheader.hidden_headers_list_view = hidden_headers_list_view;
@@ -555,7 +560,8 @@ static void prefs_display_header_list_view_set_row(gboolean hidden)
 	const gchar *entry_text;
 	GtkTreeModel *model;
 
-	entry_text = gtk_entry_get_text(GTK_ENTRY(dispheader.hdr_entry));
+	entry_text = defaults[gtk_combo_box_get_active(
+						GTK_COMBO_BOX(dispheader.hdr_combo))];
 	if (entry_text[0] == '\0') {
 		alertpanel_error(_("Header name is not set."));
 		return;
@@ -614,16 +620,17 @@ static void prefs_display_header_up(void)
 {
 	GtkTreePath *prev, *sel, *try;
 	GtkTreeIter isel;
-	GtkListStore *store;
+	GtkListStore *store = NULL;
+	GtkTreeModel *model = NULL;
 	GtkTreeIter iprev;
 	
 	if (!gtk_tree_selection_get_selected
 		(gtk_tree_view_get_selection
 			(GTK_TREE_VIEW(dispheader.headers_list_view)),
-		 (GtkTreeModel **) &store,	
+		 &model,	
 		 &isel))
 		return;
-
+	store = (GtkListStore *)model;
 	sel = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &isel);
 	if (!sel)
 		return;
@@ -647,17 +654,18 @@ static void prefs_display_header_up(void)
 
 static void prefs_display_header_down(void)
 {
-	GtkListStore *store;
+	GtkListStore *store = NULL;
+	GtkTreeModel *model = NULL;
 	GtkTreeIter next, sel;
 	GtkTreePath *try;
 	
 	if (!gtk_tree_selection_get_selected
 		(gtk_tree_view_get_selection
 			(GTK_TREE_VIEW(dispheader.headers_list_view)),
-		 (GtkTreeModel **) &store,
+		 &model,
 		 &sel))
 		return;
-
+	store = (GtkListStore *)model;
 	try = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &sel);
 	if (!try) 
 		return;
@@ -717,7 +725,8 @@ static void prefs_display_header_insert_header(GtkListStore *store,
 	/* add new */
 	gtk_list_store_append(store, &iter);
 	gtk_list_store_set(store, &iter,
-			   PREFS_HDR_HEADER, name,
+			   PREFS_HDR_HEADER,
+			   prefs_common.trans_hdr ? gettext(name) : name,
 			   PREFS_HDR_DATA, dp,
 			   -1);
 }
