@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto and the Sylpheed-Claws team
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -232,8 +232,8 @@ static void prefs_filtering_create(void)
 	gtk_widget_show (vbox);
 	gtk_container_add (GTK_CONTAINER (window), vbox);
 
-	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
-				      &cancel_btn, GTK_STOCK_CANCEL, NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area, &cancel_btn, GTK_STOCK_CANCEL,
+				      &ok_btn, GTK_STOCK_OK, NULL, NULL);
 	gtk_widget_show (confirm_area);
 	gtk_box_pack_end (GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default (ok_btn);
@@ -289,7 +289,7 @@ static void prefs_filtering_create(void)
                     	  (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
                     	  (GtkAttachOptions) (0), 0, 0);
 
-	cond_btn = gtk_button_new_with_label (_("Define ..."));
+	cond_btn = gtk_button_new_with_label (_(" Define... "));
 	gtk_widget_show (cond_btn);
   	gtk_table_attach (GTK_TABLE (table), cond_btn, 2, 3, 1, 2,
                     	  (GtkAttachOptions) (GTK_FILL),
@@ -311,7 +311,7 @@ static void prefs_filtering_create(void)
                     	  (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
                     	  (GtkAttachOptions) (0), 0, 0);
 
-	action_btn = gtk_button_new_with_label (_("Define ..."));
+	action_btn = gtk_button_new_with_label (_(" Define... "));
 	gtk_widget_show (action_btn);
   	gtk_table_attach (GTK_TABLE (table), action_btn, 2, 3, 2, 3,
                     	  (GtkAttachOptions) (GTK_FILL),
@@ -884,6 +884,8 @@ static void prefs_filtering_register_cb(void)
 	prefs_filtering_list_view_set_row(-1, prop);
 	
 	filteringprop_free(prop);
+
+	prefs_filtering_reset_dialog();
 }
 
 static void prefs_filtering_substitute_cb(void)
@@ -901,6 +903,8 @@ static void prefs_filtering_substitute_cb(void)
 	prefs_filtering_list_view_set_row(selected_row, prop);
 
 	filteringprop_free(prop);
+
+	prefs_filtering_reset_dialog();
 }
 
 static void prefs_filtering_delete_cb(void)
@@ -916,7 +920,7 @@ static void prefs_filtering_delete_cb(void)
 
 	if (alertpanel(_("Delete rule"),
 		       _("Do you really want to delete this rule?"),
-		       GTK_STOCK_YES, GTK_STOCK_NO, NULL) == G_ALERTALTERNATE)
+		       GTK_STOCK_CANCEL, "+"GTK_STOCK_DELETE, NULL) == G_ALERTDEFAULT)
 		return;
 
 	model = gtk_tree_view_get_model(list_view);	
@@ -1076,7 +1080,7 @@ static void prefs_filtering_ok(void)
 		if (!filtering_str) {
 			val = alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
-				 GTK_STOCK_YES, GTK_STOCK_NO, NULL);
+				 GTK_STOCK_CLOSE, _("_Continue editing"), NULL);
 			if (G_ALERTDEFAULT != val) {
 				g_free(filtering_str);
 				g_free(str); /* fixed two leaks: huzzah! */
@@ -1088,6 +1092,27 @@ static void prefs_filtering_ok(void)
 		g_free(filtering_str);
 		g_free(str);
 		filteringprop_free(prop); /* fixed a leak: huzzah! */
+	} else {
+		gchar *name, *condition, *action;
+		name = gtk_editable_get_chars(GTK_EDITABLE(filtering.name_entry), 0, -1);
+		condition = gtk_editable_get_chars(GTK_EDITABLE(filtering.cond_entry), 0, -1);
+		action = gtk_editable_get_chars(GTK_EDITABLE(filtering.action_entry), 0, -1);
+		if (strlen(name) || 
+		    strlen(condition) || 
+		    strlen(action)) {
+			val = alertpanel(_("Entry not saved"),
+				 _("The entry was not saved. Close anyway?"),
+				 GTK_STOCK_CLOSE, _("_Continue editing"), NULL);
+			if (G_ALERTDEFAULT != val) {
+				g_free(name);
+				g_free(condition);
+				g_free(action);
+				return;
+			}
+		}
+		g_free(name);
+		g_free(condition);
+		g_free(action);
 	}
 	prefs_filtering_set_list();
 	prefs_matcher_write_config();
