@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -138,6 +138,7 @@ typedef enum Action_ {
 	ACTION_CHANGE_SCORE,
 	ACTION_SET_SCORE,
 	ACTION_HIDE,
+	ACTION_IGNORE,
 	ACTION_STOP,
 	/* add other action constants */
 } Action;
@@ -163,6 +164,7 @@ static struct {
 	{ N_("Change score"),		ACTION_CHANGE_SCORE},
 	{ N_("Set score"),		ACTION_SET_SCORE},
 	{ N_("Hide"),		        ACTION_HIDE	},
+	{ N_("Ignore thread"),	        ACTION_IGNORE	},
 	{ N_("Stop filter"),		ACTION_STOP	},
 };
 
@@ -224,6 +226,18 @@ void prefs_filtering_action_open(GSList *action_list,
 	gtk_widget_show(filtering_action.window);
 }
 
+/*!
+ *\brief	Save Gtk object size to prefs dataset
+ */
+static void prefs_filtering_action_size_allocate_cb(GtkWidget *widget,
+					 GtkAllocation *allocation)
+{
+	g_return_if_fail(allocation != NULL);
+
+	prefs_common.filteringactionwin_width = allocation->width;
+	prefs_common.filteringactionwin_height = allocation->height;
+}
+
 /*!
  *\brief	Create the matcher dialog
  */
@@ -275,6 +289,7 @@ static void prefs_filtering_action_create(void)
 
 	GList *combo_items;
 	gint i;
+	static GdkGeometry geometry;
 
         GList * accounts;
 
@@ -290,8 +305,10 @@ static void prefs_filtering_action_create(void)
 	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
-	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
-				      &cancel_btn, GTK_STOCK_CANCEL, NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      &ok_btn, GTK_STOCK_OK,
+				      NULL, NULL);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
@@ -300,6 +317,8 @@ static void prefs_filtering_action_create(void)
 			     _("Filtering action configuration"));
 	g_signal_connect(G_OBJECT(window), "delete_event",
 			 G_CALLBACK(prefs_filtering_action_deleted), NULL);
+	g_signal_connect(G_OBJECT(window), "size_allocate",
+			 G_CALLBACK(prefs_filtering_action_size_allocate_cb), NULL);
 	g_signal_connect(G_OBJECT(window), "key_press_event",
 			 G_CALLBACK(prefs_filtering_action_key_pressed), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
@@ -513,6 +532,16 @@ static void prefs_filtering_action_create(void)
 	g_signal_connect(G_OBJECT(down_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_action_down), NULL);
 
+	if (!geometry.min_height) {
+		geometry.min_width = 490;
+		geometry.min_height = 328;
+	}
+
+	gtk_window_set_geometry_hints(GTK_WINDOW(window), NULL, &geometry,
+				      GDK_HINT_MIN_SIZE);
+	gtk_widget_set_size_request(window, prefs_common.filteringactionwin_width,
+				    prefs_common.filteringactionwin_height);
+
 	gtk_widget_show_all(window);
 
 	filtering_action.window    = window;
@@ -712,6 +741,8 @@ static gint prefs_filtering_action_get_matching_from_action(Action action_id)
 		return MATCHACTION_COLOR;
 	case ACTION_HIDE:
 		return MATCHACTION_HIDE;
+	case ACTION_IGNORE:
+		return MATCHACTION_IGNORE;
 	case ACTION_STOP:
 		return MATCHACTION_STOP;
 	case ACTION_CHANGE_SCORE:
@@ -793,6 +824,7 @@ static FilteringAction * prefs_filtering_action_dialog_to_action(gboolean alert)
                 break;
 	case ACTION_STOP:
 	case ACTION_HIDE:
+	case ACTION_IGNORE:
         case ACTION_DELETE:
         case ACTION_MARK:
         case ACTION_UNMARK:
@@ -898,16 +930,17 @@ static void prefs_filtering_action_up(void)
 {
 	GtkTreePath *prev, *sel, *try;
 	GtkTreeIter isel;
-	GtkListStore *store;
+	GtkListStore *store = NULL;
+	GtkTreeModel *model = NULL;
 	GtkTreeIter iprev;
 	
 	if (!gtk_tree_selection_get_selected
 		(gtk_tree_view_get_selection
 			(GTK_TREE_VIEW(filtering_action.action_list_view)),
-		 (GtkTreeModel **) &store,	
+		 &model,	
 		 &isel))
 		return;
-
+	store = (GtkListStore *)model;
 	sel = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &isel);
 	if (!sel)
 		return;
@@ -939,17 +972,18 @@ static void prefs_filtering_action_up(void)
  */
 static void prefs_filtering_action_down(void)
 {
-	GtkListStore *store;
+	GtkListStore *store = NULL;
+	GtkTreeModel *model = NULL;
 	GtkTreeIter next, sel;
 	GtkTreePath *try;
 	
 	if (!gtk_tree_selection_get_selected
 		(gtk_tree_view_get_selection
 			(GTK_TREE_VIEW(filtering_action.action_list_view)),
-		 (GtkTreeModel **) &store,
+		 &model,
 		 &sel))
 		return;
-
+	store = (GtkListStore *)model;
 	try = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &sel);
 	if (!try) 
 		return;
@@ -1038,7 +1072,7 @@ static gint prefs_filtering_action_deleted(GtkWidget *widget,
  * When adding new lines, remember to put 2 strings for each line
  */
 static gchar *exec_desc_strings[] = {
-	"%%",	"%",
+	"%%",	N_("literal %"),
 	"%s",	N_("Subject"),
 	"%f",	N_("From"),
 	"%t",	N_("To"),
@@ -1047,18 +1081,22 @@ static gchar *exec_desc_strings[] = {
 	"%i",	N_("Message-ID"),
 	"%n",	N_("Newsgroups"),
 	"%r",	N_("References"),
-	"%F",	N_("Filename - should not be modified"),
+	"%F",	N_("filename (should not be modified)"),
 	"\\n",	N_("new line"),
 	"\\",	N_("escape character for quotes"),
-	"\\\"",N_("quote character"),
+	"\\\"", N_("quote character"),
 	NULL, NULL
 };
 
 static DescriptionWindow exec_desc_win = { 
+	NULL,
         NULL, 
         2,
-        N_("Description of symbols"),
-        exec_desc_strings
+        N_("Filtering Action: 'Execute'"),
+	N_("'Execute' allows you to send a message or message element "
+	   "to an external program or script.\n\n"
+	   "The following symbols can be used:"),
+       exec_desc_strings
 };
 
 /*!
@@ -1173,6 +1211,7 @@ static void prefs_filtering_action_type_select(GtkList *list,
 	case ACTION_MARK_AS_UNREAD:
         case ACTION_STOP:
         case ACTION_HIDE:
+	case ACTION_IGNORE:
 		gtk_widget_show(filtering_action.account_label);
 		gtk_widget_set_sensitive(filtering_action.account_label, FALSE);
 		gtk_widget_set_sensitive(filtering_action.account_combo, FALSE);
@@ -1495,6 +1534,10 @@ static gboolean prefs_filtering_actions_selected
 		gtk_list_select_item(GTK_LIST(filtering_action.action_type_list),
 				     ACTION_HIDE);
 		break;
+	case MATCHACTION_IGNORE:
+		gtk_list_select_item(GTK_LIST(filtering_action.action_type_list),
+				     ACTION_IGNORE);
+		break;
 	}
 
 	filteringaction_free(action); /* XXX: memleak */
