@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,6 +33,7 @@
 #include "main.h"
 #include "prefs.h"
 #include "prefs_gtk.h"
+#include "prefs_common.h"
 #include "utils.h"
 #include "gtkutils.h"
 #include "passcrypt.h"
@@ -61,6 +62,9 @@ void prefs_read_config(PrefParam *param, const gchar *label,
 	g_return_if_fail(label != NULL);
 	g_return_if_fail(rcfile != NULL);
 
+	if (encoding != NULL)
+		g_warning("Encoding is ignored\n");
+
 	debug_print("Reading configuration...\n");
 
 	prefs_set_default(param);
@@ -136,13 +140,19 @@ void prefs_config_parse_one_line(PrefParam *param, const gchar *buf)
 		switch (param[i].type) {
 		case P_STRING:
 		{
-			gchar *tmp;
+			gchar *tmp = NULL;
 
-			tmp = *value ?
-				conv_codeset_strdup(value,
-						    conv_get_locale_charset_str(),
-						    CS_UTF_8)
-				: g_strdup("");
+			if (*value) {
+				if (g_utf8_validate(value, -1, NULL))
+					tmp = g_strdup(value);
+				else {
+					tmp = conv_codeset_strdup(value,
+						    conv_get_locale_charset_str_no_utf8(),
+						    CS_INTERNAL);
+				}
+			} else {
+				tmp = g_strdup("");
+			}
 			if (!tmp) {
 				g_warning("failed to convert character set.");
 				tmp = g_strdup(value);
@@ -291,11 +301,15 @@ gint prefs_write_param(PrefParam *param, FILE *fp)
 			gchar *tmp = NULL;
 
 			if (*((gchar **)param[i].data)) {
-				tmp = conv_codeset_strdup(*((gchar **)param[i].data),
-							  CS_UTF_8,
-							  conv_get_locale_charset_str());
-				if (!tmp)
+				if (g_utf8_validate(*((gchar **)param[i].data), -1, NULL))
 					tmp = g_strdup(*((gchar **)param[i].data));
+				else {
+					tmp = conv_codeset_strdup(*((gchar **)param[i].data),
+						conv_get_locale_charset_str_no_utf8(),
+						CS_INTERNAL);
+					if (!tmp)
+						tmp = g_strdup(*((gchar **)param[i].data));
+				}
 			}
 
 			g_snprintf(buf, sizeof(buf), "%s=%s\n", param[i].name,
@@ -384,7 +398,7 @@ void prefs_set_default(PrefParam *param)
 					tmp = envstr && *envstr ?
 						conv_codeset_strdup(envstr,
 								    conv_get_locale_charset_str(),
-								    CS_UTF_8)
+								    CS_INTERNAL)
 						: g_strdup("");
 					if (!tmp) {
 						g_warning("faild to convert character set.");
@@ -536,6 +550,14 @@ void prefs_button_toggled(GtkToggleButton *toggle_btn, GtkWidget *widget)
 	gtk_widget_set_sensitive(widget, is_active);
 }
 
+void prefs_button_toggled_reverse(GtkToggleButton *toggle_btn, GtkWidget *widget)
+{
+	gboolean is_active;
+
+	is_active = gtk_toggle_button_get_active(toggle_btn);
+	gtk_widget_set_sensitive(widget, !is_active);
+}
+
 void prefs_set_dialog(PrefParam *param)
 {
 	gint i;
@@ -845,7 +867,8 @@ static GSList *prefs_pages = NULL;
 
 void prefs_gtk_open(void)
 {
-	prefswindow_open(_("Preferences"), prefs_pages, NULL);
+	prefswindow_open(_("Preferences"), prefs_pages, NULL,
+			&prefs_common.prefswin_width, &prefs_common.prefswin_height);
 }
 
 void prefs_gtk_register_page(PrefsPage *page)
