@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -158,7 +158,7 @@ static const gchar *criteria_text [] = {
 	N_("Replied flag"), N_("Forwarded flag"),
 	N_("Locked flag"),
 	N_("Color label"),
-	N_("Ignore thread"),
+	N_("Ignored thread"),
 	N_("Score greater than"), N_("Score lower than"),
 	N_("Score equal to"),
 	N_("Test"),
@@ -315,6 +315,18 @@ void prefs_matcher_open(MatcherList *matchers, PrefsMatcherSignal *cb)
 	gtk_widget_show(matcher.window);
 }
 
+/*!
+ *\brief	Save Gtk object size to prefs dataset
+ */
+static void prefs_matcher_size_allocate_cb(GtkWidget *widget,
+					 GtkAllocation *allocation)
+{
+	g_return_if_fail(allocation != NULL);
+
+	prefs_common.matcherwin_width = allocation->width;
+	prefs_common.matcherwin_height = allocation->height;
+}
+
 /*!
  *\brief	Create the matcher dialog
  */
@@ -374,6 +386,7 @@ static void prefs_matcher_create(void)
 
 	GList *combo_items;
 	gint i;
+	static GdkGeometry geometry;
 
 	debug_print("Creating matcher configuration window...\n");
 
@@ -387,8 +400,8 @@ static void prefs_matcher_create(void)
 	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
-	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
-				      &cancel_btn, GTK_STOCK_CANCEL, NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area, &cancel_btn, GTK_STOCK_CANCEL,
+				      &ok_btn, GTK_STOCK_OK, NULL, NULL);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
@@ -397,6 +410,8 @@ static void prefs_matcher_create(void)
 			     _("Condition configuration"));
 	g_signal_connect(G_OBJECT(window), "delete_event",
 			 G_CALLBACK(prefs_matcher_deleted), NULL);
+	g_signal_connect(G_OBJECT(window), "size_allocate",
+			 G_CALLBACK(prefs_matcher_size_allocate_cb), NULL);
 	g_signal_connect(G_OBJECT(window), "key_press_event",
 			 G_CALLBACK(prefs_matcher_key_pressed), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
@@ -438,7 +453,7 @@ static void prefs_matcher_create(void)
 
 	g_list_free(combo_items);
 
-	gtk_widget_set_size_request(criteria_combo, 120, -1);
+	gtk_widget_set_size_request(criteria_combo, 170, -1);
 	gtk_table_attach(GTK_TABLE(criteria_table), criteria_combo, 0, 1, 1, 2,
 			  0, 0, 0, 0);
 	criteria_list = GTK_COMBO(criteria_combo)->list;
@@ -459,7 +474,7 @@ static void prefs_matcher_create(void)
 
 	header_combo = gtk_combo_new();
 	gtk_widget_show(header_combo);
-	gtk_widget_set_size_request(header_combo, 96, -1);
+	gtk_widget_set_size_request(header_combo, 120, -1);
 	gtkut_combo_set_items(GTK_COMBO (header_combo),
 			      "Subject", "From", "To", "Cc", "Reply-To",
 			      "Sender", "X-ML-Name", "X-List", "X-Sequence",
@@ -484,10 +499,10 @@ static void prefs_matcher_create(void)
 	gtk_table_attach(GTK_TABLE(criteria_table), value_entry, 2, 3, 1, 2,
 			 GTK_FILL | GTK_SHRINK | GTK_EXPAND, 0, 0, 0);
 
-	test_btn = gtk_button_new_with_label(_("Info ..."));
+	test_btn = gtk_button_new_with_label(_(" Info... "));
 	gtk_widget_show(test_btn);
 	gtk_table_attach(GTK_TABLE (criteria_table), test_btn, 3, 4, 1, 2,
-			 GTK_FILL | GTK_SHRINK | GTK_EXPAND, 0, 0, 0);
+			 0, 0, 0, 0);
 	g_signal_connect(G_OBJECT (test_btn), "clicked",
 			 G_CALLBACK(prefs_matcher_test_info),
 			 NULL);
@@ -603,7 +618,7 @@ static void prefs_matcher_create(void)
 
 	bool_op_combo = gtk_combo_new();
 	gtk_widget_show(bool_op_combo);
-	gtk_widget_set_size_request(bool_op_combo, 50, -1);
+	gtk_widget_set_size_request(bool_op_combo, 60, -1);
 	bool_op_list = GTK_COMBO(bool_op_combo)->list;
 	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(bool_op_combo)->entry),
 			       FALSE);
@@ -654,6 +669,16 @@ static void prefs_matcher_create(void)
 	g_signal_connect(G_OBJECT(down_btn), "clicked",
 			 G_CALLBACK(prefs_matcher_down), NULL);
 
+	if (!geometry.min_height) {
+		geometry.min_width = 520;
+		geometry.min_height = 368;
+	}
+
+	gtk_window_set_geometry_hints(GTK_WINDOW(window), NULL, &geometry,
+				      GDK_HINT_MIN_SIZE);
+	gtk_widget_set_size_request(window, prefs_common.matcherwin_width,
+				    prefs_common.matcherwin_height);
+
 	gtk_widget_show_all(window);
 
 	matcher.window    = window;
@@ -679,6 +704,9 @@ static void prefs_matcher_create(void)
 	matcher.criteria_table = criteria_table;
 
 	matcher.cond_list_view = cond_list_view;
+
+	matcher.selected_criteria = -1;
+	prefs_matcher_criteria_select(GTK_LIST(criteria_list), NULL, NULL);
 }
 
 /*!
@@ -1288,16 +1316,17 @@ static void prefs_matcher_up(void)
 {
 	GtkTreePath *prev, *sel, *try;
 	GtkTreeIter isel;
-	GtkListStore *store;
+	GtkListStore *store = NULL;
+	GtkTreeModel *model = NULL;
 	GtkTreeIter iprev;
 	
 	if (!gtk_tree_selection_get_selected
 		(gtk_tree_view_get_selection
 			(GTK_TREE_VIEW(matcher.cond_list_view)),
-		 (GtkTreeModel **) &store,	
+		 &model,	
 		 &isel))
 		return;
-
+	store = (GtkListStore *)model;
 	sel = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &isel);
 	if (!sel)
 		return;
@@ -1329,17 +1358,18 @@ static void prefs_matcher_up(void)
  */
 static void prefs_matcher_down(void)
 {
-	GtkListStore *store;
+	GtkListStore *store = NULL;
+	GtkTreeModel *model = NULL;
 	GtkTreeIter next, sel;
 	GtkTreePath *try;
 	
 	if (!gtk_tree_selection_get_selected
 		(gtk_tree_view_get_selection
 			(GTK_TREE_VIEW(matcher.cond_list_view)),
-		 (GtkTreeModel **) &store,
+		 &model,
 		 &sel))
 		return;
-
+	store = (GtkListStore *)model;
 	try = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &sel);
 	if (!try) 
 		return;
@@ -1394,7 +1424,10 @@ static void prefs_matcher_criteria_select(GtkList *list,
 	old_value = matcher.selected_criteria;
 	matcher.selected_criteria = value = get_sel_from_list
 		(GTK_LIST(matcher.criteria_list));
-	
+
+	if (old_value == matcher.selected_criteria)
+		return;
+
 	/* CLAWS: the value widget is currently either the color label combo box,
 	 * or a GtkEntry, so kiss for now */
 	if (matcher.selected_criteria == CRITERIA_COLORLABEL) { 
@@ -1601,8 +1634,8 @@ static void prefs_matcher_ok(void)
 
 				if (!matcher_str || strcmp(matcher_str, str) != 0) {
 	                        	val = alertpanel(_("Entry not saved"),
-       		                        	 _("The entry was not saved\nHave you really finished?"),
-               		                	 GTK_STOCK_YES, GTK_STOCK_NO, NULL);
+       		                        	 _("The entry was not saved.\nClose anyway?"),
+               		                	 GTK_STOCK_CLOSE, _("_Continue editing"), NULL);
 					if (G_ALERTDEFAULT != val) {
 						g_free(matcher_str);						 
 	        	                        g_free(str);
@@ -1643,11 +1676,7 @@ static gint prefs_matcher_deleted(GtkWidget *widget, GdkEventAny *event,
  * When adding new lines, remember to put 2 strings for each line
  */
 static gchar *test_desc_strings[] = {
-	N_("'Test' allows you to test a message or message element"), NULL,
-	N_("using an external program or script. The program will"), NULL,
-	N_("return either 0 or 1"), NULL,
-	N_("The following symbols can be used:"), NULL,
-	"%%",	"%",
+	"%%",	N_("literal %"),
 	"%s",	N_("Subject"),
 	"%f",	N_("From"),
 	"%t",	N_("To"),
@@ -1656,7 +1685,7 @@ static gchar *test_desc_strings[] = {
 	"%i",	N_("Message-ID"),
 	"%n",	N_("Newsgroups"),
 	"%r",	N_("References"),
-	"%F",	N_("Filename - should not be modified"),
+	"%F",	N_("filename (should not be modified)"),
 	"\\n",	N_("new line"),
 	"\\",	N_("escape character for quotes"),
 	"\\\"", N_("quote character"),
@@ -1664,9 +1693,14 @@ static gchar *test_desc_strings[] = {
 };
 
 static DescriptionWindow test_desc_win = { 
+	NULL,
         NULL, 
         2,
         N_("Match Type: 'Test'"),
+	N_("'Test' allows you to test a message or message element "
+	   "using an external program or script. The program will "
+	   "return either 0 or 1.\n\n"
+	   "The following symbols can be used:"),
         test_desc_strings
 };
 
