@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2005 Colin Leroy <colin@colino.net> & The Sylpheed-Claws Team
+ * Copyright (C) 2005-2006 Colin Leroy <colin@colino.net> & The Sylpheed-Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -58,8 +58,8 @@ typedef struct _SummariesPage
 
 	GtkWidget *checkbtn_always_show_msg;
 	GtkWidget *checkbtn_mark_as_read_on_newwin;
-	GtkWidget *checkbtn_openinbox;
 	GtkWidget *checkbtn_immedexec;
+	GtkWidget *checkbtn_ask_mark_all_read;
  	GtkWidget *optmenu_select_on_entry;
  	GtkWidget *optmenu_nextunreadmsgdialog;
 
@@ -257,8 +257,8 @@ static GtkWidget *date_format_create(GtkButton *button, void *data)
 	gtk_label_set_justify(GTK_LABEL(label3), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment(GTK_MISC(label3), 0, 0.5);
 
-	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
-				&cancel_btn, GTK_STOCK_CANCEL, NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area, &cancel_btn, GTK_STOCK_CANCEL,
+				      &ok_btn, GTK_STOCK_OK, NULL, NULL);
 
 	gtk_box_pack_start(GTK_BOX(vbox1), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_show(confirm_area);
@@ -325,7 +325,7 @@ static void prefs_keybind_select(void)
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
-	gtk_window_set_title (GTK_WINDOW (window), _("Key bindings"));
+	gtk_window_set_title (GTK_WINDOW (window), _("Select key bindings"));
 	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
 	gtk_window_set_resizable(GTK_WINDOW (window), FALSE);
@@ -343,9 +343,6 @@ static void prefs_keybind_select(void)
 	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 
-	hbox1 = gtk_hbox_new (FALSE, 8);
-	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
-
 	combo = gtk_combo_new ();
 	gtk_box_pack_start (GTK_BOX (hbox1), combo, TRUE, TRUE, 0);
 	gtkut_combo_set_items (GTK_COMBO (combo),
@@ -361,18 +358,20 @@ static void prefs_keybind_select(void)
 
 	label = gtk_label_new
 		(_("You can also modify each menu shortcut by pressing\n"
-		   "any key(s) when placing the mouse pointer on the item."));
+		   "any key(s) when focusing the mouse pointer on the item."));
 	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
+	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtkut_widget_set_small_font_size (label);
 
 	hbox1 = gtk_hbox_new (FALSE, 8);
 	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
 
-	gtkut_stock_button_set_create (&confirm_area, &ok_btn, GTK_STOCK_OK,
-				       &cancel_btn, GTK_STOCK_CANCEL,
+	gtkut_stock_button_set_create (&confirm_area, &cancel_btn, GTK_STOCK_CANCEL,
+				       &ok_btn, GTK_STOCK_OK,
 				       NULL, NULL);
 	gtk_box_pack_end (GTK_BOX (hbox1), confirm_area, FALSE, FALSE, 0);
-	gtk_widget_grab_default (ok_btn);
+	gtk_widget_grab_focus (ok_btn);
 
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
 	g_signal_connect (G_OBJECT (window), "delete_event",
@@ -442,21 +441,24 @@ static void prefs_keybind_apply_clicked(GtkWidget *widget)
 	gint n_menurc;
 
 	static struct KeyBind default_menurc[] = {
-		{"<Main>/File/Empty all Trash folders",		""},
+		{"<Main>/File/Empty all Trash folders",		"<shift>D"},
 		{"<Main>/File/Save as...",			"<control>S"},
-		{"<Main>/File/Print...",			""},
+		{"<Main>/File/Print...",			"<control>P"},
+		{"<Main>/File/Work offline",			"<control>W"},
+		{"<Main>/File/Synchronise folders",		"<control><shift>S"},
 		{"<Main>/File/Exit",				"<control>Q"},
 
 		{"<Main>/Edit/Copy",				"<control>C"},
 		{"<Main>/Edit/Select all",			"<control>A"},
 		{"<Main>/Edit/Find in current message...",	"<control>F"},
 		{"<Main>/Edit/Search folder...",		"<shift><control>F"},
+		{"<Main>/Edit/Quick search",			"slash"},
 
 		{"<Main>/View/Show or hide/Message View",	"V"},
 		{"<Main>/View/Thread view",			"<control>T"},
-		{"<Main>/View/Go to/Prev message",		"P"},
+		{"<Main>/View/Go to/Previous message",		"P"},
 		{"<Main>/View/Go to/Next message",		"N"},
-		{"<Main>/View/Go to/Prev unread message",	"<shift>P"},
+		{"<Main>/View/Go to/Previous unread message",	"<shift>P"},
 		{"<Main>/View/Go to/Next unread message",	"<shift>N"},
 		{"<Main>/View/Go to/Other folder...",		"G"},
 		{"<Main>/View/Open in new window",		"<control><alt>N"},
@@ -476,7 +478,7 @@ static void prefs_keybind_apply_clicked(GtkWidget *widget)
 		/* {"<Main>/Message/Forward as attachment",	 ""}, */
 		{"<Main>/Message/Move...",			"<control>O"},
 		{"<Main>/Message/Copy...",			"<shift><control>O"},
-		{"<Main>/Message/Delete",			"<control>D"},
+		{"<Main>/Message/Move to trash",		"<control>D"},
 		{"<Main>/Message/Mark/Mark",			"<shift>asterisk"},
 		{"<Main>/Message/Mark/Unmark",			"U"},
 		{"<Main>/Message/Mark/Mark as unread",		"<shift>exclam"},
@@ -486,17 +488,43 @@ static void prefs_keybind_apply_clicked(GtkWidget *widget)
 		{"<Main>/Tools/Execute",			"X"},
 		{"<Main>/Tools/Log window",			"<shift><control>L"},
 
+		{"<Compose>/Message/Send",				"<control>Return"},
+		{"<Compose>/Message/Send later",			"<shift><control>S"},
+		{"<Compose>/Message/Attach file",			"<control>M"},
+		{"<Compose>/Message/Insert file",			"<control>I"},
+		{"<Compose>/Message/Insert signature",			"<control>G"},
+		{"<Compose>/Message/Save",				"<control>S"},
 		{"<Compose>/Message/Close",				"<control>W"},
+		{"<Compose>/Edit/Undo",					"<control>Z"},
+		{"<Compose>/Edit/Redo",					"<control>Y"},
+		{"<Compose>/Edit/Cut",					"<control>X"},
+		{"<Compose>/Edit/Copy",					"<control>C"},
+		{"<Compose>/Edit/Paste",				"<control>V"},
 		{"<Compose>/Edit/Select all",				"<control>A"},
-		{"<Compose>/Edit/Advanced/Move a word backward",	""},
+		{"<Compose>/Edit/Advanced/Move a character backward",	"<control>B"},
+		{"<Compose>/Edit/Advanced/Move a character forward",	"<control>F"},
+		{"<Compose>/Edit/Advanced/Move a word backward,"	""},
 		{"<Compose>/Edit/Advanced/Move a word forward",		""},
 		{"<Compose>/Edit/Advanced/Move to beginning of line",	""},
+		{"<Compose>/Edit/Advanced/Move to end of line",		"<control>E"},
+		{"<Compose>/Edit/Advanced/Move to previous line",	"<control>P"},
+		{"<Compose>/Edit/Advanced/Move to next line",		"<control>N"},
+		{"<Compose>/Edit/Advanced/Delete a character backward",	"<control>H"},
+		{"<Compose>/Edit/Advanced/Delete a character forward",	"<control>D"},
 		{"<Compose>/Edit/Advanced/Delete a word backward",	""},
 		{"<Compose>/Edit/Advanced/Delete a word forward",	""},
+		{"<Compose>/Edit/Advanced/Delete line",			"<control>U"},
+		{"<Compose>/Edit/Advanced/Delete entire line",		""},
+		{"<Compose>/Edit/Advanced/Delete to end of line",	"<control>K"},
+		{"<Compose>/Edit/Wrap current paragraph",		"<control>L"},
+		{"<Compose>/Edit/Wrap all long lines",			"<control><alt>L"},
+		{"<Compose>/Edit/Auto wrapping",			"<shift><control>L"},
+		{"<Compose>/Edit/Edit with external editor",		"<shift><control>X"},
+		{"<Compose>/Tools/Address book",			"<shift><control>A"},
 	};
 
 	static struct KeyBind mew_wl_menurc[] = {
-		{"<Main>/File/Empty all Trash folders",			"<shift>D"},
+		{"<Main>/File/Empty all Trash folders",		"<shift>D"},
 		{"<Main>/File/Save as...",			"Y"},
 		{"<Main>/File/Print...",			"<shift>numbersign"},
 		{"<Main>/File/Exit",				"<shift>Q"},
@@ -508,9 +536,9 @@ static void prefs_keybind_apply_clicked(GtkWidget *widget)
 
 		{"<Main>/View/Show or hide/Message View",	""},
 		{"<Main>/View/Thread view",			"<shift>T"},
-		{"<Main>/View/Go to/Prev message",		"P"},
+		{"<Main>/View/Go to/Previous message",		"P"},
 		{"<Main>/View/Go to/Next message",		"N"},
-		{"<Main>/View/Go to/Prev unread message",	"<shift>P"},
+		{"<Main>/View/Go to/Previous unread message",	"<shift>P"},
 		{"<Main>/View/Go to/Next unread message",	"<shift>N"},
 		{"<Main>/View/Go to/Other folder...",		"G"},
 		{"<Main>/View/Open in new window",		"<control><alt>N"},
@@ -562,9 +590,9 @@ static void prefs_keybind_apply_clicked(GtkWidget *widget)
 
 		{"<Main>/View/Show or hide/Message view",	"V"},
 		{"<Main>/View/Thread view",			"<control>T"},
-		{"<Main>/View/Go to/Prev message",		""},
+		{"<Main>/View/Go to/Previous message",		""},
 		{"<Main>/View/Go to/Next message",		""},
-		{"<Main>/View/Go to/Prev unread message",	""},
+		{"<Main>/View/Go to/Previous unread message",	""},
 		{"<Main>/View/Go to/Next unread message",	""},
 		{"<Main>/View/Go to/Other folder...",		"C"},
 		{"<Main>/View/Open in new window",		"<control><alt>N"},
@@ -616,9 +644,9 @@ static void prefs_keybind_apply_clicked(GtkWidget *widget)
 
 		{"<Main>/View/Show or hide/Message View",	""},
 		{"<Main>/View/Thread view",			"<control>T"},
-		{"<Main>/View/Go to/Prev message",		"P"},
+		{"<Main>/View/Go to/Previous message",		"P"},
 		{"<Main>/View/Go to/Next message",		"N"},
-		{"<Main>/View/Go to/Prev unread message",	"<shift>P"},
+		{"<Main>/View/Go to/Previous unread message",	"<shift>P"},
 		{"<Main>/View/Go to/Next unread message",	"<shift>N"},
 		{"<Main>/View/Go to/Other folder...",		"<alt>G"},
 		{"<Main>/View/Open in new window",		"<shift><control>N"},
@@ -702,23 +730,29 @@ void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 	GtkWidget *label_datefmt;
 	GtkWidget *button_datefmt;
 	GtkWidget *entry_datefmt;
+	GtkTooltips *tip_datefmt;
 	GtkWidget *hbox_dispitem;
 	GtkWidget *frame_dispitem;
 	GtkWidget *button_dispitem;
 
+	GtkWidget *vbox4;
+	GtkWidget *hbox2;
 	GtkWidget *checkbtn_always_show_msg;
 	GtkWidget *checkbtn_mark_as_read_on_newwin;
-	GtkWidget *checkbtn_openinbox;
 	GtkWidget *checkbtn_immedexec;
+	GtkWidget *checkbtn_ask_mark_all_read;
 	GtkTooltips *immedexec_tooltip;
 	GtkWidget *label;
 	GtkWidget *menu;
 	GtkWidget *menuitem;
 	GtkWidget *button_keybind;
-	GtkWidget *hbox_select_on_entry;
  	GtkWidget *optmenu_select_on_entry;
-	GtkWidget *hbox_nextunreadmsgdialog;
  	GtkWidget *optmenu_nextunreadmsgdialog;
+	GtkWidget *table;
+	GtkWidget *vbox5;
+	GtkTooltips *tooltips;
+
+	tooltips = gtk_tooltips_new();
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
@@ -730,13 +764,14 @@ void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 
 	PACK_CHECK_BUTTON
 		(vbox2, chkbtn_transhdr,
-		 _("Translate header name (such as 'From:', 'Subject:')"));
+		 _("Translate header names"));
+	gtk_tooltips_set_tip(tooltips, chkbtn_transhdr,
+			     "The display of standard headers (such as 'From:', 'Subject:') "
+			     "will be translated into your language.", NULL);
 
 	PACK_CHECK_BUTTON (vbox2, chkbtn_folder_unread,
 			   _("Display unread number next to folder name"));
 
-	PACK_VSPACER(vbox2, vbox3, VSPACING_NARROW_2);
-
 	hbox1 = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox1);
 	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, TRUE, 0);
@@ -762,23 +797,21 @@ void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 
 	/* ---- Summary ---- */
 
-	vbox2 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox2);
-	gtk_container_add (GTK_CONTAINER (vbox1), vbox2);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 0);
+	vbox3 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox3);
+	gtk_container_add (GTK_CONTAINER (vbox2), vbox3);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox3), 0);
 
 	PACK_CHECK_BUTTON
-		(vbox2, chkbtn_useaddrbook,
+		(vbox3, chkbtn_useaddrbook,
 		 _("Display sender using address book"));
 	PACK_CHECK_BUTTON
-		(vbox2, chkbtn_threadsubj,
+		(vbox3, chkbtn_threadsubj,
 		 _("Thread using subject in addition to standard headers"));
 
-	PACK_VSPACER(vbox2, vbox3, VSPACING_NARROW_2);
-
 	hbox1 = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox1);
-	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox3), hbox1, FALSE, TRUE, 0);
 
 	label_datefmt = gtk_label_new (_("Date format"));
 	gtk_widget_show (label_datefmt);
@@ -794,10 +827,14 @@ void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 	gtk_box_pack_start (GTK_BOX (hbox1), button_datefmt, FALSE, FALSE, 0);
 	g_signal_connect (G_OBJECT (button_datefmt), "clicked",
 			  G_CALLBACK (date_format_create), NULL);
+	tip_datefmt = gtk_tooltips_new();
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tip_datefmt),
+			     button_datefmt,
+			     _("Date format help"), NULL);
 
 	PACK_VSPACER(vbox2, vbox3, VSPACING_NARROW);
 
-	PACK_FRAME(vbox2, frame_dispitem, _("Set displayed columns"));
+	PACK_FRAME(vbox3, frame_dispitem, _("Set displayed columns"));
 
 	hbox_dispitem = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox_dispitem);
@@ -820,9 +857,9 @@ void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 			  G_CALLBACK (prefs_summary_column_open),
 			  NULL);
 
-	vbox2 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox2);
-	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);
+	vbox4 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox4);
+	gtk_box_pack_start (GTK_BOX (vbox1), vbox4, FALSE, FALSE, 0);
 
 	/* PACK_CHECK_BUTTON (vbox2, checkbtn_emacs,
 			   _("Emulate the behavior of mouse operation of\n"
@@ -830,49 +867,54 @@ void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 	gtk_label_set_justify (GTK_LABEL (GTK_BIN (checkbtn_emacs)->child),
 			       GTK_JUSTIFY_LEFT);   */
 
-	PACK_CHECK_BUTTON
-		(vbox2, checkbtn_always_show_msg,
-		 _("Always open message when selected"));
-
-	PACK_CHECK_BUTTON
-		(vbox2, checkbtn_mark_as_read_on_newwin,
-		 _("Only mark message as read when opened in new window"));
-
-	PACK_CHECK_BUTTON
-		(vbox2, checkbtn_openinbox,
-		 _("Go to inbox after receiving new mail"));
-
-	vbox3 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox3);
-	gtk_box_pack_start (GTK_BOX (vbox2), vbox3, FALSE, FALSE, 0);
-
 	immedexec_tooltip = gtk_tooltips_new();
 
 	PACK_CHECK_BUTTON
-		(vbox3, checkbtn_immedexec,
+		(vbox4, checkbtn_immedexec,
 		 _("Execute immediately when moving or deleting messages"));
 	gtk_tooltips_set_tip(GTK_TOOLTIPS(immedexec_tooltip), checkbtn_immedexec,
 			     _("Messages will be marked until execution"
 		   	       " if this is turned off"),
 			     NULL);
 
-	hbox1 = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox1);
-	gtk_box_pack_start (GTK_BOX (vbox3), hbox1, FALSE, FALSE, 0);
+	PACK_CHECK_BUTTON
+		(vbox4, checkbtn_ask_mark_all_read,
+		 _("Confirm before marking all mails in a folder as read"));
 
-	hbox_select_on_entry = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox_select_on_entry);
-	gtk_box_pack_start (GTK_BOX (vbox1), hbox_select_on_entry, FALSE, FALSE, 0);
+	PACK_CHECK_BUTTON
+		(vbox4, checkbtn_always_show_msg,
+		 _("Always open message when selected"));
+
+	PACK_CHECK_BUTTON
+		(vbox4, checkbtn_mark_as_read_on_newwin,
+		 _("Only mark message as read when opened in a new window"));
+
+
+	vbox5 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox5);
+	gtk_box_pack_start (GTK_BOX (vbox1), vbox5, FALSE, FALSE, 0);
+
+	table = gtk_table_new(1, 1, FALSE);
+	gtk_widget_show(table);
+	gtk_container_add (GTK_CONTAINER (vbox5), table);
+	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
 
 	label = gtk_label_new (_("When entering a folder"));
 	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (hbox_select_on_entry), label, FALSE, FALSE, 8);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
+			(GtkAttachOptions) (GTK_FILL),
+			(GtkAttachOptions) (0), 0, 0);
+	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
  	optmenu_select_on_entry = gtk_option_menu_new ();
  	gtk_widget_show (optmenu_select_on_entry);
-	gtk_box_pack_start (GTK_BOX (hbox_select_on_entry),
-			    optmenu_select_on_entry, FALSE, FALSE, 8);
-	
+
+	gtk_table_attach(GTK_TABLE(table), optmenu_select_on_entry, 1, 2, 0, 1,
+			(GtkAttachOptions) (GTK_FILL),
+			(GtkAttachOptions) (0), 0, 0);
+
 	menu = gtk_menu_new ();
 	MENUITEM_ADD (menu, menuitem, _("Do nothing"), 0);
 	MENUITEM_ADD (menu, menuitem, _("Select first unread (or new) message"),
@@ -881,20 +923,29 @@ void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 		      SELECTONENTRY_NEW);
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu_select_on_entry), menu);
- 	
+
 	/* Next Unread Message Dialog */
-	hbox_nextunreadmsgdialog = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox_nextunreadmsgdialog);
-	gtk_box_pack_start (GTK_BOX (vbox1), hbox_nextunreadmsgdialog, FALSE, FALSE, 0);
+	table = gtk_table_new(1, 1, FALSE);
+	gtk_widget_show(table);
+	gtk_container_add (GTK_CONTAINER (vbox5), table);
+	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
 
 	label = gtk_label_new (_("Show \"no unread (or new) message\" dialog"));
 	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (hbox_nextunreadmsgdialog), label, FALSE, FALSE, 8);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
+			(GtkAttachOptions) (GTK_FILL),
+			(GtkAttachOptions) (0), 0, 0);
+	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
+	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
  	optmenu_nextunreadmsgdialog = gtk_option_menu_new ();
  	gtk_widget_show (optmenu_nextunreadmsgdialog);
-	gtk_box_pack_start (GTK_BOX (hbox_nextunreadmsgdialog), optmenu_nextunreadmsgdialog, FALSE, FALSE, 8);
-	
+
+	gtk_table_attach(GTK_TABLE(table), optmenu_nextunreadmsgdialog, 1, 2, 1, 2,
+			(GtkAttachOptions) (GTK_FILL),
+			(GtkAttachOptions) (0), 0, 0);
+
 	menu = gtk_menu_new ();
 	MENUITEM_ADD (menu, menuitem, _("Always"), NEXTUNREADMSGDIALOG_ALWAYS);
 	MENUITEM_ADD (menu, menuitem, _("Assume 'Yes'"), 
@@ -904,13 +955,13 @@ void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu_nextunreadmsgdialog), menu);
 
-	hbox1 = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox1);
-	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
+	hbox2 = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox2);
+	gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, FALSE, 0);
 
 	button_keybind = gtk_button_new_with_label (_(" Set key bindings... "));
 	gtk_widget_show (button_keybind);
-	gtk_box_pack_start (GTK_BOX (hbox1), button_keybind, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox2), button_keybind, FALSE, FALSE, 0);
 	g_signal_connect (G_OBJECT (button_keybind), "clicked",
 			  G_CALLBACK (prefs_keybind_select), NULL);
 
@@ -934,10 +985,10 @@ void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 			prefs_common.always_show_msg);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_mark_as_read_on_newwin),
 			prefs_common.mark_as_read_on_new_window);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_openinbox),
-			prefs_common.open_inbox_on_inc);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_immedexec),
 			prefs_common.immediate_exec);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_ask_mark_all_read),
+			prefs_common.ask_mark_all_read);
 
 	gtk_option_menu_set_history(GTK_OPTION_MENU(optmenu_select_on_entry),
 			prefs_common.select_on_entry);
@@ -953,8 +1004,8 @@ void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 
 	prefs_summaries->checkbtn_always_show_msg = checkbtn_always_show_msg;
 	prefs_summaries->checkbtn_mark_as_read_on_newwin = checkbtn_mark_as_read_on_newwin;
-	prefs_summaries->checkbtn_openinbox = checkbtn_openinbox;
 	prefs_summaries->checkbtn_immedexec = checkbtn_immedexec;
+	prefs_summaries->checkbtn_ask_mark_all_read = checkbtn_ask_mark_all_read;
 	prefs_summaries->optmenu_select_on_entry = optmenu_select_on_entry;
 	prefs_summaries->optmenu_nextunreadmsgdialog = optmenu_nextunreadmsgdialog;
 
@@ -986,10 +1037,10 @@ void prefs_summaries_save(PrefsPage *_page)
 		GTK_TOGGLE_BUTTON(page->checkbtn_always_show_msg));
 	prefs_common.mark_as_read_on_new_window = gtk_toggle_button_get_active(
 		GTK_TOGGLE_BUTTON(page->checkbtn_mark_as_read_on_newwin));
-	prefs_common.open_inbox_on_inc = gtk_toggle_button_get_active(
-		GTK_TOGGLE_BUTTON(page->checkbtn_openinbox));
 	prefs_common.immediate_exec = gtk_toggle_button_get_active(
 		GTK_TOGGLE_BUTTON(page->checkbtn_immedexec));
+	prefs_common.ask_mark_all_read = gtk_toggle_button_get_active(
+		GTK_TOGGLE_BUTTON(page->checkbtn_ask_mark_all_read));
 
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(page->optmenu_select_on_entry));
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
