@@ -1,7 +1,7 @@
 /*
  * Sylpheed templates subsystem 
  * Copyright (C) 2001 Alexander Barinov
- * Copyright (C) 2001-2005 Hiroyuki Yamamoto
+ * Copyright (C) 2001-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "defs.h"
@@ -85,7 +85,8 @@ static GtkListStore* prefs_template_create_data_store	(void);
 static void prefs_template_list_view_insert_template	(GtkWidget *list_view,
 							 GtkTreeIter *row_iter,
 							 const gchar *template,
-							 Template *data);
+							 Template *data,
+							 gboolean sorted);
 static GtkWidget *prefs_template_list_view_create	(void);
 static void prefs_template_create_list_view_columns	(GtkWidget *list_view);
 static gboolean prefs_template_selected			(GtkTreeSelection *selector,
@@ -106,6 +107,18 @@ void prefs_template_open(void)
 	gtk_widget_show(templates.window);
 }
 
+/*!
+ *\brief	Save Gtk object size to prefs dataset
+ */
+static void prefs_template_size_allocate_cb(GtkWidget *widget,
+					 GtkAllocation *allocation)
+{
+	g_return_if_fail(allocation != NULL);
+
+	prefs_common.templateswin_width = allocation->width;
+	prefs_common.templateswin_height = allocation->height;
+}
+
 #define ADD_ENTRY(entry, str, row) \
 { \
 	label1 = gtk_label_new(str); \
@@ -149,13 +162,15 @@ static void prefs_template_window_create(void)
 	GtkWidget       *confirm_area;
 	GtkWidget         *ok_btn;
 	GtkWidget         *cancel_btn;
+	static GdkGeometry geometry;
+
+	debug_print("Creating templates configuration window...\n");
 
 	/* main window */
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
-	gtk_window_set_default_size(GTK_WINDOW(window), 400, -1);
 
 	/* vpaned to separate template settings from templates list */
 	vpaned = gtk_vpaned_new();
@@ -190,7 +205,9 @@ static void prefs_template_window_create(void)
 	ADD_ENTRY(entry_to, _("To:"), 0);
 	address_completion_register_entry(GTK_ENTRY(entry_to));
 	ADD_ENTRY(entry_cc, _("Cc:"), 1)
+	address_completion_register_entry(GTK_ENTRY(entry_cc));
 	ADD_ENTRY(entry_bcc, _("Bcc:"), 2)	
+	address_completion_register_entry(GTK_ENTRY(entry_bcc));
 	ADD_ENTRY(entry_subject, _("Subject:"), 3);
 
 #undef ADD_ENTRY
@@ -251,7 +268,7 @@ static void prefs_template_window_create(void)
 	g_signal_connect(G_OBJECT(del_btn), "clicked",
 			 G_CALLBACK(prefs_template_delete_cb), NULL);
 
-	desc_btn = gtk_button_new_with_label(_(" Symbols "));
+	desc_btn = gtk_button_new_with_label(_(" Symbols... "));
 	gtk_widget_show(desc_btn);
 	gtk_box_pack_end(GTK_BOX(hbox2), desc_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(desc_btn), "clicked",
@@ -271,8 +288,8 @@ static void prefs_template_window_create(void)
 	gtk_container_add(GTK_CONTAINER(scroll1), list_view);
 
 	/* ok | cancel */
-	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
-				&cancel_btn, GTK_STOCK_CANCEL, NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area, &cancel_btn, GTK_STOCK_CANCEL,
+				      &ok_btn, GTK_STOCK_OK, NULL, NULL);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end(GTK_BOX(vbox2), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
@@ -281,6 +298,8 @@ static void prefs_template_window_create(void)
 
 	g_signal_connect(G_OBJECT(window), "delete_event",
 			 G_CALLBACK(prefs_template_deleted_cb), NULL);
+	g_signal_connect(G_OBJECT(window), "size_allocate",
+			 G_CALLBACK(prefs_template_size_allocate_cb), NULL);
 	g_signal_connect(G_OBJECT(window), "key_press_event",
 			 G_CALLBACK(prefs_template_key_pressed_cb), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
@@ -291,6 +310,16 @@ static void prefs_template_window_create(void)
 
 	address_completion_start(window);
 
+	if (!geometry.min_height) {
+		geometry.min_width = 440;
+		geometry.min_height = 500;
+	}
+
+	gtk_window_set_geometry_hints(GTK_WINDOW(window), NULL, &geometry,
+				      GDK_HINT_MIN_SIZE);
+	gtk_widget_set_size_request(window, prefs_common.templateswin_width,
+				    prefs_common.templateswin_height);
+
 	templates.window = window;
 	templates.ok_btn = ok_btn;
 	templates.list_view = list_view;
@@ -319,7 +348,7 @@ static void prefs_template_window_setup(void)
 
 	prefs_template_list_view_insert_template(templates.list_view,
 						 NULL, _("(New)"),
-						 NULL);
+						 NULL, FALSE);
 	
 	tmpl_list = template_read_config();
 
@@ -327,7 +356,7 @@ static void prefs_template_window_setup(void)
 		tmpl = (Template *)cur->data;
 		prefs_template_list_view_insert_template(templates.list_view,
 							 NULL, tmpl->name, 
-							 tmpl);
+							 tmpl, FALSE);
 	}
 
 	g_slist_free(tmpl_list);
@@ -370,7 +399,8 @@ static void prefs_template_ok_cb(void)
 
 	if (modified && alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
-				 GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT) {
+				 GTK_STOCK_CLOSE, _("_Continue editing"), 
+				 NULL) != G_ALERTDEFAULT) {
 		return;
 	}
 	modified = FALSE;
@@ -386,7 +416,8 @@ static void prefs_template_cancel_cb(void)
 {
 	if (modified && alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
-				 GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT) {
+				 GTK_STOCK_CLOSE, _("_Continue editing"),
+				 NULL) != G_ALERTDEFAULT) {
 		return;
 	}
 	modified = FALSE;
@@ -445,7 +476,8 @@ static GSList *prefs_template_get_list(void)
 	return tmpl_list;
 }
 
-static void prefs_template_list_view_set_row(GtkTreeIter *row)
+static gboolean prefs_template_list_view_set_row(GtkTreeIter *row)
+/* return TRUE if the row could be modified */
 {
 	Template *tmpl;
 	gchar *name;
@@ -470,19 +502,23 @@ static void prefs_template_list_view_set_row(GtkTreeIter *row)
 		MsgInfo dummyinfo;
 
 		memset(&dummyinfo, 0, sizeof(MsgInfo));
-		quote_fmt_init(&dummyinfo, NULL, NULL);
+		quote_fmt_init(&dummyinfo, NULL, NULL, TRUE);
 		quote_fmt_scan_string(value);
 		quote_fmt_parse();
 		parsed_buf = quote_fmt_get_buffer();
 		if (!parsed_buf) {
 			alertpanel_error(_("Template format error."));
 			g_free(value);
-			return;
+			return FALSE;
 		}
 	}
 
 	name = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_name),
 				      0, -1);
+	if (name[0] == '\0') {
+		alertpanel_error(_("Template name is not set."));
+		return FALSE;
+	}
 	subject = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_subject),
 					 0, -1);
 	to = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_to),
@@ -518,13 +554,13 @@ static void prefs_template_list_view_set_row(GtkTreeIter *row)
 	tmpl->value = value;
 
 	prefs_template_list_view_insert_template(templates.list_view,
-						 row, tmpl->name, tmpl);
+						 row, tmpl->name, tmpl, TRUE);
+	return TRUE;
 }
 
 static void prefs_template_register_cb(void)
 {
-	prefs_template_list_view_set_row(NULL);
-	modified = FALSE;
+	modified = !prefs_template_list_view_set_row(NULL);
 }
 
 static void prefs_template_substitute_cb(void)
@@ -546,8 +582,7 @@ static void prefs_template_substitute_cb(void)
 
 	if (!tmpl) return;
 
-	prefs_template_list_view_set_row(&row);
-	modified = FALSE;
+	modified = !prefs_template_list_view_set_row(&row);
 }
 
 static void prefs_template_delete_cb(void)
@@ -572,7 +607,8 @@ static void prefs_template_delete_cb(void)
 
 	if (alertpanel(_("Delete template"),
 		       _("Do you really want to delete this template?"),
-		       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
+		       GTK_STOCK_CANCEL, GTK_STOCK_DELETE,
+		       NULL) != G_ALERTALTERNATE)
 		return;
 
 	gtk_list_store_remove(GTK_LIST_STORE(model), &row);		
@@ -587,23 +623,83 @@ static GtkListStore* prefs_template_create_data_store(void)
 				  -1);
 }
 
+static gboolean prefs_template_list_view_find_insertion_point(GtkTreeModel* model,
+							const gchar* template,
+							GtkTreeIter* iter)
+{
+	GtkTreeIter i;
+	gchar *text = NULL;
+
+	if ((gtk_tree_model_get_iter_first(model, &i))) {
+		for ( ; gtk_tree_model_iter_next(model, &i); ) {
+			gtk_tree_model_get(model, &i, TEMPL_TEXT, &text, -1);
+			if (strcmp(text, template) >= 0) {
+				g_free(text);
+				*iter = i;
+				return TRUE;
+			}
+			g_free(text);
+		}
+	}
+	/* cannot add before (need to append) */
+	return FALSE;
+}
+
 static void prefs_template_list_view_insert_template(GtkWidget *list_view,
 						     GtkTreeIter *row_iter,
 						     const gchar *template,
-						     Template *data)
+						     Template *data,
+							 gboolean sorted)
 {
 	GtkTreeIter iter;
-	GtkListStore *list_store = GTK_LIST_STORE(gtk_tree_view_get_model
-					(GTK_TREE_VIEW(list_view)));
+	GtkTreeModel *model = gtk_tree_view_get_model
+					(GTK_TREE_VIEW(list_view));
+	GtkListStore *list_store = GTK_LIST_STORE(model);
 	GAuto *auto_data;					
 
 	if (row_iter == NULL) {
-		/* append new */
-		gtk_list_store_append(list_store, &iter);
-	} else {
-		iter = *row_iter;
-	}
+		if (sorted) {
+			GtkTreeIter insertion_point;
 
+			/* insert sorted */
+			if (prefs_template_list_view_find_insertion_point(model,
+					template, &insertion_point)) {
+				gtk_list_store_insert_before(list_store, &iter,
+						&insertion_point);
+			} else {
+				gtk_list_store_append(list_store, &iter);
+			}
+		} else {
+			/* append new */
+			gtk_list_store_append(list_store, &iter);
+		}
+	} else {
+		/* modify the existing */
+		iter = *row_iter;
+
+		if (sorted) {
+			GtkTreeIter insertion_point;
+			gchar *text = NULL;
+
+			/* force re-sorting if template's name changed */
+			gtk_tree_model_get(model, row_iter, TEMPL_TEXT, &text, -1);
+			if (strcmp(text, template) != 0) {
+
+				/* move the modified template */
+				if (prefs_template_list_view_find_insertion_point(model,
+						template, &insertion_point)) {
+					gtk_list_store_move_before(list_store, row_iter,
+							&insertion_point);
+				} else {
+					/* move to the end */
+					gtk_list_store_move_before(list_store, row_iter,
+							NULL);
+				}
+			}
+			g_free(text);
+		}
+	}
+		
 	auto_data = g_auto_pointer_new_with_free(data, (GFreeFunc) template_free);  
 
 	/* if replacing data in an existing row, the auto pointer takes care
@@ -703,5 +799,3 @@ static gboolean prefs_template_selected(GtkTreeSelection *selector,
 
 	return TRUE;
 }
-
-
