@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -165,154 +165,6 @@ gint procheader_get_one_field_asis(gchar *buf, size_t len, FILE *fp)
 				     FALSE);
 }
 
-#if 0
-gchar *procheader_get_unfolded_line(gchar *buf, size_t len, FILE *fp)
-{
-	gboolean folded = FALSE;
-	gint nexthead;
-	gchar *bufp;
-
-	if (fgets(buf, len, fp) == NULL) return NULL;
-	if (buf[0] == '\r' || buf[0] == '\n') return NULL;
-	bufp = buf + strlen(buf);
-
-	for (; bufp > buf &&
-	     (*(bufp - 1) == '\n' || *(bufp - 1) == '\r');
-	     bufp--)
-		*(bufp - 1) = '\0';
-
-	while (1) {
-		nexthead = fgetc(fp);
-
-		/* folded */
-		if (nexthead == ' ' || nexthead == '\t')
-			folded = TRUE;
-		else if (nexthead == EOF)
-			break;
-		else if (folded == TRUE) {
-			if ((len - (bufp - buf)) <= 2) break;
-
-			if (nexthead == '\n') {
-				folded = FALSE;
-				continue;
-			}
-
-			/* replace return code on the tail end
-			   with space */
-			*bufp++ = ' ';
-			*bufp++ = nexthead;
-			*bufp = '\0';
-
-			/* concatenate next line */
-			if (fgets(bufp, len - (bufp - buf), fp)
-			    == NULL) break;
-			bufp += strlen(bufp);
-
-			for (; bufp > buf &&
-			     (*(bufp - 1) == '\n' || *(bufp - 1) == '\r');
-			     bufp--)
-				*(bufp - 1) = '\0';
-
-			folded = FALSE;
-		} else {
-			ungetc(nexthead, fp);
-			break;
-		}
-	}
-
-	/* remove trailing return code */
-	strretchomp(buf);
-
-	return buf;
-}
-#endif
-
-#if 0
-GSList *procheader_get_header_list_from_file(const gchar *file)
-{
-	FILE *fp;
-	GSList *hlist;
-
-	if ((fp = g_fopen(file, "rb")) == NULL) {
-		FILE_OP_ERROR(file, "fopen");
-		return NULL;
-	}
-
-	hlist = procheader_get_header_list(fp);
-
-	fclose(fp);
-	return hlist;
-}
-
-GSList *procheader_get_header_list(FILE *fp)
-{
-	gchar buf[BUFFSIZE];
-	GSList *hlist = NULL;
-	Header *header;
-
-	g_return_val_if_fail(fp != NULL, NULL);
-
-	while (procheader_get_unfolded_line(buf, sizeof(buf), fp) != NULL) {
-		if ((header = procheader_parse_header(buf)) != NULL)
-			hlist = g_slist_append(hlist, header);
-		/*
-		if (*buf == ':') continue;
-		for (p = buf; *p && *p != ' '; p++) {
-			if (*p == ':') {
-				header = g_new(Header, 1);
-				header->name = g_strndup(buf, p - buf);
-				p++;
-				while (*p == ' ' || *p == '\t') p++;
-				conv_unmime_header(tmp, sizeof(tmp), p, NULL);
-				header->body = g_strdup(tmp);
-
-				hlist = g_slist_append(hlist, header);
-				break;
-			}
-		}
-		*/
-	}
-
-	return hlist;
-}
-#endif
-
-#if 0
-GPtrArray *procheader_get_header_array(FILE *fp)
-{
-	gchar buf[BUFFSIZE];
-	GPtrArray *headers;
-	Header *header;
-
-	g_return_val_if_fail(fp != NULL, NULL);
-
-	headers = g_ptr_array_new();
-
-	while (procheader_get_unfolded_line(buf, sizeof(buf), fp) != NULL) {
-		if ((header = procheader_parse_header(buf)) != NULL)
-			g_ptr_array_add(headers, header);
-		/*
-		if (*buf == ':') continue;
-		for (p = buf; *p && *p != ' '; p++) {
-			if (*p == ':') {
-				header = g_new(Header, 1);
-				header->name = g_strndup(buf, p - buf);
-				p++;
-				while (*p == ' ' || *p == '\t') p++;
-				conv_unmime_header(tmp, sizeof(tmp), p, NULL);
-				header->body = g_strdup(tmp);
-
-				g_ptr_array_add(headers, header);
-				break;
-			}
-		}
-		*/
-	}
-
-	return headers;
-}
-#endif
-
 GPtrArray *procheader_get_header_array_asis(FILE *fp)
 {
 	gchar buf[BUFFSIZE];
@@ -346,19 +198,6 @@ GPtrArray *procheader_get_header_array_asis(FILE *fp)
 	return headers;
 }
 
-#if 0
-void procheader_header_list_destroy(GSList *hlist)
-{
-	Header *header;
-
-	while (hlist != NULL) {
-		header = hlist->data;
-		procheader_header_free(header);
-		hlist = g_slist_remove(hlist, header);
-	}
-}
-#endif
-
 void procheader_header_array_destroy(GPtrArray *harray)
 {
 	gint i;
@@ -512,12 +351,13 @@ enum
 	H_FROM_SPACE	= 13,
 	H_SC_PLANNED_DOWNLOAD = 14,
 	H_SC_MESSAGE_SIZE = 15,
-	H_X_FACE	= 16,
-	H_DISPOSITION_NOTIFICATION_TO = 17,
-	H_RETURN_RECEIPT_TO = 18,
-	H_SC_PARTIALLY_RETRIEVED = 19,
-	H_SC_ACCOUNT_SERVER = 20,
-	H_SC_ACCOUNT_LOGIN = 21,
+	H_FACE		= 16,
+	H_X_FACE	= 17,
+	H_DISPOSITION_NOTIFICATION_TO = 18,
+	H_RETURN_RECEIPT_TO = 19,
+	H_SC_PARTIALLY_RETRIEVED = 20,
+	H_SC_ACCOUNT_SERVER = 21,
+	H_SC_ACCOUNT_LOGIN = 22,
 };
 
 static HeaderEntry hentry_full[] = {{"Date:",		NULL, FALSE},
@@ -536,6 +376,7 @@ static HeaderEntry hentry_full[] = {{"Date:",		NULL, FALSE},
 				   {"From ",		NULL, FALSE},
 				   {"SC-Marked-For-Download:", NULL, FALSE},
 				   {"SC-Message-Size:", NULL, FALSE},
+				   {"Face:",		NULL, FALSE},
 				   {"X-Face:",		NULL, FALSE},
 				   {"Disposition-Notification-To:", NULL, FALSE},
 				   {"Return-Receipt-To:", NULL, FALSE},
@@ -612,7 +453,22 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			if (msginfo->date) break;
 			msginfo->date_t =
 				procheader_date_parse(NULL, hp, 0);
-			msginfo->date = g_strdup(hp);
+			if (g_utf8_validate(hp, -1, NULL)) {
+				msginfo->date = g_strdup(hp);
+			} else {
+				gchar *utf = conv_codeset_strdup(
+					hp, 
+					conv_get_locale_charset_str_no_utf8(),
+					CS_INTERNAL);
+				if (utf == NULL || 
+				    !g_utf8_validate(utf, -1, NULL)) {
+					g_free(utf);
+					utf = g_malloc(strlen(buf)*2+1);
+					conv_localetodisp(utf, 
+						strlen(hp)*2+1, hp);
+				}
+				msginfo->date = utf;
+			}
 			break;
 		case H_FROM:
 			if (msginfo->from) break;
@@ -691,6 +547,10 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			MSG_UNSET_PERM_FLAGS(msginfo->flags, MSG_NEW|MSG_UNREAD);
 			break;
 #endif			
+		case H_FACE:
+			if (msginfo->face) break;
+			msginfo->face = g_strdup(hp);
+			break;
 		case H_X_FACE:
 			if (msginfo->xface) break;
 			msginfo->xface = g_strdup(hp);
