@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto & The Sylpheed-Claws Team
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto & The Sylpheed-Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,7 +28,9 @@
 #include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
-#include <locale.h>
+#if HAVE_LOCALE_H
+#  include <locale.h>
+#endif
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -116,6 +118,7 @@ static gboolean free_func(GNode *node, gpointer data)
 	g_free(mimeinfo->subtype);
 	g_free(mimeinfo->description);
 	g_free(mimeinfo->id);
+	g_free(mimeinfo->location);
 
 	g_hash_table_foreach_remove(mimeinfo->typeparameters,
 		procmime_mimeinfo_parameters_destroy, NULL);
@@ -145,53 +148,6 @@ void procmime_mimeinfo_free_all(MimeInfo *mimeinfo)
 	g_node_destroy(node);
 }
 
-#if 0 /* UNUSED */
-MimeInfo *procmime_mimeinfo_insert(MimeInfo *parent, MimeInfo *mimeinfo)
-{
-	MimeInfo *child = parent->children;
-
-	if (!child)
-		parent->children = mimeinfo;
-	else {
-		while (child->next != NULL)
-			child = child->next;
-
-		child->next = mimeinfo;
-	}
-
-	mimeinfo->parent = parent;
-	mimeinfo->level = parent->level + 1;
-
-	return mimeinfo;
-}
-
-void procmime_mimeinfo_replace(MimeInfo *old, MimeInfo *new)
-{
-	MimeInfo *parent = old->parent;
-	MimeInfo *child;
-
-	g_return_if_fail(parent != NULL);
-	g_return_if_fail(new->next == NULL);
-
-	for (child = parent->children; child && child != old;
-	     child = child->next)
-		;
-	if (!child) {
-		g_warning("oops: parent can't find it's own child");
-		return;
-	}
-	procmime_mimeinfo_free_all(old);
-
-	if (child == parent->children) {
-		new->next = parent->children->next;
-		parent->children = new;
-	} else {
-		new->next = child->next;
-		child = new;
-	}
-}
-#endif
-
 MimeInfo *procmime_mimeinfo_parent(MimeInfo *mimeinfo)
 {
 	g_return_val_if_fail(mimeinfo != NULL, NULL);
@@ -299,6 +255,7 @@ gboolean procmime_decode_content(MimeInfo *mimeinfo)
 	outfp = get_tmpfile_in_dir(get_mime_tmp_dir(), &tmpfilename);
 	if (!outfp) {
 		perror("tmpfile");
+		fclose(infp);
 		return FALSE;
 	}
 	tmp_file = TRUE;
@@ -325,6 +282,7 @@ gboolean procmime_decode_content(MimeInfo *mimeinfo)
 			if (!tmpfp) {
 				perror("tmpfile");
 				if (tmp_file) fclose(outfp);
+				fclose(infp);
 				return FALSE;
 			}
 		}
@@ -736,7 +694,7 @@ FILE *procmime_get_text_content(MimeInfo *mimeinfo)
 		oldout = dup(1);
 		
 		dup2(fileno(outfp), 1);
-		
+
 		p = popen(renderer->renderer, "w");
 		if (p != NULL) {
 			size_t count;
@@ -751,15 +709,15 @@ FILE *procmime_get_text_content(MimeInfo *mimeinfo)
 		dup2(oldout, 1);
 /* CodeConverter seems to have no effect here */
 	} else if (mimeinfo->type == MIMETYPE_TEXT && !g_ascii_strcasecmp(mimeinfo->subtype, "html")) {
-		HTMLParser *parser;
+		SC_HTMLParser *parser;
 		CodeConverter *conv;
 
 		conv = conv_code_converter_new(src_codeset);
-		parser = html_parser_new(tmpfp, conv);
-		while ((str = html_parse(parser)) != NULL) {
+		parser = sc_html_parser_new(tmpfp, conv);
+		while ((str = sc_html_parse(parser)) != NULL) {
 			fputs(str, outfp);
 		}
-		html_parser_destroy(parser);
+		sc_html_parser_destroy(parser);
 		conv_code_converter_destroy(conv);
 	} else if (mimeinfo->type == MIMETYPE_TEXT && !g_ascii_strcasecmp(mimeinfo->subtype, "enriched")) {
 		ERTFParser *parser;
@@ -980,8 +938,10 @@ gchar *procmime_get_tmp_file_name(MimeInfo *mimeinfo)
 		if (basetmp == NULL)
 			basetmp = "mimetmp";
 		basetmp = g_path_get_basename(basetmp);
-		if (*basetmp == '\0') basetmp = g_strdup("mimetmp");
+		if (*basetmp == '\0') 
+			basetmp = g_strdup("mimetmp");
 		base = conv_filename_from_utf8(basetmp);
+		g_free((gchar*)basetmp);
 		subst_for_shellsafe_filename(base);
 	}
 
@@ -1176,7 +1136,7 @@ EncodingType procmime_get_encoding_for_charset(const gchar *charset)
 		return ENC_8BIT;
 }
 
-EncodingType procmime_get_encoding_for_text_file(const gchar *file)
+EncodingType procmime_get_encoding_for_text_file(const gchar *file, gboolean *has_binary)
 {
 	FILE *fp;
 	guchar buf[BUFFSIZE];
@@ -1184,6 +1144,7 @@ EncodingType procmime_get_encoding_for_text_file(const gchar *file)
 	size_t octet_chars = 0;
 	size_t total_len = 0;
 	gfloat octet_percentage;
+	gboolean force_b64 = FALSE;
 
 	if ((fp = g_fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
@@ -1197,6 +1158,10 @@ EncodingType procmime_get_encoding_for_text_file(const gchar *file)
 		for (p = buf, i = 0; i < len; ++p, ++i) {
 			if (*p & 0x80)
 				++octet_chars;
+			if (*p == '\0') {
+				force_b64 = TRUE;
+				*has_binary = TRUE;
+			}
 		}
 		total_len += len;
 	}
@@ -1212,7 +1177,7 @@ EncodingType procmime_get_encoding_for_text_file(const gchar *file)
 		    "8bit chars: %d / %d (%f%%)\n", octet_chars, total_len,
 		    100.0 * octet_percentage);
 
-	if (octet_percentage > 0.20) {
+	if (octet_percentage > 0.20 || force_b64) {
 		debug_print("using BASE64\n");
 		return ENC_BASE64;
 	} else if (octet_chars > 0) {
@@ -1306,12 +1271,13 @@ gchar *procmime_get_content_type_str(MimeMediaType type,
 	return g_strdup_printf("%s/%s", type_str, subtype);
 }
 
-int procmime_parse_mimepart(MimeInfo *parent,
+static int procmime_parse_mimepart(MimeInfo *parent,
 			     gchar *content_type,
 			     gchar *content_encoding,
 			     gchar *content_description,
 			     gchar *content_id,
 			     gchar *content_disposition,
+			     gchar *content_location,
 			     const gchar *filename,
 			     guint offset,
 			     guint length);
@@ -1327,6 +1293,8 @@ void procmime_parse_message_rfc822(MimeInfo *mimeinfo)
 						   NULL, TRUE},
 				{"Content-Disposition:",
 				                   NULL, TRUE},
+				{"Content-Location:",
+						   NULL, TRUE},
 				{"MIME-Version:",
 						   NULL, TRUE},
 				{NULL,		   NULL, FALSE}};
@@ -1358,13 +1326,18 @@ void procmime_parse_message_rfc822(MimeInfo *mimeinfo)
                 g_free(hentry[4].body);
                 hentry[4].body = tmp;
         }                
+	if (hentry[5].body != NULL) {
+                tmp = conv_unmime_header(hentry[5].body, NULL);
+                g_free(hentry[5].body);
+                hentry[5].body = tmp;
+        }                
 	content_start = ftell(fp);
 	fclose(fp);
 	
 	procmime_parse_mimepart(mimeinfo,
 				hentry[0].body, hentry[1].body,
 				hentry[2].body, hentry[3].body,
-				hentry[4].body,
+				hentry[4].body, hentry[5].body,
 				mimeinfo->data.filename, content_start,
 				mimeinfo->length - (content_start - mimeinfo->offset));
 	
@@ -1385,6 +1358,8 @@ void procmime_parse_multipart(MimeInfo *mimeinfo)
 						   NULL, TRUE},
 				{"Content-Disposition:",
 				                   NULL, TRUE},
+				{"Content-Location:",
+						   NULL, TRUE},
 				{NULL,		   NULL, FALSE}};
 	gchar *p, *tmp;
 	gchar *boundary;
@@ -1415,7 +1390,7 @@ void procmime_parse_multipart(MimeInfo *mimeinfo)
 				result = procmime_parse_mimepart(mimeinfo,
 				                        hentry[0].body, hentry[1].body,
 							hentry[2].body, hentry[3].body, 
-							hentry[4].body, 
+							hentry[4].body, hentry[5].body,
 							mimeinfo->data.filename, lastoffset,
 							(ftell(fp) - strlen(buf)) - lastoffset - 1);
 			}
@@ -1444,6 +1419,11 @@ void procmime_parse_multipart(MimeInfo *mimeinfo)
                                 g_free(hentry[4].body);
                                 hentry[4].body = tmp;
                         }                
+                        if (hentry[5].body != NULL) {
+                                tmp = conv_unmime_header(hentry[5].body, NULL);
+                                g_free(hentry[5].body);
+                                hentry[5].body = tmp;
+                        }                
 			lastoffset = ftell(fp);
 		}
 	}
@@ -1482,6 +1462,8 @@ static void parse_parameters(const gchar *parameters, GHashTable *table)
 
 		value[0] = '\0';
 		value++;
+		while (value[0] == ' ')
+			value++;
 
 		g_strdown(attribute);
 
@@ -1518,6 +1500,18 @@ static void parse_parameters(const gchar *parameters, GHashTable *table)
 				*tmp = '\0';
 		}
 
+		if (attribute) {
+			while (attribute[0] == ' ')
+				attribute++;
+			while (attribute[strlen(attribute)-1] == ' ') 
+				attribute[strlen(attribute)-1] = '\0';
+		} 
+		if (value) {
+			while (value[0] == ' ')
+				value++;
+			while (value[strlen(value)-1] == ' ') 
+				value[strlen(value)-1] = '\0';
+		}		
 		if (strrchr(attribute, '*') != NULL) {
 			gchar *tmpattr;
 
@@ -1571,7 +1565,8 @@ static void parse_parameters(const gchar *parameters, GHashTable *table)
 		gchar *charset, *lang, *oldvalue, *newvalue;
 
 		attribute = (gchar *) cur->data;
-		if (!g_hash_table_lookup_extended(table, attribute, (gpointer *) &key, (gpointer *) &value))
+		if (!g_hash_table_lookup_extended(
+			table, attribute, (gpointer *)(gchar *) &key, (gpointer *)(gchar *) &value))
 			continue;
 
 		charset = value;
@@ -1611,10 +1606,12 @@ static void procmime_parse_content_type(const gchar *content_type, MimeInfo *mim
 		mimeinfo->type = MIMETYPE_TEXT;
 		mimeinfo->subtype = g_strdup("plain");
 		if (g_hash_table_lookup(mimeinfo->typeparameters,
-				       "charset") == NULL)
+				       "charset") == NULL) {
 			g_hash_table_insert(mimeinfo->typeparameters,
-					    g_strdup("charset"),
-					    g_strdup("us-ascii"));
+				    g_strdup("charset"),
+				    g_strdup(
+					conv_get_locale_charset_str_no_utf8()));
+		}
 	} else {
 		gchar *type, *subtype, *params;
 
@@ -1679,12 +1676,13 @@ static void procmime_parse_content_encoding(const gchar *content_encoding, MimeI
 	return;
 }
 
-int procmime_parse_mimepart(MimeInfo *parent,
+static int procmime_parse_mimepart(MimeInfo *parent,
 			     gchar *content_type,
 			     gchar *content_encoding,
 			     gchar *content_description,
 			     gchar *content_id,
 			     gchar *content_disposition,
+			     gchar *content_location,
 			     const gchar *filename,
 			     guint offset,
 			     guint length)
@@ -1715,8 +1713,12 @@ int procmime_parse_mimepart(MimeInfo *parent,
 		mimeinfo->type = MIMETYPE_TEXT;
 		mimeinfo->subtype = g_strdup("plain");
 		if (g_hash_table_lookup(mimeinfo->typeparameters,
-				       "charset") == NULL)
-			g_hash_table_insert(mimeinfo->typeparameters, g_strdup("charset"), g_strdup("us-ascii"));
+				       "charset") == NULL) {
+			g_hash_table_insert(mimeinfo->typeparameters,
+				    g_strdup("charset"),
+				    g_strdup(
+					conv_get_locale_charset_str_no_utf8()));
+		}
 	}
 
 	if (content_encoding != NULL) {
@@ -1735,6 +1737,11 @@ int procmime_parse_mimepart(MimeInfo *parent,
 	else
 		mimeinfo->id = NULL;
 
+	if (content_location != NULL)
+		mimeinfo->location = g_strdup(content_location);
+	else
+		mimeinfo->location = NULL;
+
 	if (content_disposition != NULL) 
 		procmime_parse_content_disposition(content_disposition, mimeinfo);
 	else
@@ -1989,6 +1996,9 @@ void procmime_write_mime_header(MimeInfo *mimeinfo, FILE *fp)
 	if (mimeinfo->id != NULL)
 		fprintf(fp, "Content-ID: %s\n", mimeinfo->id);
 
+	if (mimeinfo->location != NULL)
+		fprintf(fp, "Content-Location: %s\n", mimeinfo->location);
+
 	if (mimeinfo->disposition != DISPOSITIONTYPE_UNKNOWN) {
 		ParametersData *pdata = g_new0(ParametersData, 1);
 		gchar *buf = NULL;
@@ -2042,6 +2052,7 @@ gint procmime_write_message_rfc822(MimeInfo *mimeinfo, FILE *fp)
 			    g_ascii_strncasecmp(buf, "Content-Transfer-Encoding:", 26) == 0 ||
 			    g_ascii_strncasecmp(buf, "Content-Description:", 20) == 0 ||
 			    g_ascii_strncasecmp(buf, "Content-ID:", 11) == 0 ||
+			    g_ascii_strncasecmp(buf, "Content-Location:", 17) == 0 ||
 			    g_ascii_strncasecmp(buf, "Content-Disposition:", 20) == 0) {
 				skip = TRUE;
 				continue;
