@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 2.0.  */
+/* A Bison parser, made by GNU Bison 2.1.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 /* As a special exception, when this file is copied by Bison into a
    Bison output file, you may use that output file without restriction.
@@ -36,6 +36,9 @@
 /* Identify Bison output.  */
 #define YYBISON 1
 
+/* Bison version.  */
+#define YYBISON_VERSION "2.1"
+
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
 
@@ -95,6 +98,7 @@
      SET_CURSOR_POS = 297
    };
 #endif
+/* Tokens.  */
 #define SHOW_NEWSGROUPS 258
 #define SHOW_DATE 259
 #define SHOW_FROM 260
@@ -151,6 +155,7 @@
 #include "procmsg.h"
 #include "procmime.h"
 #include "utils.h"
+#include "codeconv.h"
 #include "procheader.h"
 
 #include "quote_fmt.h"
@@ -166,6 +171,7 @@ int yylex(void);
 
 static MsgInfo *msginfo = NULL;
 static gboolean *visible = NULL;
+static gboolean dry_run = FALSE;
 static gint maxsize = 0;
 static gint stacksize = 0;
 
@@ -176,7 +182,7 @@ static const gchar *quote_str = NULL;
 static const gchar *body = NULL;
 static gint error = 0;
 
-static gint cursor_pos  = 0;
+static gint cursor_pos  = -1;
 
 extern int quote_fmt_firsttime;
 
@@ -240,11 +246,12 @@ gint quote_fmt_get_cursor_pos(void)
 	}
 
 void quote_fmt_init(MsgInfo *info, const gchar *my_quote_str,
-		    const gchar *my_body)
+		    const gchar *my_body, gboolean my_dry_run)
 {
 	quote_str = my_quote_str;
 	body = my_body;
 	msginfo = info;
+	dry_run = my_dry_run;
 	stacksize = 0;
 	add_visibility(TRUE);
 	if (buffer != NULL)
@@ -255,7 +262,7 @@ void quote_fmt_init(MsgInfo *info, const gchar *my_quote_str,
          * force LEX initialization
          */
 	quote_fmt_firsttime = 1;
-	cursor_pos = 0;
+	cursor_pos = -1;
 }
 
 void quote_fmterror(char *str)
@@ -333,7 +340,24 @@ static void quote_fmt_show_date(const MsgInfo *msginfo, const gchar *format)
 			}
 		}
 		
-		INSERT(result);
+		if (g_utf8_validate(result, -1, NULL)) {
+			INSERT(result);
+		} else {
+			gchar *utf = conv_codeset_strdup(result, 
+				conv_get_locale_charset_str_no_utf8(),
+				CS_INTERNAL);
+			if (utf == NULL || 
+			    !g_utf8_validate(utf, -1, NULL)) {
+				g_free(utf);
+				utf = g_malloc(strlen(result)*2+1);
+				conv_localetodisp(utf, 
+					strlen(result)*2+1, result);
+			}
+			if (g_utf8_validate(utf, -1, NULL)) {
+				INSERT(utf);
+			}
+			g_free(utf);
+		}
 	}
 #undef STR_SIZE			
 #undef RLEFT			
@@ -428,7 +452,7 @@ static void quote_fmt_show_sender_initial(const MsgInfo *msginfo)
 	p = msginfo->fromname;
 	cur = tmp;
 	while (*p) {
-		if (*p && isalnum(*p)) {
+		if (*p && g_utf8_validate(p, 1, NULL)) {
 			*cur = toupper(*p);
 				cur++;
 			len++;
@@ -503,7 +527,7 @@ static void quote_fmt_insert_program_output(const gchar *progname)
 		while (fgets(buffer, sizeof(buffer), file)) {
 			INSERT(buffer);
 		}
-		fclose(file);
+		pclose(file);
 	}
 }
 
@@ -522,14 +546,19 @@ static void quote_fmt_insert_program_output(const gchar *progname)
 # define YYERROR_VERBOSE 0
 #endif
 
+/* Enabling the token table.  */
+#ifndef YYTOKEN_TABLE
+# define YYTOKEN_TABLE 0
+#endif
+
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 388 "quote_fmt_parse.y"
+#line 408 "quote_fmt_parse.y"
 typedef union YYSTYPE {
 	char chr;
 	char str[256];
 } YYSTYPE;
-/* Line 190 of yacc.c.  */
-#line 533 "quote_fmt_parse.c"
+/* Line 196 of yacc.c.  */
+#line 562 "quote_fmt_parse.c"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -540,18 +569,37 @@ typedef union YYSTYPE {
 /* Copy the second part of user declarations.  */
 
 
-/* Line 213 of yacc.c.  */
-#line 545 "quote_fmt_parse.c"
+/* Line 219 of yacc.c.  */
+#line 574 "quote_fmt_parse.c"
+
+#if ! defined (YYSIZE_T) && defined (__SIZE_TYPE__)
+# define YYSIZE_T __SIZE_TYPE__
+#endif
+#if ! defined (YYSIZE_T) && defined (size_t)
+# define YYSIZE_T size_t
+#endif
+#if ! defined (YYSIZE_T) && (defined (__STDC__) || defined (__cplusplus))
+# include <stddef.h> /* INFRINGES ON USER NAME SPACE */
+# define YYSIZE_T size_t
+#endif
+#if ! defined (YYSIZE_T)
+# define YYSIZE_T unsigned int
+#endif
+
+#ifndef YY_
+# if YYENABLE_NLS
+#  if ENABLE_NLS
+#   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
+#   define YY_(msgid) dgettext ("bison-runtime", msgid)
+#  endif
+# endif
+# ifndef YY_
+#  define YY_(msgid) msgid
+# endif
+#endif
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
-# ifndef YYFREE
-#  define YYFREE free
-# endif
-# ifndef YYMALLOC
-#  define YYMALLOC malloc
-# endif
-
 /* The parser invokes alloca or malloc; define the necessary symbols.  */
 
 # ifdef YYSTACK_USE_ALLOCA
@@ -560,6 +608,10 @@ typedef union YYSTYPE {
 #    define YYSTACK_ALLOC __builtin_alloca
 #   else
 #    define YYSTACK_ALLOC alloca
+#    if defined (__STDC__) || defined (__cplusplus)
+#     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
+#     define YYINCLUDED_STDLIB_H
+#    endif
 #   endif
 #  endif
 # endif
@@ -567,13 +619,39 @@ typedef union YYSTYPE {
 # ifdef YYSTACK_ALLOC
    /* Pacify GCC's `empty if-body' warning. */
 #  define YYSTACK_FREE(Ptr) do { /* empty */; } while (0)
-# else
-#  if defined (__STDC__) || defined (__cplusplus)
-#   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
-#   define YYSIZE_T size_t
+#  ifndef YYSTACK_ALLOC_MAXIMUM
+    /* The OS might guarantee only one guard page at the bottom of the stack,
+       and a page size can be as small as 4096 bytes.  So we cannot safely
+       invoke alloca (N) if N exceeds 4096.  Use a slightly smaller number
+       to allow for a few compiler-allocated temporary stack slots.  */
+#   define YYSTACK_ALLOC_MAXIMUM 4032 /* reasonable circa 2005 */
 #  endif
+# else
 #  define YYSTACK_ALLOC YYMALLOC
 #  define YYSTACK_FREE YYFREE
+#  ifndef YYSTACK_ALLOC_MAXIMUM
+#   define YYSTACK_ALLOC_MAXIMUM ((YYSIZE_T) -1)
+#  endif
+#  ifdef __cplusplus
+extern "C" {
+#  endif
+#  ifndef YYMALLOC
+#   define YYMALLOC malloc
+#   if (! defined (malloc) && ! defined (YYINCLUDED_STDLIB_H) \
+	&& (defined (__STDC__) || defined (__cplusplus)))
+void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
+#   endif
+#  endif
+#  ifndef YYFREE
+#   define YYFREE free
+#   if (! defined (free) && ! defined (YYINCLUDED_STDLIB_H) \
+	&& (defined (__STDC__) || defined (__cplusplus)))
+void free (void *); /* INFRINGES ON USER NAME SPACE */
+#   endif
+#  endif
+#  ifdef __cplusplus
+}
+#  endif
 # endif
 #endif /* ! defined (yyoverflow) || YYERROR_VERBOSE */
 
@@ -608,7 +686,7 @@ union yyalloc
 #   define YYCOPY(To, From, Count)		\
       do					\
 	{					\
-	  register YYSIZE_T yyi;		\
+	  YYSIZE_T yyi;				\
 	  for (yyi = 0; yyi < (Count); yyi++)	\
 	    (To)[yyi] = (From)[yyi];		\
 	}					\
@@ -658,7 +736,7 @@ union yyalloc
 #define YYUNDEFTOK  2
 #define YYMAXUTOK   297
 
-#define YYTRANSLATE(YYX) 						\
+#define YYTRANSLATE(YYX)						\
   ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)
 
 /* YYTRANSLATE[YYLEX] -- Bison symbol number corresponding to YYLEX.  */
@@ -733,17 +811,17 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned short int yyrline[] =
 {
-       0,   418,   418,   421,   422,   425,   426,   430,   431,   434,
-     438,   443,   457,   462,   466,   471,   476,   481,   485,   489,
-     493,   498,   503,   508,   512,   517,   523,   527,   531,   535,
-     539,   543,   547,   551,   555,   559,   563,   567,   574,   573,
-     582,   581,   590,   589,   598,   597,   606,   605,   614,   613,
-     622,   621,   630,   629,   638,   637,   648,   652
+       0,   438,   438,   441,   442,   445,   446,   450,   451,   454,
+     458,   463,   477,   482,   486,   491,   496,   501,   505,   509,
+     513,   518,   523,   528,   532,   537,   543,   547,   551,   555,
+     559,   563,   567,   571,   575,   579,   583,   587,   594,   593,
+     602,   601,   610,   609,   618,   617,   626,   625,   634,   633,
+     642,   641,   650,   649,   658,   657,   668,   674
 };
 #endif
 
-#if YYDEBUG || YYERROR_VERBOSE
-/* YYTNME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
+#if YYDEBUG || YYERROR_VERBOSE || YYTOKEN_TABLE
+/* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
    First, the terminals, then, starting at YYNTOKENS, nonterminals. */
 static const char *const yytname[] =
 {
@@ -895,22 +973,6 @@ static const unsigned char yystos[] =
       39,    39,    39,    39,    39
 };
 
-#if ! defined (YYSIZE_T) && defined (__SIZE_TYPE__)
-# define YYSIZE_T __SIZE_TYPE__
-#endif
-#if ! defined (YYSIZE_T) && defined (size_t)
-# define YYSIZE_T size_t
-#endif
-#if ! defined (YYSIZE_T)
-# if defined (__STDC__) || defined (__cplusplus)
-#  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
-#  define YYSIZE_T size_t
-# endif
-#endif
-#if ! defined (YYSIZE_T)
-# define YYSIZE_T unsigned int
-#endif
-
 #define yyerrok		(yyerrstatus = 0)
 #define yyclearin	(yychar = YYEMPTY)
 #define YYEMPTY		(-2)
@@ -940,8 +1002,8 @@ do								\
       goto yybackup;						\
     }								\
   else								\
-    { 								\
-      yyerror ("syntax error: cannot back up");\
+    {								\
+      yyerror (YY_("syntax error: cannot back up")); \
       YYERROR;							\
     }								\
 while (0)
@@ -1020,7 +1082,7 @@ do {								\
   if (yydebug)							\
     {								\
       YYFPRINTF (stderr, "%s ", Title);				\
-      yysymprint (stderr, 					\
+      yysymprint (stderr,					\
                   Type, Value);	\
       YYFPRINTF (stderr, "\n");					\
     }								\
@@ -1068,13 +1130,13 @@ yy_reduce_print (yyrule)
 #endif
 {
   int yyi;
-  unsigned int yylno = yyrline[yyrule];
-  YYFPRINTF (stderr, "Reducing stack by rule %d (line %u), ",
+  unsigned long int yylno = yyrline[yyrule];
+  YYFPRINTF (stderr, "Reducing stack by rule %d (line %lu), ",
              yyrule - 1, yylno);
   /* Print the symbols being reduced, and their result.  */
   for (yyi = yyprhs[yyrule]; 0 <= yyrhs[yyi]; yyi++)
-    YYFPRINTF (stderr, "%s ", yytname [yyrhs[yyi]]);
-  YYFPRINTF (stderr, "-> %s\n", yytname [yyr1[yyrule]]);
+    YYFPRINTF (stderr, "%s ", yytname[yyrhs[yyi]]);
+  YYFPRINTF (stderr, "-> %s\n", yytname[yyr1[yyrule]]);
 }
 
 # define YY_REDUCE_PRINT(Rule)		\
@@ -1103,7 +1165,7 @@ int yydebug;
    if the built-in stack extension method is used).
 
    Do not make this value too large; the results are undefined if
-   SIZE_MAX < YYSTACK_BYTES (YYMAXDEPTH)
+   YYSTACK_ALLOC_MAXIMUM < YYSTACK_BYTES (YYMAXDEPTH)
    evaluated with infinite-precision integer arithmetic.  */
 
 #ifndef YYMAXDEPTH
@@ -1127,7 +1189,7 @@ yystrlen (yystr)
      const char *yystr;
 #   endif
 {
-  register const char *yys = yystr;
+  const char *yys = yystr;
 
   while (*yys++ != '\0')
     continue;
@@ -1152,8 +1214,8 @@ yystpcpy (yydest, yysrc)
      const char *yysrc;
 #   endif
 {
-  register char *yyd = yydest;
-  register const char *yys = yysrc;
+  char *yyd = yydest;
+  const char *yys = yysrc;
 
   while ((*yyd++ = *yys++) != '\0')
     continue;
@@ -1163,7 +1225,55 @@ yystpcpy (yydest, yysrc)
 #  endif
 # endif
 
-#endif /* !YYERROR_VERBOSE */
+# ifndef yytnamerr
+/* Copy to YYRES the contents of YYSTR after stripping away unnecessary
+   quotes and backslashes, so that it's suitable for yyerror.  The
+   heuristic is that double-quoting is unnecessary unless the string
+   contains an apostrophe, a comma, or backslash (other than
+   backslash-backslash).  YYSTR is taken from yytname.  If YYRES is
+   null, do not copy; instead, return the length of what the result
+   would have been.  */
+static YYSIZE_T
+yytnamerr (char *yyres, const char *yystr)
+{
+  if (*yystr == '"')
+    {
+      size_t yyn = 0;
+      char const *yyp = yystr;
+
+      for (;;)
+	switch (*++yyp)
+	  {
+	  case '\'':
+	  case ',':
+	    goto do_not_strip_quotes;
+
+	  case '\\':
+	    if (*++yyp != '\\')
+	      goto do_not_strip_quotes;
+	    /* Fall through.  */
+	  default:
+	    if (yyres)
+	      yyres[yyn] = *yyp;
+	    yyn++;
+	    break;
+
+	  case '"':
+	    if (yyres)
+	      yyres[yyn] = '\0';
+	    return yyn;
+	  }
+    do_not_strip_quotes: ;
+    }
+
+  if (! yyres)
+    return yystrlen (yystr);
+
+  return yystpcpy (yyres, yystr) - yyres;
+}
+# endif
+
+#endif /* YYERROR_VERBOSE */
 
 
 
@@ -1283,13 +1393,13 @@ yyparse (void)
 #else
 int
 yyparse ()
-
+    ;
 #endif
 #endif
 {
   
-  register int yystate;
-  register int yyn;
+  int yystate;
+  int yyn;
   int yyresult;
   /* Number of tokens to shift before error messages enabled.  */
   int yyerrstatus;
@@ -1307,12 +1417,12 @@ yyparse ()
   /* The state stack.  */
   short int yyssa[YYINITDEPTH];
   short int *yyss = yyssa;
-  register short int *yyssp;
+  short int *yyssp;
 
   /* The semantic value stack.  */
   YYSTYPE yyvsa[YYINITDEPTH];
   YYSTYPE *yyvs = yyvsa;
-  register YYSTYPE *yyvsp;
+  YYSTYPE *yyvsp;
 
 
 
@@ -1344,9 +1454,6 @@ yyparse ()
   yyssp = yyss;
   yyvsp = yyvs;
 
-
-  yyvsp[0] = yylval;
-
   goto yysetstate;
 
 /*------------------------------------------------------------.
@@ -1379,7 +1486,7 @@ yyparse ()
 	   data in use in that stack, in bytes.  This used to be a
 	   conditional around just the two extra args, but that might
 	   be undefined if yyoverflow is a macro.  */
-	yyoverflow ("parser stack overflow",
+	yyoverflow (YY_("memory exhausted"),
 		    &yyss1, yysize * sizeof (*yyssp),
 		    &yyvs1, yysize * sizeof (*yyvsp),
 
@@ -1390,11 +1497,11 @@ yyparse ()
       }
 #else /* no yyoverflow */
 # ifndef YYSTACK_RELOCATE
-      goto yyoverflowlab;
+      goto yyexhaustedlab;
 # else
       /* Extend the stack our own way.  */
       if (YYMAXDEPTH <= yystacksize)
-	goto yyoverflowlab;
+	goto yyexhaustedlab;
       yystacksize *= 2;
       if (YYMAXDEPTH < yystacksize)
 	yystacksize = YYMAXDEPTH;
@@ -1404,7 +1511,7 @@ yyparse ()
 	union yyalloc *yyptr =
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
-	  goto yyoverflowlab;
+	  goto yyexhaustedlab;
 	YYSTACK_RELOCATE (yyss);
 	YYSTACK_RELOCATE (yyvs);
 
@@ -1533,14 +1640,14 @@ yyreduce:
   switch (yyn)
     {
         case 6:
-#line 427 "quote_fmt_parse.y"
+#line 447 "quote_fmt_parse.y"
     {
 		INSERT_CHARACTER((yyvsp[0].chr));
 	}
     break;
 
   case 10:
-#line 439 "quote_fmt_parse.y"
+#line 459 "quote_fmt_parse.y"
     {
 		(yyval.str)[0] = (yyvsp[0].chr);
 		(yyval.str)[1] = '\0';
@@ -1548,7 +1655,7 @@ yyreduce:
     break;
 
   case 11:
-#line 444 "quote_fmt_parse.y"
+#line 464 "quote_fmt_parse.y"
     {
 		int len;
 		
@@ -1563,7 +1670,7 @@ yyreduce:
     break;
 
   case 12:
-#line 458 "quote_fmt_parse.y"
+#line 478 "quote_fmt_parse.y"
     {
 		if (msginfo->newsgroups)
 			INSERT(msginfo->newsgroups);
@@ -1571,14 +1678,14 @@ yyreduce:
     break;
 
   case 13:
-#line 463 "quote_fmt_parse.y"
+#line 483 "quote_fmt_parse.y"
     {
 		quote_fmt_show_date(msginfo, (yyvsp[-1].str));
 	}
     break;
 
   case 14:
-#line 467 "quote_fmt_parse.y"
+#line 487 "quote_fmt_parse.y"
     {
 		if (msginfo->date)
 			INSERT(msginfo->date);
@@ -1586,7 +1693,7 @@ yyreduce:
     break;
 
   case 15:
-#line 472 "quote_fmt_parse.y"
+#line 492 "quote_fmt_parse.y"
     {
 		if (msginfo->from)
 			INSERT(msginfo->from);
@@ -1594,7 +1701,7 @@ yyreduce:
     break;
 
   case 16:
-#line 477 "quote_fmt_parse.y"
+#line 497 "quote_fmt_parse.y"
     {
 		if (msginfo->fromname)
 			INSERT(msginfo->fromname);
@@ -1602,28 +1709,28 @@ yyreduce:
     break;
 
   case 17:
-#line 482 "quote_fmt_parse.y"
+#line 502 "quote_fmt_parse.y"
     {
 		quote_fmt_show_first_name(msginfo);
 	}
     break;
 
   case 18:
-#line 486 "quote_fmt_parse.y"
+#line 506 "quote_fmt_parse.y"
     {
 		quote_fmt_show_last_name(msginfo);
 	}
     break;
 
   case 19:
-#line 490 "quote_fmt_parse.y"
+#line 510 "quote_fmt_parse.y"
     {
 		quote_fmt_show_sender_initial(msginfo);
 	}
     break;
 
   case 20:
-#line 494 "quote_fmt_parse.y"
+#line 514 "quote_fmt_parse.y"
     {
 		if (msginfo->subject)
 			INSERT(msginfo->subject);
@@ -1631,7 +1738,7 @@ yyreduce:
     break;
 
   case 21:
-#line 499 "quote_fmt_parse.y"
+#line 519 "quote_fmt_parse.y"
     {
 		if (msginfo->to)
 			INSERT(msginfo->to);
@@ -1639,7 +1746,7 @@ yyreduce:
     break;
 
   case 22:
-#line 504 "quote_fmt_parse.y"
+#line 524 "quote_fmt_parse.y"
     {
 		if (msginfo->msgid)
 			INSERT(msginfo->msgid);
@@ -1647,14 +1754,14 @@ yyreduce:
     break;
 
   case 23:
-#line 509 "quote_fmt_parse.y"
+#line 529 "quote_fmt_parse.y"
     {
 		INSERT("%");
 	}
     break;
 
   case 24:
-#line 513 "quote_fmt_parse.y"
+#line 533 "quote_fmt_parse.y"
     {
 		if (msginfo->cc)
 			INSERT(msginfo->cc);
@@ -1662,7 +1769,7 @@ yyreduce:
     break;
 
   case 25:
-#line 518 "quote_fmt_parse.y"
+#line 538 "quote_fmt_parse.y"
     {
                 /* CLAWS: use in reply to header */
 		/* if (msginfo->references)
@@ -1671,203 +1778,203 @@ yyreduce:
     break;
 
   case 26:
-#line 524 "quote_fmt_parse.y"
+#line 544 "quote_fmt_parse.y"
     {
 		quote_fmt_show_msg(msginfo, body, FALSE, TRUE, quote_str);
 	}
     break;
 
   case 27:
-#line 528 "quote_fmt_parse.y"
+#line 548 "quote_fmt_parse.y"
     {
 		quote_fmt_show_msg(msginfo, body, TRUE, TRUE, quote_str);
 	}
     break;
 
   case 28:
-#line 532 "quote_fmt_parse.y"
+#line 552 "quote_fmt_parse.y"
     {
 		quote_fmt_show_msg(msginfo, body, FALSE, FALSE, quote_str);
 	}
     break;
 
   case 29:
-#line 536 "quote_fmt_parse.y"
+#line 556 "quote_fmt_parse.y"
     {
 		quote_fmt_show_msg(msginfo, body, TRUE, FALSE, quote_str);
 	}
     break;
 
   case 30:
-#line 540 "quote_fmt_parse.y"
+#line 560 "quote_fmt_parse.y"
     {
 		INSERT("\\");
 	}
     break;
 
   case 31:
-#line 544 "quote_fmt_parse.y"
+#line 564 "quote_fmt_parse.y"
     {
 		INSERT("\t");
 	}
     break;
 
   case 32:
-#line 548 "quote_fmt_parse.y"
+#line 568 "quote_fmt_parse.y"
     {
 		INSERT("\n");
 	}
     break;
 
   case 33:
-#line 552 "quote_fmt_parse.y"
+#line 572 "quote_fmt_parse.y"
     {
 		INSERT("?");
 	}
     break;
 
   case 34:
-#line 556 "quote_fmt_parse.y"
+#line 576 "quote_fmt_parse.y"
     {
 		INSERT("|");
 	}
     break;
 
   case 35:
-#line 560 "quote_fmt_parse.y"
+#line 580 "quote_fmt_parse.y"
     {
 		INSERT("{");
 	}
     break;
 
   case 36:
-#line 564 "quote_fmt_parse.y"
+#line 584 "quote_fmt_parse.y"
     {
 		INSERT("}");
 	}
     break;
 
   case 37:
-#line 568 "quote_fmt_parse.y"
+#line 588 "quote_fmt_parse.y"
     {
 		cursor_pos = bufsize;
 	}
     break;
 
   case 38:
-#line 574 "quote_fmt_parse.y"
+#line 594 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->date != NULL);
 	}
     break;
 
   case 39:
-#line 578 "quote_fmt_parse.y"
+#line 598 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 40:
-#line 582 "quote_fmt_parse.y"
+#line 602 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->from != NULL);
 	}
     break;
 
   case 41:
-#line 586 "quote_fmt_parse.y"
+#line 606 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 42:
-#line 590 "quote_fmt_parse.y"
+#line 610 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->fromname != NULL);
 	}
     break;
 
   case 43:
-#line 594 "quote_fmt_parse.y"
+#line 614 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 44:
-#line 598 "quote_fmt_parse.y"
+#line 618 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->subject != NULL);
 	}
     break;
 
   case 45:
-#line 602 "quote_fmt_parse.y"
+#line 622 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 46:
-#line 606 "quote_fmt_parse.y"
+#line 626 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->to != NULL);
 	}
     break;
 
   case 47:
-#line 610 "quote_fmt_parse.y"
+#line 630 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 48:
-#line 614 "quote_fmt_parse.y"
+#line 634 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->newsgroups != NULL);
 	}
     break;
 
   case 49:
-#line 618 "quote_fmt_parse.y"
+#line 638 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 50:
-#line 622 "quote_fmt_parse.y"
+#line 642 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->msgid != NULL);
 	}
     break;
 
   case 51:
-#line 626 "quote_fmt_parse.y"
+#line 646 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 52:
-#line 630 "quote_fmt_parse.y"
+#line 650 "quote_fmt_parse.y"
     {
 		add_visibility(msginfo->cc != NULL);
 	}
     break;
 
   case 53:
-#line 634 "quote_fmt_parse.y"
+#line 654 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 54:
-#line 638 "quote_fmt_parse.y"
+#line 658 "quote_fmt_parse.y"
     {
                 /* CLAWS: use in-reply-to header */
 		/* add_visibility(msginfo->references != NULL); */
@@ -1875,31 +1982,36 @@ yyreduce:
     break;
 
   case 55:
-#line 643 "quote_fmt_parse.y"
+#line 663 "quote_fmt_parse.y"
     {
 		remove_visibility();
 	}
     break;
 
   case 56:
-#line 649 "quote_fmt_parse.y"
+#line 669 "quote_fmt_parse.y"
     {
-		quote_fmt_insert_file((yyvsp[-1].str));
+		if (!dry_run) {
+			quote_fmt_insert_file((yyvsp[-1].str));
+		}
 	}
     break;
 
   case 57:
-#line 653 "quote_fmt_parse.y"
+#line 675 "quote_fmt_parse.y"
     {
-		quote_fmt_insert_program_output((yyvsp[-1].str));
+		if (!dry_run) {
+			quote_fmt_insert_program_output((yyvsp[-1].str));
+		}
 	}
     break;
 
 
+      default: break;
     }
 
-/* Line 1037 of yacc.c.  */
-#line 1903 "quote_fmt_parse.c"
+/* Line 1126 of yacc.c.  */
+#line 2015 "quote_fmt_parse.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1938,12 +2050,36 @@ yyerrlab:
 
       if (YYPACT_NINF < yyn && yyn < YYLAST)
 	{
-	  YYSIZE_T yysize = 0;
 	  int yytype = YYTRANSLATE (yychar);
-	  const char* yyprefix;
-	  char *yymsg;
+	  YYSIZE_T yysize0 = yytnamerr (0, yytname[yytype]);
+	  YYSIZE_T yysize = yysize0;
+	  YYSIZE_T yysize1;
+	  int yysize_overflow = 0;
+	  char *yymsg = 0;
+#	  define YYERROR_VERBOSE_ARGS_MAXIMUM 5
+	  char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
 	  int yyx;
 
+#if 0
+	  /* This is so xgettext sees the translatable formats that are
+	     constructed on the fly.  */
+	  YY_("syntax error, unexpected %s");
+	  YY_("syntax error, unexpected %s, expecting %s");
+	  YY_("syntax error, unexpected %s, expecting %s or %s");
+	  YY_("syntax error, unexpected %s, expecting %s or %s or %s");
+	  YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s");
+#endif
+	  char *yyfmt;
+	  char const *yyf;
+	  static char const yyunexpected[] = "syntax error, unexpected %s";
+	  static char const yyexpecting[] = ", expecting %s";
+	  static char const yyor[] = " or %s";
+	  char yyformat[sizeof yyunexpected
+			+ sizeof yyexpecting - 1
+			+ ((YYERROR_VERBOSE_ARGS_MAXIMUM - 2)
+			   * (sizeof yyor - 1))];
+	  char const *yyprefix = yyexpecting;
+
 	  /* Start YYX at -YYN if negative to avoid negative indexes in
 	     YYCHECK.  */
 	  int yyxbegin = yyn < 0 ? -yyn : 0;
@@ -1951,48 +2087,68 @@ yyerrlab:
 	  /* Stay within bounds of both yycheck and yytname.  */
 	  int yychecklim = YYLAST - yyn;
 	  int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
-	  int yycount = 0;
+	  int yycount = 1;
+
+	  yyarg[0] = yytname[yytype];
+	  yyfmt = yystpcpy (yyformat, yyunexpected);
 
-	  yyprefix = ", expecting ";
 	  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 	    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
 	      {
-		yysize += yystrlen (yyprefix) + yystrlen (yytname [yyx]);
-		yycount += 1;
-		if (yycount == 5)
+		if (yycount == YYERROR_VERBOSE_ARGS_MAXIMUM)
 		  {
-		    yysize = 0;
+		    yycount = 1;
+		    yysize = yysize0;
+		    yyformat[sizeof yyunexpected - 1] = '\0';
 		    break;
 		  }
+		yyarg[yycount++] = yytname[yyx];
+		yysize1 = yysize + yytnamerr (0, yytname[yyx]);
+		yysize_overflow |= yysize1 < yysize;
+		yysize = yysize1;
+		yyfmt = yystpcpy (yyfmt, yyprefix);
+		yyprefix = yyor;
 	      }
-	  yysize += (sizeof ("syntax error, unexpected ")
-		     + yystrlen (yytname[yytype]));
-	  yymsg = (char *) YYSTACK_ALLOC (yysize);
-	  if (yymsg != 0)
-	    {
-	      char *yyp = yystpcpy (yymsg, "syntax error, unexpected ");
-	      yyp = yystpcpy (yyp, yytname[yytype]);
 
-	      if (yycount < 5)
+	  yyf = YY_(yyformat);
+	  yysize1 = yysize + yystrlen (yyf);
+	  yysize_overflow |= yysize1 < yysize;
+	  yysize = yysize1;
+
+	  if (!yysize_overflow && yysize <= YYSTACK_ALLOC_MAXIMUM)
+	    yymsg = (char *) YYSTACK_ALLOC (yysize);
+	  if (yymsg)
+	    {
+	      /* Avoid sprintf, as that infringes on the user's name space.
+		 Don't have undefined behavior even if the translation
+		 produced a string with the wrong number of "%s"s.  */
+	      char *yyp = yymsg;
+	      int yyi = 0;
+	      while ((*yyp = *yyf))
 		{
-		  yyprefix = ", expecting ";
-		  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
-		    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
-		      {
-			yyp = yystpcpy (yyp, yyprefix);
-			yyp = yystpcpy (yyp, yytname[yyx]);
-			yyprefix = " or ";
-		      }
+		  if (*yyp == '%' && yyf[1] == 's' && yyi < yycount)
+		    {
+		      yyp += yytnamerr (yyp, yyarg[yyi++]);
+		      yyf += 2;
+		    }
+		  else
+		    {
+		      yyp++;
+		      yyf++;
+		    }
 		}
 	      yyerror (yymsg);
 	      YYSTACK_FREE (yymsg);
 	    }
 	  else
-	    yyerror ("syntax error; also virtual memory exhausted");
+	    {
+	      yyerror (YY_("syntax error"));
+	      goto yyexhaustedlab;
+	    }
 	}
       else
 #endif /* YYERROR_VERBOSE */
-	yyerror ("syntax error");
+	yyerror (YY_("syntax error"));
     }
 
 
@@ -2004,18 +2160,9 @@ yyerrlab:
 
       if (yychar <= YYEOF)
         {
-          /* If at end of input, pop the error token,
-	     then the rest of the stack, then return failure.  */
+	  /* Return failure if at end of input.  */
 	  if (yychar == YYEOF)
-	     for (;;)
-	       {
-
-		 YYPOPSTACK;
-		 if (yyssp == yyss)
-		   YYABORT;
-		 yydestruct ("Error: popping",
-                             yystos[*yyssp], yyvsp);
-	       }
+	    YYABORT;
         }
       else
 	{
@@ -2034,12 +2181,11 @@ yyerrlab:
 `---------------------------------------------------*/
 yyerrorlab:
 
-#ifdef __GNUC__
-  /* Pacify GCC when the user code never invokes YYERROR and the label
-     yyerrorlab therefore never appears in user code.  */
+  /* Pacify compilers like GCC when the user code never invokes
+     YYERROR and the label yyerrorlab therefore never appears in user
+     code.  */
   if (0)
      goto yyerrorlab;
-#endif
 
 yyvsp -= yylen;
   yyssp -= yylen;
@@ -2102,23 +2248,29 @@ yyacceptlab:
 | yyabortlab -- YYABORT comes here.  |
 `-----------------------------------*/
 yyabortlab:
-  yydestruct ("Error: discarding lookahead",
-              yytoken, &yylval);
-  yychar = YYEMPTY;
   yyresult = 1;
   goto yyreturn;
 
 #ifndef yyoverflow
-/*----------------------------------------------.
-| yyoverflowlab -- parser overflow comes here.  |
-`----------------------------------------------*/
-yyoverflowlab:
-  yyerror ("parser stack overflow");
+/*-------------------------------------------------.
+| yyexhaustedlab -- memory exhaustion comes here.  |
+`-------------------------------------------------*/
+yyexhaustedlab:
+  yyerror (YY_("memory exhausted"));
   yyresult = 2;
   /* Fall through.  */
 #endif
 
 yyreturn:
+  if (yychar != YYEOF && yychar != YYEMPTY)
+     yydestruct ("Cleanup: discarding lookahead",
+		 yytoken, &yylval);
+  while (yyssp != yyss)
+    {
+      yydestruct ("Cleanup: popping",
+		  yystos[*yyssp], yyvsp);
+      YYPOPSTACK;
+    }
 #ifndef yyoverflow
   if (yyss != yyssa)
     YYSTACK_FREE (yyss);
