@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "defs.h"
@@ -132,6 +132,10 @@
 #include "pixmaps/drafts_close_mark.xpm"
 #include "pixmaps/drafts_open_mark.xpm"
 #include "pixmaps/dir_noselect.xpm"
+#include "pixmaps/spam.xpm"
+#include "pixmaps/spam_btn.xpm"
+#include "pixmaps/ham_btn.xpm"
+#include "pixmaps/empty.xpm"
 
 typedef struct _StockPixmapData	StockPixmapData;
 
@@ -292,7 +296,11 @@ static StockPixmapData pixmaps[] =
 	{read_xpm				, NULL, NULL, "read", NULL},
 	{trash_btn_xpm				, NULL, NULL, "trash_btn", NULL},
 	{sylpheed_logo_xpm			, NULL, NULL, "sylpheed_logo", NULL},
-        {dir_noselect_xpm                       , NULL, NULL, "dir_noselect" , NULL}
+        {dir_noselect_xpm                       , NULL, NULL, "dir_noselect" , NULL},
+        {spam_xpm                               , NULL, NULL, "spam" , NULL},
+        {spam_btn_xpm                           , NULL, NULL, "spam_btn" , NULL},
+        {ham_btn_xpm                            , NULL, NULL, "ham_btn" , NULL},
+        {empty_xpm                              , NULL, NULL, "empty" , NULL}
 };
 
 /* return newly constructed GtkPixmap from GdkPixmap */
