@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,8 +50,12 @@
 #include "menu.h"
 #include "utils.h"
 #include "gtkutils.h"
+#include "prefs_gtk.h"
 #include "manage_window.h"
 #include "alertpanel.h"
+#include "matcher.h"
+#include "matcher_parser.h"
+#include "prefs_matcher.h"
 
 static struct SummarySearchWindow {
 	GtkWidget *window;
@@ -63,6 +67,10 @@ static struct SummarySearchWindow {
 	GtkWidget *subject_entry;
 	GtkWidget *body_entry;
 
+	GtkWidget *adv_condition_entry;
+	GtkWidget *adv_condition_btn;
+	GtkWidget *adv_search_checkbtn;
+
 	GtkWidget *case_checkbtn;
 
 	GtkWidget *clear_btn;
@@ -70,8 +78,13 @@ static struct SummarySearchWindow {
 	GtkWidget *prev_btn;
 	GtkWidget *next_btn;
 	GtkWidget *close_btn;
+	GtkWidget *stop_btn;
 
 	SummaryView *summaryview;
+
+	MatcherList			*matcher_list;
+
+	gboolean is_searching;
 } search_window;
 
 static void summary_search_create	(void);
@@ -87,11 +100,16 @@ static void summary_search_next_clicked	(GtkButton	*button,
 					 gpointer	 data);
 static void summary_search_all_clicked	(GtkButton	*button,
 					 gpointer	 data);
+static void summary_search_stop_clicked	(GtkButton	*button,
+					 gpointer	 data);
+static void adv_condition_btn_clicked	(GtkButton	*button,
+					 gpointer	 data);
 
 static void from_activated		(void);
 static void to_activated		(void);
 static void subject_activated		(void);
 static void body_activated		(void);
+static void adv_condition_activated	(void);
 
 static gboolean key_pressed		(GtkWidget	*widget,
 					 GdkEventKey	*event,
@@ -100,10 +118,11 @@ static gboolean key_pressed		(GtkWidget	*widget,
 
 void summary_search(SummaryView *summaryview)
 {
-	if (!search_window.window)
+	if (!search_window.window) {
 		summary_search_create();
-	else
+	} else {
 		gtk_widget_hide(search_window.window);
+	}
 
 	search_window.summaryview = summaryview;
 
@@ -131,8 +150,12 @@ static void summary_search_create(void)
 	GtkWidget *subject_entry;
 	GtkWidget *body_label;
 	GtkWidget *body_entry;
+	GtkWidget *adv_condition_label;
+	GtkWidget *adv_condition_entry;
+	GtkWidget *adv_condition_btn;
 
 	GtkWidget *checkbtn_hbox;
+	GtkWidget *adv_search_checkbtn;
 	GtkWidget *case_checkbtn;
 
 	GtkWidget *confirm_area;
@@ -140,10 +163,13 @@ static void summary_search_create(void)
 	GtkWidget *prev_btn;
 	GtkWidget *next_btn;
 	GtkWidget *close_btn;
+	GtkWidget *stop_btn;
+	GtkTooltips *tooltip;
+
+	gboolean is_searching = FALSE;
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW (window), _("Search messages"));
-	gtk_widget_set_size_request(window, 450, -1);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, TRUE);
 	gtk_container_set_border_width(GTK_CONTAINER (window), 8);
 	g_signal_connect(G_OBJECT(window), "delete_event",
@@ -173,7 +199,7 @@ static void summary_search_create(void)
 	gtk_widget_show(clear_btn);
 	gtk_box_pack_end(GTK_BOX(bool_hbox), clear_btn, FALSE, FALSE, 0);
 
-	table1 = gtk_table_new (4, 3, FALSE);
+	table1 = gtk_table_new (5, 3, FALSE);
 	gtk_widget_show (table1);
 	gtk_box_pack_start (GTK_BOX (vbox1), table1, TRUE, TRUE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (table1), 4);
@@ -208,6 +234,23 @@ static void summary_search_create(void)
 	g_signal_connect(G_OBJECT(body_entry), "activate",
 			 G_CALLBACK(body_activated), NULL);
 
+	adv_condition_entry = gtk_entry_new ();
+	gtk_widget_show (adv_condition_entry);
+	gtk_table_attach (GTK_TABLE (table1), adv_condition_entry, 1, 2, 4, 5,
+			  GTK_EXPAND|GTK_FILL, 0, 0, 0);
+	g_signal_connect(G_OBJECT(adv_condition_entry), "activate",
+			 G_CALLBACK(adv_condition_activated), NULL);
+
+	adv_condition_btn = gtk_button_new_with_label(" ... ");
+	gtk_widget_show (adv_condition_btn);
+	gtk_table_attach (GTK_TABLE (table1), adv_condition_btn, 2, 3, 4, 5,
+			  GTK_FILL, 0, 0, 0);
+	g_signal_connect(G_OBJECT (adv_condition_btn), "clicked",
+			 G_CALLBACK(adv_condition_btn_clicked), search_window.window);
+	tooltip = gtk_tooltips_new();
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltip), adv_condition_btn,
+			     _("Edit search criteria"), NULL);
+
 	from_label = gtk_label_new (_("From:"));
 	gtk_widget_show (from_label);
 	gtk_table_attach (GTK_TABLE (table1), from_label, 0, 1, 0, 1,
@@ -236,6 +279,13 @@ static void summary_search_create(void)
 	gtk_label_set_justify (GTK_LABEL (body_label), GTK_JUSTIFY_RIGHT);
 	gtk_misc_set_alignment (GTK_MISC (body_label), 1, 0.5);
 
+	adv_condition_label = gtk_label_new (_("Condition:"));
+	gtk_widget_show (adv_condition_label);
+	gtk_table_attach (GTK_TABLE (table1), adv_condition_label, 0, 1, 4, 5,
+			  GTK_FILL, 0, 0, 0);
+	gtk_label_set_justify (GTK_LABEL (adv_condition_label), GTK_JUSTIFY_RIGHT);
+	gtk_misc_set_alignment (GTK_MISC (adv_condition_label), 1, 0.5);
+
 	checkbtn_hbox = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (checkbtn_hbox);
 	gtk_box_pack_start (GTK_BOX (vbox1), checkbtn_hbox, TRUE, TRUE, 0);
@@ -246,13 +296,18 @@ static void summary_search_create(void)
 	gtk_box_pack_start (GTK_BOX (checkbtn_hbox), case_checkbtn,
 			    FALSE, FALSE, 0);
 
+	adv_search_checkbtn = gtk_check_button_new_with_label (_("Advanced search"));
+	gtk_widget_show (adv_search_checkbtn);
+	gtk_box_pack_start (GTK_BOX (checkbtn_hbox), adv_search_checkbtn,
+			    FALSE, FALSE, 0);
+
 	confirm_area = gtk_hbutton_box_new();
 	gtk_widget_show (confirm_area);
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(confirm_area),
 				  GTK_BUTTONBOX_END);
 	gtk_box_set_spacing(GTK_BOX(confirm_area), 5);
 
-	all_btn = gtk_button_new_from_stock(_("Find all"));
+	all_btn = gtk_button_new_with_mnemonic(_("Find _all"));
 	GTK_WIDGET_SET_FLAGS(all_btn, GTK_CAN_DEFAULT);
 	gtk_box_pack_start(GTK_BOX(confirm_area), all_btn, TRUE, TRUE, 0);
 	gtk_widget_show(all_btn);
@@ -272,9 +327,24 @@ static void summary_search_create(void)
 	gtk_box_pack_start(GTK_BOX(confirm_area), close_btn, TRUE, TRUE, 0);
 	gtk_widget_show(close_btn);
 
+	/* stop button hidden */
+	stop_btn = gtk_button_new_from_stock(GTK_STOCK_STOP);
+	GTK_WIDGET_SET_FLAGS(stop_btn, GTK_CAN_DEFAULT);
+	gtk_box_pack_start(GTK_BOX(confirm_area), stop_btn, TRUE, TRUE, 0);
+
 	gtk_box_pack_start (GTK_BOX (vbox1), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(next_btn);
 
+	SET_TOGGLE_SENSITIVITY_REVERSE(adv_search_checkbtn, bool_menu)
+	SET_TOGGLE_SENSITIVITY_REVERSE(adv_search_checkbtn, from_entry)
+	SET_TOGGLE_SENSITIVITY_REVERSE(adv_search_checkbtn, to_entry)
+	SET_TOGGLE_SENSITIVITY_REVERSE(adv_search_checkbtn, subject_entry)
+	SET_TOGGLE_SENSITIVITY_REVERSE(adv_search_checkbtn, body_entry)
+	SET_TOGGLE_SENSITIVITY(adv_search_checkbtn, adv_condition_label)
+	SET_TOGGLE_SENSITIVITY(adv_search_checkbtn, adv_condition_entry)
+	SET_TOGGLE_SENSITIVITY(adv_search_checkbtn, adv_condition_btn)
+	SET_TOGGLE_SENSITIVITY_REVERSE(adv_search_checkbtn, case_checkbtn)
+
 	g_signal_connect(G_OBJECT(clear_btn), "clicked",
 			 G_CALLBACK(summary_search_clear), NULL);
 	g_signal_connect(G_OBJECT(all_btn), "clicked",
@@ -286,8 +356,9 @@ static void summary_search_create(void)
 	g_signal_connect_closure
 		(G_OBJECT(close_btn), "clicked",
 		 g_cclosure_new_swap(G_CALLBACK(gtk_widget_hide),
-				     window, NULL),
-		 FALSE);
+	     window, NULL), FALSE);
+	g_signal_connect(G_OBJECT(stop_btn), "clicked",
+			 G_CALLBACK(summary_search_stop_clicked), NULL);
 
 	search_window.window = window;
 	search_window.bool_optmenu = bool_optmenu;
@@ -295,12 +366,18 @@ static void summary_search_create(void)
 	search_window.to_entry = to_entry;
 	search_window.subject_entry = subject_entry;
 	search_window.body_entry = body_entry;
+	search_window.adv_condition_entry = adv_condition_entry;
+	search_window.adv_condition_btn = adv_condition_btn;
 	search_window.case_checkbtn = case_checkbtn;
+	search_window.adv_search_checkbtn = adv_search_checkbtn;
 	search_window.clear_btn = clear_btn;
 	search_window.all_btn = all_btn;
 	search_window.prev_btn = prev_btn;
 	search_window.next_btn = next_btn;
 	search_window.close_btn = close_btn;
+	search_window.stop_btn = stop_btn;
+	search_window.matcher_list = NULL;
+	search_window.is_searching = is_searching;
 }
 
 static void summary_search_execute(gboolean backward, gboolean search_all)
@@ -309,32 +386,72 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
 	GtkCTreeNode *node;
 	MsgInfo *msginfo;
-	gboolean bool_and;
-	gboolean case_sens;
+	gboolean adv_search;
+	gboolean bool_and = FALSE;
+	gboolean case_sens = FALSE;
 	gboolean all_searched = FALSE;
-	gboolean matched;
-	gboolean body_matched;
-	const gchar *from_str, *to_str, *subject_str, *body_str;
-	StrFindFunc str_find_func;
+	gboolean matched = FALSE;
+	gboolean body_matched = FALSE;
+	const gchar *from_str = NULL, *to_str = NULL, *subject_str = NULL;
+	const gchar *body_str = NULL, *adv_condition = NULL;
+	StrFindFunc str_find_func = NULL;
 
-	if (summary_is_locked(summaryview)) return;
+	if (summary_is_locked(summaryview)) {
+		return;
+	}
 	summary_lock(summaryview);
 
-	bool_and = GPOINTER_TO_INT
-		(menu_get_option_menu_active_user_data
-			(GTK_OPTION_MENU(search_window.bool_optmenu)));
-	case_sens = gtk_toggle_button_get_active
-		(GTK_TOGGLE_BUTTON(search_window.case_checkbtn));
+	adv_search = gtk_toggle_button_get_active
+		(GTK_TOGGLE_BUTTON(search_window.adv_search_checkbtn));
 
-	if (case_sens)
-		str_find_func = str_find;
-	else
-		str_find_func = str_case_find;
+	if (adv_search) {
+		if (search_window.matcher_list != NULL) {
+			matcherlist_free(search_window.matcher_list);
+			search_window.matcher_list = NULL;
+		}
+		adv_condition = gtk_entry_get_text(GTK_ENTRY(search_window.adv_condition_entry));
+		if (adv_condition[0] != '\0') {
+			search_window.matcher_list = matcher_parser_get_cond((gchar*)adv_condition);
+			/* TODO: check for condition parsing error and show an error dialog */
+		} else {
+			/* TODO: warn if no search condition? (or make buttons enabled only when
+				at least one search condition has been set */
+			summary_unlock(summaryview);
+			return;
+		}
+	} else {
+		bool_and = GPOINTER_TO_INT
+			(menu_get_option_menu_active_user_data
+				(GTK_OPTION_MENU(search_window.bool_optmenu)));
+		case_sens = gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON(search_window.case_checkbtn));
 
-	from_str    = gtk_entry_get_text(GTK_ENTRY(search_window.from_entry));
-	to_str      = gtk_entry_get_text(GTK_ENTRY(search_window.to_entry));
-	subject_str = gtk_entry_get_text(GTK_ENTRY(search_window.subject_entry));
-	body_str    = gtk_entry_get_text(GTK_ENTRY(search_window.body_entry));
+		if (case_sens) {
+			str_find_func = str_find;
+		} else {
+			str_find_func = str_case_find;
+		}
+
+		from_str    = gtk_entry_get_text(GTK_ENTRY(search_window.from_entry));
+		to_str      = gtk_entry_get_text(GTK_ENTRY(search_window.to_entry));
+		subject_str = gtk_entry_get_text(GTK_ENTRY(search_window.subject_entry));
+		body_str    = gtk_entry_get_text(GTK_ENTRY(search_window.body_entry));
+
+		if (	(from_str[0] == '\0') &&
+				(to_str[0] == '\0') &&
+				(subject_str[0] == '\0') &&
+				(body_str[0] == '\0')) {
+			/* TODO: warn if no search criteria? (or make buttons enabled only when
+				at least one search criteria has been set */
+			summary_unlock(summaryview);
+			return;
+		}
+	}
+
+	search_window.is_searching = TRUE;
+	main_window_cursor_wait(summaryview->mainwin);
+	gtk_widget_hide(search_window.close_btn);
+	gtk_widget_show(search_window.stop_btn);
 
 	if (search_all) {
 		gtk_clist_freeze(GTK_CLIST(ctree));
@@ -342,34 +459,36 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
 		backward = FALSE;
 	} else if (!summaryview->selected) {
-		if (backward)
+		if (backward) {
 			node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list_end);
-		else
+		} else {
 			node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+		}
 
 		if (!node) {
+			search_window.is_searching = FALSE;
+			gtk_widget_hide(search_window.stop_btn);
+			gtk_widget_show(search_window.close_btn);
+			main_window_cursor_normal(summaryview->mainwin);
 			summary_unlock(summaryview);
 			return;
 		}
 	} else {
-		if (backward)
+		if (backward) {
 			node = gtkut_ctree_node_prev
 				(ctree, summaryview->selected);
-		else
+		} else {
 			node = gtkut_ctree_node_next
 				(ctree, summaryview->selected);
+		}
 	}
 
-	if (*body_str)
-		main_window_cursor_wait(summaryview->mainwin);
-
-	for (;;) {
+	for (; search_window.is_searching;) {
 		if (!node) {
 			gchar *str;
 			AlertValue val;
 
 			if (search_all) {
-				gtk_clist_thaw(GTK_CLIST(ctree));
 				break;
 			}
 
@@ -387,80 +506,93 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 				str = _("End of list reached; continue from beginning?");
 
 			val = alertpanel(_("Search finished"), str,
-					 GTK_STOCK_YES, GTK_STOCK_NO, NULL);
-			if (G_ALERTDEFAULT == val) {
-				if (backward)
+					 GTK_STOCK_NO, GTK_STOCK_YES, NULL);
+			if (G_ALERTALTERNATE == val) {
+				if (backward) {
 					node = GTK_CTREE_NODE
 						(GTK_CLIST(ctree)->row_list_end);
-				else
+				} else {
 					node = GTK_CTREE_NODE
 						(GTK_CLIST(ctree)->row_list);
+				}
 
 				all_searched = TRUE;
 
 				manage_window_focus_in(search_window.window, NULL, NULL);
-			} else
+			} else {
 				break;
+			}
 		}
 
-
 		msginfo = gtk_ctree_node_get_row_data(ctree, node);
 		body_matched = FALSE;
 
-		if (bool_and) {
-			matched = TRUE;
-			if (*from_str) {
-				if (!msginfo->from ||
-				    !str_find_func(msginfo->from, from_str))
-					matched = FALSE;
-			}
-			if (matched && *to_str) {
-				if (!msginfo->to ||
-				    !str_find_func(msginfo->to, to_str))
-					matched = FALSE;
-			}
-			if (matched && *subject_str) {
-				if (!msginfo->subject ||
-				    !str_find_func(msginfo->subject, subject_str))
-					matched = FALSE;
-			}
-			if (matched && *body_str) {
-				if (procmime_find_string(msginfo, body_str,
-							 str_find_func))
-					body_matched = TRUE;
-				else
-					matched = FALSE;
-			}
-			if (matched && !*from_str && !*to_str &&
-			    !*subject_str && !*body_str)
-				matched = FALSE;
+		if (adv_search) {
+			matched = matcherlist_match(search_window.matcher_list, msginfo);
 		} else {
-			matched = FALSE;
-			if (*from_str && msginfo->from) {
-				if (str_find_func(msginfo->from, from_str))
-					matched = TRUE;
-			}
-			if (!matched && *to_str && msginfo->to) {
-				if (str_find_func(msginfo->to, to_str))
-					matched = TRUE;
-			}
-			if (!matched && *subject_str && msginfo->subject) {
-				if (str_find_func(msginfo->subject, subject_str))
-					matched = TRUE;
-			}
-			if (!matched && *body_str) {
-				if (procmime_find_string(msginfo, body_str,
-							 str_find_func)) {
-					matched = TRUE;
-					body_matched = TRUE;
+			if (bool_and) {
+				matched = TRUE;
+				if (*from_str) {
+					if (!msginfo->from ||
+				    	!str_find_func(msginfo->from, from_str)) {
+						matched = FALSE;
+					}
+				}
+				if (matched && *to_str) {
+					if (!msginfo->to ||
+				    	!str_find_func(msginfo->to, to_str)) {
+						matched = FALSE;
+					}
+				}
+				if (matched && *subject_str) {
+					if (!msginfo->subject ||
+				    	!str_find_func(msginfo->subject, subject_str)) {
+						matched = FALSE;
+					}
+				}
+				if (matched && *body_str) {
+					if (procmime_find_string(msginfo, body_str,
+								 str_find_func)) {
+						body_matched = TRUE;
+					} else {
+						matched = FALSE;
+					}
+				}
+				if (matched && !*from_str && !*to_str &&
+			    	!*subject_str && !*body_str) {
+					matched = FALSE;
+				}
+			} else {
+				matched = FALSE;
+				if (*from_str && msginfo->from) {
+					if (str_find_func(msginfo->from, from_str)) {
+						matched = TRUE;
+					}
+				}
+				if (!matched && *to_str && msginfo->to) {
+					if (str_find_func(msginfo->to, to_str)) {
+						matched = TRUE;
+					}
+				}
+				if (!matched && *subject_str && msginfo->subject) {
+					if (str_find_func(msginfo->subject, subject_str)) {
+						matched = TRUE;
+					}
+				}
+				if (!matched && *body_str) {
+					if (procmime_find_string(msginfo, body_str,
+								 str_find_func)) {
+						matched = TRUE;
+						body_matched = TRUE;
+					}
 				}
 			}
 		}
 
 		if (matched) {
-			if (search_all)
+			if (search_all) {
 				gtk_ctree_select(ctree, node);
-			else {
+			} else {
 				if (messageview_is_visible
 					(summaryview->messageview)) {
 					summary_unlock(summaryview);
@@ -482,24 +614,40 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 
 		node = backward ? gtkut_ctree_node_prev(ctree, node)
 				: gtkut_ctree_node_next(ctree, node);
+
+		GTK_EVENTS_FLUSH();
 	}
 
-	if (*body_str)
-		main_window_cursor_normal(summaryview->mainwin);
-
+	search_window.is_searching = FALSE;
+	gtk_widget_hide(search_window.stop_btn);
+	gtk_widget_show(search_window.close_btn);
+	main_window_cursor_normal(summaryview->mainwin);
+	if (search_all) {
+		gtk_clist_thaw(GTK_CLIST(ctree));
+	}
 	summary_unlock(summaryview);
 }
 
 static void summary_search_clear(GtkButton *button, gpointer data)
 {
-	gtk_editable_delete_text(GTK_EDITABLE(search_window.from_entry),
-				 0, -1);
-	gtk_editable_delete_text(GTK_EDITABLE(search_window.to_entry),
-				 0, -1);
-	gtk_editable_delete_text(GTK_EDITABLE(search_window.subject_entry),
-				 0, -1);
-	gtk_editable_delete_text(GTK_EDITABLE(search_window.body_entry),
-				 0, -1);
+	if (gtk_toggle_button_get_active
+		(GTK_TOGGLE_BUTTON(search_window.adv_search_checkbtn))) {
+		gtk_editable_delete_text(GTK_EDITABLE(search_window.adv_condition_entry),
+					 0, -1);
+	} else {
+		gtk_editable_delete_text(GTK_EDITABLE(search_window.from_entry),
+					 0, -1);
+		gtk_editable_delete_text(GTK_EDITABLE(search_window.to_entry),
+					 0, -1);
+		gtk_editable_delete_text(GTK_EDITABLE(search_window.subject_entry),
+					 0, -1);
+		gtk_editable_delete_text(GTK_EDITABLE(search_window.body_entry),
+					 0, -1);
+	}
+	/* stop searching */
+	if (search_window.is_searching) {
+		search_window.is_searching = FALSE;
+	}
 }
 
 static void summary_search_prev_clicked(GtkButton *button, gpointer data)
@@ -517,6 +665,53 @@ static void summary_search_all_clicked(GtkButton *button, gpointer data)
 	summary_search_execute(FALSE, TRUE);
 }
 
+static void adv_condition_btn_done(MatcherList * matchers)
+{
+	gchar *str;
+
+	g_return_if_fail(
+			mainwindow_get_mainwindow()->summaryview->quicksearch != NULL);
+
+	if (matchers == NULL) {
+		return;
+	}
+
+	str = matcherlist_to_string(matchers);
+
+	if (str != NULL) {
+		gtk_entry_set_text(
+			GTK_ENTRY(search_window.adv_condition_entry), str);
+		g_free(str);
+	}
+}
+
+static void summary_search_stop_clicked(GtkButton *button, gpointer data)
+{
+	search_window.is_searching = FALSE;
+}
+
+static void adv_condition_btn_clicked(GtkButton *button, gpointer data)
+{
+	const gchar * cond_str;
+	MatcherList * matchers = NULL;
+
+	g_return_if_fail( search_window.window != NULL );
+
+	/* re-use it the current quicksearch value if it's a condition expression,
+	   otherwise ignore it silently */
+	cond_str = gtk_entry_get_text(
+			GTK_ENTRY(search_window.adv_condition_entry));
+	if (*cond_str != '\0') {
+		matchers = matcher_parser_get_cond((gchar*)cond_str);
+	}
+
+	prefs_matcher_open(matchers, adv_condition_btn_done);
+
+	if (matchers != NULL) {
+		matcherlist_free(matchers);
+	}
+};
+
 static void from_activated(void)
 {
 	gtk_widget_grab_focus(search_window.to_entry);
@@ -537,10 +732,24 @@ static void body_activated(void)
 	gtk_button_clicked(GTK_BUTTON(search_window.next_btn));
 }
 
+static void adv_condition_activated(void)
+{
+	gtk_button_clicked(GTK_BUTTON(search_window.next_btn));
+}
+
 static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event,
 			    gpointer data)
 {
-	if (event && event->keyval == GDK_Escape)
-		gtk_widget_hide(search_window.window);
+	if (event && event->keyval == GDK_Escape) {
+		/* ESC key will:
+			- stop a running search
+			- close the search window if no search is running
+		*/
+		if (!search_window.is_searching) {
+			gtk_widget_hide(search_window.window);
+		} else {
+			search_window.is_searching = FALSE;
+		}
+	}
 	return FALSE;
 }
