@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,6 +34,13 @@
 #include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
+#if HAVE_LIBCOMPFACE
+#  include <compface.h>
+#endif
+
+#if HAVE_LIBCOMPFACE
+#define XPM_XFACE_HEIGHT	(HEIGHT + 3)  /* 3 = 1 header + 2 colors */
+#endif
 
 #include "main.h"
 #include "summaryview.h"
@@ -54,6 +61,7 @@
 #include "menu.h"
 #include "image_viewer.h"
 #include "filesel.h"
+#include "base64.h"
 
 struct _RemoteURI
 {
@@ -92,27 +100,18 @@ static GdkColor emphasis_color = {
 	(gushort)0xcfff
 };
 
-#if 0
-static GdkColor error_color = {
-	(gulong)0,
-	(gushort)0xefff,
-	(gushort)0,
-	(gushort)0
-};
-#endif
-
-
 static GdkCursor *hand_cursor = NULL;
 static GdkCursor *text_cursor = NULL;
+static GdkCursor *watch_cursor= NULL;
 
-#define TEXTVIEW_STATUSBAR_PUSH(textview, str)					    \
-{									    \
+#define TEXTVIEW_STATUSBAR_PUSH(textview, str)				    \
+{	if (textview->messageview->statusbar)				    \
 	gtk_statusbar_push(GTK_STATUSBAR(textview->messageview->statusbar), \
 			   textview->messageview->statusbar_cid, str);	    \
 }
 
-#define TEXTVIEW_STATUSBAR_POP(textview)						   \
-{									   \
+#define TEXTVIEW_STATUSBAR_POP(textview)				   \
+{	if (textview->messageview->statusbar)				   \
 	gtk_statusbar_pop(GTK_STATUSBAR(textview->messageview->statusbar), \
 			  textview->messageview->statusbar_cid);	   \
 }
@@ -231,6 +230,29 @@ static GtkItemFactoryEntry textview_file_popup_entries[] =
 	{N_("/_Save image..."),		NULL, save_file_cb, 0, NULL},
 };
 
+static void scrolled_cb (GtkAdjustment *adj, TextView *textview)
+{
+#ifndef WIDTH
+#  define WIDTH 48
+#  define HEIGHT 48
+#endif
+	if (textview->image) {
+		gint x, y, x1;
+		x1 = textview->text->allocation.width - WIDTH - 5;
+		gtk_text_view_buffer_to_window_coords(
+			GTK_TEXT_VIEW(textview->text),
+			GTK_TEXT_WINDOW_TEXT, x1, 5, &x, &y);
+		gtk_text_view_move_child(GTK_TEXT_VIEW(textview->text), 
+			textview->image, x1, y);
+	}
+}
+
+static void textview_size_allocate_cb	(GtkWidget	*widget,
+					 GtkAllocation	*allocation,
+					 gpointer	 data)
+{
+	scrolled_cb(NULL, (TextView *)data);
+}
 
 TextView *textview_create(void)
 {
@@ -242,6 +264,7 @@ TextView *textview_create(void)
 	GtkClipboard *clipboard;
 	GtkItemFactory *link_popupfactory, *mail_popupfactory, *file_popupfactory;
 	GtkWidget *link_popupmenu, *mail_popupmenu, *file_popupmenu;
+	GtkAdjustment *adj;
 	gint n_entries;
 
 	debug_print("Creating text view...\n");
@@ -284,6 +307,14 @@ TextView *textview_create(void)
 			 G_CALLBACK(textview_leave_notify), textview);
 	g_signal_connect(G_OBJECT(text), "visibility-notify-event",
 			 G_CALLBACK(textview_visibility_notify), textview);
+	adj = gtk_scrolled_window_get_vadjustment(
+		GTK_SCROLLED_WINDOW(scrolledwin));
+	g_signal_connect(G_OBJECT(adj), "value-changed",
+			 G_CALLBACK(scrolled_cb), textview);
+	g_signal_connect(G_OBJECT(text), "size_allocate",
+			 G_CALLBACK(textview_size_allocate_cb),
+			 textview);
+
 
 	gtk_widget_show(scrolledwin);
 
@@ -323,7 +354,7 @@ TextView *textview_create(void)
 	textview->mail_popup_factory = mail_popupfactory;
 	textview->file_popup_menu    = file_popupmenu;
 	textview->file_popup_factory = file_popupfactory;
-
+	textview->image		     = NULL;
 	return textview;
 }
 
@@ -352,7 +383,7 @@ static void textview_create_tags(GtkTextView *text, TextView *textview)
 				   "pixels-below-lines", 0,
 				   "pixels-below-lines-set", TRUE,
 				   "font-desc", font_desc,
-				   "left-margin", 0,
+				   "left-margin", 3,
 				   "left-margin-set", TRUE,
 				   NULL);
 	gtk_text_buffer_create_tag(buffer, "header_title",
@@ -390,6 +421,8 @@ void textview_init(TextView *textview)
 		hand_cursor = gdk_cursor_new(GDK_HAND2);
 	if (!text_cursor)
 		text_cursor = gdk_cursor_new(GDK_XTERM);
+	if (!watch_cursor)
+		watch_cursor = gdk_cursor_new(GDK_WATCH);
 
 	textview_reflect_prefs(textview);
 	textview_set_all_headers(textview, FALSE);
@@ -479,7 +512,6 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		textview_add_parts(textview, mimeinfo);
 	else
 		textview_write_body(textview, mimeinfo);
-
 }
 
 static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
@@ -531,7 +563,9 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
 
 	g_free(content_type);			   
 
-	if (mimeinfo->type != MIMETYPE_TEXT) {
+	if (mimeinfo->disposition == DISPOSITIONTYPE_ATTACHMENT
+	|| (mimeinfo->disposition == DISPOSITIONTYPE_INLINE && 
+	    mimeinfo->type != MIMETYPE_TEXT)) {
 		gtk_text_buffer_insert(buffer, &iter, buf, -1);
 		if (mimeinfo->type == MIMETYPE_IMAGE  &&
 		    prefs_common.inline_img ) {
@@ -540,10 +574,6 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
 			gchar *filename;
 			RemoteURI *uri;
 			gchar *uri_str;
-			FILE *fp;
-
-			fp = g_fopen(mimeinfo->data.filename, "rb");
-			fseek(fp, mimeinfo->offset, SEEK_SET);
 
 			filename = procmime_get_tmp_file_name(mimeinfo);
 			if (procmime_get_part(filename, mimeinfo) < 0) {
@@ -603,7 +633,7 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
 			g_object_unref(pixbuf);
 			g_free(filename);
 		}
-	} else if (mimeinfo->disposition != DISPOSITIONTYPE_ATTACHMENT) {
+	} else if (mimeinfo->type == MIMETYPE_TEXT) {
 		if (prefs_common.display_header && (charcount > 0))
 			gtk_text_buffer_insert(buffer, &iter, "\n", 1);
 
@@ -722,7 +752,6 @@ void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 	TEXT_INSERT(_("    (alternately double-click, or click the middle "));
 	TEXT_INSERT(_("mouse button),\n"));
 	TEXT_INSERT(_("    or 'Open with...' (Shortcut key: 'o')\n"));
-
 }
 
 #undef TEXT_INSERT
@@ -789,14 +818,14 @@ static void textview_write_body(TextView *textview, MimeInfo *mimeinfo)
 static void textview_show_html(TextView *textview, FILE *fp,
 			       CodeConverter *conv)
 {
-	HTMLParser *parser;
+	SC_HTMLParser *parser;
 	gchar *str;
 
-	parser = html_parser_new(fp, conv);
+	parser = sc_html_parser_new(fp, conv);
 	g_return_if_fail(parser != NULL);
 
-	while ((str = html_parse(parser)) != NULL) {
-	        if (parser->state == HTML_HREF) {
+	while ((str = sc_html_parse(parser)) != NULL) {
+	        if (parser->state == SC_HTML_HREF) {
 		        /* first time : get and copy the URL */
 		        if (parser->href == NULL) {
 				/* ALF - the sylpheed html parser returns an empty string,
@@ -815,7 +844,7 @@ static void textview_show_html(TextView *textview, FILE *fp,
 		        textview_write_line(textview, str, NULL);
 	}
 	textview_write_line(textview, "\n", NULL);
-	html_parser_destroy(parser);
+	sc_html_parser_destroy(parser);
 }
 
 static void textview_show_ertf(TextView *textview, FILE *fp,
@@ -851,7 +880,8 @@ static void textview_show_ertf(TextView *textview, FILE *fp,
 static void textview_make_clickable_parts(TextView *textview,
 					  const gchar *fg_tag,
 					  const gchar *uri_tag,
-					  const gchar *linebuf)
+					  const gchar *linebuf,
+					  gboolean hdr)
 {
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
 	GtkTextBuffer *buffer = gtk_text_view_get_buffer(text);
@@ -869,7 +899,8 @@ static void textview_make_clickable_parts(TextView *textview,
 		gboolean  (*parse)	(const gchar *start,
 					 const gchar *scanpos,
 					 const gchar **bp_,
-					 const gchar **ep_);
+					 const gchar **ep_,
+					 gboolean hdr);
 		/* part to URI function */
 		gchar    *(*build_uri)	(const gchar *bp,
 					 const gchar *ep);
@@ -921,7 +952,7 @@ static void textview_make_clickable_parts(TextView *textview,
 
 		if (scanpos) {
 			/* check if URI can be parsed */
-			if (parser[last_index].parse(walk, scanpos, &bp, &ep)
+			if (parser[last_index].parse(walk, scanpos, &bp, &ep, hdr)
 			    && (size_t) (ep - bp - 1) > strlen(parser[last_index].needle)) {
 					ADD_TXT_POS(bp, ep, last_index);
 					walk = ep;
@@ -992,7 +1023,6 @@ static void textview_write_line(TextView *textview, const gchar *str,
 		conv_localetodisp(buf, sizeof(buf), str);
 		
 	strcrchomp(buf);
-	//if (prefs_common.conv_mb_alnum) conv_mb_alnum(buf);
 	fg_color = NULL;
 
 	/* change color of quotation
@@ -1026,7 +1056,7 @@ static void textview_write_line(TextView *textview, const gchar *str,
 		textview->is_in_signature = TRUE;
 	}
 
-	textview_make_clickable_parts(textview, fg_color, "link", buf);
+	textview_make_clickable_parts(textview, fg_color, "link", buf, FALSE);
 }
 
 void textview_write_link(TextView *textview, const gchar *str,
@@ -1096,6 +1126,9 @@ void textview_clear(TextView *textview)
 	textview->uri_list = NULL;
 
 	textview->body_pos = 0;
+	if (textview->image) 
+		gtk_widget_destroy(textview->image);
+	textview->image = NULL;
 }
 
 void textview_destroy(TextView *textview)
@@ -1131,8 +1164,8 @@ void textview_set_font(TextView *textview, const gchar *codeset)
 						(NORMAL_FONT);
 		if (font_desc) {
 			gtk_widget_modify_font(textview->text, font_desc);
-			pango_font_description_free(font_desc);
 			CHANGE_TAG_FONT("header", font_desc);
+			pango_font_description_free(font_desc);
 		}
 		if (bold_font_desc) {
 			pango_font_description_set_weight
@@ -1192,12 +1225,8 @@ enum
 void textview_set_position(TextView *textview, gint pos)
 {
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
-	GtkTextBuffer *buffer = gtk_text_view_get_buffer(text);
-	GtkTextIter iter;
 
-	gtk_text_buffer_get_iter_at_offset(buffer, &iter, pos);
-	gtk_text_buffer_place_cursor(buffer, &iter);
-	gtk_text_view_scroll_to_iter(text, &iter, 0.0, FALSE, 0.0, 0.0);
+	gtkut_text_view_set_position(text, pos);
 }
 
 static GPtrArray *textview_scan_header(TextView *textview, FILE *fp)
@@ -1257,6 +1286,90 @@ static GPtrArray *textview_scan_header(TextView *textview, FILE *fp)
 	return sorted_headers;
 }
 
+static void textview_show_face(TextView *textview)
+{
+	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
+	MsgInfo *msginfo = textview->messageview->msginfo;
+	int x = 0;
+	
+	if (prefs_common.display_header_pane
+	||  !prefs_common.display_xface)
+		goto bail;
+	
+	if (!msginfo->face) {
+		goto bail;
+	}
+
+	if (textview->image) 
+		gtk_widget_destroy(textview->image);
+	
+	textview->image = face_get_from_header(msginfo->face);
+	g_return_if_fail(textview->image != NULL);
+
+	gtk_widget_show(textview->image);
+	
+	x = textview->text->allocation.width - WIDTH -5;
+
+	gtk_text_view_add_child_in_window(text, textview->image, 
+		GTK_TEXT_WINDOW_TEXT, x, 5);
+
+	gtk_widget_show_all(textview->text);
+	
+
+	return;
+bail:
+	if (textview->image) 
+		gtk_widget_destroy(textview->image);
+	textview->image = NULL;	
+}
+
+#if HAVE_LIBCOMPFACE
+static void textview_show_xface(TextView *textview)
+{
+	MsgInfo *msginfo = textview->messageview->msginfo;
+	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
+	int x = 0;
+
+	if (prefs_common.display_header_pane
+	||  !prefs_common.display_xface)
+		goto bail;
+	
+	if (!msginfo)
+		goto bail;
+
+	if (msginfo->face)
+		return;
+	
+	if (!msginfo->xface || strlen(msginfo->xface) < 5) {
+		goto bail;
+	}
+
+	if (textview->image) 
+		gtk_widget_destroy(textview->image);
+	
+	textview->image = xface_get_from_header(msginfo->xface,
+				&textview->text->style->white,
+				textview->text->window);
+	g_return_if_fail(textview->image != NULL);
+
+	gtk_widget_show(textview->image);
+	
+	x = textview->text->allocation.width - WIDTH -5;
+
+	gtk_text_view_add_child_in_window(text, textview->image, 
+		GTK_TEXT_WINDOW_TEXT, x, 5);
+
+	gtk_widget_show_all(textview->text);
+	
+	return;
+bail:
+	if (textview->image) 
+		gtk_widget_destroy(textview->image);
+	textview->image = NULL;
+	
+}
+#endif
+
 static void textview_show_header(TextView *textview, GPtrArray *headers)
 {
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
@@ -1272,9 +1385,20 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 		g_return_if_fail(header->name != NULL);
 
 		gtk_text_buffer_get_end_iter (buffer, &iter);
-		gtk_text_buffer_insert_with_tags_by_name
-			(buffer, &iter, header->name, -1,
-			 "header_title", "header", NULL);
+		if(prefs_common.trans_hdr == TRUE) {
+			gchar *hdr = g_strndup(header->name, strlen(header->name) - 1);
+			gchar *trans_hdr = gettext(hdr);
+			gtk_text_buffer_insert_with_tags_by_name(buffer,
+				&iter, trans_hdr, -1,
+				"header_title", "header", NULL);
+			gtk_text_buffer_insert_with_tags_by_name(buffer,
+				&iter, ":", 1, "header_title", "header", NULL);
+			g_free(hdr);
+		} else {
+			gtk_text_buffer_insert_with_tags_by_name(buffer,
+				&iter, header->name,
+				-1, "header_title", "header", NULL);
+		}
 		if (header->name[strlen(header->name) - 1] != ' ')
 		gtk_text_buffer_insert_with_tags_by_name
 				(buffer, &iter, " ", 1,
@@ -1289,84 +1413,46 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 		if ((procheader_headername_equal(header->name, "X-Mailer") ||
 		     procheader_headername_equal(header->name,
 						 "X-Newsreader")) &&
-		    strstr(header->body, "Sylpheed") != NULL) {
+		    strstr(header->body, "Sylpheed-Claws") != NULL) {
 			gtk_text_buffer_get_end_iter (buffer, &iter);
 			gtk_text_buffer_insert_with_tags_by_name
 				(buffer, &iter, header->body, -1,
 				 "header", "emphasis", NULL);
 		} else {
-			textview_make_clickable_parts(textview, "header", "link",
-						      header->body);
+			gboolean hdr = 
+			  procheader_headername_equal(header->name, "From") ||
+			  procheader_headername_equal(header->name, "To") ||
+			  procheader_headername_equal(header->name, "Cc") ||
+			  procheader_headername_equal(header->name, "Bcc");
+			textview_make_clickable_parts(textview, "header", 
+						      "link", header->body, 
+						      hdr);
 		}
 		gtk_text_buffer_get_end_iter (buffer, &iter);
 		gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, "\n", 1,
 							 "header", NULL);
 	}
+	
+	textview_show_face(textview);
+#if HAVE_LIBCOMPFACE
+	textview_show_xface(textview);
+#endif
 }
 
 gboolean textview_search_string(TextView *textview, const gchar *str,
 				gboolean case_sens)
 {
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
-	GtkTextBuffer *buffer;
-	GtkTextIter iter, match_pos;
-	GtkTextMark *mark;
-	gint len;
 
-	g_return_val_if_fail(str != NULL, FALSE);
-
-	buffer = gtk_text_view_get_buffer(text);
-
-	len = g_utf8_strlen(str, -1);
-	g_return_val_if_fail(len >= 0, FALSE);
-
-	mark = gtk_text_buffer_get_insert(buffer);
-	gtk_text_buffer_get_iter_at_mark(buffer, &iter, mark);
-
-	if (gtkut_text_buffer_find(buffer, &iter, str, case_sens,
-				   &match_pos)) {
-		GtkTextIter end = match_pos;
-
-		gtk_text_iter_forward_chars(&end, len);
-		/* place "insert" at the last character */
-		gtk_text_buffer_select_range(buffer, &end, &match_pos);
-		gtk_text_view_scroll_to_mark(text, mark, 0.0, FALSE, 0.0, 0.0);
-		return TRUE;
-	}
-
-	return FALSE;
+	return gtkut_text_view_search_string(text, str, case_sens);
 }
 
 gboolean textview_search_string_backward(TextView *textview, const gchar *str,
 					 gboolean case_sens)
 {
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
-	GtkTextBuffer *buffer;
-	GtkTextIter iter, match_pos;
-	GtkTextMark *mark;
-	gint len;
 
-	g_return_val_if_fail(str != NULL, FALSE);
-
-	buffer = gtk_text_view_get_buffer(text);
-
-	len = g_utf8_strlen(str, -1);
-	g_return_val_if_fail(len >= 0, FALSE);
-
-	mark = gtk_text_buffer_get_insert(buffer);
-	gtk_text_buffer_get_iter_at_mark(buffer, &iter, mark);
-
-	if (gtkut_text_buffer_find_backward(buffer, &iter, str, case_sens,
-					    &match_pos)) {
-		GtkTextIter end = match_pos;
-
-		gtk_text_iter_forward_chars(&end, len);
-		gtk_text_buffer_select_range(buffer, &match_pos, &end);
-		gtk_text_view_scroll_to_mark(text, mark, 0.0, FALSE, 0.0, 0.0);
-		return TRUE;
-	}
-
-	return FALSE;
+	return gtkut_text_view_search_string_backward(text, str, case_sens);
 }
 
 void textview_scroll_one_line(TextView *textview, gboolean up)
@@ -1590,6 +1676,7 @@ static gint textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	case GDK_y:
 	case GDK_t:
 	case GDK_l:
+	case GDK_c:
 		if ((event->state & (GDK_MOD1_MASK|GDK_CONTROL_MASK)) == 0) {
 			KEY_PRESS_EVENT_STOP();
 			mimeview_pass_key_press_event(messageview->mimeview,
@@ -1652,6 +1739,22 @@ static gboolean textview_visibility_notify(GtkWidget *widget,
 	return FALSE;
 }
 
+void textview_cursor_wait(TextView *textview)
+{
+	GdkWindow *window = gtk_text_view_get_window(
+			GTK_TEXT_VIEW(textview->text),
+			GTK_TEXT_WINDOW_TEXT);
+	gdk_window_set_cursor(window, watch_cursor);
+}
+
+void textview_cursor_normal(TextView *textview)
+{
+	GdkWindow *window = gtk_text_view_get_window(
+			GTK_TEXT_VIEW(textview->text),
+			GTK_TEXT_WINDOW_TEXT);
+	gdk_window_set_cursor(window, text_cursor);
+}
+
 static void textview_uri_update(TextView *textview, gint x, gint y)
 {
 	GtkTextBuffer *buffer;
@@ -1710,7 +1813,11 @@ static void textview_uri_update(TextView *textview, gint x, gint y)
 		
 		window = gtk_text_view_get_window(GTK_TEXT_VIEW(textview->text),
 						  GTK_TEXT_WINDOW_TEXT);
-		gdk_window_set_cursor(window, uri ? hand_cursor : text_cursor);
+		if (textview->messageview->mainwin->cursor_count == 0) {
+			gdk_window_set_cursor(window, uri ? hand_cursor : text_cursor);
+		} else {
+			gdk_window_set_cursor(window, watch_cursor);
+		}
 
 		TEXTVIEW_STATUSBAR_POP(textview);
 
@@ -1735,24 +1842,7 @@ static gboolean textview_get_uri_range(TextView *textview,
 				       GtkTextIter *start_iter,
 				       GtkTextIter *end_iter)
 {
-	GtkTextIter _start_iter, _end_iter;
-
-	_end_iter = *iter;
-	if (!gtk_text_iter_forward_to_tag_toggle(&_end_iter, tag)) {
-		debug_print("Can't find end");
-		return FALSE;
-	}
-
-	_start_iter = _end_iter;
-	if (!gtk_text_iter_backward_to_tag_toggle(&_start_iter, tag)) {
-		debug_print("Can't find start.");
-		return FALSE;
-	}
-
-	*start_iter = _start_iter;
-	*end_iter = _end_iter;
-
-	return TRUE;
+	return get_tag_range(iter, tag, start_iter, end_iter);
 }
 
 static RemoteURI *textview_get_uri_from_range(TextView *textview,
@@ -1920,7 +2010,7 @@ static gboolean textview_uri_security_check(TextView *textview, RemoteURI *uri)
 
 		uri_path = get_uri_path(uri->uri);
 		visible_uri_path = get_uri_path(visible_str);
-		if (strcmp(uri_path, visible_uri_path) != 0)
+		if (path_cmp(uri_path, visible_uri_path) != 0)
 			retval = FALSE;
 	}
 
@@ -1934,10 +2024,10 @@ static gboolean textview_uri_security_check(TextView *textview, RemoteURI *uri)
 					"Open it anyway?"),
 				      uri->uri, visible_str);
 		aval = alertpanel_full(_("Fake URL warning"), msg,
-				       GTK_STOCK_YES, GTK_STOCK_NO, NULL, FALSE,
-				       NULL, ALERT_WARNING, G_ALERTALTERNATE);
+				       GTK_STOCK_CANCEL, _("_Open URL"), NULL, FALSE,
+				       NULL, ALERT_WARNING, G_ALERTDEFAULT);
 		g_free(msg);
-		if (aval == G_ALERTDEFAULT)
+		if (aval == G_ALERTALTERNATE)
 			retval = TRUE;
 	}
 
@@ -2075,10 +2165,10 @@ static void save_file_cb (TextView *textview, guint action, void *data)
 		
 		res = g_strdup_printf(_("Overwrite existing file '%s'?"),
 				      filename);
-		aval = alertpanel(_("Overwrite"), res, GTK_STOCK_OK, 
-				  GTK_STOCK_CANCEL, NULL);
+		aval = alertpanel(_("Overwrite"), res, GTK_STOCK_CANCEL, 
+				  GTK_STOCK_OK, NULL);
 		g_free(res);					  
-		if (G_ALERTDEFAULT != aval) 
+		if (G_ALERTALTERNATE != aval)
 			return;
 	}
 
