@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2001-2003 Hiroyuki Yamamoto and the Sylpheed-Claws team
+ * Copyright (C) 2001-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /*
@@ -97,6 +97,9 @@ static void toolbar_trash_cb			(GtkWidget	*widget,
 static void toolbar_compose_cb			(GtkWidget	*widget,
 					    	 gpointer	 data);
 
+static void toolbar_learn_cb			(GtkWidget	*widget,
+					    	 gpointer	 data);
+
 static void toolbar_reply_cb		   	(GtkWidget	*widget,
 					    	 gpointer	 data);
 
@@ -173,6 +176,7 @@ struct {
 	{ "A_GOTO_NEXT",     	N_("Go to Next Unread Message")            },
 	{ "A_IGNORE_THREAD", 	N_("Ignore thread")			   },
 	{ "A_PRINT",	     	N_("Print")				   },
+	{ "A_LEARN_SPAM",	N_("Learn Spam or Ham")			   },
 
 	{ "A_SEND",          	N_("Send Message")                         },
 	{ "A_SENDL",         	N_("Put into queue folder and send later") },
@@ -187,7 +191,7 @@ struct {
 #ifdef USE_ASPELL
 	{ "A_CHECK_SPELLING",	N_("Check spelling")                       },
 #endif
-	{ "A_SYL_ACTIONS",   	N_("Sylpheed Actions Feature")             }, 
+	{ "A_SYL_ACTIONS",   	N_("Sylpheed-Claws Actions Feature")	   }, 
 	{ "A_SEPARATOR",     	"Separator"				}
 };
 
@@ -302,15 +306,10 @@ GList *toolbar_get_action_items(ToolbarType source)
 					A_REPLY_ALL,     A_REPLY_ML,      A_FORWARD, 
 					A_TRASH , A_DELETE_REAL,       A_EXECUTE,       A_GOTO_PREV, 
 					A_GOTO_NEXT,	A_IGNORE_THREAD,  A_PRINT,
-					A_ADDRBOOK, 	 A_SYL_ACTIONS };
+					A_ADDRBOOK, 	A_LEARN_SPAM, A_SYL_ACTIONS };
 
 		for (i = 0; i < sizeof main_items / sizeof main_items[0]; i++)  {
 			items = g_list_append(items, gettext(toolbar_text[main_items[i]].descr));
-			if (main_items[i] == A_PRINT) {
-				g_print("$$$ descr %s, trans %s\n",
-					toolbar_text[main_items[i]].descr,
-					gettext(toolbar_text[main_items[i]].descr));
-			}
 		}	
 	}
 	else if (source == TOOLBAR_COMPOSE) {
@@ -330,7 +329,7 @@ GList *toolbar_get_action_items(ToolbarType source)
 		gint msgv_items[] =   { A_COMPOSE_EMAIL, A_REPLY_MESSAGE, A_REPLY_SENDER,
 				        A_REPLY_ALL,     A_REPLY_ML,      A_FORWARD,
 				        A_TRASH, A_DELETE_REAL,       A_GOTO_PREV,	  A_GOTO_NEXT,
-					A_ADDRBOOK,	 A_SYL_ACTIONS };	
+					A_ADDRBOOK,	 A_LEARN_SPAM, A_SYL_ACTIONS };	
 
 		for (i = 0; i < sizeof msgv_items / sizeof msgv_items[0]; i++) 
 			items = g_list_append(items, gettext(toolbar_text[msgv_items[i]].descr));
@@ -387,11 +386,10 @@ static void toolbar_set_default_main(void)
 		gint icon;
 		gchar *text;
 	} default_toolbar[] = {
-		{ A_RECEIVE_CUR,   STOCK_PIXMAP_MAIL_RECEIVE,         _("Get")     },
-		{ A_RECEIVE_ALL,   STOCK_PIXMAP_MAIL_RECEIVE_ALL,     _("Get All") },
+		{ A_RECEIVE_ALL,   STOCK_PIXMAP_MAIL_RECEIVE_ALL,     _("Get Mail") },
 		{ A_SEPARATOR,     0,                                 ("")         }, 
 		{ A_SEND_QUEUED,   STOCK_PIXMAP_MAIL_SEND_QUEUE,      _("Send")    },
-		{ A_COMPOSE_EMAIL, STOCK_PIXMAP_MAIL_COMPOSE,         _("Email")   },
+		{ A_COMPOSE_EMAIL, STOCK_PIXMAP_MAIL_COMPOSE,         _("Compose") },
 		{ A_SEPARATOR,     0,                                 ("")         },
 		{ A_REPLY_MESSAGE, STOCK_PIXMAP_MAIL_REPLY,           _("Reply")   }, 
 		{ A_REPLY_ALL,     STOCK_PIXMAP_MAIL_REPLY_TO_ALL,    _("All")     },
@@ -399,8 +397,10 @@ static void toolbar_set_default_main(void)
 		{ A_FORWARD,       STOCK_PIXMAP_MAIL_FORWARD,         _("Forward") },
 		{ A_SEPARATOR,     0,                                 ("")         },
 		{ A_TRASH,         STOCK_PIXMAP_TRASH,                _("Trash")   },
-		{ A_EXECUTE,       STOCK_PIXMAP_EXEC,                 _("Execute") },
-		{ A_GOTO_PREV,     STOCK_PIXMAP_UP_ARROW,             _("Previous")},
+#ifdef USE_SPAMASSASSIN_PLUGIN
+		{ A_LEARN_SPAM,	   STOCK_PIXMAP_SPAM_BTN,             _("Spam")    },
+#endif
+		{ A_SEPARATOR,     0,                                 ("")         },
 		{ A_GOTO_NEXT,     STOCK_PIXMAP_DOWN_ARROW,           _("Next")    }
 	};
 	
@@ -444,11 +444,6 @@ static void toolbar_set_default_compose(void)
 		{ A_SEPARATOR, 		0,                               ("")           	}, 
 		{ A_INSERT,    		STOCK_PIXMAP_INSERT_FILE,       _("Insert")     	},
 		{ A_ATTACH,    		STOCK_PIXMAP_MAIL_ATTACH,       _("Attach")     	},
-		{ A_SIG,       		STOCK_PIXMAP_MAIL_SIGN,         _("Signature")  	},
-		{ A_SEPARATOR, 		0,                               ("")           	},
-		{ A_EXTEDITOR, 		STOCK_PIXMAP_EDIT_EXTERN,       _("Editor")     	},
-		{ A_LINEWRAP_CURRENT,	STOCK_PIXMAP_LINEWRAP_CURRENT,  _("Wrap paragraph")   	},
-		{ A_LINEWRAP_ALL,  	STOCK_PIXMAP_LINEWRAP_ALL,      _("Wrap all")   	},
 		{ A_SEPARATOR, 		0,                               ("")           	},
 		{ A_ADDRBOOK,  		STOCK_PIXMAP_ADDRESS_BOOK,      _("Address")    	}
 	};
@@ -493,6 +488,9 @@ static void toolbar_set_default_msgview(void)
 		{ A_FORWARD,       STOCK_PIXMAP_MAIL_FORWARD,         _("Forward") },
 		{ A_SEPARATOR,     0,                                 ("")         },
 		{ A_TRASH,         STOCK_PIXMAP_TRASH,                _("Trash")   },
+#ifdef USE_SPAMASSASSIN_PLUGIN
+		{ A_LEARN_SPAM,	   STOCK_PIXMAP_SPAM_BTN,             _("Spam")    },
+#endif
 		{ A_GOTO_NEXT,     STOCK_PIXMAP_DOWN_ARROW,           _("Next")    }
 	};
 	
@@ -753,6 +751,29 @@ void toolbar_set_compose_button(Toolbar            *toolbar,
 					compose_btn_type);
 }
 
+static void activate_learn_button (Toolbar           *toolbar,
+				     ToolbarStyle      style,
+				     LearnButtonType type)
+{
+	if ((!toolbar->learn_spam_btn) || (!toolbar->learn_ham_btn))
+		return;
+
+	gtk_widget_hide(type == LEARN_SPAM ? toolbar->learn_ham_btn 
+			: toolbar->learn_spam_btn);
+	gtk_widget_show(type == LEARN_SPAM ? toolbar->learn_spam_btn
+			: toolbar->learn_ham_btn);
+	toolbar->learn_btn_type = type;	
+}
+
+void toolbar_set_learn_button(Toolbar            *toolbar, 
+				LearnButtonType  learn_btn_type)
+{
+	if (toolbar->learn_btn_type != learn_btn_type)
+		activate_learn_button(toolbar, 
+					prefs_common.toolbar_style,
+					learn_btn_type);
+}
+
 void toolbar_toggle(guint action, gpointer data)
 {
 	MainWindow *mainwin = (MainWindow*)data;
@@ -792,6 +813,10 @@ void toolbar_set_style(GtkWidget *toolbar_wid, GtkWidget *handlebox_wid, guint a
 		gtk_toolbar_set_style(GTK_TOOLBAR(toolbar_wid),
 				      GTK_TOOLBAR_BOTH);
 		break;
+	case TOOLBAR_BOTH_HORIZ:
+		gtk_toolbar_set_style(GTK_TOOLBAR(toolbar_wid),
+				      GTK_TOOLBAR_BOTH_HORIZ);
+		break;
 	default:
 		return;
 	}
@@ -980,6 +1005,34 @@ static void toolbar_compose_cb(GtkWidget *widget, gpointer data)
 	}
 }
 
+static void toolbar_learn_cb(GtkWidget *widget, gpointer data)
+{
+	ToolbarItem *toolbar_item = (ToolbarItem*)data;
+	MainWindow *mainwin;
+	MessageView *msgview;
+
+	g_return_if_fail(toolbar_item != NULL);
+
+	switch (toolbar_item->type) {
+	case TOOLBAR_MAIN:
+		mainwin = (MainWindow*)toolbar_item->parent;
+		if (mainwin->toolbar->learn_btn_type == LEARN_SPAM) 
+			mainwindow_learn(mainwin, TRUE);
+		else
+			mainwindow_learn(mainwin, FALSE);
+		break;
+	case TOOLBAR_MSGVIEW:
+		msgview = (MessageView*)toolbar_item->parent;
+		if (msgview->toolbar->learn_btn_type == LEARN_SPAM) 
+			messageview_learn(msgview, TRUE);
+		else
+			messageview_learn(msgview, FALSE);
+		break;
+	default:
+		debug_print("toolbar event not supported\n");
+	}
+}
+
 
 /*
  * Reply Message
@@ -1075,7 +1128,15 @@ static void toolbar_prev_unread_cb(GtkWidget *widget, gpointer data)
 		
 	case TOOLBAR_MSGVIEW:
 		msgview = (MessageView*)toolbar_item->parent;
+		msgview->updating = TRUE;
 		summary_select_prev_unread(msgview->mainwin->summaryview);
+		msgview->updating = FALSE;
+
+		if (msgview->deferred_destroy) {
+			debug_print("messageview got away!\n");
+			messageview_destroy(msgview);
+			return;
+		}
 		
 		/* Now we need to update the messageview window */
 		if (msgview->mainwin->summaryview->selected) {
@@ -1114,8 +1175,16 @@ static void toolbar_next_unread_cb(GtkWidget *widget, gpointer data)
 		
 	case TOOLBAR_MSGVIEW:
 		msgview = (MessageView*)toolbar_item->parent;
+		msgview->updating = TRUE;
 		summary_select_next_unread(msgview->mainwin->summaryview);
-		
+		msgview->updating = FALSE;
+
+		if (msgview->deferred_destroy) {
+			debug_print("messageview got away!\n");
+			messageview_destroy(msgview);
+			return;
+		}
+
 		/* Now we need to update the messageview window */
 		if (msgview->mainwin->summaryview->selected) {
 			GtkCTree *ctree = GTK_CTREE(msgview->mainwin->summaryview->ctree);
@@ -1312,6 +1381,7 @@ static void toolbar_buttons_cb(GtkWidget   *widget,
 		{ A_GOTO_NEXT,      	toolbar_next_unread_cb		},
 		{ A_IGNORE_THREAD,	toolbar_ignore_thread_cb	},
 		{ A_PRINT,		toolbar_print_cb		},
+		{ A_LEARN_SPAM,		toolbar_learn_cb		},
 
 		{ A_SEND,		toolbar_send_cb       		},
 		{ A_SENDL,		toolbar_send_later_cb 		},
@@ -1353,10 +1423,12 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 	GtkWidget *toolbar;
 	GtkWidget *icon_wid = NULL;
 	GtkWidget *icon_news;
+	GtkWidget *icon_ham;
 	GtkWidget *item;
 	GtkWidget *item_news;
-
+	GtkWidget *item_ham;
 	guint n_menu_entries;
+	ComboButton *getall_combo;
 	ComboButton *reply_combo;
 	ComboButton *replyall_combo;
 	ComboButton *replylist_combo;
@@ -1401,7 +1473,6 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 		toolbar_data->item_list = 
 			g_slist_append(toolbar_data->item_list, 
 				       toolbar_item);
-
 		icon_wid = stock_pixmap_widget(container, stock_pixmap_get_icon(toolbar_item->file));
 		item  = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 						toolbar_item->text,
@@ -1417,6 +1488,14 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 			gtk_tooltips_set_tip(GTK_TOOLTIPS(toolbar_tips), 
 					     toolbar_data->getall_btn, 
 					   _("Receive Mail on all Accounts"), NULL);
+			getall_combo = gtkut_combo_button_create(toolbar_data->getall_btn, NULL, 0,
+					"<GetAll>", (gpointer)toolbar_item);
+			gtk_button_set_relief(GTK_BUTTON(getall_combo->arrow),
+					      GTK_RELIEF_NONE);
+			gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
+				  		  GTK_WIDGET_PTR(getall_combo),
+				 		  _("Receive Mail on selected Account"), "Reply");
+			toolbar_data->getall_combo = getall_combo;
 			break;
 		case A_RECEIVE_CUR:
 			toolbar_data->get_btn = item;
@@ -1433,7 +1512,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 		case A_COMPOSE_EMAIL:
 			icon_news = stock_pixmap_widget(container, STOCK_PIXMAP_NEWS_COMPOSE);
 			item_news = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
-							    _("News"),
+							    _("Compose"),
 							    (""),
 							    (""),
 							    icon_news, G_CALLBACK(toolbar_buttons_cb), 
@@ -1447,6 +1526,23 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 					     toolbar_data->compose_news_btn,
 					   _("Compose News"), NULL);
 			break;
+		case A_LEARN_SPAM:
+			icon_ham = stock_pixmap_widget(container, STOCK_PIXMAP_HAM_BTN);
+			item_ham = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
+							    _("Ham"),
+							    (""),
+							    (""),
+							    icon_ham, G_CALLBACK(toolbar_buttons_cb), 
+							    toolbar_item);
+			toolbar_data->learn_spam_btn = item; 
+			gtk_tooltips_set_tip(GTK_TOOLTIPS(toolbar_tips), 
+					     toolbar_data->learn_spam_btn,
+					   _("Learn Spam"), NULL);
+			toolbar_data->learn_ham_btn = item_ham;
+			gtk_tooltips_set_tip(GTK_TOOLTIPS(toolbar_tips), 
+					     toolbar_data->learn_ham_btn,
+					   _("Learn Ham"), NULL);			
+			break;
 		case A_REPLY_MESSAGE:
 			toolbar_data->reply_btn = item;
 			gtk_tooltips_set_tip(GTK_TOOLTIPS(toolbar_tips), 
@@ -1666,6 +1762,9 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 		activate_compose_button(toolbar_data, 
 					prefs_common.toolbar_style, 
 					toolbar_data->compose_btn_type);
+	if (type != TOOLBAR_COMPOSE)
+		activate_learn_button(toolbar_data, prefs_common.toolbar_style,
+				LEARN_SPAM);
 	
 	gtk_widget_show_all(toolbar);
 	
@@ -1725,8 +1824,10 @@ void toolbar_update(ToolbarType type, gpointer data)
 
 	toolbar_style(type, prefs_common.toolbar_style, data);
 
-	if (type == TOOLBAR_MAIN)
+	if (type == TOOLBAR_MAIN) {
 		toolbar_main_set_sensitive((MainWindow*)data);
+		account_set_menu_only_toolbar();
+	}
 }
 
 void toolbar_main_set_sensitive(gpointer data)
@@ -1755,6 +1856,9 @@ void toolbar_main_set_sensitive(gpointer data)
 
 	SET_WIDGET_COND(toolbar->get_btn, M_HAVE_ACCOUNT|M_UNLOCKED);
 	SET_WIDGET_COND(toolbar->getall_btn, M_HAVE_ACCOUNT|M_UNLOCKED);
+	if (toolbar->getall_btn)
+		SET_WIDGET_COND(GTK_WIDGET_PTR(toolbar->getall_combo),
+			M_HAVE_ACCOUNT|M_UNLOCKED);
 	SET_WIDGET_COND(toolbar->compose_news_btn, M_HAVE_ACCOUNT);
 	SET_WIDGET_COND(toolbar->reply_btn,
 			M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST);
@@ -1790,12 +1894,20 @@ void toolbar_main_set_sensitive(gpointer data)
 	} else {
 		SET_WIDGET_COND(toolbar->next_btn, 0);
 	}
-
-	SET_WIDGET_COND(toolbar->trash_btn,
-			M_TARGET_EXIST|M_ALLOW_DELETE|M_UNLOCKED);
-	SET_WIDGET_COND(toolbar->delete_btn,
-			M_TARGET_EXIST|M_ALLOW_DELETE|M_UNLOCKED);
-	SET_WIDGET_COND(toolbar->exec_btn, M_DELAY_EXEC);
+	
+	if (toolbar->trash_btn)
+		SET_WIDGET_COND(toolbar->trash_btn,
+			M_TARGET_EXIST|M_ALLOW_DELETE);
+	if (toolbar->delete_btn)
+		SET_WIDGET_COND(toolbar->delete_btn,
+			M_TARGET_EXIST|M_ALLOW_DELETE);
+	if (toolbar->exec_btn)
+		SET_WIDGET_COND(toolbar->exec_btn, M_DELAY_EXEC);
+	
+	if (toolbar->learn_ham_btn)
+		SET_WIDGET_COND(toolbar->learn_ham_btn, M_TARGET_EXIST|M_CAN_LEARN_SPAM);
+	if (toolbar->learn_spam_btn)
+		SET_WIDGET_COND(toolbar->learn_spam_btn, M_TARGET_EXIST|M_CAN_LEARN_SPAM);
 
 	for (cur = toolbar->action_list; cur != NULL;  cur = cur->next) {
 		ToolbarSylpheedActions *act = (ToolbarSylpheedActions*)cur->data;
@@ -1874,6 +1986,7 @@ void toolbar_init(Toolbar * toolbar) {
 	toolbar->toolbar          	= NULL;
 	toolbar->get_btn          	= NULL;
 	toolbar->getall_btn       	= NULL;
+	toolbar->getall_combo       	= NULL;
 	toolbar->send_btn         	= NULL;
 	toolbar->compose_mail_btn 	= NULL;
 	toolbar->compose_news_btn 	= NULL;
@@ -1964,7 +2077,8 @@ void send_queue_cb(gpointer data, guint action, GtkWidget *widget)
 	if (prefs_common.work_offline)
 		if (alertpanel(_("Offline warning"), 
 			       _("You're working offline. Override?"),
-			       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
+			       GTK_STOCK_NO, GTK_STOCK_YES,
+			       NULL) != G_ALERTALTERNATE)
 		return;
 
 	/* ask for confirmation before sending queued messages only
@@ -1983,7 +2097,8 @@ void send_queue_cb(gpointer data, guint action, GtkWidget *widget)
 		if (found && !prefs_common.work_offline) {
 			if (alertpanel(_("Send queued messages"), 
 			    	   _("Send all queued messages?"),
-			    	   GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
+			    	   GTK_STOCK_CANCEL, _("_Send"),
+				   NULL) != G_ALERTALTERNATE)
 				return;
 		}
 	}
@@ -1996,7 +2111,6 @@ void send_queue_cb(gpointer data, guint action, GtkWidget *widget)
 					       prefs_common.savemsg) < 0)
 				alertpanel_error(_("Some errors occurred while "
 						   "sending queued messages."));
-			folder_item_scan(folder->queue);
 		}
 	}
 }
@@ -2008,7 +2122,7 @@ void compose_mail_cb(gpointer data, guint action, GtkWidget *widget)
 	FolderItem *item = mainwin->summaryview->folder_item;	
         GList * list;
         GList * cur;
-
+	
 	if (item) {
 		ac = account_find_from_item(item);
 		if (ac && ac->protocol != A_NNTP) {
@@ -2048,9 +2162,8 @@ void compose_news_cb(gpointer data, guint action, GtkWidget *widget)
 	if (mainwin->summaryview->folder_item) {
 		ac = mainwin->summaryview->folder_item->folder->account;
 		if (ac && ac->protocol == A_NNTP) {
-			compose_new(ac,
-				    mainwin->summaryview->folder_item->path,
-				    NULL);
+			compose_new_with_folderitem(ac,
+				    mainwin->summaryview->folder_item);
 			return;
 		}
 	}
@@ -2059,7 +2172,8 @@ void compose_news_cb(gpointer data, guint action, GtkWidget *widget)
 	for(cur = list ; cur != NULL ; cur = g_list_next(cur)) {
 		ac = (PrefsAccount *) cur->data;
 		if (ac->protocol == A_NNTP) {
-			compose_new(ac, NULL, NULL);
+			compose_new_with_folderitem(ac,
+				    mainwin->summaryview->folder_item);
 			return;
 		}
 	}
