@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /* code ported from gedit */
@@ -233,40 +233,23 @@ static gint undo_merge(GList *list, guint start_pos, guint end_pos,
 	}
 
 	if (action == UNDO_ACTION_DELETE) {
-		gboolean checkit = TRUE;
-
 		if (last_undo->start_pos != end_pos &&
 		    last_undo->start_pos != start_pos) {
 			last_undo->mergeable = FALSE;
 			return FALSE;
 		} else if (last_undo->start_pos == start_pos) {
 			/* Deleted with the delete key */
-			if (text[0] != ' ' && text[0] != '\t' &&
-			    (last_undo->text[last_undo->end_pos - last_undo->start_pos - 1] == ' ' ||
-			     last_undo->text[last_undo->end_pos - last_undo->start_pos - 1] == '\t'))
-				checkit = FALSE;
-
 			temp_string = g_strdup_printf("%s%s", last_undo->text, text);
 			last_undo->end_pos++;
 			g_free(last_undo->text);
 			last_undo->text = temp_string;
 		} else {
 			/* Deleted with the backspace key */
-			if (text[0] != ' ' && text[0] != '\t' &&
-			    (last_undo->text[0] == ' ' ||
-			     last_undo->text[0] == '\t'))
-				checkit = FALSE;
-
 			temp_string = g_strdup_printf("%s%s", text, last_undo->text);
 			last_undo->start_pos = start_pos;
 			g_free(last_undo->text);
 			last_undo->text = temp_string;
 		}
-
-		if (!checkit) {
-			last_undo->mergeable = FALSE;
-			return FALSE;
-		}
 	} else if (action == UNDO_ACTION_INSERT) {
 		if (last_undo->end_pos != start_pos) {
 			last_undo->mergeable = FALSE;
@@ -537,18 +520,39 @@ void undo_unblock(UndoMain *undostruct)
 					  undostruct);
 }
 
+void undo_wrapping(UndoMain *undostruct, gboolean wrap)
+{
+	debug_print("undo wrapping now %d\n", wrap);
+	undostruct->wrap = wrap;
+}
+
 void undo_insert_text_cb(GtkTextBuffer *textbuf, GtkTextIter *iter,
 			 gchar *new_text, gint new_text_length,
 			 UndoMain *undostruct) 
 {
 	gchar *text_to_insert;
 	gint pos;
-
 	if (prefs_common.undolevels <= 0) return;
 
 	pos = gtk_text_iter_get_offset(iter);
-
+	if (undostruct->wrap && undostruct->undo) {
+		UndoInfo *last_undo = undostruct->undo->data;
+		if (last_undo && last_undo->action == UNDO_ACTION_INSERT
+		&&  last_undo->start_pos < pos && last_undo->end_pos > pos) {
+			GtkTextIter start,end;
+			last_undo->end_pos += g_utf8_strlen(new_text, -1);
+			gtk_text_buffer_get_iter_at_offset(textbuf, &start, last_undo->start_pos);
+			gtk_text_buffer_get_iter_at_offset(textbuf, &end, last_undo->end_pos);
+			g_free(last_undo->text);
+			last_undo->text = gtk_text_buffer_get_text(textbuf, &start, &end, FALSE);
+			debug_print("add:undo upd %d-%d\n", last_undo->start_pos, last_undo->end_pos);
+			return;
+		} else debug_print("add:last: %d, %d-%d (%d)\n", last_undo->action,
+			last_undo->start_pos, last_undo->end_pos, pos);
+		
+	} 
 	Xstrndup_a(text_to_insert, new_text, new_text_length, return);
+	debug_print("add:undo add %d-%ld\n", pos, pos + g_utf8_strlen(text_to_insert, -1));
 	undo_add(text_to_insert, pos, pos + g_utf8_strlen(text_to_insert, -1),
 		 UNDO_ACTION_INSERT, undostruct);
 }
@@ -567,6 +571,23 @@ void undo_delete_text_cb(GtkTextBuffer *textbuf, GtkTextIter *start,
 	start_pos = gtk_text_iter_get_offset(start);
 	end_pos   = gtk_text_iter_get_offset(end);
 
+	if (undostruct->wrap && undostruct->undo) {
+		UndoInfo *last_undo = undostruct->undo->data;
+		if (last_undo && last_undo->action == UNDO_ACTION_INSERT
+		&&  last_undo->start_pos < start_pos && last_undo->end_pos > end_pos) {
+			GtkTextIter start,end;
+			last_undo->end_pos -= g_utf8_strlen(text_to_delete, -1);
+			gtk_text_buffer_get_iter_at_offset(textbuf, &start, last_undo->start_pos);
+			gtk_text_buffer_get_iter_at_offset(textbuf, &end, last_undo->end_pos);
+			g_free(last_undo->text);
+			last_undo->text = gtk_text_buffer_get_text(textbuf, &start, &end, FALSE);
+			debug_print("del:undo upd %d-%d\n", last_undo->start_pos, last_undo->end_pos);
+			return;
+		} else debug_print("del:last: %d, %d-%d (%d)\n", last_undo->action,
+			last_undo->start_pos, last_undo->end_pos, start_pos);
+		
+	} 
+	debug_print("del:undo add %d-%d\n", start_pos, end_pos);
 	undo_add(text_to_delete, start_pos, end_pos, UNDO_ACTION_DELETE,
 		 undostruct);
 	g_free(text_to_delete);
@@ -574,10 +595,6 @@ void undo_delete_text_cb(GtkTextBuffer *textbuf, GtkTextIter *start,
 
 void undo_paste_clipboard_cb(GtkTextView *textview, UndoMain *undostruct)
 {
-#if 0
-	if (editable->clipboard_text == NULL) return;
-#endif
-
 	if (prefs_common.undolevels > 0)
 		if (undo_get_selection(textview, NULL, NULL))
 			undostruct->paste = TRUE;
